//
// Tutorial2.java
//
// A basic JMF tutorial
//
// stage 2
//
// Copyright (C) 1995,1996,1997,1998 RealNetworks, Inc.
// All rights reserved.
//


import java.awt.*;
import java.awt.event.*;

import javax.media.*;  


public class Tutorial2 extends WindowAdapter implements ControllerListener
{
	private javax.media.Player player = null;
	private String urlString = null;
	private java.awt.Component visualComponent = null;
	private java.awt.Frame frame = null;

	public static void main(String[] args)
	{
		System.out.println("Welcome to JMF for RealSystem G2 Tutorial2.");

		if (args.length == 0 || args[0] == null || args[0].length() == 0)
		{
			System.out.println("Tutorial2: Usage - java Tutorial2 <url>");
			return;
		}

		new Tutorial2(args[0]);
	}

	public Tutorial2(String urlString)
	{
		// save URL for later use

		this.urlString = urlString;

		// first create MediaLocator

		MediaLocator mrl = new MediaLocator(urlString);

		// create Player object 

		try
		{
			player = Manager.createPlayer(mrl);
		}
		catch (Exception e)
		{
			System.err.println("Tutorial2: Unable to create player for URL = " + urlString);
			System.err.println(e);
			return;
		}

		System.out.println("Tutorial2: beginning playback now.");

		// now we have our player, lets start it! 
		player.addControllerListener(this);
		player.start();
	}

	public void windowClosing(WindowEvent e)
	{
		// if the player is still playing, stop it before we close	
		if (player.getState() == Controller.Started)
		{
			player.stop();
		}

		frame.hide();
		player.close();
	}

	public void controllerUpdate(ControllerEvent e)
	{
		// realizing is done - we can get the VisualComponent now! 
		if (e instanceof RealizeCompleteEvent)
		{
			visualComponent = player.getVisualComponent();
			if (visualComponent != null)
			{
				frame = new Frame(urlString);
				frame.setLayout(new BorderLayout());
				frame.addNotify();

				Dimension vsize = visualComponent.getSize();
				int xInsets = frame.getInsets().left + frame.getInsets().right;
				int yInsets = frame.getInsets().top + frame.getInsets().bottom;

				frame.setSize(vsize.width + xInsets, vsize.height + yInsets + 1);

				frame.add(visualComponent, "Center");
				frame.setVisible(true);

				frame.addWindowListener(this);
			}
		}
		// playback is complete, shut down the player
		else if (e instanceof EndOfMediaEvent)
		{
			player.close();
		}
		else if (e instanceof ControllerClosedEvent)
		{
			if (frame != null)
			{
				frame.dispose();
			}

			System.exit(0);
		}
	}
}
 
