//
// Tutorial4.java
//
// This tutorial demonstrates how to add a progress control bar.
// 
// Copyright (C) 1995,1996,1997,1998 RealNetworks, Inc.
// All rights reserved.
//


import java.awt.*;
import java.awt.event.*;

import javax.media.*;  


public class Tutorial4 extends WindowAdapter implements ControllerListener
{
	private javax.media.Player player = null;
	private String urlString = null;
	private java.awt.Component visualComponent = null;
	private java.awt.Frame frame = null;

	private javax.media.CachingControl cacheControl = null;

	private TutorialPanel lowerPanel = null;
	private java.awt.Component controlPanel = null;
	private java.awt.Component progBar = null;

	private long currentPercentDone;

	public static void main(String[] args)
	{
		System.out.println("Welcome to JMF for RealSystem G2 Tutorial4.");

		if (args.length == 0 || args[0] == null || args[0].length() == 0)
		{
			System.out.println("Tutorial4: Usage - java Tutorial4 <url>");
			return;
		}

		new Tutorial4(args[0]);
	}

	public Tutorial4(String urlString)
	{
		//com.real.util.DebugOut.setDebug(true);

		// save URL for later use

		this.urlString = urlString;

		// first create MediaLocator

		MediaLocator mrl = new MediaLocator(urlString);

		// create Player object 

		try
		{
			player = Manager.createPlayer(mrl);
		}
		catch (Exception e)
		{
			System.err.println("Tutorial4: Unable to create player for URL = " + urlString);
			System.err.println(e);
			return;
		}

		System.out.println("Tutorial4: beginning playback now.");

		// now we have our player, lets start it! 
		player.addControllerListener(this);
		player.start();
	}

	public void windowClosing(WindowEvent e)
	{
		// if the player is still playing, stop it before we close	
		if (player.getState() == Controller.Started)
		{
			player.stop();
		}

		frame.hide();
		player.close();
	}

	public void controllerUpdate(ControllerEvent e)
	{
		// realizing is done - we can get the VisualComponent now! 
		if (e instanceof RealizeCompleteEvent)
		{
			// create frame first
			frame = new Frame(urlString);
			frame.setLayout(new BorderLayout());
			frame.addNotify();

			// get frame insets 
			int xInsets = frame.getInsets().left + frame.getInsets().right;
			int yInsets = frame.getInsets().top + frame.getInsets().bottom;

			// get the control panel 
			controlPanel = player.getControlPanelComponent();
			Dimension csize = controlPanel.getSize();

			// get progress bar component
			cacheControl = (javax.media.CachingControl) player.getControl("com.real.media.RMCachingControl");
			progBar = cacheControl.getProgressBarComponent();

			// create lower panel using controlPanel & progBar then add to frame
			lowerPanel = new TutorialPanel(controlPanel, progBar, currentPercentDone);
			frame.add(lowerPanel, "South");

			int height = csize.height + yInsets;
			int width = csize.width + xInsets;

			// handle the visual component 
			Dimension vsize = null; 
			visualComponent = player.getVisualComponent();
			if (visualComponent != null)
			{
				vsize = visualComponent.getSize();
				frame.add(visualComponent, "Center");

				width = vsize.width + xInsets;
				height += vsize.height;
			}

			frame.setSize(width, height);
			frame.setVisible(true);
			frame.addWindowListener(this);
		}
		else if (e instanceof com.real.media.RMTACChangedEvent)
		{
			com.real.media.RMTACChangedEvent tac = (com.real.media.RMTACChangedEvent) e;
			System.out.println("TAC = Title: " + tac.getTitle() + " Author: " + tac.getAuthor() + " Copyright: " + tac.getCopyright());
		}
		else if (e instanceof ControllerClosedEvent)
		{
			if (frame != null)
			{
				frame.dispose();
			}

			System.exit(0);
		}
		else if (e instanceof CachingControlEvent)
		{
			// for file based playback, 100% buffering occurs almost
			// instantly - consequently, the RealizeComplete event 
			// has not yet been sent so no lowerPanel has been
			// created yet. 

			CachingControlEvent cce = (CachingControlEvent) e;
			currentPercentDone = cce.getContentProgress();

			if (lowerPanel != null)
			{
				if (currentPercentDone > 0 && currentPercentDone < 100 && 
									lowerPanel.isCPShowing())
				{
					lowerPanel.showProgBar();	
				}
				else if (currentPercentDone == 100 && lowerPanel.isPBShowing())
				{
					lowerPanel.showControlPanel();
				}
			}
		}
	}
}

