// 	 
// ICanvas.java
//
// Part of the JMF demonstration Applet
//


import java.awt.*;
import java.net.*;
import java.io.*;


// canvas class which loads and displays an image
public class ICanvas extends Canvas implements Runnable
{
	// class data
	private MediaTracker m_tracker;
	private Image m_image;
	private String m_imageName;

	private boolean m_enable;
	public void setEnable(boolean b)	{ m_enable = b; }
	public boolean getEnable()	{ return m_enable; }
	
	private boolean isErr;
	public boolean isError()	{ return(isErr); } 

	private URL codeBaseURL;
	private URL imageURL;
	private Thread t;


	// constructors	
	public ICanvas(String imgName) throws MalformedURLException
	{
		this();
		loadImage(imgName);
	}

	public ICanvas()
	{
		m_imageName = "";
		m_tracker = null;
		m_image = null;
		isErr = false;
		m_enable = true;
		codeBaseURL = null;
		imageURL = null;
		t = null;
	}

	public void setCodebase(URL codeBaseURL)
	{
		this.codeBaseURL = codeBaseURL;
	}

	public void run()
	{
		if (imageURL != null)
		{
			System.out.println("ICanvas: run - calling Toolkit.getImage().");

			// get image from toolkit using URL
			m_image = getToolkit().getImage(imageURL);

			// create media tracker 
			m_tracker = new MediaTracker(this);
			m_tracker.addImage(m_image, 0);

			// wait for image to load 
			while (!m_tracker.checkID(0, true)) {};

			if (m_tracker.isErrorID(0))
			{
				System.err.println("ICanvas: run - Error loading image " + m_imageName  + ".");
				isErr = true;
			}
		}
		else
		{
			System.err.println("ICanvas: loadImage - Image URL is 'null'.");
		}

		t = null;
	}

	public void loadImage(String imgName)
	{
		System.out.println("ICanvas: loadImage - entered.");

		m_imageName = imgName;

		if (codeBaseURL == null)
		{
			return;
		}

		System.out.println("ICanvas: loadImage - CodeBase is: " + codeBaseURL + ".");
		try
		{
			// use alternate method for creating URL
			imageURL = new URL(codeBaseURL, m_imageName);

			System.out.println("ICanvas: loadImage - Image URL: " + imageURL + ".");
		}
		catch (Exception ee)
		{
			System.err.println(ee.toString());
			System.err.println("URL for " + m_imageName + " is STILL 'null'.");
			return;
		}

		// begin image loading thread
		t = new Thread(this);
		t.start();
	}
	
	public void update(Graphics g)
	{
		paint(g);
	}

	public void paint(Graphics g)
	{
		if (m_enable)
		{
			if (m_image != null && !isErr) 
			{
				
				g.drawImage(m_image, 1, 1, this);
			}
		}
	}
}
																	   