/*
 * PApplet.java
 *
 * Copyright (C) 1998 RealNetworks, Inc.
 * All rights reserved.
 *  
 * http://www.real.com/devzone
 *
 * This program contains proprietary information of RealNetworks, Inc.,
 * and is licensed subject to restrictions on use and distribution.
 *
 * PApplet.java
 *
 * Demonstrates playing RM clips in Java applet
 */

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;
import java.util.*;

import javax.media.*;
import javax.media.protocol.*;

/**
 * Media player applet
 */
public class PApplet extends Applet implements ControllerListener, 
					       ActionListener, 
					       ItemListener
{
    // class data 
    private javax.media.Player player = null;
    private Component visualComponent = null;

    private Component controlComponent = null;
    private com.real.media.gui.RMProgressWindow progWindow = null;
    private com.real.media.RMTACControl tac = null;
    private com.real.media.gui.RMTACDlg tacDialog = null;

    private GainControl gainControl = null;
    private Component gainComponent = null;
    private String currentURL = "";
    private boolean destroying = false;

    // components
    private Label label1;
    private Label label2;
    private Label label3;
    private Label label4;
    private Choice catalog;
    private TextField stopTime;
    private TextField url;
    private Button openURLButton;
    private ICanvas frameBox;
    private Checkbox loopCheckbox;
    private TextField percentCompleteComponent;
    private Button tacButton;

    /**
     *  Initialize the applet
     */
    public void init()
    {
		System.out.println("PApplet: Welcome to the JMF Player for RealSystem G2!\n");

		com.real.util.DebugOut.setDebug(true);

        // position the applet controls
        placeControls();

        // position the volume control panel
        placeGainControl();

        // center the image placeholder box
        placeFrameBox();
        frameBox.setCodebase(getCodeBase());

        try
        {
            frameBox.loadImage("realplayer.gif");
        }
        catch (Exception e)
        {
            System.out.println("PApplet: Exception trying to call ICanvas.loadImage. " + e.toString() + ".");
        }

        // set up event handler for open button
        openURLButton.addActionListener(this);
		tacButton.addActionListener(this);

        // open catalog file if one was provided in the HTML
        String catFileName = super.getParameter("CATALOG_FILE");
        if (catFileName != null && catFileName.length() > 0)
        {
            try
            {
                // open file for reading
                URL catURL = new URL(getCodeBase(), catFileName);
                InputStream is = catURL.openStream();
                DataInputStream dis = new DataInputStream(is);

                // read a line at a time & load choice component 
                String line = dis.readLine();
                while (dis.available() > 0 && line != null)
                {
                    catalog.add(line);
                    line = dis.readLine();
                }
            }
            catch(Exception ce)
            {
                System.err.println("Unable to load catalog: " + ce);
            }
        }
        catalog.addItemListener(this);
    }

    public void start()
    {
        if (player != null)
        {
            player.start();
        }
    }

    public void stop()
    {
        if (player != null)
        {
            player.deallocate();
        }
    }

    public void destroy()
    {
        destroying = true;

        if (player != null)
        {
            player.close();
        }
    }

    /**
     *  Draw a frame around the visual component
     */
    public void paint(Graphics g)
    {
        super.paint(g);

        // clear background first
        Color c = g.getColor();
        g.setColor( getBackground() );
        g.fillRect(0, 0, getBounds().width, getBounds().height);
        g.setColor(c);

        // get the vis components location & size (if one exists at this time)
        Rectangle rect;
        if (visualComponent != null)
        {
            // shut down the frame box & hide it for now
            frameBox.enable(false);
            frameBox.setVisible(false);

            // get dimensions of the visual component
            rect = visualComponent.getBounds();
        }
        else    // there is no visual component yet, so lets draw a nice empty hole 
        {
            rect = frameBox.getBounds();
        }

        // draw frame around visual component
        rect.grow(2,2);
        Rectangle box = new Rectangle(rect.x-2, rect.y-2, rect.width+3, rect.height+3);
        g.setColor( getBackground() );
        g.draw3DRect( box.x, box.y, box.width, box.height, true /* raise */ );
        box.grow( -1, -1 );
        g.draw3DRect( box.x, box.y, box.width, box.height, false /*  lower */ );

        // draw dropped border between video portion and data portion 
        box = new Rectangle(0, 115, getBounds().width, 3);
        g.setColor( getBackground() );
        g.draw3DRect( box.x, box.y, box.width, box.height, false /* raise */ );
        box = new Rectangle(0, 116, getBounds().width, 1);        
        g.draw3DRect( box.x, box.y, box.width, box.height, true /*  lower */ );
    }

    /**
     *  ControllerListener implementation
     */
    public synchronized void controllerUpdate(ControllerEvent event) 
    {
        if (player == null)
        {
                return;
        }

        // When the player is Realized, get the visual
        // and control components and add them to the Applet's panel
        if (event instanceof RealizeCompleteEvent) 
        {
		    System.out.println("PApplet: RealizeCompleteEvent - Entered.");

	        // if we don't already have a control component
	        if (controlComponent == null)
	        {
	            if (( controlComponent = player.getControlPanelComponent()) != null)
	            {
			add(controlComponent);

			// position the control panel
			placeControlPanel();
	            }
	        }

	        // if we don't already have a visual component
	        if (visualComponent == null)
	        {
	            // check if we have a visual component for this url
	            if (( visualComponent = player.getVisualComponent()) != null)
	            {
	                add(visualComponent);
	            }
	            else
	            {
	                // make our default image display for now
	                frameBox.enable(true);
	                frameBox.setVisible(true);
	            }
	        }

	        // by now we can be sure our vis comp is sized and ready for final placement
	        placeVisualComponent();

	        // if we don't have a gain control component yet, get it now!
	        if (gainComponent == null)
	        {
	            gainControl = player.getGainControl();
	            if (gainControl != null)
	            {
	                gainComponent = gainControl.getControlComponent();
	                if (gainComponent != null)
	                {
	                    add(gainComponent);
	                    gainComponent.setVisible(true);

	                    placeGainControl();
	                }
	            }
	        }

		    // turn off "sticky" size for visual component - comp will adjust size
		    // when new clip is played as part of RAM or SMIL presentation.
		    // With "stickySize" on (default) the first image sets the size for all.

		    //((com.real.media.RMPlayer) player).setStickySize(false);

		    // force applet to repaint its controls
		    validate();
		    repaint();
        }
        else if (event instanceof DeallocateEvent)
		{
	    	System.out.println("PApplet: DeallocateEvent - Entered.");
		}
        else if (event instanceof PrefetchCompleteEvent)
        {
	    	System.out.println("PApplet: PrefetchCompleteEvent - Entered.");

            // see if the user entered a "Stop Time"
            try
            {
                String st = stopTime.getText();
                int nst = Integer.parseInt(st);
                if (nst > 0 && nst != player.getStopTime().getNanoseconds())
                {
                    player.setStopTime(new javax.media.Time(nst));
                }
            }
            catch (NumberFormatException nfe)
            {
                player.setStopTime(Clock.RESET);
            }

  	    	// get TAC control from player
		    tac = (com.real.media.RMTACControl) player.getControl("com.real.media.RMTACControl");
		    if (tac != null && tacDialog != null)
	    	{
				// create TAC dialog (but don't display)
				tacDialog.updateFromTAC(url.getText(), tac);
		    }
    	}
        else if (event instanceof CachingControlEvent)
        {
            CachingControlEvent cce = (CachingControlEvent) event;
            long percentComplete = cce.getContentProgress();

            if (progWindow == null && cce.getCachingControl() instanceof com.real.media.RMCachingControl)
            {
                progWindow = com.real.media.gui.RMProgressWindow.makeProgressWindow(getTopFrame(this), player, cce.getCachingControl());
            }

            if (progWindow != null)
            {
                if (percentComplete == 0)
                {
                    progWindow.showAt(480,180);
                }
                else if (percentComplete == 100)
                {
                    progWindow.setVisible(false);
                    progWindow.dispose();
                    progWindow = null;
                }
            }

            String percentText = String.valueOf(percentComplete) + "%";
            percentCompleteComponent.setText(percentText);
        }
        else if (event instanceof com.real.media.RMSizeChangingEvent)
        {
	    	System.out.println("PApplet: RMSizeChangingEvent - Entered.");

            // visual component has changed its size - so lets re-position it
            Rectangle newRect = ((com.real.media.RMSizeChangingEvent) event).getBounds();

	    	// position visual component now
            if (visualComponent != null)
            {
                placeVisualComponent(newRect.width, newRect.height);
            }
        }
        else if (event instanceof ResourceUnavailableEvent)    // realize or prefetch failed
        {
            if (!destroying)
            {
                  // start hour glass
                setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));            
                System.err.println("PApplet: controllerUpdate - ResourceUnavailableEvent. " + ((ResourceUnavailableEvent) event).getMessage());
            }

            if (progWindow != null)
            {
                progWindow.setVisible(false);
                progWindow.dispose();
                progWindow = null;
            }
        }
        else if (event instanceof ControllerErrorEvent)
        {
            if (!destroying)
            {
                  // start hour glass
                setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));            
                System.err.println("PApplet: controllerUpdate - ControllerErrorEvent. " + ((ControllerErrorEvent) event).getMessage());
            }
        }
        else if (event instanceof StopTimeChangeEvent)
        {
        }
        else if (event instanceof RestartingEvent)
        {
            // start hour glass
            setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));            
        }
        else if (event instanceof StartEvent)
        {
            // reset status text
            showStatus("");

            // stop hour glass
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));            
        }
        else if (event instanceof DurationUpdateEvent)
        {
        }
        else if (event instanceof EndOfMediaEvent)
        {
	    	System.out.println("PApplet: EndOfMediaEvent.");

            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));            

            // if the "loop" checkbox is checked, re-start playback here! 
            if (loopCheckbox.getState())
            {
				System.out.println("PApplet: EndOfMediaEvent received. Restarting playback.");

				player.setMediaTime(new javax.media.Time(0));
                player.start();
            }
        }
        else if (event instanceof StopEvent)
        {
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));            

            if (!destroying)
            {
                // stop hour glass
                setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));            
            }

            if (progWindow != null)
            {
                progWindow.setVisible(false);
                progWindow.dispose();
                progWindow = null;
            }
        }
        else if (event instanceof TransitionEvent)
        {
            TransitionEvent te = (TransitionEvent) event;
            if (te.getTargetState() == Controller.Prefetched)
            {
                // start hour glass
                setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));            
            }
        }
    }

    /**
     *  ActionListener implementation
     */
    public void actionPerformed(ActionEvent event)
    {
        // if our button caused this event
        if (event.getSource() == openURLButton) 
        {
			openURLButtonClicked();
        }
		else if (event.getSource() == tacButton)
		{
			if (tac != null)
			{
				if (tacDialog == null )
				{
					tacDialog = com.real.media.gui.RMTACDlg.makeTACDialog(getTopFrame(this), url.getText(), tac);
					
				}
				tacDialog.show();
			}
			else
			{
				showStatus("You cannot display the TAC dialog until Prefetching is complete.");
			}
		}
    }

    /**
     *  ItemListener implementation
     */
    public void itemStateChanged(ItemEvent e)
    {
        showStatus("");

        String urlToPlay = (String) e.getItem();
        if (urlToPlay != null && urlToPlay.length() > 0 &&
            urlToPlay.indexOf("http") >= 0 ||
            urlToPlay.indexOf("file") >= 0 ||
            urlToPlay.indexOf("rtsp") >= 0 ||
            urlToPlay.indexOf("pnm") >= 0)
        {
            String urlString = urlToPlay.trim();

            if (!urlString.equalsIgnoreCase(currentURL))
            {
                url.setText(urlString);
                playURL(urlString);
            }
        }
    }

    /**
     *  Create a media player with the given URL
     */
    protected boolean createPlayer(String urlString)
    {
        boolean res = true;

        String msg = "";
        Exception ee = null;
        MediaLocator mrl = new MediaLocator(urlString);

        if (mrl != null)
        {
            // first create DataSource
            try 
            {
                // Create an instance of a player for this media
                DataSource ds = javax.media.Manager.createDataSource(mrl);
                if (ds != null)
                {
                    try
                    {
                        player = javax.media.Manager.createPlayer(ds);
                    }
                    catch (Exception e)
                    {
                        msg = "unable to create Player. Exception: " + e.toString();
                        res = false;
                        ee = e;
                    }
                }
            }
            catch (Exception e)
            {
                msg = "unable to create DataSource. Exception: " + e.toString();
                res = false;
                ee = e;
            }
        }

        // report error to user if we could not create a player
        if (player == null || ee != null)
        {
            String text = "PApplet: createPlayer - " + msg;
            showStatus(text);

            System.err.println(text);
        }
        else if (player != null)
        {
            player.addControllerListener(this);
        }

        return res;
    }

    /**
     *  Handle Open button click
     */
    protected void openURLButtonClicked()
    {
        showStatus("");

        // get the url text from the TextField & see if it seems valid (a rudimentary analysis of course)
        String urlString = url.getText();
        if (urlString != null && urlString.length() > 0 && !urlString.equalsIgnoreCase(currentURL))
        {
            playURL(urlString);

        }
        else if (urlString != null && urlString.equalsIgnoreCase(currentURL))       // same exact URL - act like "Play"
        {
            if (player != null && player.getState() != Player.Started)
            {
                //player.syncStart(new Time(14000000));
                player.start();
            }
        }
    }

    /**
     *  Create a new player and play a new URL
     */
    private void playURL(String urlString)
    {
        currentURL = urlString;

        // start hour glass
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));            

		int oldPlayerState = Controller.Started;
		if (player != null)
		{
	    		oldPlayerState = player.getState();
		}

        // save old player 
        Player oldPlayer = player;
        player = null;

        // if we have a previous player, close it down        
        if (oldPlayer != null)
        {
            // stop player if it is in progress
            if (oldPlayerState == Controller.Started ||
                oldPlayerState == Controller.Realizing || 
                oldPlayerState == Controller.Prefetching)
            {
                oldPlayer.stop();
            }

            if (progWindow != null)
            {
                progWindow.setVisible(false);
                progWindow.dispose();
                progWindow = null;
            }

            // remove any old control component from the applet
            if (controlComponent != null)
            {
                oldPlayer.removeControllerListener((ControllerListener) controlComponent);
                controlComponent.setVisible(false);

                remove(controlComponent);
                controlComponent = null;

                invalidate();
            }

            // remove any old visual component from the applet
            if (visualComponent != null)
            {
                remove(visualComponent);
                visualComponent = null;
            }

            if (gainComponent != null)
            {
                gainComponent.setVisible(false);
                remove(gainComponent);
                gainComponent = null;
            }

            invalidate();
        }

        try
        {
            // create a new player - saves the new player's object in 'player' 
            if (!createPlayer(urlString))
            {
                  // start hour glass
                setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));            
            }
        }
        catch (Exception ex)
        {
            System.err.println("PApplet: playURL - createPlayer() threw unexpected exception: " + ex.toString() +".");
        }

        // destroy old player if we have one
        if (oldPlayer != null)
        {
            System.err.println("PApplet: playURL - about to close old player.");

            // deallocate the player
            oldPlayer.close();
            oldPlayer = null;
        }

        // now start the player if we were able to create on
        if (player != null)
        {
            player.start();
        }
    }

    private Frame getTopFrame(Container c)
    {
        if (c instanceof Frame)
        {
            return (Frame) c;
        }

        Container parent = c.getParent();
        
        while (parent != null && !(parent instanceof Frame))
        {
            parent = parent.getParent();
        }

        if (parent == null)
        {
            System.out.println("PApplet: getTopFrame - unable to find frame. Creating dummy");
            parent = new Frame();
        }

        return (Frame) parent;
    }

    private void placeControlPanel()
    {
        if (controlComponent != null)
        {
		Rectangle cRect = controlComponent.getBounds();
		Rectangle rect = getBounds();

		controlComponent.setBounds(0, 
				(rect.height - cRect.height) - (getInsets().bottom + getInsets().top) -5, 
				rect.width, cRect.height);

		controlComponent.setVisible(true);
        }
    }

    private void placeGainControl()
    {
        if (gainComponent != null)
        {
            Rectangle rect = getBounds();						// entire window client space
            Rectangle gainRect = gainComponent.getBounds();		// gain component
			Rectangle cpRect = controlComponent.getBounds();	// control panel component

            // calc new position
            int x =  rect.width - gainRect.width - 6;
            int y = rect.height - cpRect.height - gainRect.height - 10;
							 
            if (x >= 0 && y >= 0)
            {
                gainComponent.setBounds(x, y, gainRect.width, gainRect.height);
            }
        }
    }

    private void placeFrameBox()
    {
        if (frameBox != null)
        {
            // visual component bounds 
            Rectangle frameRect = frameBox.getBounds();

            // applet bounds 
            Rectangle rect = getBounds();

            // calc new position
            int x = (rect.width / 2) - (frameRect.width / 2);
            int y = (rect.height / 2) - (frameRect.height / 2);
            if (x >= 0 && y >= 0)
            {
                frameBox.setBounds(x, y, frameRect.width, frameRect.height);
            }

            // update display
            frameBox.setVisible(true);
        }
    }

    private void placeVisualComponent(int width, int height)
    {
        if (visualComponent != null)
        {
            // applet bounds 
            Rectangle rect = getBounds();

            // calc new position
            int x = (rect.width / 2) - (width / 2);
            int y = (rect.height / 2) - (height / 2);

	    	//System.out.println("PApplet: placeVisualComponent - new width = " + width + " new height = " + height);
	    	//System.out.println("PApplet: placeVisualComponent - new x = " + x + " new y = " + y);

            if (x >= 0 && y >= 0)
            {
                visualComponent.setLocation(x, y);
                Rectangle vRect = visualComponent.getBounds();
            }

            // update display
            validate();
        }
    }

    private void placeVisualComponent()
    {
        if (visualComponent != null)
        {
            // visual component bounds 
            Rectangle videoRect = visualComponent.getBounds();
            placeVisualComponent(videoRect.width, videoRect.height);
        }
    }

    private final void placeControls()
    {
        setLayout(null);
        setBackground(new Color(192, 192, 192));
        addNotify();
        resize(590,525);

        label1 = new java.awt.Label("URL:");
        label1.setBounds(37,18,37,16);
        label1.setFont(new Font("Dialog", Font.BOLD, 12));
        add(label1);

        url = new java.awt.TextField();
        url.setText( "pnm://video.real.com/welcome.rm" );
        url.setBounds(84,13,381,30);
        add(url);

        label2 = new Label( "Catalog:", Label.RIGHT );
        label2.setFont( new Font( "Dialog", Font.BOLD, 12 ));
        add( label2 );
        label2.reshape(18,48,56,16);

        catalog = new Choice();
        catalog.setBackground( new Color( 255, 255, 255 ));
        add( catalog );
        catalog.reshape(84,47,484,21);

        label3 = new java.awt.Label("Stop At:", Label.RIGHT);
        label3.setBounds(18,84,56,16);
        label3.setFont(new Font("Dialog", Font.BOLD, 12));
        add(label3);

        stopTime = new java.awt.TextField();
        stopTime.setText("0");
        stopTime.setBounds(84,80,106,30);
        add(stopTime);

        loopCheckbox = new Checkbox("Loop");
        loopCheckbox.setBounds(220, 80, 50, 22);
        add(loopCheckbox);

	tacButton = new Button("TAC");
	tacButton.setBounds(310, 80, 80, 26);
	add(tacButton);

        label4 = new java.awt.Label("Fetching:", Label.RIGHT);
        label4.setBounds(420, 84, 60, 16);
        label4.setFont(new Font("Dialog", Font.BOLD, 12));
        add(label4);

        percentCompleteComponent = new java.awt.TextField();
        percentCompleteComponent.setText(" 0 %");
        percentCompleteComponent.setBounds(487, 80, 80, 30);
        add(percentCompleteComponent);

        openURLButton = new java.awt.Button("Open");
        openURLButton.setBounds(487,12,80,26);
        add(openURLButton);

        // this is centered on the applet & represents a "standard" vid site of 176x144 ??
        frameBox = new ICanvas();
        frameBox.setBounds(215, 202, 160, 120);
        add(frameBox);

        super.init();
    }
}

