/****************************************************************************
 * 
 *  $Id: exaudio.h,v 1.7 1997/10/21 20:51:39 christof Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 * 
 *  exaudio.h
 *
 *  Example Renderer plug-in which shows the use of audio services.
 */
#ifndef _EXAUDIO_H_
#define _EXAUDIO_H_


/****************************************************************************
 *
 *  CExampleRenderer Class
 *
 *  Example implementation of a basic renderer.
 */
class CExampleRenderer :    public IRMAPlugin, 
			    public IRMARenderer, 
			    public IRMASiteUser
{
    public:
    /****** Public Class Methods ******************************************/
    CExampleRenderer()
	: m_lRefCount(0)
	, m_pContext(NULL)
	, m_pStream(NULL)
	, m_pPlayer(NULL)
	, m_pHeader(NULL)
	, m_pLastPacket(NULL)
	, m_ulLastTime(0)
	, m_ulLastPacketTime(0)
	, m_ulLastRealPacketTime(0)
	, m_pAudioPlayer(NULL)
	, m_pAudioStream(NULL)
	, m_pAudioBuffer(NULL)
	, m_pFile(NULL)
	, m_pNewPlayer(NULL)
	, m_pCommonClassFactory(0)
	, m_pMISUS(NULL)
	, m_pMISUSSite(NULL)
	, m_bInSeekMode(FALSE)
	{ };
    void HandleClick();
    void Draw();


    /************************************************************************
     *  IRMARenderer Interface Methods                       ref:  rmarendr.h
     */
    STDMETHOD(StartStream) (THIS_ IRMAStream* pStream, IRMAPlayer* pPlayer);
    STDMETHOD(EndStream )  (THIS);
    STDMETHOD(OnHeader  )  (THIS_ IRMAValues* pStreamHeaderObj);
    STDMETHOD(OnPacket  )  (THIS_ IRMAPacket* pPacket, INT32 timeOffset);
    STDMETHOD(OnTimeSync)  (THIS_ UINT32 currentPlayBackTime);
    STDMETHOD(OnPreSeek )  (THIS_ UINT32 timeBefore, UINT32 timeAfter);
    STDMETHOD(OnPostSeek)  (THIS_ UINT32 timeBefore, UINT32 timeAfter);
    STDMETHOD(OnPause   )  (THIS_ UINT32 timeBeforePause);
    STDMETHOD(OnBegin   )  (THIS_ UINT32 timeAfterBegin);
    STDMETHOD(OnBuffering) (THIS_ UINT32 reason, UINT16 percentComplete);
    STDMETHOD(GetRendererInfo)
        (THIS_
         REF(const char**) pStreamMimeTypes,
         REF(UINT32)       initialGranularity
        );
    STDMETHOD(GetDisplayType)
        (THIS_
         REF(RMA_DISPLAY_TYPE) displayType,
         REF(IRMABuffer*)      pDisplayInfo
        );
    STDMETHOD(OnEndofPackets) (THIS);


    /************************************************************************
     *  IRMASiteUser Interface Methods                         ref:  rmawin.h
     */
    STDMETHOD(AttachSite)  (THIS_ IRMASite* pSite);
    STDMETHOD(DetachSite)  (THIS);
    STDMETHOD(HandleEvent) (THIS_ PNxEvent* pEvent);
    STDMETHOD_(BOOL,NeedsWindowedSites)	(THIS);
    
    
    /************************************************************************
     *  IRMAPlugin Interface Methods                         ref:  rmaplugn.h
     */
    STDMETHOD(GetPluginInfo)
        (THIS_ 
          REF(BOOL)        bLoadMultiple,
          REF(const char*) pDescription,
          REF(const char*) pCopyright,
          REF(const char*) pMoreInfoURL,
          REF(UINT32)      versionNumber
        );
    STDMETHOD(InitPlugin) (THIS_ IUnknown* pContext);


    /************************************************************************
     *  IUnknown COM Interface Methods                          ref:  pncom.h
     */
    STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
    STDMETHOD_(UINT32, AddRef ) (THIS);
    STDMETHOD_(UINT32, Release) (THIS);


    private:
    /****** Private Class Variables ****************************************/
    INT32		m_lRefCount;
    IUnknown*		m_pContext;
    IRMAStream*		m_pStream;
    IRMAAudioPlayer*	m_pAudioPlayer;
    IRMAAudioStream*	m_pAudioStream;
    IRMAPlayer*		m_pPlayer;
    IRMAValues*		m_pHeader;
    IRMAPacket*		m_pLastPacket;
    IRMABuffer*		m_pAudioBuffer;
    IRMAPlayer*		m_pNewPlayer;
    FILE*		m_pFile;
    UINT32		m_ulLastTime;
    UINT32		m_ulLastPacketTime;
    UINT32		m_ulLastRealPacketTime;
    PNxSize		m_size;
    IRMAMultiInstanceSiteUserSupplier*	m_pMISUS;
    IRMASite*				m_pMISUSSite;
    IRMACommonClassFactory*		m_pCommonClassFactory;
    BOOL				m_bInSeekMode;

    #ifdef _WINDOWS
	HDC	        m_hDC;
    #endif

    /****** Private Static Class Variables *********************************/
    static const char*	zm_pDescription;
    static const char*	zm_pCopyright;
    static const char*	zm_pMoreInfoURL;
    static const char*	zm_pStreamMimeTypes[];

    /****** Private Class Methods ******************************************/
    ~CExampleRenderer();

    PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME       // Avoids GCC compiler warning
};


#endif // ndef _EXAUDIO_H_
