/****************************************************************************
 * 
 *  $Id: exblt.cpp,v 1.2 2000/05/19 22:57:11 ahamel Exp $
 *  
 *  Copyright (C) 1995,1996,1997 Progressive Networks.
 *  All rights reserved.
 *
 *  This program contains proprietary 
 *  information of Progressive Networks, Inc, and is licensed
 *  subject to restrictions on use and distribution.
 *
 */

#include <stdio.h>

#include "pncom.h"
#include "pntypes.h"
#include "pntypes.h"
#include "pnwintyp.h"

///#include "cpnxtype.h"

#include "rmapckts.h"
#include "rmawin.h"
#include "rmasite2.h"
//#include "rmavctrl.h"
#include "rmavsurf.h"
#include "rmacomm.h"
#include "rmargn.h"
#include "rmavsurf.h"

#include "fivemlist.h"

#include "exnwsite.h"
#include "exblt.h"

#ifdef _DEBUG
#undef PN_THIS_FILE		
static char PN_THIS_FILE[] = __FILE__;
#endif

ExampleBltService::ExampleBltService(IUnknown* pContext, ExampleWindowlessSite* pSiteWindowless)
    :	m_lRefCount(0)
    ,	m_pContext(pContext)
    ,	m_pSite(pSiteWindowless)
    ,	m_nRGNPitch(0)
    ,	m_pRGBInfo(0)
    ,	m_pBits(0)
{
    m_pContext->AddRef();
    m_pSite->AddRef();

}

ExampleBltService::~ExampleBltService()
{
    PN_RELEASE(m_pContext);
    PN_RELEASE(m_pSite);
    PN_DELETE(m_pRGBInfo);
    PN_DELETE(m_pBits);
    
}

// *** IUnknown methods ***

/////////////////////////////////////////////////////////////////////////
//  Method:
//      IUnknown::QueryInterface
//  Purpose:
//      Implement this to export the interfaces supported by your 
//      object.
//
STDMETHODIMP 
ExampleBltService::QueryInterface(REFIID riid, void** ppvObj)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
        AddRef();
        *ppvObj = this;
        return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMAVideoSurface))
    {
        AddRef();
        *ppvObj = (IRMAVideoSurface*)this;
        return PNR_OK;
    }
    
    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}

/////////////////////////////////////////////////////////////////////////
//  Method:
//      IUnknown::AddRef
//  Purpose:
//      Everyone usually implements this the same... feel free to use
//      this implementation.
//
STDMETHODIMP_(ULONG32) 
ExampleBltService::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}

/////////////////////////////////////////////////////////////////////////
//  Method:
//      IUnknown::Release
//  Purpose:
//      Everyone usually implements this the same... feel free to use
//      this implementation.
//
STDMETHODIMP_(ULONG32) 
ExampleBltService::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
        return m_lRefCount;
    }

    delete this;
    return 0;
}

STDMETHODIMP ExampleBltService::Blt(	UCHAR*		    pImageData,
					RMABitmapInfoHeader*   pBitmapInfo,
					REF(PNxRect)	    rDestRect,
					REF(PNxRect)	    rSrcRect)
{
    int returnVal = 1;

    CheckBitmap();

    int dxDst = rDestRect.right  - rDestRect.left;
    int dyDst = rDestRect.bottom - rDestRect.top;
    int dxSrc = rSrcRect.right   - rSrcRect.left;
    int dySrc = rSrcRect.bottom  - rSrcRect.top;

    /* 
     *	At this point the source bits (pImageData) must be color converted onto 
     *	the destination bitmap (m_pBits).
     */
    

    return PNR_FAIL;
}

void     ExampleBltService::CheckBitmap()
{
    if (!m_pRGBInfo)
    {
	m_pRGBInfo = new RMABitmapInfoHeader;
	memset(m_pRGBInfo, 0, sizeof(RMABitmapInfoHeader));
    }

    PNxSize size;
    m_pSite->GetSize(size);
    
    if (size.cx > m_pRGBInfo->biWidth || 
	size.cy > m_pRGBInfo->biHeight)
    {
	PN_DELETE(m_pBits);
	m_pRGBInfo->biSize = sizeof(BITMAPINFOHEADER);
	/* set image parameters: */
	m_pRGBInfo->biWidth		= size.cx;
	m_pRGBInfo->biHeight		= size.cy;
	m_pRGBInfo->biPlanes		= 1;
        m_pRGBInfo->biCompression	= BI_RGB;
        m_pRGBInfo->biBitCount		= 24;
	m_pRGBInfo->biSizeImage		= m_pRGBInfo->biHeight * (( (m_pRGBInfo->biWidth * 3) + 3) & ~3);
	m_pBits = new UCHAR[m_pRGBInfo->biSizeImage];
	m_nRGNPitch  = (( (m_pRGBInfo->biWidth * 3) + 3) & ~3);
    }
}

PN_RESULT ExampleBltService::SaveFrame(char* filename)
{   
    if (!m_pRGBInfo) 
	return PNR_FAIL;

    FILE* file = fopen(filename, "wb");

    if (file)
    {
	BITMAPFILEHEADER bitmapFileHeader;

	// Now we can save the bits to the file...
	bitmapFileHeader.bfType = 0x4d42;
	bitmapFileHeader.bfSize = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + (m_pRGBInfo->biClrUsed * sizeof(RGBQUAD)) + m_pRGBInfo->biSizeImage;
	bitmapFileHeader.bfReserved1 = 0;
	bitmapFileHeader.bfReserved2 = 0;
	bitmapFileHeader.bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + (m_pRGBInfo->biClrUsed * sizeof(RGBQUAD));

	// Write out the bitmapfileheader...
	fwrite((char *)(&bitmapFileHeader), sizeof(BITMAPFILEHEADER), 1, file);

	// Now write out the bitmapinfoheader...
	fwrite((char *)m_pRGBInfo, sizeof(BITMAPINFOHEADER), 1, file);

	// Write out the rgb quads...
	if (m_pRGBInfo->biClrUsed)
	{
	    fwrite((char *)(m_pRGBInfo) + sizeof(BITMAPINFOHEADER), m_pRGBInfo->biClrUsed, sizeof(RGBQUAD), file);
	}

	// Write out the bits...
	fwrite((char *)m_pBits, m_pRGBInfo->biSizeImage, 1, file);
	fclose(file);
	return TRUE;
    }
    
    return FALSE;
}


STDMETHODIMP ExampleBltService::BeginOptimizedBlt(RMABitmapInfoHeader* pBitmapInfo)
{
    return PNR_NOTIMPL;
}

STDMETHODIMP
ExampleBltService::OptimizedBlt(UCHAR* pImageBits,			
				      REF(PNxRect) rDestRect, 
				      REF(PNxRect) rSrcRect)
{
    return PNR_NOTIMPL;
}


STDMETHODIMP
ExampleBltService::EndOptimizedBlt(void)
{
    return PNR_NOTIMPL;
}

STDMETHODIMP
ExampleBltService::GetOptimizedFormat(REF(RMA_COMPRESSION_TYPE) ulType)
{
    return PNR_NOTIMPL;
}


STDMETHODIMP
ExampleBltService::GetPreferredFormat(REF(RMA_COMPRESSION_TYPE) ulType)
{
    return PNR_NOTIMPL;
}
