/****************************************************************************
 * 
 *  $Id: exnwsite.cpp,v 1.6 2000/05/19 22:57:11 ahamel Exp $
 *  
 *  Copyright (C) 1995,1996,1997 Progressive Networks.
 *  All rights reserved.
 *  
 *  Progressive Networks Confidential and Proprietary information.
 *  Do not redistribute.
 *
 *  Basic PN implementation of the IRMASite classes.
 *
 *
 */

#include <stdio.h>

#include "pncom.h"
#include "pntypes.h"
#include "pnwintyp.h"
#include "rmapckts.h"
#include "rmawin.h"
#include "rmaengin.h"
#include "rmasite2.h"
#include "rmaevent.h"
#include "rmacomm.h"

#include "fivemlist.h"
#include "rmargn.h"

#include "rmavsurf.h"

#include "exvsurf.h"
#include "exnwsite.h"

#include "rmapckts.h"


/************************************************************************
 *  Method:
 *    Constructor
 */
ExampleWindowlessSite::ExampleWindowlessSite(IUnknown* pContext, 
					     IUnknown* pUnkOuter)
    : m_lRefCount(0)
    , m_pUser(NULL)
    , m_pParentSite(NULL)
    , m_pUnkOuter(pUnkOuter)
    , m_pWatcher(NULL)
    , m_pContext(pContext)
    , m_pVideoSurface(NULL)
    , m_bIsVisible(TRUE)
    , m_pValues(NULL)
    , m_lZOrder(0)
    , m_pCCF(NULL)
    , m_bDamaged(FALSE)
    , m_bInRedraw(FALSE)
    , m_pRegion(NULL)
    , m_pBltService(NULL)
    , m_pRegionWithoutChildren(NULL)
{
    
    // addref the context 
    m_pContext->AddRef();

    // get the CCF 
    m_pContext->QueryInterface(IID_IRMACommonClassFactory,
			       (void**)&m_pCCF);

    // addref CCF
    m_pCCF->CreateInstance(CLSID_IRMAValues,(void**)&m_pValues);

    /* If we are not being aggregated, then point our outer
     * unknown to our own implementation of a non-delegating
     * unknown... this is like aggregating ourselves<g>
     */
    if (pUnkOuter == NULL)
    {
	m_pUnkOuter = (IUnknown*)(NonDelegatingUnknown*) this;
    }

    m_size.cx = 0;
    m_size.cy = 0;
    m_position.x = 0;
    m_position.y = 0;

    // create video surface
    
    m_pVideoSurface = new ExampleVideoSurface(m_pContext, this);
    m_pVideoSurface->AddRef();
    

    m_bIsChildWindow = TRUE;
}


/************************************************************************
 *  Method:
 *    Destructor
 */
ExampleWindowlessSite::~ExampleWindowlessSite()
{
    // Clean up all child sites

    ExampleWindowlessSite* pSite;
    while(pSite= (ExampleWindowlessSite*) m_Children.GetFirst())
    {
	DestroyChild((IRMASite*)pSite);
    }

    // clean up passive site watchers 

    IRMAPassiveSiteWatcher* pWatcher;
    while(pWatcher =(IRMAPassiveSiteWatcher*) m_PassiveSiteWatchers.RemoveHead())
    {
	PN_RELEASE(pWatcher);
    }

    // release intefaces
    PN_RELEASE(m_pValues);
    PN_RELEASE(m_pCCF);
    PN_RELEASE(m_pContext);
}


/************************************************************************
 *  Method:
 *    ExampleWindowlessSite::SetParentSite
 */
void		
ExampleWindowlessSite::SetParentSite(ExampleWindowlessSite* pParentSite)
{
    m_pParentSite = pParentSite;
    
}


/************************************************************************
 *  Method:
 *    IUnknown::QueryInterface
 */
STDMETHODIMP 
ExampleWindowlessSite::NonDelegatingQueryInterface(REFIID riid, void** ppvObj)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
	AddRef();
	*ppvObj = (IUnknown*)(IRMASite*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMASite))
    {
        
	AddRef();
	*ppvObj = (IRMASite*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMASite2))
    {
	AddRef();
	*ppvObj = (IRMASite2*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMASiteWindowless))
    {
        
	AddRef();
	*ppvObj = (IRMASiteWindowless*)this;
	return PNR_OK;
    }
    else if (m_pValues && m_pValues->QueryInterface(riid, ppvObj) == PNR_OK)
    {
	return PNR_OK;
    }

    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}


/************************************************************************
 *  Method:
 *    IUnknown::AddRef
 */
STDMETHODIMP_(ULONG32) 
ExampleWindowlessSite::NonDelegatingAddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}


/************************************************************************
 *  Method:
 *    IUnknown::Release
 */
STDMETHODIMP_(ULONG32) 
ExampleWindowlessSite::NonDelegatingRelease()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
        return m_lRefCount;
    }

    delete this;
    return 0;
}


/************************************************************************
 *  Method:
 *    IUnknown::QueryInterface
 */
STDMETHODIMP 
ExampleWindowlessSite::QueryInterface(REFIID riid, void** ppvObj)
{
    return m_pUnkOuter->QueryInterface(riid,ppvObj);
   
}


/************************************************************************
 *  Method:
 *    IUnknown::AddRef
 */
STDMETHODIMP_(ULONG32) 
ExampleWindowlessSite::AddRef()
{
    return m_pUnkOuter->AddRef();
}


/************************************************************************
 *  Method:
 *    IUnknown::Release
 */
STDMETHODIMP_(ULONG32) 
ExampleWindowlessSite::Release()
{
    return m_pUnkOuter->Release();
}


/************************************************************************
 *  Method:
 *    IRMASite::AttachUser
 */
STDMETHODIMP 
ExampleWindowlessSite::AttachUser(IRMASiteUser* /*IN*/ pUser)
{
    HRESULT result = PNR_FAIL;

    if (m_pUser) return PNR_UNEXPECTED;

    IRMASite* pOuterSite;
    m_pUnkOuter->QueryInterface(IID_IRMASite, (void**)&pOuterSite);
    if (pOuterSite)
    {
	result = pUser->AttachSite(pOuterSite);

	pOuterSite->Release();
    }
    
    if (PNR_OK == result)
    {
	m_pUser = pUser;
	m_pUser->AddRef();
    }
    

    return result;
}

/************************************************************************
 *  Method:
 *    IRMASite::DetachUser
 */
STDMETHODIMP 
ExampleWindowlessSite::DetachUser()
{
    HRESULT result = PNR_OK;
    if (!m_pUser) return PNR_UNEXPECTED;

    result = m_pUser->DetachSite();
    
    if (PNR_OK == result)
    {
	m_pUser->Release();
	m_pUser = NULL;
    }
   
    return result;
}

/************************************************************************
 *  Method:
 *    IRMASite::GetUser
 */
STDMETHODIMP 
ExampleWindowlessSite::GetUser(REF(IRMASiteUser*) /*OUT*/ pUser)
{
    HRESULT result = PNR_OK;
    if (!m_pUser) return PNR_UNEXPECTED;

    pUser = m_pUser;
    pUser->AddRef();

    return result;
}


/************************************************************************
 *  Method:
 *    IRMASite::CreateChild
 */
STDMETHODIMP 
ExampleWindowlessSite::CreateChild(REF(IRMASite*) /*OUT*/ pChildSite)
{
    HRESULT result = PNR_OK;

    /* Create an instance of ExampleWindowlessSite, let it know it's
     * in child window mode.
     */
    ExampleWindowlessSite* pChildSiteWindowless = new ExampleWindowlessSite(m_pContext);
    pChildSiteWindowless->AddRef();
    pChildSiteWindowless->SetParentSite(this);

    /* Get the IRMASite interface from the child to return to the
     * outside world.
     */
    pChildSiteWindowless->QueryInterface(IID_IRMASite, (void**)&pChildSite);
    m_Children.Add(pChildSiteWindowless);

    return result;
}


/************************************************************************
 *  Method:
 *    IRMASite::DestroyChild
 */
STDMETHODIMP 
ExampleWindowlessSite::DestroyChild(IRMASite* /*IN*/ pChildSite)
{
    //fprintf(stdout, "ExampleWindowlessSite::DestroyChild()\n");

    ExampleWindowlessSite* pSite = (ExampleWindowlessSite*)pChildSite;

    /* 
     * Cleanup the list items for this child site.
     */
    ListPosition pos  = m_Children.GetHeadPosition();
    ListPosition oldPos;

    ExampleWindowlessSite* pTempSite;
    while(pTempSite = (ExampleWindowlessSite*) m_Children.GetAt(pos))
    {
	if (pTempSite == pSite)
	{
	    if (pos == m_Children.GetHeadPosition())
	    {
		m_Children.RemoveHead();
		pos = m_Children.GetHeadPosition();
	    }
	    else
	    {
		m_Children.RemoveAfter(oldPos);
		pos = m_Children.GetNextPosition(oldPos);
	    }
	    PN_RELEASE(pSite);
	}
	else
	{
	    pos = m_Children.GetNextPosition(pos);
	}
	oldPos = pos;
    }

    return PNR_OK;
}

/************************************************************************
 *  Method:
 *    MapSiteToSiteInfo
 */
BOOL ExampleWindowlessSite::MapSiteToSiteInfo(IRMASite* pChildSite, ExampleWindowlessSite*& pSiteInfo, ListPosition& prevPos)
{
    BOOL res = FALSE;

    // iterate child site list 
    pSiteInfo = NULL;
    ListPosition pos = m_Children.GetHeadPosition();
    prevPos = pos;
    while (pos)
    {
        pSiteInfo = (ExampleWindowlessSite*) m_Children.GetAt(pos);
	IRMASite* pTempSite = (IRMASite*) pSiteInfo;

        if (pTempSite == pChildSite)
        {
            res = TRUE;

	    if (prevPos == m_Children.GetHeadPosition())
	    {
		prevPos = NULL;
	    }
	    break;
        }
	
	prevPos = pos;
	pos = m_Children.GetNextPosition(pos);	
    }

    return res;
}

/************************************************************************
 *  Method:
 *    IRMASite::AttachWatcher
 */
STDMETHODIMP 
ExampleWindowlessSite::AttachWatcher(IRMASiteWatcher* /*IN*/ pWatcher)
{
    if (m_pWatcher) return PNR_UNEXPECTED;

    m_pWatcher = pWatcher;

    if (m_pWatcher)
    {
	m_pWatcher->AddRef();
	m_pWatcher->AttachSite(this);
       
    }

    return PNR_OK;
}

/************************************************************************
 *  Method:
 *    IRMASite::DetachWatcher
 */
STDMETHODIMP 
ExampleWindowlessSite::DetachWatcher()
{
    if (!m_pWatcher) return PNR_UNEXPECTED;

    m_pWatcher->DetachSite();
    PN_RELEASE(m_pWatcher);

    return PNR_OK;
}

/************************************************************************
 *  Method:
 *    IRMASite::SetSize
 */
STDMETHODIMP 
ExampleWindowlessSite::SetSize(PNxSize size)
{
    HRESULT hres = PNR_OK;
    

    /* before we do anything, we give the SiteWatcher a chance to 
     * influence this operation.
     */
    if (m_pWatcher)
    {
	hres = m_pWatcher->ChangingSize(m_size, size);
    }
    
    if (PNR_OK == hres && size.cx != 0 && size.cy != 0)
    {
	m_size = size;

	// iterate child site list 
	IRMAPassiveSiteWatcher* pWatcher = NULL;
	ListPosition pos = m_PassiveSiteWatchers.GetHeadPosition();
	while (pos)
	{
	    pWatcher = (IRMAPassiveSiteWatcher*) m_PassiveSiteWatchers.GetAt(pos);
	    pWatcher->SizeChanged(&m_size);

	    pos = m_PassiveSiteWatchers.GetNextPosition(pos);	
	}
	RecomputeRegion();
    }

    return hres;
}

/************************************************************************
 *  Method:
 *    IRMASite::SetPosition
 */
STDMETHODIMP 
ExampleWindowlessSite::SetPosition(PNxPoint position)
{
    HRESULT hres = PNR_OK;

    /*
     * Before we do anything, we give the SiteWatcher a chance to 
     * influence this operation.
     */
    if (m_pWatcher)
    {
	hres = m_pWatcher->ChangingPosition(m_position, position);
    }
    
    if (PNR_OK == hres)
    {
	/* Record the position of posterity sake */
	m_position = position;

	// iterate child site list 
	IRMAPassiveSiteWatcher* pWatcher = NULL;
	ListPosition pos = m_PassiveSiteWatchers.GetHeadPosition();
	while (pos)
	{
	    pWatcher = (IRMAPassiveSiteWatcher*) m_PassiveSiteWatchers.GetAt(pos);
	    pWatcher->PositionChanged(&m_position);

	    pos = m_PassiveSiteWatchers.GetNextPosition(pos);	
	}
	RecomputeRegion();
    }
    

    return hres;
}


/************************************************************************
 *  Method:
 *    IRMASite::GetSize
 */
STDMETHODIMP 
ExampleWindowlessSite::GetSize(REF(PNxSize) size)
{
    size = m_size;
    return PNR_OK;
}


/************************************************************************
 *  Method:
 *    IRMASite::GetPosition
 */
STDMETHODIMP 
ExampleWindowlessSite::GetPosition(REF(PNxPoint) position)
{
    position = m_position;
    return PNR_OK;
}


/************************************************************************
 *  Method:
 *    IRMASite::DamageRect
 */
STDMETHODIMP 
ExampleWindowlessSite::DamageRect(PNxRect rect)
{
   
    m_bDamaged = TRUE;
    return PNR_OK;
}


/************************************************************************
 *  Method:
 *    IRMASite::DamageRegion
 */
STDMETHODIMP 
ExampleWindowlessSite::DamageRegion(PNxRegion region)
{
    m_bDamaged = TRUE;
    
    return PNR_OK;
}


/************************************************************************
 *  Method:
 *    IRMASite::ForceRedraw
 */
STDMETHODIMP 
ExampleWindowlessSite::ForceRedraw()
{
    
    // make sure we have a visible window and are not re-enterering and we have damage
    if (!m_bInRedraw && m_bDamaged && m_bIsVisible)
    {
	
//        AddRef(); 

	m_bInRedraw = TRUE; 

	PNxEvent event = {RMA_SURFACE_UPDATE, NULL, m_pVideoSurface, NULL, 0, 0};
	m_pUser->HandleEvent(&event);

	m_bInRedraw = FALSE;
	m_bDamaged = FALSE;

//	Release();
    }
    

    return PNR_OK;
}

/************************************************************************
 *  Method:
 *    IRMASite2::UpdateSiteWindow
 *    	
 *	Not used on Windows platform
 */
STDMETHODIMP 
ExampleWindowlessSite::UpdateSiteWindow
(
    PNxWindow* /*IN*/ pWindow
)
{
    return PNR_OK;
}


/************************************************************************
 *  Method:
 *    IRMASite2::ShowSite
 */
STDMETHODIMP 
ExampleWindowlessSite::ShowSite
(
    BOOL    bShow
)
{
    m_bIsVisible = bShow;
    RecomputeRegion();
    return PNR_OK;
}

/************************************************************************
 *  Method:
 *    IRMASite2::IsSiteVisible
 */
STDMETHODIMP_(BOOL) 
ExampleWindowlessSite::IsSiteVisible()
{
    BOOL bIsVisible = m_bIsVisible;
    if (m_pParentSite)
    {
        bIsVisible &= m_pParentSite->IsSiteVisible();
    }
    
    return bIsVisible;
}

/************************************************************************
 *  Method:
 *    IRMASite2::SetZOrder
 */
STDMETHODIMP
ExampleWindowlessSite::SetZOrder(INT32 lZOrder)
{
    if(!m_pParentSite) return PNR_UNEXPECTED;

    if (lZOrder == -1)
    {
	lZOrder = m_pParentSite->GetNumberOfChildSites() - 1; 
    }
    
    if (lZOrder >= (INT32) m_pParentSite->GetNumberOfChildSites())
    {
	lZOrder = m_pParentSite->GetNumberOfChildSites() - 1;
    }

    if (m_lZOrder != lZOrder)
    {
	m_pParentSite->UpdateZOrder(this, m_lZOrder, lZOrder);
    }

    m_lZOrder = lZOrder;
    return PNR_OK;
}

void ExampleWindowlessSite::UpdateZOrder( ExampleWindowlessSite* pUpdatedChildSite, INT32 lOldZOrder, INT32 lNewZOrder)
{
    ListPosition pos = m_Children.GetHeadPosition();
    while (pos)
    {
        ExampleWindowlessSite* pSite = (ExampleWindowlessSite*) m_Children.GetAt(pos);
	INT32 lItsOldZOrder;
	pSite->GetZOrder( lItsOldZOrder );

	if (pSite != pUpdatedChildSite)
	{
	    if (lOldZOrder < lNewZOrder)
	    {
		if (lItsOldZOrder >= lOldZOrder && lItsOldZOrder < lNewZOrder)
		{
		    pSite->SetInternalZOrder(lItsOldZOrder-1);
		}
	    }
	    else
	    {
		if (lItsOldZOrder >= lNewZOrder && lItsOldZOrder < lOldZOrder)
		{
		    pSite->SetInternalZOrder(lItsOldZOrder + 1);
		}
	    }
	}
	else
	{
	    pSite->SetInternalZOrder(lNewZOrder);
	}

	pos = m_Children.GetNextPosition(pos);	
    }
}

/************************************************************************
 *  Method:
 *    IRMASite2::GetZOrder
 */
STDMETHODIMP
ExampleWindowlessSite::GetZOrder(REF(INT32) lZOrder)
{
    if(!m_pParentSite) return PNR_UNEXPECTED;
    lZOrder = m_lZOrder;
    return PNR_OK;
}

/************************************************************************
 *  Method:
 *    IRMASite2::MoveSiteToTop
 */
STDMETHODIMP
ExampleWindowlessSite::MoveSiteToTop()
{
    if(!m_pParentSite) return PNR_UNEXPECTED;
    return PNR_NOTIMPL;
}

/************************************************************************
 *  Method:
 *    IRMASite2::GetVideoSurface
 */
STDMETHODIMP
ExampleWindowlessSite::GetVideoSurface(REF(IRMAVideoSurface*) pSurface)
{
    PN_RESULT res = PNR_FAIL;

    
    if (m_pVideoSurface)
    {
	res = m_pVideoSurface->QueryInterface(IID_IRMAVideoSurface, 
					   (void**)&pSurface);
    }

    return res;
}

/************************************************************************
 *  Method:
 *    IRMASite2::GetNumberOfChildSites
 */
STDMETHODIMP_(UINT32)
ExampleWindowlessSite::GetNumberOfChildSites()
{
    return (UINT32)m_Children.Count();
}

/************************************************************************
 *  Method:
 *    IRMASite2::AddPassiveSiteWatcher
 */
STDMETHODIMP
ExampleWindowlessSite::AddPassiveSiteWatcher(IRMAPassiveSiteWatcher* pWatcher)
{
   
    pWatcher->AddRef();
    m_PassiveSiteWatchers.Add(pWatcher);
    return PNR_OK;
}

/************************************************************************
 *  Method:
 *    IRMASite2::RemovePassiveSiteWatcher
 */
STDMETHODIMP
ExampleWindowlessSite::RemovePassiveSiteWatcher(IRMAPassiveSiteWatcher* pWatcher)
{
    // iterate child site list 
    IRMAPassiveSiteWatcher* pThisWatcher = NULL;
    ListPosition pos = m_PassiveSiteWatchers.GetHeadPosition();
    ListPosition prevPos = pos;
    while (pos)
    {
	pThisWatcher = (IRMAPassiveSiteWatcher*) m_PassiveSiteWatchers.GetAt(pos);
	if(pWatcher == pThisWatcher)
	{
	    PN_RELEASE(pWatcher);

	    if (prevPos == m_PassiveSiteWatchers.GetHeadPosition())
	    {
		m_PassiveSiteWatchers.RemoveHead();
	    }
	    else
	    {
		m_PassiveSiteWatchers.RemoveAfter(prevPos);
	    }

	    break;
	}

	prevPos = pos;
	pos = m_PassiveSiteWatchers.GetNextPosition(pos);	
    }

    return PNR_OK;
}

/*
 * IRMASiteWindowless methods called by owners of the site.
 */

STDMETHODIMP
ExampleWindowlessSite::EventOccurred(PNxEvent* /*IN*/ pEvent)
{
    return PNR_NOTIMPL;  /* not necessary within our implementation */
}

/*
 * IRMASiteWindowless method. Returns some parent window that
 * owns the windowless site. Useful for right-click menus and
 * dialog box calls.
 */

PNxWindow*
ExampleWindowlessSite::GetParentWindow()
{
    return NULL;
}

/************************************************************************
 *  Method:
 *    IRMASite2::SetCursor
 */
STDMETHODIMP
ExampleWindowlessSite::SetCursor(PNxCursor cursor, REF(PNxCursor) oldCursor)
{
    return PNR_NOTIMPL;
}

/*
 *  This function is going to be called a LOT. 
 *  If one wishes to make this somewhat efficient, all calls to this function
 *  should be done async. ie: when someone sizes a region a note should be made
 *  and if a blt which could be affected is made RecomputeRegion should then be called.
 *  and if no one makes a BLT then it should be called in some timeout value (~100 ms).
 *  
 *  If the above is not done, this program WILL consume more CPU than it should. 
 */

void	ExampleWindowlessSite::RecomputeRegion()
{
    if (m_pParentSite)
    {
	m_pParentSite->RecomputeRegion();
    }
    else
    {
	InternalRecomputeRegion();
    }
}

void	ExampleWindowlessSite::InternalRecomputeRegion()
{
    ComputeRegion();

    ListPosition pos = m_Children.GetHeadPosition();
    while (pos)
    {
        ExampleWindowlessSite* pSite = (ExampleWindowlessSite*) m_Children.GetAt(pos);
	if (pSite)
	{
	    pSite->InternalRecomputeRegion();
	}
    	pos = m_Children.GetNextPosition(pos);	
    }
}

IRMAVideoSurface* ExampleWindowlessSite::GetBltService()
{
    if (m_pParentSite)
    {
	return m_pParentSite->GetBltService();
    }
    else
    {
	if (m_pBltService)
	{
	    m_pBltService->AddRef();
	    return m_pBltService;
	}
	else
	{
	    if (m_pParentSite)
		m_pParentSite->GetBltService();
	}

	return 0;
    }
}

void	ExampleWindowlessSite::SetBltService(IRMAVideoSurface* pService) 
{
    if (m_pBltService)
    {
	PN_RELEASE(m_pBltService);
    }
    m_pBltService = pService; 
    if (m_pBltService)
    {
	m_pBltService->AddRef();
    }
}

void ExampleWindowlessSite::GetTopLeft(PNxPoint* pPoint)
{
    pPoint->x += m_position.x;
    pPoint->y += m_position.y;

    if (m_pParentSite)
    {
	m_pParentSite->GetTopLeft(pPoint);
    }
}

void ExampleWindowlessSite::SubtractSite(REGION* pRegion)
{
    PNxPoint topLeft;
    memset(&topLeft, 0, sizeof(PNxPoint));
    GetTopLeft(&topLeft);
    
    REGION* pTempRegion = RMACreateRectRegion(topLeft.x, topLeft.y, 
	topLeft.x + m_size.cx, topLeft.y + m_size.cy);

    RMASubtractRegion(pRegion, pTempRegion, pRegion);
    RMADestroyRegion(pTempRegion);
}

void ExampleWindowlessSite::PrintDataFromTop()
{
    if (m_pParentSite)
    {
	m_pParentSite->PrintDataFromTop();
    }
    else
    {
	PrintData();
    }
}

void ExampleWindowlessSite::PrintData()
{
    ExampleWindowlessSite*  pChildSite;
    ListPosition	    pos;

    FILE* f1;
    f1 = fopen("c:\\sitedata.txt", "a+");
    ::fprintf(f1, "Site %p\n", this);
    ::fprintf(f1, "Parent List:");
    ExampleWindowlessSite* pSite = m_pParentSite;
    while(pSite)
    {
	fprintf(f1, "%p,",  pSite);
	pSite = pSite->m_pParentSite;
    }
    fprintf(f1, "\n");
    fprintf(f1, "Size (%d, %d) \n", m_size.cx, m_size.cy);
    fprintf(f1, "Bounding Rect: (%d, %d) - (%d, %d)  .... zorder: %d  visible: %d\n", m_position.x, m_position.y, m_position.x + m_size.cx, m_position.y+m_size.cy, m_lZOrder, IsSiteVisible());
    PNxPoint topleft;
    memset(&topleft, 0, sizeof(PNxPoint));
    GetTopLeft(&topleft);
    fprintf(f1, "Bounding Rect (absolulte): (%d, %d) -(%d, %d) \n", topleft.x, topleft.y, topleft.x+m_size.cx, topleft.y+m_size.cy);
    fprintf(f1, "list of rects:");
    if (m_pRegion)
    {
	if (m_pRegion->numRects)
	{
	    fprintf(f1, "\n");
	    for(int i = 0; i < m_pRegion->numRects; i++)
	    {
		PNxRect* pRect= m_pRegion->rects+i;
		::fprintf(f1, "Rect (%d, %d) -(%d, %d)\n", pRect->left, pRect->top, pRect->right, pRect->bottom);
	    }
	}
	else
	{
	    fprintf(f1, "NO RECTS\n");
	}
    }
    else
    {
	fprintf(f1, "NO REGION\n");
    }

    fprintf(f1, "list of rects without children:");
    if (m_pRegionWithoutChildren)
    {
	if (m_pRegionWithoutChildren->numRects)
	{
	    fprintf(f1, "\n");
	    for(int i = 0; i < m_pRegionWithoutChildren->numRects; i++)
	    {
		PNxRect* pRect = m_pRegionWithoutChildren->rects+i;
		::fprintf(f1, "Rect (%d, %d) -(%d, %d)\n", pRect->left, pRect->top, pRect->right, pRect->bottom);
	    }
	}
	else
	{
	    fprintf(f1, "NO RECTS\n");
	}
    }
    else
    {
	fprintf(f1, "NO REGION\n");
    }
    fprintf(f1, "****************************************************************\n");
    fclose(f1);

    pos = m_Children.GetHeadPosition();
    while (pos)
    {
	pChildSite = (ExampleWindowlessSite*) m_Children.GetAt(pos);
	pChildSite->PrintData();
	pos = m_Children.GetNextPosition(pos);
    }
}

void ExampleWindowlessSite::ComputeRegion()
{
    /*
     *	Save off old region so we can compare.
     */ 

    REGION* pTempRegion = NULL;

    if (m_pRegion)
    {
	pTempRegion = RMACreateRegion();
	RMAUnionRegion(pTempRegion, m_pRegion, pTempRegion);
	RMADestroyRegion(m_pRegion);
    }

    if (m_pRegionWithoutChildren)
    {
	RMADestroyRegion(m_pRegionWithoutChildren);
    }

    PNxPoint topleft;
    memset(&topleft, 0, sizeof(PNxPoint));
    GetTopLeft(&topleft);

    if (IsSiteVisible())
    {
	// All sites are rectangles -- this may change in the future.

	m_pRegionWithoutChildren = RMACreateRectRegion(topleft.x, topleft.y, 
							topleft.x + m_size.cx, 
							topleft.y + m_size.cy);

	if (m_pParentSite)
	{
	    // We must intersect with our parent's region (without children!).
	    RMAIntersectRegion(m_pRegionWithoutChildren, m_pParentSite->m_pRegionWithoutChildren, m_pRegionWithoutChildren);

    	    // subtract siblings that have a higher z-order.

	    ExampleWindowlessSite*  pSiblingSite;
	    ListPosition	    pos;

	    pos = m_pParentSite->m_Children.GetHeadPosition();
	    
	    while (pos)
	    {
		pSiblingSite = (ExampleWindowlessSite*) m_pParentSite->m_Children.GetAt(pos);
		if (pSiblingSite != this) 
		{
		    INT32 zOrder;
		    pSiblingSite->GetZOrder(zOrder);

		    if (zOrder > m_lZOrder && pSiblingSite->IsSiteVisible())
		    {
			pSiblingSite->SubtractSite(m_pRegionWithoutChildren);
		    }
		}
		pos = m_pParentSite->m_Children.GetNextPosition(pos);
	    }
	}

	// copy region without children to region.

	m_pRegion = RMACreateRegion();
	RMAUnionRegion(m_pRegion, m_pRegionWithoutChildren, m_pRegion);

	// now subtract our children to obtain our actual region.

	ExampleWindowlessSite*  pChildSite;
	ListPosition		pos;
    
	pos = m_Children.GetHeadPosition();
	
	while (pos)
	{
	    pChildSite = (ExampleWindowlessSite*) m_Children.GetAt(pos);
	    if (pChildSite->IsSiteVisible())
	    {
		pChildSite->SubtractSite(m_pRegion);
	    }
	    pos = m_Children.GetNextPosition(pos);
	}
    }
    else
    {
	m_pRegionWithoutChildren    = RMACreateRectRegion(0,0,0,0);
	m_pRegion		    = RMACreateRectRegion(0,0,0,0);
    }

    // check to see if every destination new region and the
    // old region are the same.

    if (pTempRegion)
    {
	if (!RMAEqualRegion(m_pRegion, pTempRegion))
	{
	    ForceRedraw();
	}
    }

    RMADestroyRegion(pTempRegion);
}
