/****************************************************************************
 * 
 *  $Id: exvsurf.cpp,v 1.3 2000/05/19 22:57:11 ahamel Exp $
 *  
 *  Copyright (C) 1995,1996,1997 Progressive Networks.
 *  All rights reserved.
 *
 *  This program contains proprietary 
 *  information of Progressive Networks, Inc, and is licensed
 *  subject to restrictions on use and distribution.
 *
 */


#include <stdio.h>

#include "pncom.h"
#include "pntypes.h"
#include "pntypes.h"
#include "pnwintyp.h"

///#include "cpnxtype.h"

#include "rmapckts.h"
#include "rmawin.h"
#include "rmasite2.h"
//#include "rmavctrl.h"
#include "rmavsurf.h"
#include "rmacomm.h"

#include "fivemlist.h"

#include "rmargn.h"
#include "exvsurf.h"
#include "exnwsite.h"


#ifdef _DEBUG
#undef PN_THIS_FILE		
static char PN_THIS_FILE[] = __FILE__;
#endif



ExampleVideoSurface::ExampleVideoSurface(IUnknown* pContext, ExampleWindowlessSite* pSiteWindowless)
    : m_lRefCount(0)
    , m_pContext(pContext)
    , m_pSiteWindowless(pSiteWindowless)
    ,	m_pBitmap(NULL)
{ 
    if (m_pContext)
    {
	m_pContext->AddRef();
    }

    memset(&m_lastBitmapInfo, 0, sizeof(RMABitmapInfoHeader));
}

ExampleVideoSurface::~ExampleVideoSurface()
{
    PN_RELEASE(m_pContext);
}

// *** IUnknown methods ***

/////////////////////////////////////////////////////////////////////////
//  Method:
//      IUnknown::QueryInterface
//  Purpose:
//      Implement this to export the interfaces supported by your 
//      object.
//
STDMETHODIMP 
ExampleVideoSurface::QueryInterface(REFIID riid, void** ppvObj)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
        AddRef();
        *ppvObj = this;
        return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMAVideoSurface))
    {
        AddRef();
        *ppvObj = (IRMAVideoSurface*)this;
        return PNR_OK;
    }
    
    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}

/////////////////////////////////////////////////////////////////////////
//  Method:
//      IUnknown::AddRef
//  Purpose:
//      Everyone usually implements this the same... feel free to use
//      this implementation.
//
STDMETHODIMP_(ULONG32) 
ExampleVideoSurface::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}

/////////////////////////////////////////////////////////////////////////
//  Method:
//      IUnknown::Release
//  Purpose:
//      Everyone usually implements this the same... feel free to use
//      this implementation.
//
STDMETHODIMP_(ULONG32) 
ExampleVideoSurface::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
        return m_lRefCount;
    }

    delete this;
    return 0;
}


PN_RESULT	
ExampleVideoSurface::Init()
{
    return PNR_OK;
}


STDMETHODIMP
ExampleVideoSurface::Blt(UCHAR*		    pImageData,
			     RMABitmapInfoHeader*   pBitmapInfo,
			     REF(PNxRect)	    inDestRect,
			     REF(PNxRect)	    inSrcRect)
{
    BeginOptimizedBlt(pBitmapInfo);
    return OptimizedBlt(pImageData, inDestRect, inSrcRect);
}


STDMETHODIMP
ExampleVideoSurface::BeginOptimizedBlt(RMABitmapInfoHeader* pBitmapInfo)
{
    if (memcmp(&m_bitmapInfo, pBitmapInfo, sizeof(RMABitmapInfoHeader)))
    {
	memcpy(&m_bitmapInfo, pBitmapInfo, sizeof(RMABitmapInfoHeader));
	
	// format of image has changed somehow.
	// do something here if this affects you.
    }
    return PNR_OK;
}

int z_printData = 0;

STDMETHODIMP
ExampleVideoSurface::OptimizedBlt(UCHAR* pImageBits,			
				      REF(PNxRect) rDestRect, 
				      REF(PNxRect) rSrcRect)
{
    if (!m_pSiteWindowless)
    {
	return PNR_UNEXPECTED;
    }

    if (z_printData)
    {
	m_pSiteWindowless->PrintDataFromTop();
	ExampleWindowlessSite* pSite = m_pSiteWindowless;
	while(pSite->m_pParentSite)
	{
	    pSite = pSite->m_pParentSite;
	}

	pSite->RecomputeRegion();
	m_pSiteWindowless->PrintDataFromTop();
    }

    IRMAVideoSurface* pBltter = m_pSiteWindowless->GetBltService();
    REGION* pRegion = m_pSiteWindowless->GetRegion();

    if (!pBltter || !pRegion)
    {
	PN_RELEASE(pBltter);
	return PNR_UNEXPECTED;
    }

    PNxPoint origin;
    memset(&origin, 0, sizeof(PNxPoint));
    m_pSiteWindowless->GetTopLeft(&origin);
    PNxRect adjustedDestRect;
    adjustedDestRect.left   = rDestRect.left	+ origin.x;
    adjustedDestRect.top    = rDestRect.top	+ origin.y;
    adjustedDestRect.right  = rDestRect.right	+ origin.x;
    adjustedDestRect.bottom = rDestRect.bottom	+ origin.y;

    for(int i = 0; i < pRegion->numRects; i++)
    {
	PNxRect* pRect = pRegion->rects+i;
	// intersect the dest rect with the rect from the
	// region to get the final dest rect.
	PNxRect finalDestRect;
	IntersectRect(&adjustedDestRect, pRect, &finalDestRect);
	
	// now compute the src rect for this blt.

	double xStretch = (double) (rDestRect.right - rDestRect.left)   / (double) (rSrcRect.right - rSrcRect.left);
	double yStretch = (double) (rDestRect.bottom - rDestRect.top)   / (double) (rSrcRect.bottom - rSrcRect.top);

	PNxRect finalSrcRect;

	finalSrcRect.left   = (INT32) ((double)(finalDestRect.left-origin.x)	/ xStretch + 0.5);
	finalSrcRect.top    = (INT32) ((double)(finalDestRect.top-origin.y)	/ xStretch + 0.5);
	finalSrcRect.right  = (INT32) ((double)(finalDestRect.right-origin.x)	/ xStretch + 0.5);
	finalSrcRect.bottom = (INT32) ((double)(finalDestRect.bottom-origin.y)	/ xStretch + 0.5);
	
	pBltter->Blt(pImageBits, &m_bitmapInfo, finalDestRect, finalSrcRect);
    }

    PN_RELEASE(pBltter);
    return PNR_OK;
}

void ExampleVideoSurface::IntersectRect(PNxRect * pRect1, PNxRect* pRect2, PNxRect * pRetVal)
{
    pRetVal->left   = max(pRect1->left, pRect2->left);
    pRetVal->top    = max(pRect1->top, pRect2->top);
    pRetVal->right  = min(pRect1->right, pRect2->right);
    pRetVal->bottom = min(pRect1->bottom, pRect2->bottom);
}

STDMETHODIMP
ExampleVideoSurface::EndOptimizedBlt(void)
{
    memset(&m_bitmapInfo, 0, sizeof(RMABitmapInfo));

    return PNR_OK;
}


STDMETHODIMP
ExampleVideoSurface::GetOptimizedFormat(REF(RMA_COMPRESSION_TYPE) ulType)
{
    ulType =  m_bitmapInfo.biCompression;
    return PNR_OK;
}


STDMETHODIMP
ExampleVideoSurface::GetPreferredFormat(REF(RMA_COMPRESSION_TYPE) ulType)
{
    ulType = RMA_RGB;
    return PNR_OK;
}
