#include "pntypes.h"
#include "pnwintyp.h"
#include "rmargn.h"

#ifdef _WIN32

#include "windows.h"

void ExtractRects(REGION* pRegion)
{
    LPRGNDATA	lpRgnData;

    DWORD sizeNeeed = GetRegionData((HRGN)pRegion->pOSRegion, 0, NULL); 
 
    lpRgnData = (LPRGNDATA) new char[sizeNeeed];
    DWORD returnValue = GetRegionData((HRGN)pRegion->pOSRegion, sizeNeeed, lpRgnData); 
    
    PN_DELETE(pRegion->rects);

    pRegion->numRects		= lpRgnData->rdh.nCount;
    pRegion->extents.left	= lpRgnData->rdh.rcBound.left;
    pRegion->extents.top	= lpRgnData->rdh.rcBound.top;
    pRegion->extents.right	= lpRgnData->rdh.rcBound.right;
    pRegion->extents.bottom	= lpRgnData->rdh.rcBound.bottom;
    
    if (lpRgnData->rdh.nCount)
    {
	pRegion->rects = new PNxRect[lpRgnData->rdh.nCount];

	// now extract the information.

	for(int j = 0; j < (int) lpRgnData->rdh.nCount;j++)
	{
	    RECT* pRect = (RECT*) lpRgnData->Buffer;
		
	    pRegion->rects[j].left	= pRect[j].left;
	    pRegion->rects[j].top	= pRect[j].top;
	    pRegion->rects[j].right	= pRect[j].right;
	    pRegion->rects[j].bottom	= pRect[j].bottom;
	    
	}
    }
    PN_DELETE(lpRgnData);
}

REGION* RMACreateRectRegion(int left, int top, int right, int bottom)
{
    REGION* retVal = new REGION;
    retVal->pOSRegion = (void*) CreateRectRgn(left, top, right, bottom);
    ExtractRects(retVal);
    return retVal;
}

void RMASubtractRegion(REGION* regM, REGION* regS, REGION* regD)
{
    CombineRgn((HRGN)regD->pOSRegion, (HRGN)regM->pOSRegion, (HRGN)regS->pOSRegion, RGN_DIFF);
    ExtractRects(regD);
}

void RMAUnionRegion(REGION* reg1, REGION* reg2, REGION* regD)
{
    CombineRgn((HRGN)regD->pOSRegion, (HRGN)reg1->pOSRegion, (HRGN)reg2->pOSRegion, RGN_OR);
    ExtractRects(regD);
}

void RMAIntersectRegion(REGION* reg1, REGION* reg2, REGION* regD)
{
    CombineRgn((HRGN)regD->pOSRegion, (HRGN)reg1->pOSRegion, (HRGN)reg2->pOSRegion, RGN_AND);
    ExtractRects(regD);
}

BOOL RMAEqualRegion(REGION* reg1, REGION* reg2)
{
    if (EqualRgn((HRGN)reg1->pOSRegion, (HRGN)reg2->pOSRegion))
    {
	if (!memcmp(&reg1->extents, &reg2->extents, sizeof(PNxRect)))
	{
	    return  TRUE;
	}
    }

    return FALSE;
}

void RMADestroyRegion(REGION* reg)
{
    if (reg)
    {
	DeleteObject((HRGN)reg->pOSRegion);
    }
    PN_DELETE(reg);
}

REGION* RMACreateRegion()
{
    return RMACreateRectRegion(0,0,0,0);
}

#elif defined(_MACINTOSH)

REGION* RMACreateRectRegion(int left, int top, int right, int bottom)
{
    REGION* retval = new REGION;
    return retval;
}

void	RMASubtractRegion(REGION* regM, REGION* regS, REGION* regD);
{
}

void	RMAUnionRegion(REGION* reg1, REGION* reg2, REGION* regD);
{
}

void	RMAIntersectRegion(REGION* reg1, REGION* reg2, REGION* regD);
{
}

BOOL	RMAEqualRegion(REGION* reg1, REGION* reg2);
{
    return FALSE;
}

void	RMADestroyRegion(REGION* reg);
{
    PN_DELETE(reg);
}

REGION* RMACreateRegion();
{
    REGION* retval = new REGION;
    return retval;
}

#else

REGION* RMACreateRectRegion(int left, int top, int right, int bottom)
{
    REGION* retval = new REGION;
    return retval;
}

void	RMASubtractRegion(REGION* regM, REGION* regS, REGION* regD);
{
}

void	RMAUnionRegion(REGION* reg1, REGION* reg2, REGION* regD);
{
}

void	RMAIntersectRegion(REGION* reg1, REGION* reg2, REGION* regD);
{
}

BOOL	RMAEqualRegion(REGION* reg1, REGION* reg2);
{
    return FALSE;
}

void	RMADestroyRegion(REGION* reg);
{
    PN_DELETE(reg);
}

REGION* RMACreateRegion();
{
    REGION* retval = new REGION;
    return retval;
}

#endif




