/****************************************************************************
 * 
 *  $Id: netswap.cpp,v 1.3 1997/10/13 20:57:12 christof Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 *
 *  netswap.cpp
 *
 *  Sample Netbyte Order support routines for RealMedia Architecture (RMA).    
 *
 */


/****************************************************************************
 * Includes
 */
#include "netswap.h"
#include "pntypes.h"
	

/****************************************************************************
 *  Portable implementation of bit manipulation routines
 */
#ifndef NET_ENDIAN

    NetWord WToNet(HostWord wHost)
    {
	NetWord wNet;
	unsigned char *uc = (unsigned char *)&wNet;
	*uc++ = (unsigned char) ((wHost & 0xFF00) >> 8);
	*uc++ = (unsigned char) (wHost & 0x00FF);
	return wNet;
    } 	

    HostWord WToHost(NetWord wNet)
    {
	HostWord wHost;
	unsigned char *uc = (unsigned char *)&wNet;
	wHost = (*uc++) << 8;
	wHost += (*uc++);
	return wHost;
    }
    
    NetDWord DwToNet(HostDWord dwHost)
    {
	NetDWord dwNet;
	unsigned char *uc = (unsigned char *)&dwNet;
	*uc++ = (unsigned char) ((dwHost & 0xFF000000L) >> 24);
	*uc++ = (unsigned char) ((dwHost & 0x00FF0000L) >> 16);
	*uc++ = (unsigned char) ((dwHost & 0x0000FF00L) >> 8);
	*uc++ = (unsigned char) (dwHost & 0x000000FFL);
	return dwNet;
    }
    	
    HostDWord DwToHost(NetDWord dwNet) 
    {
	HostDWord dwHost;
	unsigned char *uc = (unsigned char *)&dwNet;
	dwHost = ((unsigned long)(*uc++)) << 24;
	dwHost += ((unsigned long)(*uc++)) << 16;
	dwHost += ((unsigned long)(*uc++)) << 8;
	dwHost += (unsigned long)(*uc++);
	return dwHost;
    }
#endif /* NET_ENDIAN */


/****************************************************************************
 *  TestBigEndian
 */
int 
TestBigEndian(void)
{
    unsigned long test = 0x000000FF;
    char *temp = (char *)&test;

    /* on big endian machines, the first byte should be 0x00, on little endian 0xFF.*/

    return (temp[0] == 0);
}


/****************************************************************************
 *  SwapWordBytes
 *
 *  Converts 2 byte ints from big endian to little endian or vice versa
 */
void 
SwapWordBytes(HostWord *data, int numInts)
{
    INT16     i,temp;
    char    *temp1, *temp2;

    /* temp2 points to our temporary int that will hold the swapped bytes */
    temp2 = (char *) &temp;

    for(i = 0; i < numInts; i++)
    {
	temp1 = (char *) &data[i];

        temp2[0] = temp1[1];
	temp2[1] = temp1[0];

	data[i] = temp;
    }
}


/****************************************************************************
 *  SwapDWordBytes
 *
 *  Converts 4 byte ints from big endian to little endian or vice versa
 */
void 
SwapDWordBytes(HostDWord *data, int numInts)
{
    INT32     i,temp;
    char    *temp1, *temp2;

    /* temp2 points to our temporary int that will hold the swapped bytes */
    temp2 = (char *) &temp;

    for(i = 0; i < numInts; i++)
    {
	temp1 = (char *) &data[i];

        temp2[0] = temp1[3];
        temp2[1] = temp1[2];
        temp2[2] = temp1[1];
	temp2[3] = temp1[0];

	data[i] = temp;
    }
}
