/****************************************************************************
 * 
 *  $Id: pcmrendr.h,v 1.7 1998/07/14 03:33:01 ehodge Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 * 
 *  pcmrendr.h
 *
 *  PCM audio Renderer plug-in for the RealMedia Architecture (RMA).
 *
 */
#ifndef _PCMRENDR_H_
#define _PCMRENDR_H_


/****************************************************************************
 * Constants
 */

/* used for endianness in the pcm header */
#define PN_PCM_LITTLEENDIAN 0
#define PN_PCM_BIGENDIAN    1
/* used for usFormatTag in the pcm header */
#define PN_PCM_TWOS_COMPLEMENT 1
#define PN_PCM_ZERO_OFFSET     2


/****************************************************************************
 * typedef's
 */
typedef struct tPCMHEADER
{
    UINT16      usFormatTag;         /* format type */
    UINT16      usChannels;          /* number of channels (i.e. mono, stereo..*/
    UINT32      ulSamplesPerSec;     /* sample rate */
    UINT16      usBitsPerSample;     /* number of bits per sample of mono data*/
    UINT16	usSampleEndianness;  /* for 16-bit samples, 0==little, 1==big */
} PCMHEADER;



/****************************************************************************
 *
 *  CMyExampleRenderer Class
 *
 *  Example implementation of a basic renderer.
 */
class CPCMRenderer :    public IRMAPlugin, 
			public IRMARenderer,
			public IRMADryNotification
{
    public:
    /****** Public Class Methods ******************************************/
    CPCMRenderer();

    /************************************************************************
     *  IRMARenderer Interface Methods                       ref:  rmarendr.h
     */
    STDMETHOD(StartStream) (THIS_ IRMAStream* pStream, IRMAPlayer* pPlayer);
    STDMETHOD(EndStream )  (THIS);
    STDMETHOD(OnHeader  )  (THIS_ IRMAValues* pStreamHeaderObj);
    STDMETHOD(OnPacket  )  (THIS_ IRMAPacket* pPacket, INT32 timeOffset);
    STDMETHOD(OnTimeSync)  (THIS_ UINT32 currentPlayBackTime);
    STDMETHOD(OnPreSeek )  (THIS_ UINT32 timeBefore, UINT32 timeAfter);
    STDMETHOD(OnPostSeek)  (THIS_ UINT32 timeBefore, UINT32 timeAfter);
    STDMETHOD(OnPause   )  (THIS_ UINT32 timeBeforePause);
    STDMETHOD(OnBegin   )  (THIS_ UINT32 timeAfterBegin);
    STDMETHOD(OnBuffering) (THIS_ UINT32 reason, UINT16 percentComplete);
    STDMETHOD(GetRendererInfo)
        (THIS_
         REF(const char**) pStreamMimeTypes,
         REF(UINT32)       initialGranularity
        );
    STDMETHOD(GetDisplayType)
        (THIS_
         REF(RMA_DISPLAY_TYPE) displayType,
         REF(IRMABuffer*)      pDisplayInfo
        );
    STDMETHOD(OnEndofPackets) (THIS);


    /************************************************************************
     *  IRMAPlugin Interface Methods                         ref:  rmaplugn.h
     */
    STDMETHOD(GetPluginInfo)
        (THIS_ 
          REF(BOOL)        bLoadMultiple,
          REF(const char*) pDescription,
          REF(const char*) pCopyright,
          REF(const char*) pMoreInfoURL,
          REF(UINT32)      versionNumber
        );
    STDMETHOD(InitPlugin) (THIS_ IUnknown* pContext);


    /************************************************************************
     *  IRMADryNotification Interface Methods                ref:  rmaausvc.h
     */
    STDMETHOD(OnDryNotification)
	(THIS_
	 UINT32 /*IN*/ ulCurrentStreamTime,
	 UINT32 /*IN*/ ulMinimumDurationRequired
	);


    /************************************************************************
     *  IUnknown COM Interface Methods                          ref:  pncom.h
     */
    STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
    STDMETHOD_(UINT32, AddRef ) (THIS);
    STDMETHOD_(UINT32, Release) (THIS);


    private:
    /****** Private Class Variables ****************************************/
    INT32		    m_lRefCount;
    IUnknown*		    m_pContext;
    IRMAStream*		    m_pStream;
    IRMAPlayer*		    m_pPlayer;
    IRMAValues*		    m_pHeader;
    IRMAPacket*		    m_pLastPacket;
    IRMAAudioPlayer*	    m_pAudioPlayer;
    IRMAAudioStream*	    m_pAudioStream;
    IRMACommonClassFactory* m_pCommonClassFactory;
    UINT32		    m_ulLastTime;
    UINT32		    m_ulGranularity;
    UINT32		    m_ulCurrentTime;
    FILE*		    m_logfile;
    int			    m_pcmfile;
    BOOL		    m_bSwapSampleBytes;
    BOOL		    m_bZeroOffsetPCM;
    BOOL		    m_bIsGapInStreaming;
    FiveMinuteQueue	    m_PacketQueue;
    RMAAudioFormat	    m_audioFmt;
    BOOL		    m_bInSeekMode;

    /****** Private Static Class Variables *********************************/
    static const char*	    zm_pDescription;
    static const char*	    zm_pCopyright;
    static const char*	    zm_pMoreInfoURL;
    static const char*	    zm_ppStreamMimeTypes[];

    /****** Private Class Methods ******************************************/
    UINT32 CalcMs(ULONG32 ulNumBytes);
    ~CPCMRenderer() 
	{ 
	    EndStream();
	};

    PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME       // Avoids GCC compiler warning
};


#define AUDIO_PCMHEADER_MAGIC_NUMBER    0x4350
#define AUDIO_WAVHEADER_MAGIC_NUMBER    0x7677

/*Some pcm streams coming from a RealNetworks audio or video
 * file format plugin may have a header structure in the stream
 * that contains the following (otherwise their header will
 * start with a PCMHEADER structure).  AudioPCMHEADER::unpack()
 * takes care of byte order of this header data:
 */
class AudioPCMHEADER
{
public:
    UINT8*	unpack(UINT8* buf, UINT32 len);
    const UINT32	static_size() {return 16;}

    UINT16	usVersion;
    UINT16	usMagicNumberTag;
    UINT16	usFormatTag;
    UINT16	usChannels;
    UINT32	ulSamplesPerSec;
    UINT16	usBitsPerSample;
    UINT16	usSampleEndianness;
};

inline UINT8*
AudioPCMHEADER::unpack(UINT8* buf, UINT32 len)
{
    if (len <= 0)
	return 0;
    UINT8* off = buf;

    {usVersion = *off++<<8; usVersion |= *off++;}
    {usMagicNumberTag = *off++<<8; usMagicNumberTag |= *off++;}
    {usFormatTag = *off++<<8; usFormatTag |= *off++;}
    {usChannels = *off++<<8; usChannels |= *off++;}
    {
	ulSamplesPerSec = ((UINT32)*off++)<<24; ulSamplesPerSec |= ((UINT32)*off++)<<16;
	ulSamplesPerSec |= ((UINT32)*off++)<<8; ulSamplesPerSec |= ((UINT32)*off++);
    }
    {usBitsPerSample = *off++<<8; usBitsPerSample |= *off++;}
    {usSampleEndianness = *off++<<8; usSampleEndianness |= *off++;}
    return off;
}


#endif // ndef _PCMRENDR_H_

