/****************************************************************************
 * 
 *  $Id: ex2strms.cpp,v 1.15 1999/03/30 17:57:15 darrens Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 * 
 *  ex2strms.cpp
 *
 *
 *  Example File Format for RealMedia Architecture that opens a container
 *  file with n streams for reading (this example only does 2 streams)
 *
 */


/****************************************************************************
 * 
 *  EXAMPLE FILE FORMAT for (.EF2, application/x-rma-example2)
 *
 *  This is a very simple file format, that has a 152 byte file header,
 *  n 28 byte stream headers, and then fixed packets of 100 bytes each
 *  which represent 1 second chunks of data for each stream.
 *
 */


/****************************************************************************
 * Defines
 */
#define   INITGUID     /* Interface ID's */

#ifdef _DEBUG
#undef PN_THIS_FILE             
static char PN_THIS_FILE[] = __FILE__;
#endif


/****************************************************************************
 * Includes
 */
#include <stdio.h>
#include <string.h>

#include "pntypes.h"
#include "pncom.h"
#include "pnmarsh.h"
#include "rmapckts.h"
#include "rmafiles.h"
#include "rmaformt.h"
#include "rmaplugn.h"
#include "rmacomm.h"

#include "ex2strms.h"
#include "ex2strms.ver"



/****************************************************************************
 *  RMACreateInstance                                        ref:  rmaplugn.h
 *
 *  This routine creates a new instance of the CMyExampleFileFormat class.
 *  It is called when the RMA core application is launched, and whenever
 *  an URL associated with this plug-in is opened.
 */
STDAPI
RMACreateInstance(IUnknown** ppIUnknown)
{
    *ppIUnknown = (IUnknown*)(IRMAPlugin*)new CExampleFileFormat();
    if (*ppIUnknown)
    {
	(*ppIUnknown)->AddRef();
	return PNR_OK;
    }
    return PNR_OUTOFMEMORY;
}


/****************************************************************************
 *  CMyExampleFileFormat static variables                    ref:  ex2strms.h
 *
 *  These variables are passed to the RMA core to provide information about
 *  this plug-in. They are required to be static in order to remain valid
 *  for the lifetime of the plug-in.
 */
const IID   CExampleFileFormat::zm_myIID           = IID_IRMAFileFormatObject;
const char* CExampleFileFormat::zm_pDescription    = "Example File Format 2 Plugin";
const char* CExampleFileFormat::zm_pCopyright      = "(c) 1995,1996,1997 RealNetworks, All rights reserved.";
const char* CExampleFileFormat::zm_pMoreInfoURL    = "http://www.real.com";
const char* CExampleFileFormat::zm_pFileMimeTypes[]  = {"application/x-pn-example2fileformat", NULL};
const char* CExampleFileFormat::zm_pFileExtensions[] = {"ef2", NULL};
const char* CExampleFileFormat::zm_pFileOpenNames[]  = {"Example File Format 2 (*.ef2)", NULL};


/****************************************************************************
 *  IRMAPlugin::GetPluginInfo                                ref:  rmaplugn.h
 *
 *  This routine returns descriptive information about the plug-in, most
 *  of which is used in the About box of the user interface. It is called
 *  when the RMA core application is launched.
 */
STDMETHODIMP
CExampleFileFormat::GetPluginInfo
(
    REF(BOOL)       /*OUT*/ bLoadMultiple,
    REF(const char*)/*OUT*/ pDescription,
    REF(const char*)/*OUT*/ pCopyright,
    REF(const char*)/*OUT*/ pMoreInfoURL,
    REF(ULONG32)    /*OUT*/ ulVersionNumber
)
{
    bLoadMultiple   = TRUE;    /* Must be true for file formats. */
    pDescription    = zm_pDescription;
    pCopyright	    = zm_pCopyright;
    pMoreInfoURL    = zm_pMoreInfoURL;
    ulVersionNumber = TARVER_ULONG32_VERSION;

    return PNR_OK;
}


/****************************************************************************
 *  IRMAFileFormatObject::GetFileFormatInfo                  ref:  rmaformt.h
 *
 *  This routine returns crucial information required to associate this
 *  plug-in with a given MIME type. This information tells the core which
 *  File Format plug-in to use for a particular URL. The routine is called
 *  when the RMA core application is launched.
 */
STDMETHODIMP
CExampleFileFormat::GetFileFormatInfo
(
    REF(const char**) /*OUT*/ pFileMimeTypes,
    REF(const char**) /*OUT*/ pFileExtensions,
    REF(const char**) /*OUT*/ pFileOpenNames
)
{
    pFileMimeTypes  = zm_pFileMimeTypes;
    pFileExtensions = zm_pFileExtensions;
    pFileOpenNames  = zm_pFileOpenNames;

    return PNR_OK;
}


/****************************************************************************
 *  IRMAPlugin::InitPlugin                                   ref:  rmaplugn.h
 *
 *  This routine performs initialization steps such as determining whether
 *  required interfaces are available. It is called when the RMA core 
 *  application is launched, and whenever an URL associated with this
 *  plug-in is opened.
 */
STDMETHODIMP CExampleFileFormat::InitPlugin(IUnknown* /*IN*/ pContext)
{
    m_pContext = pContext;
    m_pContext->AddRef();
    if (PNR_OK != m_pContext->QueryInterface(IID_IRMACommonClassFactory,
                                           (void**)&m_pCommonClassFactory))
    {
	m_pCommonClassFactory = 0;
    }

    return PNR_OK;
}


/****************************************************************************
 *  IRMAFileFormatObject::InitFileFormat                     ref:  rmaformt.h
 *
 *  This routine initializes the file, and stores references to objects used
 *  throughout the example. It is called whenever an URL associated with this
 *  plug-in is opened.
 */
STDMETHODIMP CExampleFileFormat::InitFileFormat
(
    IRMARequest*            /*IN*/  pRequest, 
    IRMAFormatResponse*     /*IN*/  pFormatResponse,
    IRMAFileObject*         /*IN*/  pFileObject
)
{
    PN_RESULT hresult;

    pRequest->AddRef();
    m_pRequest = pRequest;

    pFormatResponse->AddRef();
    m_pFFResponse = pFormatResponse;

    pFileObject->AddRef();
    m_pFileObject = pFileObject;

    m_pStreamArray = new StreamObject*[MAX_STREAMS];

    for (UINT32 i = 0; i < MAX_STREAMS; i++)
    {
	m_pStreamArray[i] = 0;
    }

    hresult = m_pCommonClassFactory->CreateInstance(CLSID_IRMAFileSystemManager,
                                                    (void**)&m_pFSManager);
    if (hresult != PNR_OK)
    {
	m_pFSManager = 0;
	goto InitError;
    }

    m_pFSManagerResponse = new CExampleFSManagerResponse(this);
    m_pFSManagerResponse->AddRef();

    IRMAFileSystemManagerResponse* pResponse;
    m_pFSManagerResponse->QueryInterface(IID_IRMAFileSystemManagerResponse, (void **)&pResponse);
    hresult = m_pFSManager->Init(pResponse);
    pResponse->Release();

    if (hresult != PNR_OK)
    {
	goto InitError;
    }

    return PNR_OK;

InitError:
    m_pFFResponse->InitDone(PNR_FAIL);

    return hresult;
}       


/****************************************************************************
 *  CExampleFileFormat::InitDone                             ref:  ex2strms.h
 *
 *  This routine notifies the RMA core that the initialization of the file is
 *  done. It is called by the IRMAFileObject when the initialization of the
 *  file has completed.
 */
PN_RESULT
CExampleFileFormat::InitDone
(
    PN_RESULT   status,
    UINT16      uStreamNumber
)
{
    if (status != PNR_OK)
    {
	if (uStreamNumber == 0)
	{
	    m_pFFResponse->InitDone(status);
	}
	else
	{
	    m_pFFResponse->StreamHeaderReady(status, 0);
	}

	return PNR_OK;
    }

    StreamObject* pStream = m_pStreamArray[uStreamNumber];

    /* If we are not ready then something has gone wrong */

    if (pStream->m_state != InitPending) return PNR_UNEXPECTED;

    if (uStreamNumber == 0)
    {
	/*
	 * Since this the InitDone for the first FileObject, go ahead and
	 * signal that the FileFormatObject is ready
	 */

	pStream->m_state = Ready;

	m_pFFResponse->InitDone(status);
    }
    else
    {
	/*
	 * The Init() call must have come from GetStreamHeader, so start
	 * the stream header process
	 */

	pStream->m_state = GetStreamHeaderSeekPending;

	UINT32 offset = FILE_HEADER_OFFSET +
			FILE_HEADER_SIZE +
			STREAM_HEADER_SIZE*uStreamNumber;

	/* Actually seek... */

	pStream->m_pFileObject->Seek(offset,FALSE);

    }

    return PNR_OK;
}


/****************************************************************************
 *  IRMAFileFormatObject::GetFileHeader                      ref:  rmaformt.h
 *
 *  This routine returns to the RMA core an object containing the file
 *  header information. Several routines are actually required to complete
 *  the process due to the asynchronous nature of the RMA file system. This
 *  method is called by the RMA core after the file has been initialized.
 */
STDMETHODIMP
CExampleFileFormat::GetFileHeader()
{
    /* If we are not ready then something has gone wrong */

    if (m_pStreamArray[0]->m_state != Ready) return PNR_UNEXPECTED;

    /*
     * To give the file header we need to be make sure the file is 
     * positioned at the start of the file. We need to call the file 
     * object's seek method.
     *
     * Since this is asynchronous we need to note our state so we can
     * correctly respond to the seek complete response from the file
     * object.
     */

    m_pStreamArray[0]->m_state = GetFileHeaderSeekPending;

    /* Actually seek... */

    m_pStreamArray[0]->m_pFileObject->Seek(FILE_HEADER_OFFSET,FALSE);

    /* See SeekDone() for next "step" of the GetStreamHeader process. */

    return PNR_OK;
}


/****************************************************************************
 *  IRMAFileFormatObject::GetStreamHeader                    ref:  rmaformt.h
 *
 *  This routine returns to the RMA core an object containing the stream
 *  header information for a particular stream. Several routines are actually
 *  required to complete the process due to the asynchronous nature of the
 *  RMA file system. This method is called (after the file header has been
 *  read) by the RMA core for each stream in the file format.
 */
STDMETHODIMP
CExampleFileFormat::GetStreamHeader(UINT16 uStreamNumber)
{
    /*
     * Need to open a new FileObject for each stream after the first.
     * The seek for the header will occur in InitDone
     */

    if (uStreamNumber > 0)
    {
	return HandleFileObject(uStreamNumber);
    }
   
    StreamObject* pStream = m_pStreamArray[uStreamNumber];

    /* If we are not ready then something has gone wrong */

    if (pStream->m_state != Ready) return PNR_UNEXPECTED;

    /*
     * To give the header we need to be make sure the file is positioned
     * at the start of the file. We need to call the file object's
     * seek method.
     *
     * Since this is asynchronous we need to note our state so we can
     * correctly respond to the seek complete response from the file
     * object.
     */

    pStream->m_state = GetStreamHeaderSeekPending;

    UINT32 offset = FILE_HEADER_OFFSET +
		    FILE_HEADER_SIZE +
		    STREAM_HEADER_SIZE*uStreamNumber;

    /* Actually seek... */

    pStream->m_pFileObject->Seek(offset,FALSE);

    /* See SeekDone() for next "step" of the GetStreamHeader process. */

    return PNR_OK;
}


/****************************************************************************
 *  IRMAFileFormatObject::GetPacket                          ref:  rmaformt.h
 *
 *  This routine returns to the RMA core an object containing the packet
 *  data for a particular stream. Several routines are actually required to
 *  complete the process due to the asynchronous nature of the RMA file
 *  system. This method is called by the RMA core each time it needs another
 *  packet.
 */
STDMETHODIMP
CExampleFileFormat::GetPacket(UINT16 uStreamNumber)
{
    StreamObject* pStream = m_pStreamArray[uStreamNumber];
    PN_RESULT result = PNR_OK;

    /* If we are not ready then something may have gone wrong */

    if (pStream->m_state != Ready)
    {
	/* The only legal non-Ready case */

	if (pStream->m_state != FirstPacketPending)
	{
	    return PNR_UNEXPECTED;
	}
    }

    if (!m_bHeaderSent)
    {
	result = PNR_UNEXPECTED;
    }
    else
    {
	if (pStream->m_ulCurrentOffset == pStream->m_ulDataLength)
	{
	    /* All the data for this stream has been sent */

	    m_pFFResponse->StreamDone(uStreamNumber);
	    return PNR_OK;
	}

	if (pStream->m_state == FirstPacketPending)
	{
	    pStream->m_state = GetPacketSeekPending;

	    pStream->m_pFileObject->Seek(pStream->m_ulDataOffset,FALSE);

	    return result;
	}

	/*
	 * We are being asked to get the next packet and we have sent
	 * the header, so basically we know the next PACKET_SIZE bytes
	 * should be the next packet... again, since read is asynchronous, 
	 * we need to set up our state...
	 */

	pStream->m_state = GetPacketReadPending;

	/* Actually read... */

	pStream->m_pFileObject->Read(PACKET_SIZE);

	/* See ReadDone() for next "step" of GetPacket() */
    }

    return result;
}


/****************************************************************************
 *  CExampleFileFormat::SeekDone                             ref:  ex2strms.h
 *
 *  This routine is called by the IRMAFileObject when the Seek() operation
 *  in the processing file is complete. Other actions are then taken
 *  dependent on the current processing state.
 */
PN_RESULT CExampleFileFormat::SeekDone
(
    PN_RESULT   status,
    UINT16      uStreamNumber
)
{
    StreamObject* pStream = m_pStreamArray[uStreamNumber];
    PN_RESULT result = PNR_OK;

    /*
     * Note, the seek may be done because we needed to seek to produce
     * a packet or because as a file format we were asked to seek to
     * a time position. We need to remember why we tried to seek and
     * respond accordingly...
     */

    switch (pStream->m_state)
    {
	case GetFileHeaderSeekPending:
	{
	    /*
	     * We are in the process of responding to a GetFileHeader()
	     * request and the seek has completed, so now we will try
	     * to read the first FILE_HEADER_SIZE bytes... again, since 
	     * this is asynchronous, we need to set up our state...
	     */

	    pStream->m_state = GetFileHeaderReadPending;

	    /* Actually read...*/

	    pStream->m_pFileObject->Read(FILE_HEADER_SIZE);

	    /* See ReadDone() for next "step" of GetFileHeader() */
	}
	break;

	case GetStreamHeaderSeekPending:
	{
	    /*
	     * We are in the process of responding to a GetStreamHeader()
	     * request and the seek has completed, so now we will try
	     * to read the first STREAM_HEADER_SIZE bytes... again, since 
	     * this is asynchronous, we need to set up our state...
	     */

	    pStream->m_state = GetStreamHeaderReadPending;

	    /* Actually read... */

	    pStream->m_pFileObject->Read(STREAM_HEADER_SIZE);

	    /* See ReadDone() for next "step" of GetStreamHeader() */
	}
	break;

	case GetPacketSeekPending:
	{
	    /*
	     * We are in the process of responding to a GetPacket()
	     * request and the seek has completed, so now we will try
	     * to read the first PACKET_SIZE bytes... again, since 
	     * this is asynchronous, we need to set up our state...
	     */

	    pStream->m_state = GetPacketReadPending;

	    /* Actually read... */

	    pStream->m_pFileObject->Read(PACKET_SIZE);

	    /* See ReadDone() for next "step" of GetStreamHeader() */
	}
	break;

	case SeekSeekPending:
	{
	    /*
	     * We are in the process of responding to a Seek()
	     * request and the seek has completed for this stream. If
	     * all the streams have responded, the seek is done so
	     * inform the FormatResponse.
	     */

	    pStream->m_state = Ready;

	    m_ulSeekDoneCount++;

	    if (m_ulSeekDoneCount == m_ulNumStreams)
	    {
		/* Tell the FormatResponse of our success in seeking. */

		m_pFFResponse->SeekDone(status);
	    }
	}
	break;

	default:
	{
	    result = PNR_UNEXPECTED;
	}
	break;
    }

    return result;
}


/****************************************************************************
 *  CExampleFileFormat::ReadDone                             ref:  ex2strms.h
 *
 *  This routine is called by the IRMAFileObject when the Read() from the
 *  file is done. Other actions are then taken dependent on the current
 *  processing state.
 */
PN_RESULT CExampleFileFormat::ReadDone
(
    PN_RESULT           status,
    UINT16              uStreamNumber,
    IRMABuffer*         pBuffer
)
{
    StreamObject* pStream = m_pStreamArray[uStreamNumber];
    PN_RESULT result = PNR_OK;

    /*
     * Note, the read may be done because we needed to read to produce
     * a header or a packet. We need to remember why we tried to read and
     * respond accordingly...
     */

    switch (pStream->m_state)
    {
	case GetFileHeaderReadPending:
	{
	    /*
	     * We are in the process of responding to a GetFileHeader()
	     * request and the read has completed, which means we
	     * are done and can go back to the ready state...
	     */

	    pStream->m_state = Ready;

	    /*
	     * We now need to form a "header" object and pass it
	     * off to our controller. Notice since our header data
	     * comes straight out of the file as is, we don't need
	     * to copy data at all, the IRMABuffer returned by read
	     * can be placed into the header object and passed off.
	     *
	     * Create header object here, notice that we call the
	     * CreateInstance method of the controller, but we could
	     * have implemented our own object that exposed the IRMAPacket
	     * interface.
	     */

	    IRMAValues* pHeader;
	    if (PNR_OK == m_pCommonClassFactory->CreateInstance(CLSID_IRMAValues,
						(void**)&pHeader))
	    {
		IRMABuffer* pTitle = NULL;
		IRMABuffer* pAuthor = NULL;
		IRMABuffer* pCopyright = NULL;
		char*       pTitleData;
		char*       pAuthorData;
		char*       pCopyrightData;
		char*       pBufferData;

		m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,(void**)&pTitle);
		m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,(void**)&pAuthor);
		m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,(void**)&pCopyright);

		pTitle->SetSize(INFO_SIZE+1);
		pAuthor->SetSize(INFO_SIZE+1);
		pCopyright->SetSize(INFO_SIZE+1);

		pTitleData      = (char*)pTitle->GetBuffer();
		pAuthorData     = (char*)pAuthor->GetBuffer();
		pCopyrightData  = (char*)pCopyright->GetBuffer();
		pBufferData     = (char*)pBuffer->GetBuffer();

		m_ulNumStreams = getshort((Byte*)pBufferData);
		pBufferData += 2;

		strncpy(pTitleData,pBufferData,INFO_SIZE);
		pTitleData[INFO_SIZE] = '\0';
		pBufferData += INFO_SIZE;

		strncpy(pAuthorData,pBufferData,INFO_SIZE);
		pAuthorData[INFO_SIZE] = '\0';
		pBufferData += INFO_SIZE;

		strncpy(pCopyrightData,pBufferData,INFO_SIZE);
		pCopyrightData[INFO_SIZE] = '\0';

		/* Fill in the Header with the relevant data... */

		pHeader->SetPropertyBuffer ("Title",        pTitle);
		pHeader->SetPropertyBuffer ("Author",       pAuthor);
		pHeader->SetPropertyBuffer ("Copyright",    pCopyright);
		pHeader->SetPropertyULONG32("StreamCount",  m_ulNumStreams);

		/*
		 * Tell the FormatResponse of our success in 
		 * getting the header.
		 */

		m_pFFResponse->FileHeaderReady(status, pHeader);

		/* Release our reference on the header! */

		pHeader->Release();
		pTitle->Release();
		pAuthor->Release();
		pCopyright->Release();
	    }
	}
	break;

	case GetStreamHeaderReadPending:
	{
	    /*
	     * We are in the process of responding to a GetStreamHeader()
	     * request and the read has completed, which means we
	     * are done and must now wait for to seek to the beginning
	     * of the data section
	     */

	    pStream->m_state = FirstPacketPending;

	    /*
	     * We now need to form a "header" object and pass it
	     * off to our controller. Notice since our header data
	     * comes straight out of the file as is, we don't need
	     * to copy data at all, the IRMABuffer returned by read
	     * can be placed into the header object and passed off.
	     *
	     * Create header object here, notice that we call the
	     * CreateInstance method of the controller, but we could
	     * have implemented our own object that exposed the IRMAPacket
	     * interface.
	     */

	    IRMAValues* pHeader;
	    if (PNR_OK == m_pCommonClassFactory->CreateInstance(CLSID_IRMAValues,
						(void**)&pHeader))
	    {
		Byte*       pBufferData;

		/*
		 * Remember that we have sent the header and packets
		 * are now valid to be sent... (just a little bit of
		 * sanity checking).
		 */

		m_bHeaderSent = TRUE;

		/*
		 * The first 4 bytes contains the data offset
		 * The second 4 bytes contains the data length
		 */

		pBufferData     = pBuffer->GetBuffer();

		pStream->m_ulDataOffset = getlong(pBufferData);
		pStream->m_ulDataLength = getlong(pBufferData+4);

		ULONG32 ulMaxBitRate = 8*PACKET_SIZE*(TIME_PER_PACKET/1000);
		ULONG32 ulAvgBitRate = 8*PACKET_SIZE*(TIME_PER_PACKET/1000);
		ULONG32 ulMaxPacketSize = PACKET_SIZE;
		ULONG32 ulAvgPacketSize = PACKET_SIZE;
		ULONG32 ulStartTime = 0;
		ULONG32 ulPreroll = 0;
		ULONG32 ulDuration = (pStream->m_ulDataLength/PACKET_SIZE) *
				     TIME_PER_PACKET;
		IRMABuffer*     pStreamName = NULL;
		IRMABuffer*     pMimeType = NULL;
		IRMABuffer*	pABMRules = NULL;

		if ((PNR_OK != 
		    m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,
						(void**)&pStreamName)) ||
		    (PNR_OK !=
		    m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,
						(void**)&pMimeType))   ||
		    (PNR_OK !=
		    m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,
						(void**)&pABMRules)))
		{
		    pHeader->Release();
		    m_pFFResponse->StreamHeaderReady(PNR_FAILED, 0);
		    return PNR_FAILED;
		}

		/* ABM Rules for this stream 
		 * Representation probably needs to change
		 */
		char szStreamName[100];
		char szMimeType[] = "application/x-pn-examplestream";
		char szABMRules[] = "substream=0\nsubstream=1";

		sprintf(szStreamName,"%s%d", "example stream", uStreamNumber);

		pStreamName->Set((BYTE*)szStreamName, strlen(szStreamName)+1);
		pMimeType->Set((BYTE*)szMimeType, strlen(szMimeType)+1);
		pABMRules->Set((BYTE*)szABMRules, strlen(szABMRules)+1);

		/* Fill in the Header with the relevant data... */

		pHeader->SetPropertyBuffer ("OpaqueData",    pBuffer);
		pHeader->SetPropertyULONG32("StreamNumber",  uStreamNumber);
		pHeader->SetPropertyULONG32("MaxBitRate",    ulMaxBitRate);
		pHeader->SetPropertyULONG32("AvgBitRate",    ulAvgBitRate);
		pHeader->SetPropertyULONG32("MaxPacketSize", ulMaxPacketSize);
		pHeader->SetPropertyULONG32("AvgPacketSize", ulAvgPacketSize);
		pHeader->SetPropertyULONG32("StartTime",     ulStartTime);
		pHeader->SetPropertyULONG32("Preroll",       ulPreroll);
		pHeader->SetPropertyULONG32("Duration",      ulDuration);
		pHeader->SetPropertyCString("StreamName",    pStreamName);
		pHeader->SetPropertyCString("MimeType",      pMimeType);
		pHeader->SetPropertyCString("ABMRules",      pABMRules);

		//force popup window
		pHeader->SetPropertyULONG32("PopupWindow",	 1);

		/*
		 * Tell the FormatResponse of our success in 
		 * getting the header.
		 */

		m_pFFResponse->StreamHeaderReady(status, pHeader);

		/* Release our references */

		pHeader->Release();
		pStreamName->Release();
		pMimeType->Release();
		pABMRules->Release();
	    }
	}
	break;

	case GetPacketReadPending:
	{
	    /*
	     * We are in the process of responding to a GetPacket()
	     * request and the read has completed, which means we
	     * are done and can go back to the ready state...
	     */

	    pStream->m_state = Ready;

	    if (status == PNR_OK)
	    {
		/*
		 * We now need to form a "packet" object and pass it
		 * off to our controller. Notice since our packet data
		 * comes straight out of the file as is, we don't need
		 * to copy data at all, the IRMABuffer returned by read
		 * can be placed into the packet object and passed off.
		 *
		 * Create packet object here, notice that we call the
		 * CreateInstance method of the controller, but we could
		 * have implemented our own object that exposed the IRMAPacket
		 * interface.
		 */

		IRMAPacket* pPacket;
		if (PNR_OK == m_pCommonClassFactory->CreateInstance(CLSID_IRMAPacket,
						       (void**)&pPacket))
		{
		    /* Fill in the Packet with the relevant data... */

		    pPacket->Set(pBuffer,
				 pStream->m_ulCurrentTime,
				 uStreamNumber,
				 RMA_ASM_SWITCH_ON,
				 0);

		    /* All packets are 1 second apart */

		    pStream->m_ulCurrentTime += TIME_PER_PACKET;
		    pStream->m_ulCurrentOffset += PACKET_SIZE;

		    /*
		     * Tell the FormatResponse of our success in 
		     * getting the packet.
		     */

		    m_pFFResponse->PacketReady(status, pPacket);
	
		    /* Release our reference on the packet! */

		    pPacket->Release();
		}
	    }

	    /*
	     * If the read failed then we will call PacketReady() with
	     * a NULL packet!
	     */

	    else
	    {
		pStream->m_ulCurrentTime = 0;
		pStream->m_ulCurrentOffset = 0;

		/*
		 * Tell the FormatResponse of our success in 
		 * getting the packet.
		 */

		m_pFFResponse->StreamDone(uStreamNumber);
	    }
	}
	break;

	default:
	{
	    result = PNR_UNEXPECTED;
	}
	break;
    }

    return result;
}


/****************************************************************************
 *  IRMAFileFormatObject::Seek                               ref:  rmaformt.h
 *
 *  This routine moves to the packet nearest to the requested time in the
 *  file. It is called, for example, in response to the user moving the
 *  slider to a different location in the playback timeline.
 */
STDMETHODIMP
CExampleFileFormat::Seek(ULONG32 ulOffset)
{
    /*
     * To actually seek to the correct place in the time line,
     * we will seek in the file. We need to call all of the file
     * objects'seek method.
     *
     * Since this is asynchronous we need to note our state so we can
     * correctly respond to the seek complete response from the file
     * object.
     */

    StreamObject* pStream;
    ULONG32 ulPacketOffset;
    m_ulSeekDoneCount = 0;
    for (UINT32 i = 0; i < m_ulNumStreams; i++)
    {
	pStream = m_pStreamArray[i];

	/*
	 * Notice that the seek is passed as time in milliseconds, we
	 * need to convert this to our packet number and its offset
	 * in the file...
	 */

	ULONG32 ulPacket = ulOffset/TIME_PER_PACKET;

	pStream->m_state = SeekSeekPending;

	/*
	 * Note that the real time we are seeked to is not exactly what
	 * was requested...
	 */

	pStream->m_ulCurrentTime = ulPacket*TIME_PER_PACKET;
	pStream->m_ulCurrentOffset = ulPacket*PACKET_SIZE;

	ulPacketOffset = pStream->m_ulDataOffset + (ulPacket*PACKET_SIZE);

	/* Actually seek... */

	pStream->m_pFileObject->Seek(ulPacketOffset,FALSE);
    }

    /* See SeekDone() for next "step" of the Seek() process. */

    return PNR_OK;
}


/****************************************************************************
 *  IRMAFileFormatObject::Close                              ref:  rmaformt.h
 *
 *  This routine handles any cleanup required before closing the plug-in. All
 *  references to objects should be released and memory deallocated. This is
 *  called by the RMA core when the playback is finished or stopped.
 */
STDMETHODIMP
CExampleFileFormat::Close()
{
    if (m_pContext)
    {
	m_pContext->Release();
    }

    if (m_pRequest)
    {
	m_pRequest->Release();
    }

    if (m_pFFResponse)
    {
	m_pFFResponse->Release();
    }

    if (m_pFileObject)
    {
	m_pFileObject->Close();
	m_pFileObject->Release();
    }

    if (m_pCommonClassFactory)
    {
	m_pCommonClassFactory->Release();
    }

    if (m_pFSManager)
    {
	m_pFSManager->Release();
    }

    m_pFSManagerResponse->Release();

    if (m_pStreamArray)
    {
	for (UINT32 i = 0; i < m_ulNumStreams; i++)
	{
	    if (m_pStreamArray[i])
	    {
		m_pStreamArray[i]->m_pFileObject->Close();
		m_pStreamArray[i]->m_pFileObject->Release();
		delete m_pStreamArray[i];
	    }
	}

	delete m_pStreamArray;
    }

    return PNR_OK;
}


/****************************************************************************
 *  CExampleFileFormat::CloseDone                            ref:  ex2strms.h
 *
 *  Notification interface provided by users of the IRMAFileObject
 *  interface. This method is called by the IRMAFileObject when the
 *  close of the file is complete.
 */
PN_RESULT CExampleFileFormat::CloseDone
(
    PN_RESULT   status,
    UINT16      uStreamNumber
)
{
    return PNR_OK;
}


/****************************************************************************
 *  CExampleFileFormat::WriteDone                            ref:  ex2strms.h
 *
 *  Notification interface provided by users of the IRMAFileObject
 *  interface. This method is called by the IRMAFileObject when the
 *  write of the file is complete.
 */
PN_RESULT CExampleFileFormat::WriteDone
(
    PN_RESULT   status,
    UINT16      uStreamNumber
)
{
    /* We don't ever write, so we don't expect to get this... */

    return PNR_UNEXPECTED;
}


/****************************************************************************
 *  CExampleFileFormat::FileSystemReady                      ref:  ex2strms.h
 *
 *  Notification interface provided by users of the IRMAFileObject
 *  interface. This method is called by the
 *  IRMAFileSystemManagerResponse object when the file system is
 *  ready to handle requests.
 */
PN_RESULT
CExampleFileFormat::FileSystemReady
(
    PN_RESULT status
)
{
    if (status == PNR_OK)
    {
	m_pFSManagerResponse->m_state = READY;
    }
    else
    {
	m_pFSManagerResponse->m_state = FAILED;
    }

    return HandleFileObject(0);
}

/****************************************************************************
 *  CExampleFileFormat::FileReady                            ref:  ex2strms.h
 *
 *  Notification interface provided by users of the IRMAFileObject
 *  interface. This method is called by the
 *  IRMAFileSystemManagerResponse object when the requested
 *  FileObject is ready. It may return NULL with PN_RESULT_FAIL
 *  if the requested filename did not exist.
 */
PN_RESULT
CExampleFileFormat::FileReady
(
    PN_RESULT status,
    UINT16    uStreamNumber,
    IUnknown* pIUnknown
)
{
    IRMAFileObject* pFileObject;

    if (PNR_OK != pIUnknown->QueryInterface(IID_IRMAFileObject,
                                          (void**)&pFileObject))
    {
	m_pFSManagerResponse->m_state = FAILED;

	return InitDone(status, uStreamNumber);
    }

    m_pFSManagerResponse->m_state = READY;

    /*
     * We want to make sure the file object is initialized, we can't 
     * actually return the header count until the file init is done...
     * (See InitDone).
     */

    m_pStreamArray[uStreamNumber]                = new StreamObject;
    m_pStreamArray[uStreamNumber]->m_state       = InitPending;
    m_pStreamArray[uStreamNumber]->m_pFileObject = pFileObject;

    pFileObject->Init(PN_FILE_READ | PN_FILE_BINARY,
		      new CExampleFileFormat::CExampleFileResponse(this, uStreamNumber));

    return PNR_OK;
}


/****************************************************************************
 *  CExampleFileFormat::HandleFileObject                     ref:  ex2strms.h
 *
 *  Updates the necessary data structures when the file format
 *  opens a new FileObject
 */
PN_RESULT
CExampleFileFormat::HandleFileObject
(
    UINT16 uStreamNumber
)
{
    /*
     * Note, we need to create a new FileResponse object for each
     * FileObject
     */

    if (uStreamNumber == 0)
    {
	/*
	 * This is the original IRMAFileObject passed to InitFileFormatInfo
	 */

	return FileReady(PNR_OK,
	                 0,
	                 (IUnknown*)(IRMAFileObject*)m_pFileObject);
    }
    else
    {
	const CHAR* pFileName;
	PN_RESULT hresult;

	hresult = m_pFileObject->GetFilename(pFileName);

	if (hresult != PNR_OK)
	{
	    return hresult;
	}

	if (!m_pFSManager || m_pFSManagerResponse->m_state != READY)
	{
	    return PNR_FAIL;
	}

	/*
	 * Only allow one outstanding GetRelativeFileObject call at a time
	 * so that the m_pFSManagerResponse can keep track of which stream
	 * the IRMAFileObject belongs to
	 */

	m_pFSManagerResponse->m_state = BUSY;
	m_pFSManagerResponse->m_uStreamNumber = uStreamNumber;

	/*
	 * It so happens that we want the same file as that which was
	 * opened for the first IRMAFileObject, so we pass in the file
	 * name unchanged
	 */

	hresult = m_pFSManager->GetRelativeFileObject(m_pFileObject, pFileName);

	if (hresult != PNR_OK)
	{
	    m_pFSManagerResponse->m_state = FAILED;
	}

	return hresult;
    }
}


// IUnknown COM Interface Methods

/****************************************************************************
 *  IUnknown::AddRef                                            ref:  pncom.h
 *
 *  This routine increases the object reference count in a thread safe
 *  manner. The reference count is used to manage the lifetime of an object.
 *  This method must be explicitly called by the user whenever a new
 *  reference to an object is used.
 */
STDMETHODIMP_(ULONG32) CExampleFileFormat::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}


/****************************************************************************
 *  IUnknown::Release                                           ref:  pncom.h
 *
 *  This routine decreases the object reference count in a thread safe
 *  manner, and deletes the object if no more references to it exist. It must
 *  be called explicitly by the user whenever an object is no longer needed.
 */
STDMETHODIMP_(ULONG32) CExampleFileFormat::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
	return m_lRefCount;
    }

    delete this;
    return 0;
}


/****************************************************************************
 *  IUnknown::QueryInterface                                    ref:  pncom.h
 *
 *  This routine indicates which interfaces this object supports. If a given
 *  interface is supported, the object's reference count is incremented, and
 *  a reference to that interface is returned. Otherwise a NULL object and
 *  error code are returned. This method is called by other objects to
 *  discover the functionality of this object.
 */
STDMETHODIMP CExampleFileFormat::QueryInterface(REFIID riid, void** ppvObj)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
	AddRef();
	*ppvObj = (IUnknown*)(IRMAPlugin*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMAPlugin))
    {
	AddRef();
	*ppvObj = (IRMAPlugin*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMAFileFormatObject))
    {
	AddRef();
	*ppvObj = (IRMAFileFormatObject*)this;
	return PNR_OK;
    }

    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}


// CExampleFileResponse Class Implementation

/****************************************************************************
 *  CMyExampleFileFormat::CMyExampleFileFormat              ref:  ex2strms.h
 *
 *  Constructor
 */
CExampleFileFormat::CExampleFileResponse::CExampleFileResponse
(
    CExampleFileFormat* pFFObject,
    UINT16              uStreamNumber
)
{
    pFFObject->AddRef();
    m_pFFObject     = pFFObject;
    m_uStreamNumber = uStreamNumber;
    m_lRefCount     = 0;
}


/****************************************************************************
 *  CMyExampleFileFormat::CMyExampleFileFormat              ref:  ex2strms.h
 *
 *  Destructor
 */
CExampleFileFormat::CExampleFileResponse::~CExampleFileResponse()
{
    m_pFFObject->Release();
}


/****************************************************************************
 *  IRMAFileResponse::InitDone                               ref:  rmafiles.h
 *
 */
STDMETHODIMP
CExampleFileFormat::CExampleFileResponse::InitDone
(
    PN_RESULT status
)
{
    return m_pFFObject->InitDone(status, m_uStreamNumber);
}


/****************************************************************************
 *  IRMAFileResponse::CloseDone                              ref:  rmafiles.h
 *
 */
STDMETHODIMP
CExampleFileFormat::CExampleFileResponse::CloseDone
(
    PN_RESULT status
)
{
    return m_pFFObject->CloseDone(status, m_uStreamNumber);
}


/****************************************************************************
 *  IRMAFileResponse::ReadDone                               ref:  rmafiles.h
 *
 */
STDMETHODIMP
CExampleFileFormat::CExampleFileResponse::ReadDone
(
    PN_RESULT           status,
    IRMABuffer*         pBuffer
)
{
    return m_pFFObject->ReadDone(status, m_uStreamNumber, pBuffer);
}


/****************************************************************************
 *  IRMAFileResponse::WriteDone                             ref:  rmafiles.h
 *
 */
STDMETHODIMP
CExampleFileFormat::CExampleFileResponse::WriteDone
(
    PN_RESULT status
)
{
    return m_pFFObject->WriteDone(status, m_uStreamNumber);
}


/****************************************************************************
 *  IRMAFileResponse::SeekDone                               ref:  rmafiles.h
 *
 */
STDMETHODIMP
CExampleFileFormat::CExampleFileResponse::SeekDone
(
    PN_RESULT status
)
{
    return m_pFFObject->SeekDone(status, m_uStreamNumber);
}


/****************************************************************************
 *  IUnknown::AddRef                                            ref:  pncom.h
 */
STDMETHODIMP_(ULONG32)
CExampleFileFormat::CExampleFileResponse::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}


/****************************************************************************
 *  IUnknown::Release                                           ref:  pncom.h
 */
STDMETHODIMP_(ULONG32)
CExampleFileFormat::CExampleFileResponse::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
	return m_lRefCount;
    }

    delete this;
    return 0;
}


/****************************************************************************
 *  IUnknown::QueryInterface                                    ref:  pncom.h
 */
STDMETHODIMP
CExampleFileFormat::CExampleFileResponse::QueryInterface
(
    REFIID riid,
    void** ppvObj
)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
	AddRef();
	*ppvObj = (IUnknown*)(IRMAFileResponse*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMAFileResponse))
    {
	AddRef();
	*ppvObj = (IRMAFileResponse*)this;
	return PNR_OK;
    }

    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}


// CExampleFSManagerResponse Class Implementation

/****************************************************************************
 *  CExampleFSManagerResponse::CExampleFSManagerResponse     ref:  ex2strms.h
 *
 *  Constructor
 */
CExampleFSManagerResponse::CExampleFSManagerResponse
(
    CExampleFileFormat* pFFObject
)
{
    pFFObject->AddRef();
    m_pFFObject     = pFFObject;
    m_uStreamNumber = 0;
    m_lRefCount     = 0;
    m_state         = UNINITIALIZED;
}


/****************************************************************************
 *  CExampleFSManagerResponse::~CExampleFSManagerResponse    ref:  ex2strms.h
 *
 *  Destructor
 */
CExampleFSManagerResponse::~CExampleFSManagerResponse()
{
    m_pFFObject->Release();
}


/****************************************************************************
 *  IRMAFileSystemManagerResponse::InitDone                  ref:  rmafiles.h
 *
 */
STDMETHODIMP
CExampleFSManagerResponse::InitDone
(
    PN_RESULT status
)
{
    return m_pFFObject->FileSystemReady(status);
}


/****************************************************************************
 *  IRMAFileSystemManagerResponse::FileObjectReady           ref:  rmafiles.h
 *
 */
STDMETHODIMP
CExampleFSManagerResponse::FileObjectReady
(
    PN_RESULT status,
    IUnknown* pObject
)
{
    return m_pFFObject->FileReady(status, m_uStreamNumber, pObject);
}


/****************************************************************************
 *  IRMAFileSystemManagerResponse::DirObjectReady            ref:  rmafiles.h
 *
 */
STDMETHODIMP
CExampleFSManagerResponse::DirObjectReady
(
    PN_RESULT      status,
    IUnknown* pDirObject
)
{
    return PNR_NOTIMPL;
}


/****************************************************************************
 *  IUnknown::AddRef                                            ref:  pncom.h
 */
STDMETHODIMP_(ULONG32)
CExampleFSManagerResponse::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}


/****************************************************************************
 *  IUnknown::Release                                           ref:  pncom.h
 */
STDMETHODIMP_(ULONG32)
CExampleFSManagerResponse::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
	return m_lRefCount;
    }

    delete this;
    return 0;
}

/****************************************************************************
 *  IUnknown::QueryInterface                                    ref:  pncom.h
 */
STDMETHODIMP
CExampleFSManagerResponse::QueryInterface
(
    REFIID riid,
    void** ppvObj
)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
	AddRef();
	*ppvObj = (IUnknown*)(IRMAFileSystemManagerResponse*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMAFileSystemManagerResponse))
    {
	AddRef();
	*ppvObj = (IRMAFileSystemManagerResponse*)this;
	return PNR_OK;
    }

    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}
