/****************************************************************************
 * 
 *  $Id: ex2strms.h,v 1.11 1998/05/21 20:29:39 smpatel Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 * 
 *  ex2strms.h
 * 
 *  Example File Format for RealMedia Architecture that opens a container
 *  file with n streams for reading (this example only does 2 streams)
 */
#ifndef _EX2FILES_H_
#define _EX2FILES_H_

/*
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +                                                                           + 
  +                       .ef2 file layout                                    + 
  +                                                                           + 
  + <Number of streams (2-bytes)><Rest of File Header (50-bytes)><Stream0     +
  + Data Offset (4-bytes)><Stream0 Data Length (4-bytes)><Rest of Stream0     +
  + Header (20-bytes)>...<StreamN Data Offset (4-bytes)><StreamN Data Length  +
  + (4-bytes)><Rest of StreamN Header (20-bytes)><Stream0 Data (20-byte       +
  + packets)>...<StreamN Data (20-byte packets)>                              +
  +                                                                           + 
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/


/****************************************************************************
 * Constants
 */
#define FILE_HEADER_OFFSET  0
#define INFO_SIZE           50
#define FILE_HEADER_SIZE    (2 + (INFO_SIZE*3))
#define STREAM_HEADER_SIZE  (8 + 20)
#define PACKET_SIZE         20
#define TIME_PER_PACKET     1000
#define MAX_STREAMS         2


/****************************************************************************
 * Typedefs
 */
typedef enum
{
    Ready, InitPending,
    GetFileHeaderSeekPending, GetFileHeaderReadPending, 
    GetStreamHeaderSeekPending, GetStreamHeaderReadPending, 
    GetPacketSeekPending, GetPacketReadPending,
    SeekSeekPending, FirstPacketPending
} ExampleState;

typedef enum
{
    FAILED = 1,
    UNINITIALIZED,
    BUSY,
    READY
} STATE;


/****************************************************************************
 * Forward declarations
 */
class CExampleFSManagerResponse;


/****************************************************************************
 *
 *  CMyExampleFileFormat Class
 *
 *  Example implementation of a basic file format.
 */
class CExampleFileFormat : public IRMAPlugin, 
                           public IRMAFileFormatObject
{
    public:
    /****** Public Class Methods ******************************************/
    CExampleFileFormat()
	    : m_lRefCount(0)
	    , m_pContext(NULL)
	    , m_pRequest(NULL)
	    , m_pFFResponse(NULL)
	    , m_pFileObject(NULL)
	    , m_pCommonClassFactory(NULL)
	    , m_bHeaderSent(FALSE)
            , m_pStreamArray(NULL)
            , m_pFSManager(NULL)
            , m_pFSManagerResponse(NULL)
	    {
	    };
    /*** Analogs for IRMAFileResponse methods ***/
    PN_RESULT InitDone (PN_RESULT status, UINT16 uStreamNumber);
    PN_RESULT CloseDone(PN_RESULT status, UINT16 uStreamNumber);
    PN_RESULT ReadDone (PN_RESULT status, UINT16 uStreamNumber, IRMABuffer* pBuffer);
    PN_RESULT WriteDone(PN_RESULT status, UINT16 uStreamNumber);
    PN_RESULT SeekDone (PN_RESULT status, UINT16 uStreamNumber);

    /*** Analogs for IRMAFileSystemManagerResponse methods ***/
    PN_RESULT FileSystemReady (PN_RESULT status);
    PN_RESULT FileReady (PN_RESULT status, UINT16 uStreamNumber, IUnknown* pIUnknown);

    /*** Helper methods ***/
    PN_RESULT(HandleFileObject)	(UINT16	uStreamNumber);


    /************************************************************************
     *  IRMAFileFormatObject Interface Methods               ref:  rmaformt.h
     */
    STDMETHOD(GetFileFormatInfo)
	(THIS_
	 REF(const char**) pFileMimeTypes,
	 REF(const char**) pFileExtensions,
	 REF(const char**) pFileOpenNames
	);

    STDMETHOD(InitFileFormat)
	(THIS_
	 IRMARequest*        pRequest,
	 IRMAFormatResponse* pFormatResponse,
	 IRMAFileObject*     pFileObject
	);

    STDMETHOD(GetFileHeader  ) (THIS);
    STDMETHOD(GetStreamHeader) (THIS_ UINT16 streamNo);
    STDMETHOD(GetPacket      ) (THIS_ UINT16 streamNo);
    STDMETHOD(Seek           ) (THIS_ UINT32 requestedTime);
    STDMETHOD(Close          ) (THIS);


    /************************************************************************
     *  IRMAPlugin Interface Methods                         ref:  rmaplugn.h
     */
    STDMETHOD(GetPluginInfo)
	(THIS_ 
	 REF(BOOL)        bLoadMultiple,
	 REF(const char*) pDescription,
	 REF(const char*) pCopyright,
	 REF(const char*) pMoreInfoURL,
	 REF(UINT32)      versionNumber
	);
    STDMETHOD(InitPlugin) (THIS_ IUnknown* pRMACore);


    /************************************************************************
    *  IUnknown COM Interface Methods                          ref:  pncom.h
    */
    STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
    STDMETHOD_(UINT32, AddRef ) (THIS);
    STDMETHOD_(UINT32, Release) (THIS);


    /************************************************************************
    *  CExampleFileResponse Class
    */
    class CExampleFileResponse : public IRMAFileResponse
    {
        public:
	/****** Public Class Methods ***************************************/
	CExampleFileResponse(CExampleFileFormat* pFFObject,
	                 UINT16 uStreamNumber);

	/********************************************************************
	*  IRMAFileResponse Interface Methods                ref:  rmafiles.h
	*/
	STDMETHOD(InitDone ) (THIS_ PN_RESULT status);
	STDMETHOD(SeekDone ) (THIS_ PN_RESULT status);
	STDMETHOD(ReadDone ) (THIS_ PN_RESULT status, IRMABuffer* pBuffer);
	STDMETHOD(WriteDone) (THIS_ PN_RESULT status);
	STDMETHOD(CloseDone) (THIS_ PN_RESULT status);

	/********************************************************************
	*  IUnknown COM Interface Methods                       ref:  pncom.h
	*/
	STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
	STDMETHOD_(UINT32, AddRef ) (THIS);
	STDMETHOD_(UINT32, Release) (THIS);

        private:
	/****** Private Class Methods **************************************/
	~CExampleFileResponse();
	
	/****** Private Class Variables ************************************/
	INT32			m_lRefCount;
	CExampleFileFormat*	m_pFFObject;
	UINT16			m_uStreamNumber;

	PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME    // Avoids GCC compiler warning
    };

    
    private:
    class StreamObject
    {

	public:
	/****** Public Class Variables ************************************/
	IRMAFileObject*		m_pFileObject;
	ExampleState		m_state;
	UINT32			m_ulDataOffset;
	UINT32			m_ulDataLength;
	UINT32			m_ulCurrentTime;
	UINT32			m_ulCurrentOffset;

	/****** Public Class Methods *************************************/
	StreamObject()
		: m_pFileObject(0)
		, m_state(Ready)
		, m_ulDataOffset(0)
		, m_ulDataLength(0)
		, m_ulCurrentTime(0)
		, m_ulCurrentOffset(0)
	{
	};
    };

    /****** Private Class Variables ****************************************/
    INT32			m_lRefCount;
    IUnknown*			m_pContext;
    StreamObject**		m_pStreamArray;
    IRMARequest*		m_pRequest;
    IRMAFormatResponse*		m_pFFResponse;
    IRMAFileObject*		m_pFileObject;
    IRMACommonClassFactory*	m_pCommonClassFactory;
    IRMAFileSystemManager*	m_pFSManager;
    CExampleFSManagerResponse*	m_pFSManagerResponse;
    BOOL			m_bHeaderSent;
    UINT32			m_ulSeekDoneCount;
    UINT32			m_ulNumStreams;

    /****** Private Static Class Variables *********************************/
    static const IID		zm_myIID;
    static const char*		zm_pDescription;
    static const char*		zm_pCopyright;
    static const char*		zm_pMoreInfoURL;
    static const char*		zm_pFileMimeTypes[];
    static const char*		zm_pFileExtensions[];
    static const char*		zm_pFileOpenNames[];
 
    /****** Private Class Methods ******************************************/
   ~CExampleFileFormat() {};

    PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME      // Avoids GCC compiler warning
};



/****************************************************************************
 *
 *  CExampleFSManagerResponse Class
 *
 */
class CExampleFSManagerResponse : public IRMAFileSystemManagerResponse
{
    friend CExampleFileFormat;


    public:
    /****** Public Class Methods ******************************************/
    CExampleFSManagerResponse(CExampleFileFormat* pFFObject);


    /************************************************************************
    *  IUnknown COM Interface Methods                          ref:  pncom.h
    */
    STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
    STDMETHOD_(UINT32, AddRef ) (THIS);
    STDMETHOD_(UINT32, Release) (THIS);


    /************************************************************************
    *  IRMAFileSystemManagerResponse Interface Methods       ref:  rmafiles.h
    */
    STDMETHOD(InitDone) (THIS_ PN_RESULT status);
    STDMETHOD(FileObjectReady)
	(THIS_
	 PN_RESULT status,
	 IUnknown* pObject
	);
    STDMETHOD(DirObjectReady)
	(THIS_
	 PN_RESULT      status,
	 IUnknown*	pDirObject
	);


    private:
    /****** Private Class Variables ****************************************/
    INT32		m_lRefCount;
    CExampleFileFormat*	m_pFFObject;
    UINT16		m_uStreamNumber;
    STATE		m_state;

    /****** Private Class Methods ******************************************/
    ~CExampleFSManagerResponse();

    PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME      // Avoids GCC compiler warning
};

#endif /* ndef _EX2FILES_H_ */

