#ifndef __EXAUDDEV_H
#define __EXAUDDEV_H



/****************************************************************************
 *  FakeAudioServices
 *
 *  This class is here simply to show how one's audio device might work
 *  with the audio services of an OS.
 */

class FakeAudioServices
{
public:
    FakeAudioServices();

    void    Init();
    void    Close();
    void    AudioOut(void* pData, int nDataLen, int nMSecs);
    ULONG32 GetAudioClock();

protected:
    ULONG32 m_ulAudioTime;
};



class ExampleAudioDevice :  public IRMAAudioDevice, 
			    public IRMACallback
{
public:
    ExampleAudioDevice(IRMAPlayer* pPlayer);


   /************************************************************************
    *  IUnknown COM Interface Methods                          ref:  pncom.h
    */
    STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
    STDMETHOD_(UINT32, AddRef ) (THIS);
    STDMETHOD_(UINT32, Release) (THIS);

    /************************************************************************
    *	    IRMAAudioDevice methods
    */
    STDMETHOD(Open)		    (THIS_
				    const RMAAudioFormat*    /*IN*/ pFormat,
				    IRMAAudioDeviceResponse* /*IN*/ pResponse);
    STDMETHOD(Close)		    (THIS_
				    const BOOL  /*IN*/ bFlush );
    STDMETHOD(Resume)		    (THIS);
    STDMETHOD(Pause)		    (THIS);
    STDMETHOD(Write)		    (THIS_
				    const RMAAudioData* /*IN*/ pAudioData);
    STDMETHOD_(BOOL,InitVolume)	    (THIS_
				    const UINT16	/*IN*/ uMinVolume,
				    const UINT16	/*IN*/ uMaxVolume);
    STDMETHOD(SetVolume)	    (THIS_
				    const UINT16    /*IN*/ uVolume);
    STDMETHOD_(UINT16,GetVolume)    (THIS);
    STDMETHOD(Reset)		    (THIS);
    STDMETHOD(Drain)		    (THIS);
    STDMETHOD(CheckFormat)	    (THIS_
				    const RMAAudioFormat* /*IN*/ pFormat);
    STDMETHOD(GetCurrentAudioTime)  (THIS_
				    REF(ULONG32) /*OUT*/ ulCurrentTime);

   /************************************************************************
    *  IRMACallback COM Interface Methods                    ref: rmaengin.h
    */

    /************************************************************************
     *	Method:
     *	    IRMACallback::Func
     *	Purpose:
     *	    This is the function that will be called when a callback is
     *	    to be executed.
     */
    STDMETHOD(Func)		(THIS);

private:
    /****** Private Class Variables ****************************************/
    INT32			m_lRefCount;
    IRMAAudioDeviceResponse*	m_pResponse;
    UINT16			m_uVolume;
    BOOL			m_bPaused;
    BOOL			m_bInitialized;
    RMAAudioFormat		m_format;
    FakeAudioServices*		m_pAudioServices;
    IRMAPlayer*			m_pPlayer;
    CallbackHandle		m_hCallback;
    FiveMinuteQueue*		m_pQueue;
    BOOL			m_bFirstWrite;
    ULONG32			m_ulGranularity;

    void			ScheduleCallback();
    void			RemoveCallback();

    /****** Private Class Methods ******************************************/
    ~ExampleAudioDevice();
    PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME      // Avoids GCC compiler warning
};


#endif
