/****************************************************************************
 * 
 *  $Id: main.cpp,v 1.3 1999/02/19 23:34:40 nhart Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 *
 *  main.cpp
 *
 *  Example client for RealMedia Architecture (RMA).
 *
 */

/****************************************************************************
 * 
 *  Example client:
 *  
 *  This is an example client running without a graphical user interface.
 *
 *  It could be used as:
 *    - Sample source to develop a customized RMA client.
 *    - A testing tool for plugins.
 *
 */ 


/****************************************************************************
 * Defines
 */
#define INITGUID     /* Interface ID's */


/****************************************************************************
 * Includes
 */
#include "pntypes.h"
#include "pnwintyp.h"
#include "pncom.h"
#include "rmapckts.h"
#include "rmacomm.h"
#include "rmamon.h"
#include "rmafiles.h"
#include "rmaengin.h"
#include "rmacore.h"
#include "rmaclsnk.h"
#include "rmaerror.h"
#include "rmaauth.h"
#include "rmaausvc.h"
#include "rmawin.h"

#include "os.h"
#include "fivemmap.h"
#include "fivemque.h"
#include "exadvsnk.h"
#include "exerror.h"
#include "exsitsup.h"
#include "exaumgr.h"
#include "excontxt.h"
#include "exauddev.h"


/****************************************************************************
 * Globals
 */
FPRMCREATEENGINE	m_fpCreateEngine;
FPRMCLOSEENGINE	 	m_fpCloseEngine;
BOOL			bShowMeTheStatistics = FALSE;


/****************************************************************************
 *  main
 *
 */
int main( int argc, char *argv[] )
{
    PN_RESULT		    theErr		= PNR_OK;
    ExampleClientContext*   exContext		= NULL;
    IRMAPlayer*		    pPlayer		= NULL;
    IRMAClientEngine*	    pEngine		= NULL;
#ifdef _UNIX
    IRMAClientEngineSelector* pSelector         = NULL;
#endif
    IRMAErrorSink*	    pErrorSink		= NULL;
    IRMAErrorSinkControl* pErrorSinkControl	= NULL;
    HINSTANCE		    hDll		= NULL;
    char*		    pszURL		= NULL;
    ExampleAudioDevice*	    pAudDev		= NULL;

#ifdef _MACINTOSH
	// Emulate command line on Macintosh
	argc = ccommand( &argv );
	SIOUXSettings.standalone     = FALSE;
	SIOUXSettings.setupmenus     = FALSE;
	SIOUXSettings.showstatusline = TRUE;
#endif

    if (argc < 2 ||
	((0 == stricmp(argv[1], "-s")) && argc < 3))
    {
	fprintf(stdout,"USAGE: %s [-s] file:yourfile.ext\n", argv[0]);
	fprintf(stdout,"       -s : optional flag to show the statistics\n");
	return -1;
    }
    
    if (0 == stricmp(argv[1], "-s"))
    {
	bShowMeTheStatistics = TRUE;
	pszURL = argv[2];
    }
    else
    {
	bShowMeTheStatistics = FALSE;
	pszURL = argv[1];
    }

    if (!(exContext = new ExampleClientContext()))
    {
	theErr = PNR_UNEXPECTED;
	goto cleanup;
    }
    exContext->AddRef();		

    // initialize the globals
    m_fpCreateEngine	= NULL;
    m_fpCloseEngine	= NULL;
    
    // prepare/load the RMACore module
    char   szDllName[_MAX_PATH];
    
#if defined(_UNIX)
    strcpy(szDllName, "rmacore.dll");

#elif defined(_MACINTOSH)
    // ASSUME rmacore is in System Folder:Extensions:Real:Common
    // SHOULD obtain value from preference file
    OSErr	macErr;

    // set szDllName to empty string
    szDllName[0] = '\0'; 

    // find the Extensions folder
    short	vRefNum;
    long	dirID;
    macErr = FindFolder (kOnSystemDisk, kApplicationSupportFolderType, kDontCreateFolder, &vRefNum, &dirID);

    // find the path of the Extensions folder
    DirInfo	block;
    Str63	directoryName;
    char	tmpName[_MAX_PATH];

    block.ioDrParID = dirID;
    block.ioNamePtr = directoryName;
    do
    {
	block.ioVRefNum = vRefNum;
	block.ioFDirIndex = -1;
	block.ioDrDirID = block.ioDrParID;

	macErr = PBGetCatInfoSync ((CInfoPBPtr)&block);
	if (macErr == noErr)
	{
	    strcpy (tmpName, szDllName);
	    szDllName[0] = '\0';
	    strncat (szDllName, (char*)&directoryName[1], directoryName[0]);
	    if (strlen(tmpName))
	    {
		strcat (szDllName, ":");
		strcat (szDllName, tmpName);
	    }
	}
    }
    while ((macErr == noErr) && (block.ioDrDirID != 2)); // 2 means root directory of a volume
    strcat (szDllName, ":Real:Common:rmacore60.dll");
//    strcpy(szDllName, "rmacore60.dll");

#elif defined(_WIN32) 
    // get location of rmacore from windows registry 
    DWORD bufSize;
    HKEY hKey; 
    PN_RESULT hRes;
    
    szDllName[0] = '\0'; 
    bufSize = sizeof(szDllName) - 1;

    if(ERROR_SUCCESS == RegOpenKey(HKEY_CLASSES_ROOT,
	"Software\\RealNetworks\\Preferences\\DT_Common", &hKey)) 
    { 
	// get the path to pnen 
	hRes = RegQueryValue(hKey, "", szDllName, (long *)&bufSize); 
	RegCloseKey(hKey); 
    }

    strcat(szDllName, "pnen3260.dll");

#elif defined(_WIN16) 
    strcpy(szDllName, "pnen1660.dll");
#endif
    
    if (!(hDll = LoadLibrary(szDllName))) 
    {
	theErr = PNR_FAILED;
#if defined(_UNIX)
 	fprintf(stdout, "Failed to load the 'rmacore.so.6.0' shared library\n");
	fprintf(stdout, dlerror());
#else
 	fprintf(stdout, "Failed to load the '%s' library.\n", szDllName);
#endif
	goto cleanup;
    }

    fprintf(stdout, "Loaded rmacore\n");
    

    // retrieve the proc addresses from the module
    m_fpCreateEngine = (FPRMCREATEENGINE) GetProcAddress(hDll, "CreateEngine");
    m_fpCloseEngine  = (FPRMCLOSEENGINE)  GetProcAddress(hDll, "CloseEngine");
 
    if (m_fpCreateEngine == NULL ||
 	m_fpCloseEngine == NULL
       )
    {
	theErr = PNR_FAILED;
	goto cleanup;
    }

    // create client engine 
    if (PNR_OK != m_fpCreateEngine((IRMAClientEngine**) &pEngine))
    {
	theErr = PNR_FAILED;
	goto cleanup;
    }

    // create player
    if (PNR_OK != pEngine->CreatePlayer(pPlayer))
    {
	theErr = PNR_FAILED;
	goto cleanup;
    }

    // replace the audio device before we initialize the player
    pAudDev = new ExampleAudioDevice(pPlayer);
    if (pAudDev)
    {
	pAudDev->AddRef();

	IRMAAudioDeviceManager* pAudDevMgr = NULL;
	if (pEngine->QueryInterface(IID_IRMAAudioDeviceManager,
				    (void**)&pAudDevMgr) == PNR_OK)
	{
	    if (pAudDevMgr->Replace(pAudDev) != PNR_OK)
	    {
		// an error replacing the audio device occured
		pAudDev->Release();
		pAudDev = NULL;
	    }

	    pAudDevMgr->Release();
	}
    }

    // initialize the example context
    exContext->Init(pPlayer);

    pPlayer->SetClientContext(exContext);

    pPlayer->QueryInterface(  IID_IRMAErrorSinkControl, 
					(void**) &pErrorSinkControl);
    if ( pErrorSinkControl )
    {	
	exContext->QueryInterface(  IID_IRMAErrorSink, 
					(void**) &pErrorSink);
		
	if (pErrorSink)
	{
	    pErrorSinkControl->AddErrorSink(pErrorSink, PNLOG_EMERG, PNLOG_INFO);
	}
    }	

    // There is only one URL for this presentation
    if (PNR_OK == pPlayer->OpenURL(pszURL))
    {
	pPlayer->Begin();

#if defined( _WIN32 ) || defined( _WINDOWS )
	MSG msg;
	while (GetMessage(&msg, NULL, 0, 0) && !pPlayer->IsDone())
	{
	    DispatchMessage(&msg);
	}

#elif defined (_UNIX)
        struct _PNxEvent* pNothing = 0;
        int     ret = 0;
        struct  timeval  mtime;


	pEngine->QueryInterface(IID_IRMAClientEngineSelector, 
				(void**) &pSelector);

	while (!pPlayer->IsDone())
	{
            mtime.tv_sec  = 0; 
            mtime.tv_usec = 10000;  // 20 ms is minimum
	    
            ret = pSelector->Select(0, 
				    (fd_set *) 0, 
				    (fd_set *) 0,
				    (fd_set *) 0,
				    &mtime);
				    
	    pEngine->EventOccurred(pNothing);
	}

#elif defined (_MACINTOSH)
	EventRecord event;
	char        theChar;

	while (!pPlayer->IsDone())
	{
	    WaitNextEvent(everyEvent, &event, 30, nil);
	    switch (event.what)
	    {
		case keyDown:
		case autoKey:
		    // Determine if command-period pressed
		    if( (event.modifiers & cmdKey) != 0 )
		    {
			theChar = (event.message & charCodeMask);
			if (theChar == kPeriodKey)
			{
			   goto cleanup;
			}
		    }
		    break;
	    }
	    
	    // Send all events to the player engine
	    PNxEvent pPNxEvent;   
	    pPNxEvent.event  = ((EventRecord*)&event)->what;
	    pPNxEvent.param1 = &event;
	    pEngine->EventOccurred(&pPNxEvent);
	}

#endif
	pPlayer->Stop();
    }

cleanup:

#ifdef _UNIX
    if (pSelector)
    {
        pSelector->Release();
    }
#endif

    if (pErrorSinkControl)
    {
	pErrorSinkControl->RemoveErrorSink(pErrorSink);
	pErrorSinkControl->Release();
	pErrorSinkControl = NULL;
    }

    if (exContext)
    {
	exContext->Release();
	exContext = NULL;
    }
    
    if (pPlayer)
    {
	pEngine->ClosePlayer(pPlayer);
	pPlayer->Release();
	pPlayer = NULL;
    }

    if (pAudDev)
    {
	IRMAAudioDeviceManager* pAudDevMgr = NULL;
	if (pEngine->QueryInterface(IID_IRMAAudioDeviceManager,
				    (void**)&pAudDevMgr) == PNR_OK)
	{
	    pAudDevMgr->Remove(pAudDev);
	    pAudDevMgr->Release();
	}

	pAudDev->Release();
	pAudDev = NULL;
    }

    if (pEngine)
    {
	m_fpCloseEngine(pEngine);
	pEngine = NULL;
    }

    if (hDll)
    {
	FreeLibrary(hDll);
	hDll = NULL;
    }

    return theErr;
}
