/****************************************************************************
 * 
 *  $Id: exaudio.cpp,v 1.34 1999/01/08 19:07:11 juliah Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 *
 *  exaudio.cpp
 *
 *  Example Renderer plug-in which shows the use of audio services.
 *
 */


/****************************************************************************
 * Defines
 */
#define INITGUID     /* Interface ID's */
#define STRICT
#define	LENGTH   4096*5


/****************************************************************************
 * Includes
 */
#include "pntypes.h"

#include <stdio.h>

#ifdef _WINDOWS
    #include <windows.h>
#endif

#ifdef _UNIX
    #include <stdio.h>
    #include <stdlib.h>
    #include <unistd.h>
    #include <X11/Intrinsic.h>
    #include <X11/StringDefs.h>
    #include <X11/Shell.h>
    #include <X11/Xaw/Label.h>
#endif

#include "pnwintyp.h"
#include "pncom.h"
#include "rmacomm.h"
#include "rmapckts.h"
#include "rmaausvc.h"
#include "rmacore.h"
#include "rmarendr.h"
#include "rmaplugn.h"
#include "rmawin.h"

#include "exaudio.h"
#include "exaudio.ver"


/****************************************************************************
 * Globals
 */
 
// Replace this with the fully qualified URL
const char*    g_pURL =	"frog.pcm";

#ifdef _UNIX
    Widget     g_Label1;
    Widget     g_Label2;
#endif


/****************************************************************************
 *  RMACreateInstance                                        ref:  rmaplugn.h
 *
 *  This routine creates a new instance of the CExampleRenderer class.
 *  It is called when the RMA core application is launched, and whenever
 *  a stream associated with this plug-in needs to be rendered.
 */
STDAPI
RMACreateInstance(IUnknown** ppIUnknown)
{
    *ppIUnknown = (IUnknown*)(IRMAPlugin*)new CExampleRenderer();
    if (*ppIUnknown)
    {
	(*ppIUnknown)->AddRef();
	return PNR_OK;
    }
    return PNR_OUTOFMEMORY;
}


/****************************************************************************
 *  CExampleRenderer static variables                         ref:  exaudio.h
 *
 *  These variables are passed to the RMA core to provide information about
 *  this plug-in. They are required to be static in order to remain valid
 *  for the lifetime of the plug-in.
 */
const char* CExampleRenderer::zm_pDescription    = "Example Renderer Plugin";
const char* CExampleRenderer::zm_pCopyright      = "(c) 1995,1996,1997 RealNetworks, All rights reserved.";
const char* CExampleRenderer::zm_pMoreInfoURL    = "http://www.real.com";
const char* CExampleRenderer::zm_pStreamMimeTypes[] = {"application/x-pn-examplestream", NULL};


/****************************************************************************
 *  IRMAPlugin::GetPluginInfo                                ref:  rmaplugn.h
 *
 *  This routine returns descriptive information about the plug-in, most
 *  of which is used in the About box of the user interface. It is called
 *  when the RMA core application is launched.
 */
STDMETHODIMP
CExampleRenderer::GetPluginInfo
(
    REF(BOOL)        /*OUT*/ bLoadMultiple,
    REF(const char*) /*OUT*/ pDescription,
    REF(const char*) /*OUT*/ pCopyright,
    REF(const char*) /*OUT*/ pMoreInfoURL,
    REF(UINT32)      /*OUT*/ ulVersionNumber
)
{
    bLoadMultiple = TRUE;   // Must be true for file formats.

    pDescription    = zm_pDescription;
    pCopyright	    = zm_pCopyright;
    pMoreInfoURL    = zm_pMoreInfoURL;
    ulVersionNumber = TARVER_ULONG32_VERSION;

    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::GetRendererInfo                            ref:  rmarendr.h
 *
 *  This routine returns crucial information required to associate this
 *  plug-in with a given stream MIME type. This information tells the RMA
 *  core which Renderer to use to display a particular type of stream. This
 *  method is called when the RMA core application is launched.
 */
STDMETHODIMP
CExampleRenderer::GetRendererInfo
(
    REF(const char**) /*OUT*/ pStreamMimeTypes,
    REF(UINT32)      /*OUT*/ unInitialGranularity
)
{
    pStreamMimeTypes = zm_pStreamMimeTypes;
    unInitialGranularity = 100; // OnTimeSync() every 100ms

    return PNR_OK;
}


/****************************************************************************
 *  IRMAPlugin::InitPlugin                                   ref:  rmaplugn.h
 *
 *  This routine performs initialization steps such as determining if
 *  required interfaces are available. It is called when the RMA core 
 *  application is launched, and whenever a stream associated with this
 *  plug-in needs to be rendered.
 */
STDMETHODIMP
CExampleRenderer::InitPlugin(IUnknown* /*IN*/ pContext)
{
    PN_RESULT hr = PNR_OK;
    UINT32 actual = 0;

    m_pContext = pContext;
    m_pContext->AddRef();

    if((m_pFile = ::fopen(g_pURL, "rb")) == NULL)
    {
	hr = PNR_UNEXPECTED;
	goto cleanup;
    }
    
    if (m_pContext->QueryInterface(IID_IRMACommonClassFactory,(void**)&m_pCommonClassFactory))
    {
	hr = PNR_UNEXPECTED;
	goto cleanup;
    }

    if (PNR_OK != m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,(void**)&m_pAudioBuffer))
    {
	hr = PNR_UNEXPECTED;
	goto cleanup;
    }

    m_pAudioBuffer->SetSize(LENGTH);
    actual = ::fread(m_pAudioBuffer->GetBuffer(), sizeof(UCHAR), LENGTH, m_pFile);
    
    // Tell the buffer about it's real meaningful size
    m_pAudioBuffer->SetSize(actual);
   
cleanup:

    if (m_pFile)
    {
	::fclose(m_pFile);
	m_pFile = NULL;
    }
    return hr;
}


/****************************************************************************
 *  IRMARenderer::StartStream                                ref:  rmarendr.h
 *
 *  The RMA core calls this routine to provide access to the stream and
 *  player. It is called when the plug-in is being initialized.
 */
STDMETHODIMP
CExampleRenderer::StartStream
(
    IRMAStream*	    pStream,
    IRMAPlayer*	    pPlayer
)
{
    m_pStream  = pStream;
    m_pPlayer  = pPlayer;

    if (m_pStream ) m_pStream->AddRef();
    if (m_pPlayer ) m_pPlayer->AddRef();

    if (PNR_OK != m_pCommonClassFactory->CreateInstance(
			CLSID_IRMAMultiInstanceSiteUserSupplier,
	    		(void**)&m_pMISUS))
    {
	return PNR_UNEXPECTED;
    }
    
    m_pMISUS->SetSingleSiteUser((IUnknown*)(IRMASiteUser*)this);

    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnHeader                                   ref:  rmarendr.h
 *
 *  This routine is passed the stream header object created by the associated
 *  file format plug-in. Use the methods defined in the IRMAValues interface
 *  to access the stream header data. This method is called from the RMA
 *  core when the plug-in is being initialized.
 */
STDMETHODIMP
CExampleRenderer::OnHeader(IRMAValues* pHeader)
{
    PN_RESULT hr = PNR_OK;

    // Keep this for later use...
    m_pHeader = pHeader;
    m_pHeader->AddRef();

    // Set up the audio stream...
    if ( PNR_OK != m_pPlayer->QueryInterface(IID_IRMAAudioPlayer, (void**) &m_pAudioPlayer ))
    {
        hr = PNR_UNEXPECTED;
	goto cleanup;
    }

    m_pAudioPlayer->CreateAudioStream(&m_pAudioStream);
    RMAAudioFormat AudioFmt;
    AudioFmt.uChannels 		= 1;
    AudioFmt.uBitsPerSample 	= 16;
    AudioFmt.ulSamplesPerSec 	= 22050;
    AudioFmt.uMaxBlockSize  	= (UINT16)LENGTH;
    m_pAudioStream->Init( &AudioFmt, pHeader);

cleanup:
    return hr;
}


/****************************************************************************
 *  IRMARenderer::OnBegin                                   ref:  rmarendr.h
 *
 *  This routine is called by the RMA core to inform the Renderer that 
 *  playback has just begun or has been resumed. The stream's time value just
 *  after resuming the playback is provided.
 */
STDMETHODIMP
CExampleRenderer::OnBegin(UINT32 ulTime)
{
    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::GetDisplayType                             ref:  rmarendr.h
 *
 *  This routine returns the preferred display type of the Renderer. Any
 *  other additional information necessary for the display can also be
 *  provided. This method is called by the RMA core when the plug-in is being
 *  initialized.
 */
STDMETHODIMP
CExampleRenderer::GetDisplayType
(
    REF(RMA_DISPLAY_TYPE)   ulFlags,
    REF(IRMABuffer*)	    pBuffer
)
{
    ulFlags = RMA_DISPLAY_WINDOW;

    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnPacket                                   ref:  rmarendr.h
 *
 *  This routine is passed the packet object streamed by the associated
 *  file format plug-in. It is called by the RMA core when a packet is due
 *  to be delivered. In most cases, the actual rendering of the packet is
 *  done in this method. The stream's time offset with respect to the master
 *  timeline is provided.
 */
STDMETHODIMP
CExampleRenderer::OnPacket(IRMAPacket* pPacket, INT32 lTimeOffset)
{
    /* Ignore pre-seek packets. Some renderers may want to use them though */
    if (m_bInSeekMode)
    {
	return PNR_OK;
    }

#ifdef _UNIX
    if(rand()%2 > 0)
    {
	HandleClick();
    }
#endif

    // Release the last packet if we had one...
    if (m_pLastPacket)
    {
        m_pLastPacket->Release();
    }

    // Keep this one for later use...
    m_pLastPacket = pPacket;

    if (m_pLastPacket)
    {
	m_pLastPacket->AddRef();
    }

    // NOTE: If your stream has been offset in time due to start, end, or
    // groupstart properties on the stream, you may be getting packets which
    // need to be adjusted before writing into the audio services, or before
    // comparing against the time line. Use the following code to determine
    // to offset of this packet's delivery time in the real timeline.
    UINT32 ulPacketDeliveryTime = pPacket->GetTime(); 
    UINT32 ulRealDeliveryTime;
 
    if ((UINT32)lTimeOffset > ulPacketDeliveryTime)
    {
        ulRealDeliveryTime = 0;
    }
    else
    {
	ulRealDeliveryTime = ulPacketDeliveryTime - lTimeOffset;
    }

    // For example purposes we remember this information
    m_ulLastPacketTime = ulPacketDeliveryTime;
    m_ulLastRealPacketTime = ulRealDeliveryTime;

    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnTimeSync                                 ref:  rmarendr.h
 *
 *  This routine is called by the RMA core periodically passing the current
 *  playback time. The Renderer should use this time value to synchronize the
 *  playback of various streams.
 */
STDMETHODIMP
CExampleRenderer::OnTimeSync(UINT32 ulTime)
{
    // Here's a good time to actually render the data!

    // Remember the time we are!
    m_ulLastTime = ulTime;

    // Redraw our data by damaging the entire area of our data
    PNxRect damageRect = {0, 0, m_size.cx, m_size.cy };
    m_pMISUSSite->DamageRect(damageRect);
    m_pMISUSSite->ForceRedraw();
	
    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnPreSeek                                  ref:  rmarendr.h
 *
 *  This routine is called by the RMA core before a seek is about to occur.
 *  The stream's time value just before the seek, and the time value just
 *  after the seek are provided.
 */
STDMETHODIMP
CExampleRenderer::OnPreSeek(UINT32 ulOldTime, UINT32 ulNewTime)
{
    m_bInSeekMode   = TRUE;
    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnPostSeek                                 ref:  rmarendr.h
 *
 *  This routine is called by the RMA core just after a seek has occured.
 *  The stream's time value just before the seek, and the time value just
 *  after the seek are provided.
 */
STDMETHODIMP
CExampleRenderer::OnPostSeek(UINT32 ulOldTime, UINT32 ulNewTime)
{
    m_bInSeekMode   = FALSE;
    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnPause                                    ref:  rmarendr.h
 *
 *  This routine is called by the RMA core just after a pause has occured.
 *  The stream's time value just before pausing is provided.
 */
STDMETHODIMP
CExampleRenderer::OnPause(UINT32 ulTime)
{
    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnBuffering                                ref:  rmarendr.h
 *
 *  This routine is called by the RMA core to inform the Renderer that
 *  buffering of data is occuring. The reason for buffering (e.g. start-up
 *  of stream, seek has occured, network congestion, etc.), as well as the
 *  percentage of the buffering process complete are provided.
 */
STDMETHODIMP
CExampleRenderer::OnBuffering(UINT32 ulFlags, UINT16 unPercentComplete)
{
    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnEndofPackets                             ref:  rmarendr.h
 *
 *  Called by client engine to inform the renderer that all the
 *  packets have been delivered. However, if the user seeks before
 *  EndStream() is called, renderer may start getting packets again
 *  and the client engine will eventually call this function again.
 */
STDMETHODIMP
CExampleRenderer::OnEndofPackets(void)
{
    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::EndStream                                  ref:  rmarendr.h
 *
 *  This routine is called by the RMA core when the rendering of the stream
 *  has completed. Deallocation of any resources should be done here.
 */
STDMETHODIMP
CExampleRenderer::EndStream()
{
    // You can't use IRMAStream, IRMASourceStream, IRMABackChannel, or
    // IRMAASMStream after EndStream has been called.
    PN_RELEASE(m_pStream);

    return PNR_OK;
}


/****************************************************************************
 *  CExampleRenderer::~CExampleRenderer                      ref:  exaudio.h
 *
 *  Destructor
 */
CExampleRenderer::~CExampleRenderer()
{
    // NOTE: You should do your renderer cleanup here, instead of
    // in EndStream(), because your renderer is still left around
    // after the stream is ended in case it is a display renderer
    // and it needs to "paint" it's display area.

    // We're done with these...

    PN_RELEASE(m_pHeader);
    PN_RELEASE(m_pLastPacket);
    PN_RELEASE(m_pContext);
    PN_RELEASE(m_pPlayer);
    PN_RELEASE(m_pCommonClassFactory);
    PN_RELEASE(m_pAudioBuffer);
    PN_RELEASE(m_pAudioPlayer);
    PN_RELEASE(m_pAudioStream);
}


/****************************************************************************
 *  CExampleRenderer::Draw                                    ref:  exaudio.h
 *
 */
#define MAX_PACKET_SIZE	    400 
void CExampleRenderer::Draw()
{
    // Add draw code here!
    char	szText1[128];
    char	szText2[128];
    char	szText3[128];
    char	szText4[MAX_PACKET_SIZE + 50];
    char	szPacketFormat[40];
    int		nLength1,nLength2,nLength3,nLength4;
    int		time,hours,min,secs,tenths;
    IRMABuffer*	pBuffer = NULL;

    time    = m_ulLastTime;
    tenths  = time % 1000;
    time   /= 1000;
    secs    = time % 60;
    time   /= 60;
    min	    = time % 60;
    hours   = time / 60;

    nLength1 = sprintf(szText1,"Last Time Sync: %d:%02d:%02d.%03d",
		hours,min,secs,tenths);

    time    = m_ulLastPacketTime;
    tenths  = time % 1000;
    time   /= 1000;
    secs    = time % 60;
    time   /= 60;
    min	    = time % 60;
    hours   = time / 60;

    nLength2 = sprintf(szText2,"Last Packet Time: %d:%02d:%02d.%03d",
		hours,min,secs,tenths);

    time    = m_ulLastRealPacketTime;
    tenths  = time % 1000;
    time   /= 1000;
    secs    = time % 60;
    time   /= 60;
    min	    = time % 60;
    hours   = time / 60;

    nLength3 = sprintf(szText3,"Last Offset Packet Time: %d:%02d:%02d.%03d",
		hours,min,secs,tenths);

    nLength4 = sprintf(szText4,"Last Packet was NULL");

    if (m_pLastPacket)
    {
        pBuffer = m_pLastPacket->GetBuffer();
	if (pBuffer)
	{
	    sprintf(szPacketFormat,"Last Packet Contained: %%.%is",
		(pBuffer->GetSize() < MAX_PACKET_SIZE ? 
		    pBuffer->GetSize() : MAX_PACKET_SIZE));
	    nLength4 = sprintf(szText4,szPacketFormat,pBuffer->GetBuffer());
	    pBuffer->Release();
	}
    }

#if _WINDOWS
    ::Rectangle(m_hDC, 0, 0, m_size.cx+1, m_size.cy+1);
    ::TextOut(m_hDC,10, 10,szText1,nLength1);
    ::TextOut(m_hDC,10,30,szText2,nLength2);
    ::TextOut(m_hDC,10,50,szText3,nLength3);
    ::TextOut(m_hDC,10,70,szText4,nLength4);
#endif

#if _UNIX
    printf("exaudio: %s\n", szText1);
    printf("exaudio: %s\n", szText2);
    printf("exaudio: %s\n", szText3);
    printf("exaudio: %s\n", szText4);
    printf("\n");
    XtUnmanageChild(g_Label1);
    XtUnmanageChild(g_Label2);
    XtVaSetValues( g_Label1, XtNlabel, szText1, NULL );
    XtVaSetValues( g_Label2, XtNlabel, szText2, NULL );
    XtManageChild(g_Label1);
    XtManageChild(g_Label2);
#endif

#ifdef _MACINTOSH
    PNxRect bounds = {0, 0, m_size.cx, m_size.cy };
    
    Rect rErase;
    ::SetRect(&rErase, bounds.left, bounds.top, bounds.right, bounds.bottom);
    ::EraseRect(&rErase);
    
    ::TextSize(9);
    ::MoveTo(rErase.left + 20, rErase.top + 20);
    ::DrawText(szText1, 0, nLength1);
    ::MoveTo(rErase.left + 20, rErase.top + 50);
    ::DrawText(szText2, 0, nLength2);
#endif

}


/****************************************************************************
 *  CExampleRenderer::HandleClick                             ref:  exaudio.h
 *
 */
void CExampleRenderer::HandleClick()
{
    RMAAudioData AudioData;
   
    if (m_pAudioBuffer && m_pAudioStream)
    {
	AudioData.pData		    = m_pAudioBuffer;
	AudioData.uAudioStreamType  = INSTANTANEOUS_AUDIO;
	/* Audio time is of no significance in instantaneous playback */
	AudioData.ulAudioTime	    = 0;

	m_pAudioStream->Write(&AudioData);
    }
}


// IRMASiteUser Interface Methods

/****************************************************************************
 *  IRMASiteUser::AttachSite                                   ref:  rmawin.h
 *
 */
STDMETHODIMP
CExampleRenderer::AttachSite(IRMASite* /*IN*/ pSite)
{
    if (m_pMISUSSite) return PNR_UNEXPECTED;
    m_pMISUSSite = pSite;
    m_pMISUSSite->AddRef();

    /*
     * This is the best time to set the size of the site, we
     * know for example the the header has already been received
     * at this point.
     *
     * In most display renderers, you will store size information
     * in your stream header. In this example, we assume a specific
     * size, but we will initialize that size here as if it had
     * come from our header.
     *
     */
    m_size.cx = 400;
    m_size.cy = 100;

    m_pMISUSSite->SetSize(m_size);
    
    return PNR_OK;
}


/****************************************************************************
 *  IRMASiteUser::DetachSite                                   ref:  rmawin.h
 *
 */
STDMETHODIMP
CExampleRenderer::DetachSite()
{
    PN_RELEASE(m_pMISUSSite);

    // We're done with these...
    if (m_pMISUS) m_pMISUS->ReleaseSingleSiteUser();
    PN_RELEASE(m_pMISUS);

    return PNR_OK;
}


/****************************************************************************
 *  IRMASiteUser::NeedsWindowedSites                           ref:  rmawin.h
 *
 */
STDMETHODIMP_(BOOL)
CExampleRenderer::NeedsWindowedSites() 
{ 
    return FALSE; 
};


/****************************************************************************
 *  IRMASiteUser::HandleEvent                                  ref:  rmawin.h
 *
 */
STDMETHODIMP
CExampleRenderer::HandleEvent(PNxEvent* /*IN*/ pEvent)
{
    pEvent->handled = FALSE;
    pEvent->result  = 0;

    switch (pEvent->event)
    {
#ifdef _WINDOWS
	case WM_PAINT:
	{
	    HWND hWnd = (HWND)pEvent->window;
	    PAINTSTRUCT ps;
	    m_hDC = ::BeginPaint(hWnd,&ps);
	    Draw();
	    ::EndPaint(hWnd,&ps);
	    pEvent->handled = TRUE;
	}
	break;
	
	case WM_LBUTTONDOWN:
	{
	    HandleClick();
	}
	break;
#endif
#ifdef _MACINTOSH

	case updateEvt:
	case activateEvt:
	{
	    Draw();
	}
	break;

	case mouseDown:
	{
	    HandleClick();
	}
	break;
#endif
    }
    return PNR_OK;
}


// IUnknown COM Interface Methods

/****************************************************************************
 *  IUnknown::AddRef                                            ref:  pncom.h
 *
 *  This routine increases the object reference count in a thread safe
 *  manner. The reference count is used to manage the lifetime of an object.
 *  This method must be explicitly called by the user whenever a new
 *  reference to an object is used.
 */
STDMETHODIMP_(UINT32) CExampleRenderer::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}

/****************************************************************************
 *  IUnknown::Release                                           ref:  pncom.h
 *
 *  This routine decreases the object reference count in a thread safe
 *  manner, and deletes the object if no more references to it exist. It must
 *  be called explicitly by the user whenever an object is no longer needed.
 */
STDMETHODIMP_(UINT32) CExampleRenderer::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
        return m_lRefCount;
    }

    delete this;
    return 0;
}

/****************************************************************************
 *  IUnknown::QueryInterface                                    ref:  pncom.h
 *
 *  This routine indicates which interfaces this object supports. If a given
 *  interface is supported, the object's reference count is incremented, and
 *  a reference to that interface is returned. Otherwise a NULL object and
 *  error code are returned. This method is called by other objects to
 *  discover the functionality of this object.
 */
STDMETHODIMP CExampleRenderer::QueryInterface(REFIID riid, void** ppvObj)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
	AddRef();
	*ppvObj = (IUnknown*)(IRMAPlugin*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMAPlugin))
    {
	AddRef();
	*ppvObj = (IRMAPlugin*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMARenderer))
    {
	AddRef();
	*ppvObj = (IRMARenderer*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMASiteUser))
    {
        AddRef();
        *ppvObj = (IRMASiteUser*)this;
        return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMASiteUserSupplier))
    {
	if (m_pMISUS)
	{
	    return m_pMISUS->QueryInterface(IID_IRMASiteUserSupplier,ppvObj);
	}
	else
	{
	    *ppvObj = NULL;
	    return PNR_UNEXPECTED;
	}
    }

    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}
