/****************************************************************************
 * 
 *  $Id: exauth.cpp,v 1.5 1999/01/16 02:17:33 juliah Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 * 
 *  exauth.cpp
 *
 *  Example Authentication plug-in for the RealMedia Architecture (RMA).
 */

/****************************************************************************
 *
 *  EXAMPLE AUTHENTICATOR
 *
 *  This implements a very simple authentication protocol as described in the 
 *  HTTP/1.1 standard (rfc2068)
 */

/****************************************************************************
 * Defines
 */
#define   INITGUID     /* Interface ID's */

#ifdef _DEBUG
#undef PN_THIS_FILE		
static char PN_THIS_FILE[] = __FILE__;
#endif

/****************************************************************************
 * Includes
 */
#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "pncom.h"
#include "pntypes.h"
#include "rmaplugn.h"
#include "rmaprefs.h"
#include "rmafiles.h"
#include "rmapckts.h"
#include "rmaauthn.h"
#include "rmaplgns.h"
#include "rmacomm.h"
#include "rmadb.h"

#include "exauth.h"
#include "exauth.ver"

const char* CBasicAuthenticator::zm_pDescription = 
(
    "Example Basic Authenticator Plugin"
);
const char* CBasicAuthenticator::zm_pCopyright      = "(c) 1995,1996,1997 RealNetworks, All rights reserved.";
const char* CBasicAuthenticator::zm_pMoreInfoURL    = "http://www.real.com";

INT32 BinFrom64(const char* pInBuf, INT32 len, BYTE* pOutBuf);
INT32 BinTo64(const BYTE* pInBuf, INT32 len, char* pOutBuf);

#ifndef strncasecmp
#define strncasecmp strnicmp
#endif

#ifndef strcasecmp
#define strcasecmp stricmp
#endif

#ifdef _MACINTOSH
#include "os.h"
#endif

/****************************************************************************
 *
 *  Function:
 *
 *	RMACreateInstance()
 *
 *  Purpose:
 *
 *	Function implemented by all plugin DLL's to create an instance of
 *	any of the objects supported by the DLL. This method is similar to
 *	Window's CoCreateInstance() in its purpose, except that it only
 *	creates objects from this plugin DLL.
 *
 *	NOTE: Aggregation is never used. Therefore an outer unknown is
 *	not passed to this function, and you do not need to code for this
 *	situation.
 *
 */
STDAPI RMACreateInstance
(
    IUnknown**  /*OUT*/	ppUnknown
)
{
    PN_RESULT ResultStatus = PNR_OUTOFMEMORY;

    CBasicAuthenticator* pBasicAuth = new CBasicAuthenticator();
    if (pBasicAuth)
    {
	ResultStatus = pBasicAuth->QueryInterface
	(
	    IID_IUnknown, 
	    (void**)ppUnknown
	);
    }

    return ResultStatus;
}

/****************************************************************************
 *
 *  Function:
 *
 *	RMAShutdown()
 *
 *  Purpose:
 *
 *	Function implemented by all plugin DLL's to free any *global*
 *	resources. This method is called just before the DLL is unloaded.
 *
 */
STDAPI RMAShutdown(void)
{
    return PNR_OK;
}

CBasicAuthenticator::CBasicAuthenticator()
    : m_pRequestCurrent(NULL)
    , m_pClientAuthResponseRequester(NULL)
    , m_bFinished(FALSE)
    , m_pAuthenticationDBManagerResponseRequester(NULL)
    , m_pAuthenticationDBManagerPlugin(NULL)
    , m_pAuthenticationDBAccessCredentials(NULL)
    , m_pServerAuthResponseRequester(NULL)
    , m_pRequestContextCurrent(NULL)
    , m_pRequestResponse(NULL)
    , m_pBufferDatabaseID(NULL)
    , m_pBufferPrincipalID(NULL)
    , m_pBufferPassword(NULL)
    , m_pBufferRealm(NULL)
    , m_bAuthenticated(FALSE)
    , m_pValuesConfig(NULL)
    , m_pUnknownContext(NULL)
    , m_pCommonClassFactory(NULL)
    , m_lRefCount(0)
{
}

CBasicAuthenticator::~CBasicAuthenticator()
{
    PN_RELEASE(m_pRequestCurrent);
    PN_RELEASE(m_pClientAuthResponseRequester);
    PN_RELEASE(m_pAuthenticationDBManagerResponseRequester);
    PN_RELEASE(m_pAuthenticationDBManagerPlugin);
    PN_RELEASE(m_pAuthenticationDBAccessCredentials);
    PN_RELEASE(m_pServerAuthResponseRequester);
    PN_RELEASE(m_pRequestContextCurrent);
    PN_RELEASE(m_pRequestResponse);
    PN_RELEASE(m_pBufferDatabaseID);
    PN_RELEASE(m_pBufferPrincipalID);
    PN_RELEASE(m_pBufferPassword);
    PN_RELEASE(m_pBufferRealm);
    PN_RELEASE(m_pValuesConfig);
    PN_RELEASE(m_pUnknownContext);
    PN_RELEASE(m_pCommonClassFactory);
}

// IUnknown methods 
UINT32 
CBasicAuthenticator::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}

UINT32 
CBasicAuthenticator::Release()
{
    if (InterlockedDecrement(&m_lRefCount) == 0)
    {
	delete this;
    }
    return m_lRefCount;
}

STDMETHODIMP 
CBasicAuthenticator::QueryInterface
(
    REFIID IIDRequested,
    void** ppUnknownReturned
)
{
    PN_RESULT ResultStatus = PNR_OK;

    if (IsEqualGUID(IID_IUnknown, IIDRequested))
    {
	*ppUnknownReturned = (IUnknown*)(IRMAPlugin*)this;
    }
    else if (IsEqualGUID(IID_IRMAPlugin, IIDRequested))
    {
	*ppUnknownReturned = (IRMAPlugin*)this;
    }
    else if (IsEqualGUID(IID_IRMAPluginProperties, IIDRequested))
    {
	*ppUnknownReturned = (IRMAPluginProperties*)this;
    }
    else if (IsEqualGUID(IID_IRMAObjectConfiguration, IIDRequested))
    {
	*ppUnknownReturned = (IRMAObjectConfiguration*)this;
    }
    else if (IsEqualGUID(IID_IRMAClientAuthConversation, IIDRequested))
    {
	*ppUnknownReturned = (IRMAClientAuthConversation*)this;
    }
    else if (IsEqualGUID(IID_IRMAServerAuthConversation, IIDRequested))
    {
	*ppUnknownReturned = (IRMAServerAuthConversation*)this;
    }
    else if (IsEqualGUID(IID_IRMAUserProperties, IIDRequested))
    {
	*ppUnknownReturned = (IRMAUserProperties*)this;
    }
    else if (IsEqualGUID(IID_IRMACredRequestResponse, IIDRequested))
    {
	*ppUnknownReturned = (IRMACredRequestResponse*)this;
    }
    else if (IsEqualGUID(IID_IRMAAuthenticationDBAccessResponse, IIDRequested))
    {
	*ppUnknownReturned = (IRMAAuthenticationDBAccessResponse*)this;
    }
    else if (IsEqualGUID(IID_IRMAAuthenticationDBManager, IIDRequested))
    {
	*ppUnknownReturned = (IRMAAuthenticationDBManager*)this;
    }
    else if (IsEqualGUID(IID_IRMAAuthenticationDBManagerResponse, IIDRequested))
    {
	*ppUnknownReturned = (IRMAAuthenticationDBManagerResponse*)this;
    }
    else
    {
	ResultStatus = PNR_NOINTERFACE;
	*ppUnknownReturned = NULL;
    }

    return ResultStatus;
}

/************************************************************************
 *	Method:
 *	    IRMAObjectConfiguration::SetContext
 *	Purpose:
 *	    Initializes the plugin for use. This method 
 *	    (Or IRMAPlugin::InitPlugin()) must always be
 *	    called before any other method is called. This 
 *	    is primarily needed so that the plugin can 
 *	    have access to the context for creation of
 *	    IRMABuffers and IMalloc.
 */
STDMETHODIMP
CBasicAuthenticator::SetContext(IUnknown* pUnknownContext)
{
    PN_RELEASE(m_pUnknownContext);

    m_pUnknownContext = pUnknownContext;
    if (m_pUnknownContext)
    {
	m_pUnknownContext->AddRef();

	PN_RELEASE(m_pCommonClassFactory);

	m_pUnknownContext->QueryInterface
	(
	    IID_IRMACommonClassFactory,
	    (void**)&m_pCommonClassFactory
	);
    }

    return PNR_OK;
}

/************************************************************************
 *	Method:
 *	    IRMAObjectConfiguration::SetConfiguration
 *	Purpose:
 *	    Configures this plugin.  Often contains a list of vars
 *	    from the config file.
 */
STDMETHODIMP 
CBasicAuthenticator::SetConfiguration
(
    IRMAValues* pValuesConfiguration
)
{
    if (!m_pUnknownContext)
    {
	return PNR_UNEXPECTED;
    }

    m_pValuesConfig = pValuesConfiguration;

    if (m_pValuesConfig)
    {
	m_pValuesConfig->AddRef();

	if (!m_pBufferRealm)
	{
	    m_pValuesConfig->GetPropertyCString
	    (
		"Realm",
		m_pBufferRealm
	    );
	}

	if (!m_pBufferDatabaseID)
	{
	    m_pValuesConfig->GetPropertyCString
	    (
		"DatabaseID",
		m_pBufferDatabaseID
	    );

	    if (m_pBufferDatabaseID)
	    {
		IUnknown* pUnknownDatabase = NULL;
		IUnknown* pUnknownDatabaseManager = NULL;
		IRMADatabaseManager* pDatabaseManagerThis = NULL;

		if (m_pCommonClassFactory)
		{
		    m_pCommonClassFactory->CreateInstance
		    (
			CLSID_CRMADatabaseManager,
			(void**)&pUnknownDatabaseManager
		    );

		    if (pUnknownDatabaseManager)
		    {
			pUnknownDatabaseManager->QueryInterface
			(
			    IID_IRMADatabaseManager, 
			    (void**)&pDatabaseManagerThis
			);

			PN_RELEASE(pUnknownDatabaseManager);

			if (pDatabaseManagerThis)
			{
			    pDatabaseManagerThis->GetInstanceFromID
			    (
				m_pBufferDatabaseID,
				pUnknownDatabase
			    );

			    // Store DB Access interface
			    pUnknownDatabase->QueryInterface
			    (
				IID_IRMAAuthenticationDBAccess, 
				(void**)&m_pAuthenticationDBAccessCredentials
			    );

			    // Store DB Managment interface
			    pUnknownDatabase->QueryInterface
			    (
				IID_IRMAAuthenticationDBManager, 
				(void**)&m_pAuthenticationDBManagerPlugin
			    );

			    PN_RELEASE(pUnknownDatabase);
			}

			PN_RELEASE(pDatabaseManagerThis);
		    }
		}
	    }
	}
    }

    return PNR_OK;
}

/************************************************************************
 *  Method:
 *    IRMAPlugin::InitPlugin
 *  Purpose:
 *    Initializes the plugin for use. This interface must always be
 *    called before any other method is called. This is primarily needed
 *    so that the plugin can have access to the context for creation of
 *    IRMABuffers and IMalloc.
 */
STDMETHODIMP 
CBasicAuthenticator::InitPlugin(IUnknown* /*IN*/ pUnknownContext)
{
    return SetContext(pUnknownContext);
}

/************************************************************************
 *  Method:
 *    IRMAPlugin::GetPluginInfo
 *  Purpose:
 *    Returns the basic information about this plugin. Including:
 *
 *    bLoadMultiple	whether or not this plugin DLL can be loaded
 *			multiple times. All File Formats must set
 *			this value to TRUE.
 *    pDescription	which is used in about UIs (can be NULL)
 *    pCopyright	which is used in about UIs (can be NULL)
 *    pMoreInfoURL	which is used in about UIs (can be NULL)
 */
STDMETHODIMP 
CBasicAuthenticator::GetPluginInfo
(
    REF(BOOL)        /*OUT*/ bLoadMultiple,
    REF(const char*) /*OUT*/ pDescription,
    REF(const char*) /*OUT*/ pCopyright,
    REF(const char*) /*OUT*/ pMoreInfoURL,
    REF(ULONG32)     /*OUT*/ ulVersionNumber
)
{
    bLoadMultiple = TRUE;

    pDescription    = zm_pDescription;
    pCopyright	    = zm_pCopyright;
    pMoreInfoURL    = zm_pMoreInfoURL;
    ulVersionNumber = TARVER_ULONG32_VERSION;

    return PNR_OK;
}

/************************************************************************
 *	Method:
 *	    IRMAPluginProperties::GetProperties
 *	Purpose:
 *	    Returns a list of properties that describe this plugin
 */
STDMETHODIMP
CBasicAuthenticator::GetProperties(REF(IRMAValues*) pValuesOptions)
{
    CreateValues(&pValuesOptions);

    if (pValuesOptions)
    {
	IRMABuffer* pBufferPropValue = NULL;

	FromCharArray
	(
	    "Authenticator", 
	    &pBufferPropValue
	);
	pValuesOptions->SetPropertyCString
	(
	    "PluginType", 
	    pBufferPropValue
	);

	PN_RELEASE(pBufferPropValue);
	FromCharArray
	(
	    "rn-auth-example", 
	    &pBufferPropValue
	);
	pValuesOptions->SetPropertyCString
	(
	    "PluginID", 
	    pBufferPropValue
	);

	PN_RELEASE(pBufferPropValue);
	FromCharArray
	(
	    "Basic", 
	    &pBufferPropValue
	);
	pValuesOptions->SetPropertyCString
	(
	    "AuthenticationProtocolID", 
	    pBufferPropValue
	);

	PN_RELEASE(pBufferPropValue);
    }
    else
    {
	return PNR_OUTOFMEMORY;
    }
    return PNR_OK;
}

/************************************************************************
 *	Method:
 *	    IRMAClientAuthConversation::MakeResponse
 *	Purpose:
 *	    Creates a response to the specified request for authentication 
 *	    from the server.  For conversations that require multiple round 
 *	    trips this method is called for each trip.
 */
STDMETHODIMP 
CBasicAuthenticator::MakeResponse
(
    IRMAClientAuthResponse* pClientAuthResponseRequester,
    IRMARequest* pRequestChallenge
)
{
    /********************************************************************
     *	This protocol is a single round trip conversation.
     *	
     */

    if(!pClientAuthResponseRequester)
    {
	return PNR_UNEXPECTED;
    }

    if(!pRequestChallenge)
    {
	pClientAuthResponseRequester->ResponseReady
	(
	    PNR_UNEXPECTED, 
	    pRequestChallenge
	);

	return PNR_UNEXPECTED;
    }

    m_pRequestCurrent = pRequestChallenge;
    m_pRequestCurrent->AddRef();

    pRequestChallenge->QueryInterface
    (
	IID_IRMARequestContext,
	(void**)&m_pRequestContextCurrent
    );

    m_pClientAuthResponseRequester = pClientAuthResponseRequester;
    m_pClientAuthResponseRequester->AddRef();

    IRMAValues* pValuesChallengeHeaders = NULL;

    m_pRequestCurrent->GetResponseHeaders
    (
	pValuesChallengeHeaders
    );

    if(!pValuesChallengeHeaders)
    {
	m_pClientAuthResponseRequester->ResponseReady
	(
	    PNR_UNEXPECTED, 
	    pRequestChallenge
	);

	// Must release this to prevent possible circular 
	// references.
	PN_RELEASE(m_pClientAuthResponseRequester);

	return PNR_UNEXPECTED;
    }

    PN_RESULT ResultRet = PNR_FAIL;
    IRMABuffer* pBufferChallenge = NULL;

    pValuesChallengeHeaders->GetPropertyCString
    (
	"WWW-Authenticate", 
	pBufferChallenge
    );

    if (pBufferChallenge)
    {
	const char* pcharChallenge = 
	(
	    const char*
	) pBufferChallenge->GetBuffer();

	if(strncasecmp(pcharChallenge, "Basic", 5) == 0)
	{
	    IRMACredRequest* pCredRequester = NULL;
	    IRMAValues* pValuesCredentials = NULL;

	    _DescribeCredentials
	    (
		pValuesChallengeHeaders, 
		&pValuesCredentials
	    );

	    m_pClientAuthResponseRequester->QueryInterface
	    (
		IID_IRMACredRequest, 
		(void**)&pCredRequester
	    );

	    ResultRet = pCredRequester->GetCredentials
	    (
		this, 
		pValuesCredentials
	    );

	    // Flow continues in CredentialsReady()

	    PN_RELEASE(pCredRequester);
	    PN_RELEASE(pValuesCredentials);
	}
	else
	{
	    m_pClientAuthResponseRequester->ResponseReady(PNR_FAIL, NULL);

	    // Must release this to prevent possible circular 
	    // references.
	    PN_RELEASE(m_pClientAuthResponseRequester);
	}
    }
    else
    {
	m_pClientAuthResponseRequester->ResponseReady(PNR_FAIL, NULL);

	// Must release this to prevent possible circular 
	// references.
	PN_RELEASE(m_pClientAuthResponseRequester);
    }

    PN_RELEASE(pValuesChallengeHeaders);
    PN_RELEASE(pBufferChallenge);

    return ResultRet;
}

/************************************************************************
 *	Method:
 *	    IRMAClientAuthConversation::IsDone
 *	Purpose:
 *	    Returns FALSE until the last trip in the conversation has been 
 *	    processed.
 */
BOOL 
CBasicAuthenticator::IsDone()
{
    return m_bFinished;
}

/************************************************************************
 *	Method:
 *	    IRMAClientAuthConversation::Authenticated
 *	Purpose:
 *	    After the conversation is complete this function will be called
 *	    to report whether the server accepted or rejected the credentials.
 */
STDMETHODIMP
CBasicAuthenticator::Authenticated(BOOL bAuthenticated)
{
    return PNR_OK;
}

/************************************************************************
 *	Method:
 *	    IRMACredRequestResponse::CredentialsReady
 *	Purpose:
 *	    Called by the client application in response to a request for 
 *	    the users credentials.  Often the credentials will consist of
 *	    CString(Username) and CString(Password).  Usually the client 
 *	    will get the credentials through interaction with the user.
 */
STDMETHODIMP
CBasicAuthenticator::CredentialsReady
(
    PN_RESULT	ResultStatus,
    IRMAValues* pValuesCredentials
)
{
    PN_RESULT ResultRet = PNR_FAIL;

    if(!m_pClientAuthResponseRequester)
    {
	return PNR_UNEXPECTED;
    }

    if(!m_pRequestContextCurrent)
    {
	m_pClientAuthResponseRequester->ResponseReady
	(
	    PNR_UNEXPECTED, 
	    NULL
	);

	// Must release this to prevent possible circular 
	// references.
	PN_RELEASE(m_pClientAuthResponseRequester);

	return PNR_UNEXPECTED;
    }

    if(FAILED(ResultStatus))
    {
	m_pClientAuthResponseRequester->ResponseReady
	(
	    ResultStatus, 
	    m_pRequestCurrent
	);

	// Must release this to prevent possible circular 
	// references.
	PN_RELEASE(m_pClientAuthResponseRequester);

	return ResultStatus;
    }
    else if(pValuesCredentials)
    {
	IRMABuffer*	pBufferStoreable = NULL;
	IRMAValues*	pValuesResponseHeaders = NULL;

	ResultRet = _CredentialsToStorage
	(
	    pValuesCredentials,
	    &pBufferStoreable
	);

	if(SUCCEEDED(ResultRet))
	{
	    ResultRet = _StorageToHeader
	    (
		pValuesCredentials,
		pBufferStoreable,
		&pValuesResponseHeaders
	    );
	}

	PN_RELEASE(pBufferStoreable);

	m_pRequestCurrent->SetRequestHeaders
	(
	    pValuesResponseHeaders
	);

	PN_RELEASE(pValuesResponseHeaders);

	// we are all done..
	m_bFinished = TRUE;

	m_pClientAuthResponseRequester->ResponseReady
	(
	    PNR_OK, 
	    m_pRequestCurrent
	);
    }
    else
    {
	m_pClientAuthResponseRequester->ResponseReady
	(
	    PNR_FAIL, 
	    m_pRequestCurrent
	);
    }

    // Must release this to prevent possible circular 
    // references.
    PN_RELEASE(m_pClientAuthResponseRequester);

    return ResultRet;
}

PN_RESULT 
CBasicAuthenticator::_DescribeCredentials
(
    IRMAValues* pValuesChallengeHeaders,
    IRMAValues** ppValuesParameterDescriptions
)
{
    PN_RESULT ResultRet = _ChallengeToCredentials
    (
	pValuesChallengeHeaders, 
	ppValuesParameterDescriptions
    );

    if
    (
	SUCCEEDED(ResultRet)
	&&
	(*ppValuesParameterDescriptions)
    )
    {
	IRMABuffer* pBufferPropValue = NULL;

	FromCharArray
	(
	    "The Realm %Realm% has indicated that %URI% is secure \
	    content. Please fill out the credentials requested below \
	    to gain access. (WARNING: This type of authentication sends\
	    your password to the server)", 
	    &pBufferPropValue
	);
	(*ppValuesParameterDescriptions)->SetPropertyCString
	(
	    "Prompt", 
	    pBufferPropValue
	);

	PN_RELEASE(pBufferPropValue);
	FromCharArray
	(
	    "?", 
	    &pBufferPropValue
	);
	(*ppValuesParameterDescriptions)->SetPropertyCString
	(
	    "User", 
	    pBufferPropValue
	);

	PN_RELEASE(pBufferPropValue);
	FromCharArray
	(
	    "?*", 
	    &pBufferPropValue
	);
	(*ppValuesParameterDescriptions)->SetPropertyCString
	(
	    "Password", 
	    pBufferPropValue
	);

	PN_RELEASE(pBufferPropValue);
    }

    return ResultRet;
}

/************************************************************************
 *	Method:
 *	    IRMAServerAuthConversation::MakeChallenge
 *	Purpose:
 *	    Begins a conversation to authenticate the specified client request.
 *	    For conversations that require multiple round trips A callback 
 *	    interface implemented by the protocol is retrieved from the request 
 *	    object.
 */
STDMETHODIMP 
CBasicAuthenticator::MakeChallenge
(
    IRMAServerAuthResponse* pServerAuthResponseRequester,
    IRMARequest* pRequestResponse
)
{
    /********************************************************************
     *	This protocol is a single round trip conversation.
     *	
     */

    if(!pServerAuthResponseRequester)
    {
	return PNR_UNEXPECTED;
    }

    m_pRequestResponse = pRequestResponse;
    m_pRequestResponse->AddRef();

    pRequestResponse->QueryInterface
    (
	IID_IRMARequestContext,
	(void**)&m_pRequestContextCurrent
    );

    m_pServerAuthResponseRequester = pServerAuthResponseRequester;
    m_pServerAuthResponseRequester->AddRef();

    if(!m_pRequestContextCurrent)
    {
	m_pServerAuthResponseRequester->ChallengeReady
	(
	    PNR_UNEXPECTED, 
	    pRequestResponse
	);

	// Must release this to prevent possible circular 
	// references.
	PN_RELEASE(m_pRequestResponse);
	PN_RELEASE(m_pRequestContextCurrent);
	PN_RELEASE(m_pServerAuthResponseRequester);

	return PNR_UNEXPECTED;
    }

    IRMAValues* pValuesResponseHeaders = NULL;

    pRequestResponse->GetRequestHeaders
    (
	pValuesResponseHeaders
    );

    if(!pValuesResponseHeaders)
    {
	m_pServerAuthResponseRequester->ChallengeReady
	(
	    PNR_UNEXPECTED, 
	    pRequestResponse
	);

	// Must release this to prevent possible circular 
	// references.
	PN_RELEASE(m_pRequestResponse);
	PN_RELEASE(m_pRequestContextCurrent);
	PN_RELEASE(m_pServerAuthResponseRequester);

	return PNR_UNEXPECTED;
    }

    IRMABuffer* pBufferPropValue = NULL;
    PN_RESULT ResultReturn = PNR_OK;

    if
    (
	pValuesResponseHeaders
	&&
	SUCCEEDED
	(
	    pValuesResponseHeaders->GetPropertyCString
	    (
		"Authorization", 
		pBufferPropValue
	    )
	)
	&&
	pBufferPropValue
	&&
	!strncasecmp((char*)pBufferPropValue->GetBuffer(), "Basic", 5)
    )
    {
	char* pcharToken = (char*)pBufferPropValue->GetBuffer() + 5;
	char* pcharUserPasswd = new char[strlen(pcharToken)];
	INT32 outlen;

	for
	(
	    pcharToken = (char*)pBufferPropValue->GetBuffer() + 5;
	    *pcharToken && isspace(*pcharToken); 
	    pcharToken++
	);

	// check credentials
	outlen = BinFrom64
	(
	    pcharToken, 
	    strlen(pcharToken), 
	    (BYTE*)pcharUserPasswd
	);

	if (outlen > 0)
	{
	    pcharUserPasswd[outlen] = 0;

	    char* colon = strchr(pcharUserPasswd, ':');

	    if (colon)
	    {
		*colon = 0;

		FromCharArray
		(
		    pcharUserPasswd,
		    &m_pBufferPrincipalID
		);

		// Store for use in GetCredentialsDone()
		FromCharArray
		(
		    colon+1,
		    &m_pBufferPassword
		);

		if (m_pAuthenticationDBAccessCredentials)
		{
		    ResultReturn = m_pAuthenticationDBAccessCredentials
		    ->
		    GetCredentials(this, m_pBufferPrincipalID);
		    // Flow continues in GetCredentialsDone()
		}

	    }
	}

	delete [] pcharUserPasswd;

    }
    else
    {
	ResultReturn = _SendChallengeResponse();
    }

    PN_RELEASE(pBufferPropValue);
    PN_RELEASE(pValuesResponseHeaders);

    return ResultReturn;
}

PN_RESULT
CBasicAuthenticator::_SendChallengeResponse()
{
    IRMAValues* pValuesChallengeHeaders = NULL;

    if (!m_bAuthenticated)
    {
	IRMABuffer* pBufferValue = NULL;
	char pcharValue[512];

	// tell the client we support this protocol
	CreateValues(&pValuesChallengeHeaders);

	strcpy(pcharValue, "Basic realm=\"");
	if (m_pBufferRealm)
	{
	    strcat(pcharValue, (const char*)m_pBufferRealm->GetBuffer());
	}
	strcat(pcharValue, "\"");

	FromCharArray
	(
	    pcharValue,
	    &pBufferValue
	);

	pValuesChallengeHeaders->SetPropertyCString
	(
	    "WWW-Authenticate",
	    pBufferValue
	);

	PN_RELEASE(pBufferValue);
    }

    if (pValuesChallengeHeaders)
    {
	m_pRequestResponse->SetResponseHeaders
	(
	    pValuesChallengeHeaders
	);
    }

    PN_RELEASE(pValuesChallengeHeaders);

    m_pServerAuthResponseRequester->ChallengeReady
    (
	PNR_OK, 
	m_pRequestResponse
    );

    // Must release this to prevent possible circular 
    // references.
    PN_RELEASE(m_pRequestResponse);
    PN_RELEASE(m_pRequestContextCurrent);
    PN_RELEASE(m_pServerAuthResponseRequester);

    return PNR_OK;
}

/************************************************************************
 *	Method:
 *	    IRMAServerAuthConversation::IsAuthenticated
 *	Purpose:
 *	    Returns FALSE until the client is authenticated.
 */
STDMETHODIMP_(BOOL) 
CBasicAuthenticator::IsAuthenticated()
{
    return m_bAuthenticated;
}

/************************************************************************
 *	Method:
 *	    IRMAServerAuthConversation::GetUserContext
 *	Purpose:
 *	    Returns an object representing the authenticated user.
 *	    Returns nothing if there is no authenticated user yet.
 */
STDMETHODIMP 
CBasicAuthenticator::GetUserContext(REF(IUnknown*) pUnknownUser)
{
    /********************************************************************
     *	For simplicity we use this same object as the user object.
     *	
     */

    return QueryInterface(IID_IUnknown, (void**)&pUnknownUser);
}

/************************************************************************
 *	Method:
 *	    IRMAUserProperties::GetPrincipalID
 *	Purpose:
 *	    Returns the name of the currently authenticated user.
 */
STDMETHODIMP 
CBasicAuthenticator::GetPrincipalID
(
    REF(IRMABuffer*) pBufferPrincipalID
)
{
    pBufferPrincipalID = m_pBufferPrincipalID;

    if (pBufferPrincipalID)
    {
	pBufferPrincipalID->AddRef();
	return PNR_OK;
    }

    return PNR_FAIL;
}

/************************************************************************
 *	Method:
 *	    IRMAUserProperties::GetAuthorityName
 *	Purpose:
 *	    Returns the authorizer of the currently authenticated user.
 *	    (Realm, Domain, etc..)
 */
STDMETHODIMP 
CBasicAuthenticator::GetAuthorityName
(
    REF(IRMABuffer*) pBufferAuthorityName
)
{
    pBufferAuthorityName = m_pBufferRealm;

    if (pBufferAuthorityName)
    {
	pBufferAuthorityName->AddRef();
	return PNR_OK;
    }

    return PNR_FAIL;
}

/************************************************************************
 *	Method:
 *	    IRMAAuthenticationDBAccessResponse::ExistenceCheckDone
 *	Purpose:
 *	    Called by the Database in response to a test for the existance
 *	    of a specified user.  
 */
STDMETHODIMP
CBasicAuthenticator::ExistenceCheckDone
(
    PN_RESULT		ResultStatus,
    IRMABuffer*		pBufferPrincipalID
)
{
    /********************************************************************
     *	This protocol does not perform an existance check.
     *	
     */

    return PNR_NOTIMPL;
}

/************************************************************************
 *	Method:
 *	    IRMAAuthenticationDBAccessResponse::GetCredentialsDone
 *	Purpose:
 *	    Called by the Database in response to a request for Credentials
 *	    for a specified user.  
 */
STDMETHODIMP
CBasicAuthenticator::GetCredentialsDone
(
    PN_RESULT		ResultStatus,
    IRMABuffer*		pBufferPrincipalID,
    IRMABuffer*		pBufferCredentials
)
{
    if 
    (
	SUCCEEDED(ResultStatus)
	&&
	pBufferCredentials
	&&
	!strcasecmp
	(
	    (char*)m_pBufferPassword->GetBuffer(), 
	    (char*)pBufferCredentials->GetBuffer()
	)
    )
    {
	// allow access
	//

	m_bAuthenticated = TRUE;

	IUnknown* pUnknownUser = NULL;

	GetUserContext(pUnknownUser);

	m_pRequestContextCurrent->SetUserContext
	(
	    pUnknownUser
	);

	PN_RELEASE(pUnknownUser);
    }
    else
    {
	// disallow access
	m_bAuthenticated = FALSE;

	// This user didn't pan out..
	PN_RELEASE(m_pBufferPrincipalID);
    }

    // Clean This up
    memset
    (
	m_pBufferPassword->GetBuffer(), 
	0, 
	m_pBufferPassword->GetSize()
    );

    PN_RELEASE(m_pBufferPassword);

    // Finish Up.
    _SendChallengeResponse();

    return PNR_OK;
}

/************************************************************************
 *	Method:
 *	    IRMAAuthenticationDBManager::AddPrincipal
 *	Purpose:
 *	    Add a user to the database.
 */
STDMETHODIMP
CBasicAuthenticator::AddPrincipal
(
    IRMAAuthenticationDBManagerResponse* pAuthenticationDBManagerResponseNew,
    IRMABuffer*		pBufferPrincipalID
)
{
    /********************************************************************
     *	This method proxies to the real database.
     *	
     */

    if(!pAuthenticationDBManagerResponseNew)
    {
	return PNR_UNEXPECTED;
    }

    m_pAuthenticationDBManagerResponseRequester
    =
    pAuthenticationDBManagerResponseNew;
    m_pAuthenticationDBManagerResponseRequester->AddRef();

    if (!m_pAuthenticationDBManagerPlugin)
    {
	m_pAuthenticationDBManagerResponseRequester->AddPrincipalDone
	(
	    PNR_NOINTERFACE, 
	    pBufferPrincipalID
	);

	// Must release this to prevent possible circular 
	// references.
	PN_RELEASE(m_pAuthenticationDBManagerResponseRequester);

	return PNR_NOINTERFACE;
    }

    m_pAuthenticationDBManagerPlugin->AddPrincipal
    (
	this,
	pBufferPrincipalID
    );

    return PNR_OK;
}

/************************************************************************
 *	Method:
 *	    IRMAAuthenticationDBManager::RemovePrincipal
 *	Purpose:
 *	    Remove a user to the database.
 */
STDMETHODIMP
CBasicAuthenticator::RemovePrincipal
(
    IRMAAuthenticationDBManagerResponse* pAuthenticationDBManagerResponseNew,
    IRMABuffer*		pBufferPrincipalID
)
{
    /********************************************************************
     *	This method proxies to the real database.
     *	
     */

    if(!pAuthenticationDBManagerResponseNew)
    {
	return PNR_UNEXPECTED;
    }

    m_pAuthenticationDBManagerResponseRequester
    =
    pAuthenticationDBManagerResponseNew;
    m_pAuthenticationDBManagerResponseRequester->AddRef();

    if (!m_pAuthenticationDBManagerPlugin)
    {
	m_pAuthenticationDBManagerResponseRequester->RemovePrincipalDone
	(
	    PNR_NOINTERFACE, 
	    pBufferPrincipalID
	);

	// Must release this to prevent possible circular 
	// references.
	PN_RELEASE(m_pAuthenticationDBManagerResponseRequester);

	return PNR_NOINTERFACE;
    }

    m_pAuthenticationDBManagerPlugin->RemovePrincipal
    (
	this,
	pBufferPrincipalID
    );

    return PNR_OK;
}

/************************************************************************
 *	Method:
 *	    IRMAAuthenticationDBManager::SetCredentials
 *	Purpose:
 *	    Set new credentials for a user to the database.
 */
STDMETHODIMP
CBasicAuthenticator::SetCredentials
(
    IRMAAuthenticationDBManagerResponse* pAuthenticationDBManagerResponseNew,
    IRMABuffer*		pBufferPrincipalID, 
    IRMABuffer*		pBufferCredentials
)
{
    /********************************************************************
     *	This method proxies to the real database.
     *
     *	The real reason why all database access is proxied through the
     *	authentication plugins is so that they can encrypt/obfusicate
     *	these credentials before writting them to the database and
     *	un-encrypt/un-obfusicate them when reading them back in.
     *	Again, for simplicity, this is not implemented.
     *
     *	(Also notice that it does not proxy the IRMAAuthenticationDBAccess
     *	methods thereby leaving no supported means of access to the 
     *	un-encrypted/un-obfusicated credentials.)
     */

    if(!pAuthenticationDBManagerResponseNew)
    {
	return PNR_UNEXPECTED;
    }

    m_pAuthenticationDBManagerResponseRequester
    =
    pAuthenticationDBManagerResponseNew;
    m_pAuthenticationDBManagerResponseRequester->AddRef();

    if (!m_pAuthenticationDBManagerPlugin)
    {
	m_pAuthenticationDBManagerResponseRequester->SetCredentialsDone
	(
	    PNR_NOINTERFACE, 
	    pBufferPrincipalID
	);

	// Must release this to prevent possible circular 
	// references.
	PN_RELEASE(m_pAuthenticationDBManagerResponseRequester);

	return PNR_NOINTERFACE;
    }

    m_pAuthenticationDBManagerPlugin->SetCredentials
    (
	this,
	pBufferPrincipalID,
	pBufferCredentials
    );

    return PNR_OK;
}

/************************************************************************
 *	Method:
 *	    IRMAAuthenticationDBManagerResponse::AddPrincipalDone
 *	Purpose:
 *	    Called by a database in response to a request to add a user.
 */
STDMETHODIMP
CBasicAuthenticator::AddPrincipalDone
(
    PN_RESULT		ResultStatus,
    IRMABuffer*		pBufferPrincipalID
)
{
    // We are done..
    m_pAuthenticationDBManagerResponseRequester->AddPrincipalDone
    (
	ResultStatus, 
	pBufferPrincipalID
    );

    // Must release this to prevent possible circular 
    // references.
    PN_RELEASE(m_pAuthenticationDBManagerResponseRequester);

    return PNR_OK;
}

/************************************************************************
 *	Method:
 *	    IRMAAuthenticationDBManagerResponse::RemovePrincipalDone
 *	Purpose:
 *	    Called by a database in response to a request to remove a user.
 */
STDMETHODIMP
CBasicAuthenticator::RemovePrincipalDone
(
    PN_RESULT		ResultStatus,
    IRMABuffer*		pBufferPrincipalID
)
{
    // We are done..
    m_pAuthenticationDBManagerResponseRequester->RemovePrincipalDone
    (
	ResultStatus, 
	pBufferPrincipalID
    );

    // Must release this to prevent possible circular 
    // references.
    PN_RELEASE(m_pAuthenticationDBManagerResponseRequester);

    return PNR_OK;
}

/************************************************************************
 *	Method:
 *	    IRMAAuthenticationDBManagerResponse::SetCredentialsDone
 *	Purpose:
 *	    Called by a database in response to a request to set a users
 *	    credentials.
 */
STDMETHODIMP
CBasicAuthenticator::SetCredentialsDone
(
    PN_RESULT		ResultStatus,
    IRMABuffer*		pBufferPrincipalID
)
{
    // We are done..
    m_pAuthenticationDBManagerResponseRequester->SetCredentialsDone
    (
	ResultStatus, 
	pBufferPrincipalID
    );

    // Must release this to prevent possible circular 
    // references.
    PN_RELEASE(m_pAuthenticationDBManagerResponseRequester);

    return PNR_OK;
}

PN_RESULT
CBasicAuthenticator::_CredentialsToStorage
(
    IRMAValues* pValuesCredentials,
    IRMABuffer** ppBufferStoreablePassword
)
{
    pValuesCredentials->GetPropertyCString
    (
	"Password", 
	*ppBufferStoreablePassword
    );
    return PNR_OK;
}

PN_RESULT
CBasicAuthenticator::_StorageToHeader
(
    IRMAValues* pValuesCredentials,
    IRMABuffer* pBufferStoredPassword,
    IRMAValues** ppValuesResponseHeaders
)
{
    IRMABuffer* pBufferUserName = NULL;
    IRMABuffer* pBufferHeaderValue = NULL;
    PN_RESULT ResultRet = PNR_UNEXPECTED;

    if
    (
	pBufferStoredPassword
    )
    {
	if
	(
	    SUCCEEDED
	    (
		pValuesCredentials->GetPropertyCString
		(
		    "UserName", 
		    pBufferUserName
		)
	    )
	)
	{
	    char prebuf[1024];
	    char encodedbuf[1024];
	    INT32 encodedlen;

	    sprintf
	    (
		prebuf, 
		"%s:%s", 
		pBufferUserName->GetBuffer(), 
		pBufferStoredPassword->GetBuffer()
	    );

	    encodedlen = BinTo64
	    (
		(const BYTE*)prebuf, 
		strlen(prebuf),
		encodedbuf
	    );

	    encodedbuf[encodedlen] = 0;

	    sprintf
	    (
		prebuf, 
		"Basic %s", 
		encodedbuf
	    );

	    FromCharArray
	    (
		prebuf,
		&pBufferHeaderValue
	    );

	    CreateValues(ppValuesResponseHeaders);

	    ResultRet = (*ppValuesResponseHeaders)->SetPropertyCString
	    (
		"Authorization", 
		pBufferHeaderValue
	    );
	}

	PN_RELEASE(pBufferUserName);
	PN_RELEASE(pBufferHeaderValue);
    }

    return ResultRet;
}

BOOL
CBasicAuthenticator::_GetQuotedValue
(
    const char*& instr, 
    char* valname, 
    char* valbuf
)
{
    char* equals = strchr(instr, '=');
    if(!equals)
	return FALSE;

    while(isspace(*(equals - 1)) && equals > instr) equals--;
    if(equals <= instr)
	return FALSE;

    strncpy(valname, instr, equals - instr);
    valname[equals -instr] = 0;

    char* firstquote = strchr(equals, '"');
    if(!firstquote)
	return FALSE;

    char* secondquote = strchr(firstquote + 1, '"');
    if(!secondquote)
	return FALSE;

    strncpy(valbuf, firstquote + 1, secondquote - firstquote - 1);
    valbuf[secondquote - firstquote - 1] = 0;
    instr = secondquote + 1;
    return TRUE;
}

PN_RESULT
CBasicAuthenticator::_ChallengeToCredentials
(
    IRMAValues* pValuesChallengeHeaders, 
    IRMAValues** ppValuesCredentials
)
{
    IRMABuffer* pBufferChallenge = NULL;
    IRMABuffer* pBufferValue = NULL;

    pValuesChallengeHeaders->GetPropertyCString
    (
	"WWW-Authenticate", 
	pBufferChallenge
    );

    const char* pcharChallenge = (const char*)pBufferChallenge->GetBuffer();

    if(strncasecmp(pcharChallenge, "Basic", 5) == 0)
    {
	BOOL done = FALSE;

	CreateValues(ppValuesCredentials);

	pcharChallenge += 5;

	char* valbuf = new char[256];
	char* valname = new char[256];

	while(!done)
	{
	    while
	    (
		(
		    isspace(*pcharChallenge)  
		    || 
		    *pcharChallenge == ','
		)
		&& 
		*pcharChallenge
	    ) 
	    {
		pcharChallenge++;
	    }

	    if(!(*pcharChallenge))
		break;

	    if(_GetQuotedValue(pcharChallenge, valname, valbuf))
	    {
		FromCharArray(valbuf, &pBufferValue);

		(*ppValuesCredentials)->SetPropertyCString(valname, pBufferValue);

		PN_RELEASE(pBufferValue);
	    }
	    else
		done = TRUE;
	}

	delete[] valbuf;
	delete[] valname;

	PN_RELEASE(pBufferChallenge);

	return PNR_OK;
    }

    PN_RELEASE(pBufferChallenge);

    return PNR_FAIL;
}

PN_RESULT
CBasicAuthenticator::FromCharArray
(
    const char* szIn, 
    IRMABuffer** ppBufferOut
)
{
    IUnknown* pUnknownNew = NULL;

    if (m_pCommonClassFactory)
    {
	m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer, (void**)&pUnknownNew);

	if(pUnknownNew)
	{
	    pUnknownNew->QueryInterface(IID_IRMABuffer, (void**)ppBufferOut);

	    pUnknownNew->Release();

	    (*ppBufferOut)->Set((const unsigned char*)szIn, strlen(szIn)+1);

	    return PNR_OK;
	}
    }

    (*ppBufferOut) = NULL;

    return PNR_OUTOFMEMORY;
}

PN_RESULT 
CBasicAuthenticator::CreateValues(IRMAValues** ppValuesNew)
{
    IUnknown* pUnknownNew = NULL;

    if (m_pCommonClassFactory)
    {
	m_pCommonClassFactory->CreateInstance(CLSID_IRMAValues, (void**)&pUnknownNew);

	if(pUnknownNew)
	{
	    pUnknownNew->QueryInterface(IID_IRMAValues, (void**)ppValuesNew);

	    pUnknownNew->Release();

	    return PNR_OK;
	}
    }

    (*ppValuesNew) = NULL;

    return PNR_OUTOFMEMORY;
}

/*
 * Table for encoding base64
 */
static char Base64[] =
   "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

#define XX 127
/*
 * Table for decoding base64
 */
static char Index64[256] = {
    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,62, XX,XX,XX,63,
    52,53,54,55, 56,57,58,59, 60,61,XX,XX, XX,XX,XX,XX,
    XX, 0, 1, 2,  3, 4, 5, 6,  7, 8, 9,10, 11,12,13,14,
    15,16,17,18, 19,20,21,22, 23,24,25,XX, XX,XX,XX,XX,
    XX,26,27,28, 29,30,31,32, 33,34,35,36, 37,38,39,40,
    41,42,43,44, 45,46,47,48, 49,50,51,XX, XX,XX,XX,XX,
    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX
};
#define CHAR64(c)  (Index64[(unsigned char)(c)])

static void 
Output64Chunk(int c1, int c2, int c3, int pads, char* pBuf,
    INT32 bufOffset)
{
    pBuf[bufOffset++] = Base64[c1>>2];
    pBuf[bufOffset++] = Base64[((c1 & 0x3)<< 4) | ((c2 & 0xF0) >> 4)];
    if (pads == 2) 
    {
	pBuf[bufOffset++] = '=';
	pBuf[bufOffset++] = '=';
    }
    else if (pads) 
    {
	pBuf[bufOffset++] = Base64[((c2 & 0xF) << 2) | ((c3 & 0xC0) >>6)];
	pBuf[bufOffset++] = '=';
    } 
    else 
    {
	pBuf[bufOffset++] = Base64[((c2 & 0xF) << 2) | ((c3 & 0xC0) >>6)];
	pBuf[bufOffset++] = Base64[c3 & 0x3F];
    }
}

INT32 
BinTo64(const BYTE* pInBuf, INT32 len, char* pOutBuf)
{
    int c1, c2, c3;
    INT32 inOffset = 0;
    INT32 outOffset = 0;

    while(inOffset < len)
    {
	c1 = pInBuf[inOffset++];
	if(inOffset == len)
	{
	    Output64Chunk(c1, 0, 0, 2, pOutBuf, outOffset);
	    outOffset += 4;
	}
	else
	{
	    c2 = pInBuf[inOffset++];
	    if(inOffset == len)
	    {
		Output64Chunk(c1, c2, 0, 1, pOutBuf, outOffset);
		outOffset += 4;
	    }
	    else
	    {
		c3 = pInBuf[inOffset++];
		Output64Chunk(c1, c2, c3, 0, pOutBuf, outOffset);
		outOffset += 4;
	    }
	}
    }
    pOutBuf[outOffset++] = '\0';
    return outOffset;
}

INT32
BinFrom64(const char* pInBuf, INT32 len, BYTE* pOutBuf)
{
    int c1, c2, c3, c4;
    INT32 offset = 0;
    INT32 outOffset = 0;
    INT32 done = 0;
    INT32 outch = 0;

    while(offset < len)
    {
	c1 = pInBuf[offset++];
	if(c1 != '=' && CHAR64(c1) == XX)
	{
	    continue;
	}
	do
	{
	    c2 = pInBuf[offset++];
	} 
	while(offset < len && c2 != '=' && CHAR64(c2) == XX);
	do
	{
	    c3 = pInBuf[offset++];
	}
	while(offset < len && c3 != '=' && CHAR64(c3) == XX);
	do
	{
	    c4 = pInBuf[offset++];
	}
	while(offset < len && c4 != '=' && CHAR64(c4) == XX);
	if(offset > len)
	{
	    /* BAD data */
	    return -1;
	}
	if(c1 == '=' || c2 == '=')
	{
	    continue;
	}
	c1 = CHAR64(c1);
	c2 = CHAR64(c2);
	pOutBuf[outOffset++]  = ((c1 << 2) | ((c2 & 0x30) >> 4));
	if(c3 == '=')
	{
	    continue;
	}
	else
	{
	    c3 = CHAR64(c3);
	    pOutBuf[outOffset++]  = (((c2 & 0x0f) << 4) | ((c3 & 0x3c) >> 2));
	    if(c4 == '=')
	    {
		continue;
	    }
	    else
	    {
		c4 = CHAR64(c4);
		pOutBuf[outOffset++] = (((c3 & 0x03) << 6) | c4);
	    }
	}
    }
    return outOffset;
}
