/****************************************************************************
 * 
 *  $Id: exauth.h,v 1.2 1998/12/18 10:37:45 kshoop Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 * 
 *  exauth.h
 *
 *  Example Authentication plug-in for the RealMedia Architecture (RMA).
 */
#ifndef __CBasicAuthenticator__
#define __CBasicAuthenticator__

/****************************************************************************
 * 
 *  Function:
 * 
 *	RMACreateInstance()
 * 
 *  Purpose:
 * 
 *	Function implemented by all plugin DLL's to create an instance of 
 *	any of the objects supported by the DLL. This method is similar to 
 *	Window's CoCreateInstance() in its purpose, except that it only 
 *	creates objects from this plugin DLL.
 *
 *	NOTE: Aggregation is never used. Therefore an outer unknown is
 *	not passed to this function, and you do not need to code for this
 *	situation.
 * 
 */
STDAPI RMACreateInstance(IUnknown**  /*OUT*/	ppUnknown);


/****************************************************************************
 * 
 *  Function:
 * 
 *	RMAShutdown()
 * 
 *  Purpose:
 * 
 *	Function implemented by all plugin DLL's to free any *global* 
 *	resources. This method is called just before the DLL is unloaded.
 *
 */
STDAPI RMAShutdown(void);

class CBasicAuthenticator
    : public IRMAPlugin
    , public IRMAPluginProperties
    , public IRMAObjectConfiguration
    , public IRMAClientAuthConversation
    , public IRMACredRequestResponse
    , public IRMAServerAuthConversation
    , public IRMAUserProperties
    , public IRMAAuthenticationDBAccessResponse
    , public IRMAAuthenticationDBManager
    , public IRMAAuthenticationDBManagerResponse
{
public:
    CBasicAuthenticator();

    /************************************************************************
    *  IUnknown COM Interface Methods                          ref:  pncom.h
    */
    STDMETHOD_(UINT32, AddRef)();
    STDMETHOD_(UINT32, Release)();
    STDMETHOD(QueryInterface)
    (
	REFIID IIDRequested,
	void** ppUnknownReturned
    );

    /************************************************************************
    *  IRMAPlugin COM Interface Methods                        ref:  rmaplgn.h
    */

    /************************************************************************
     *	Method:
     *	    IRMAPlugin::GetPluginInfo
     *	Purpose:
     *	    Returns the basic information about this plugin. Including:
     *
     *	    bLoadMultiple	whether or not this plugin DLL can be loaded
     *				multiple times. All File Formats must set
     *				this value to TRUE.
     *	    pDescription	which is used in about UIs (can be NULL)
     *	    pCopyright		which is used in about UIs (can be NULL)
     *	    pMoreInfoURL	which is used in about UIs (can be NULL)
     */
    STDMETHOD(GetPluginInfo)	(THIS_
				REF(BOOL)        /*OUT*/ bLoadMultiple,
				REF(const char*) /*OUT*/ pDescription,
				REF(const char*) /*OUT*/ pCopyright,
				REF(const char*) /*OUT*/ pMoreInfoURL,
				REF(ULONG32)	 /*OUT*/ ulVersionNumber
				);

    /************************************************************************
     *	Method:
     *	    IRMAPlugin::InitPlugin
     *	Purpose:
     *	    Initializes the plugin for use. This interface must always be
     *	    called before any other method is called. This is primarily needed 
     *	    so that the plugin can have access to the context for creation of
     *	    IRMABuffers and IMalloc.
     */
    STDMETHOD(InitPlugin)   (THIS_
			    IUnknown*   /*IN*/  pContext);


    /************************************************************************
    *  IRMAObjectConfiguration COM Interface Methods           ref: rmaplgns.h
    */

    /************************************************************************
     *	Method:
     *	    IRMAObjectConfiguration::SetContext
     *	Purpose:
     *	    Initializes the plugin for use. This method 
     *	    (Or IRMAPlugin::InitPlugin()) must always be
     *	    called before any other method is called. This 
     *	    is primarily needed so that the plugin can 
     *	    have access to the context for creation of
     *	    IRMABuffers and IMalloc.
     */
    STDMETHOD(SetContext)(IUnknown* pUnknownContext);

    /************************************************************************
     *	Method:
     *	    IRMAObjectConfiguration::SetConfiguration
     *	Purpose:
     *	    Configures this plugin.  Often contains a list of vars
     *	    from the config file.
     */
    STDMETHOD(SetConfiguration)(IRMAValues* pValuesConfiguration);


    /************************************************************************
    *  IRMAPluginProperties COM Interface Methods              ref: rmaplgns.h
    */

    /************************************************************************
     *	Method:
     *	    IRMAPluginProperties::GetProperties
     *	Purpose:
     *	    Returns a list of properties that describe this plugin
     */
    STDMETHOD(GetProperties)(REF(IRMAValues*) pValuesOptions);


    /************************************************************************
    *  IRMAClientAuthConversation COM Interface Methods        ref: rmaauthn.h
    */

    /************************************************************************
     *	Method:
     *	    IRMAClientAuthConversation::MakeResponse
     *	Purpose:
     *	    Creates a response to the specified request for authentication 
     *	    from the server.  For conversations that require multiple round 
     *	    trips this method is called for each trip.
     */
    STDMETHOD(MakeResponse)
    (
	IRMAClientAuthResponse* pClientAuthResponseRequester,
	IRMARequest* pRequestChallenge
    );

    /************************************************************************
     *	Method:
     *	    IRMAClientAuthConversation::IsDone
     *	Purpose:
     *	    Returns FALSE until the last trip in the conversation has been 
     *	    processed.
     */
    STDMETHOD_(BOOL,IsDone)();

    /************************************************************************
     *	Method:
     *	    IRMAClientAuthConversation::Authenticated
     *	Purpose:
     *	    After the conversation is complete this function will be called
     *	    to report whether the server accepted or rejected the credentials.
     */
    STDMETHOD(Authenticated)(BOOL bAuthenticated);


    /************************************************************************
    *  IRMACredRequestResponse COM Interface Methods           ref: rmaauthn.h
    */

    /************************************************************************
     *	Method:
     *	    IRMACredRequestResponse::CredentialsReady
     *	Purpose:
     *	    Called by the client application in response to a request for 
     *	    the users credentials.  Often the credentials will consist of
     *	    CString(Username) and CString(Password).  Usually the client 
     *	    will get the credentials through interaction with the user.
     */
    STDMETHOD(CredentialsReady)
    (
	PN_RESULT	ResultStatus,
	IRMAValues* pValuesCredentials
    );


    /************************************************************************
    *  IRMAServerAuthConversation COM Interface Methods        ref: rmaauthn.h
    */

    /************************************************************************
     *	Method:
     *	    IRMAServerAuthConversation::MakeChallenge
     *	Purpose:
     *	    Begins a conversation to authenticate the specified client request.
     *	    For conversations that require multiple round trips A callback 
     *	    interface implemented by the protocol is retrieved from the request 
     *	    object.
     */
    STDMETHOD(MakeChallenge)
    (
	IRMAServerAuthResponse* pServerAuthResponseRequester,
	IRMARequest* pRequestResponse
    );

    /************************************************************************
     *	Method:
     *	    IRMAServerAuthConversation::IsAuthenticated
     *	Purpose:
     *	    Returns FALSE until the client is authenticated.
     */
    STDMETHOD_(BOOL,IsAuthenticated)();

    /************************************************************************
     *	Method:
     *	    IRMAServerAuthConversation::GetUserContext
     *	Purpose:
     *	    Returns an object representing the authenticated user.
     *	    Returns nothing if there is no authenticated user yet.
     */
    STDMETHOD(GetUserContext)(REF(IUnknown*) pUnknownUser);


    /************************************************************************
    *  IRMAUserProperties COM Interface Methods                ref: rmaauthn.h
    */

    /************************************************************************
     *	Method:
     *	    IRMAUserProperties::GetPrincipalID
     *	Purpose:
     *	    Returns the name of the currently authenticated user.
     */
    STDMETHOD(GetPrincipalID)(REF(IRMABuffer*) pBufferPrincipalID);

    /************************************************************************
     *	Method:
     *	    IRMAUserProperties::GetAuthorityName
     *	Purpose:
     *	    Returns the authorizer of the currently authenticated user.
     *	    (Realm, Domain, etc..)
     */
    STDMETHOD(GetAuthorityName)(REF(IRMABuffer*) pBufferAuthorityName);


    /************************************************************************
    *  IRMAAuthenticationDBAccessResponse COM Interface Methods   ref:  rmadb.h
    */

    /************************************************************************
     *	Method:
     *	    IRMAAuthenticationDBAccessResponse::ExistenceCheckDone
     *	Purpose:
     *	    Called by the Database in response to a test for the existance
     *	    of a specified user.  
     */
    STDMETHOD(ExistenceCheckDone)
    (
	PN_RESULT		ResultStatus,
	IRMABuffer*		pBufferPrincipalID
    );

    /************************************************************************
     *	Method:
     *	    IRMAAuthenticationDBAccessResponse::GetCredentialsDone
     *	Purpose:
     *	    Called by the Database in response to a request for Credentials
     *	    for a specified user.  
     */
    STDMETHOD(GetCredentialsDone)
    (
	PN_RESULT		ResultStatus,
	IRMABuffer*		pBufferPrincipalID,
	IRMABuffer*		pBufferCredentials
    );


    /************************************************************************
    *  IRMAAuthenticationDBManager COM Interface Methods          ref:  rmadb.h
    */

    /************************************************************************
     *	Method:
     *	    IRMAAuthenticationDBManager::AddPrincipal
     *	Purpose:
     *	    Add a user to the database.
     */
    STDMETHOD(AddPrincipal)
    (
	IRMAAuthenticationDBManagerResponse* pAuthenticationDBManagerResponseNew,
	IRMABuffer*		pBufferPrincipalID
    );

    /************************************************************************
     *	Method:
     *	    IRMAAuthenticationDBManager::RemovePrincipal
     *	Purpose:
     *	    Remove a user to the database.
     */
    STDMETHOD(RemovePrincipal)
    (
	IRMAAuthenticationDBManagerResponse* pAuthenticationDBManagerResponseNew,
	IRMABuffer*		pBufferPrincipalID
    );

    /************************************************************************
     *	Method:
     *	    IRMAAuthenticationDBManager::SetCredentials
     *	Purpose:
     *	    Set new credentials for a user to the database.
     */
    STDMETHOD(SetCredentials)
    (
	IRMAAuthenticationDBManagerResponse* pAuthenticationDBManagerResponseNew,
	IRMABuffer*		pBufferPrincipalID, 
	IRMABuffer*		pBufferCredentials
    );


    /************************************************************************
    *  IRMAAuthenticationDBManagerResponse COM Interface Methods  ref:  rmadb.h
    */

    /************************************************************************
     *	Method:
     *	    IRMAAuthenticationDBManagerResponse::AddPrincipalDone
     *	Purpose:
     *	    Called by a database in response to a request to add a user.
     */
    STDMETHOD(AddPrincipalDone)
    (
	PN_RESULT		ResultStatus,
	IRMABuffer*		pBufferPrincipalID
    );

    /************************************************************************
     *	Method:
     *	    IRMAAuthenticationDBManagerResponse::RemovePrincipalDone
     *	Purpose:
     *	    Called by a database in response to a request to remove a user.
     */
    STDMETHOD(RemovePrincipalDone)
    (
	PN_RESULT		ResultStatus,
	IRMABuffer*		pBufferPrincipalID
    );

    /************************************************************************
     *	Method:
     *	    IRMAAuthenticationDBManagerResponse::SetCredentialsDone
     *	Purpose:
     *	    Called by a database in response to a request to set a users
     *	    credentials.
     */
    STDMETHOD(SetCredentialsDone)
    (
	PN_RESULT		ResultStatus,
	IRMABuffer*		pBufferPrincipalID
    );

private:

    /****** Private Class Methods ******************************************/
    ~CBasicAuthenticator();

    PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME      // Avoids GCC compiler warning

    PN_RESULT _DescribeCredentials
    (
	IRMAValues* pValuesChallengeHeaders,
	IRMAValues** ppValuesParameterDescriptions
    );

    PN_RESULT _ChallengeToCredentials
    (
	IRMAValues* pValuesChallengeHeaders, 
	IRMAValues** ppValuesCredentials
    );

    PN_RESULT _CredentialsToStorage
    (
	IRMAValues* pValuesCredentials,
	IRMABuffer** ppBufferStorablePassword
    );

    PN_RESULT _StorageToHeader
    (
	IRMAValues* pValuesCredentials,
	IRMABuffer* pBufferStoredPassword,
	IRMAValues** ppValuesResponseHeaders
    );

    BOOL _GetQuotedValue
    (
	const char*& instr, 
	char* valname, 
	char* valbuf
    );

    PN_RESULT _SendChallengeResponse();

    PN_RESULT FromCharArray(const char* szIn, IRMABuffer** ppBufferOut);
    PN_RESULT CreateValues(IRMAValues** ppValuesNew);

    // Client Members
    IRMARequest*		m_pRequestCurrent;
    IRMAClientAuthResponse*	m_pClientAuthResponseRequester;
    BOOL			m_bFinished;

    // Server Members
    IRMAAuthenticationDBManagerResponse*    m_pAuthenticationDBManagerResponseRequester;
    IRMAAuthenticationDBManager*	m_pAuthenticationDBManagerPlugin;
    IRMAAuthenticationDBAccess*		m_pAuthenticationDBAccessCredentials;
    IRMAServerAuthResponse*		m_pServerAuthResponseRequester;
    IRMARequestContext*			m_pRequestContextCurrent;
    IRMARequest*			m_pRequestResponse;
    IRMABuffer*				m_pBufferDatabaseID;
    IRMABuffer*				m_pBufferPrincipalID;
    IRMABuffer*				m_pBufferPassword;
    IRMABuffer*				m_pBufferRealm;
    BOOL				m_bAuthenticated;

    // Plugin Members
    IRMAValues*			m_pValuesConfig;
    IUnknown*			m_pUnknownContext;
    IRMACommonClassFactory*	m_pCommonClassFactory;

    INT32			m_lRefCount;

    static const char*		zm_pDescription;
    static const char*		zm_pCopyright;
    static const char*		zm_pMoreInfoURL;
};


#endif // !__CBasicAuthenticator__
