/****************************************************************************
 *
 *  $Id: exencaud.h,v 1.2 1999/10/05 19:17:10 ehodge Exp $
 *
 *  Copyright (C) 1999 RealNetworks, Inc.
 *  All rights reserved.
 *
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 *
 *  exencaud.h
 *
 *  ExAudio Encoder for RealMedia Architecture (RMA).
 */ 
#ifndef _EXENCAUD_H_
#define _EXENCAUD_H_

/****************************************************************************
 * Includes
 */
#include "rmaengin.h"  /* for IRMACallback */


/****************************************************************************
 * Defines
 */
/*
NOTE: some of these may not yet be supported in the RealNetworks system.
 Those that are supported and have been tested with this encoder as of 3/1999:
 are: 0, 5, 6, 8, 10, and 11.
    Audio
    Payload	Encoding	Samples per
    Type	Name		Second		Channels
    ---------------------------------------------------------
    0		PCMU		8000		1
    1		1016		8000		1
    2		G721		8000		1
    3		GSM		13000		1
    4		unassigned	8000		1
    5		DVI4		8000		1
    6		DVI4		16000		1
    7		LPC		8000		1
    8		PCMA		8000		1
    9		G722		8000		1
    10		L16		44100		2
    11		L16		44100		1
    12		unassigned
    13		unassigned
    14		MPA		90000		-
    15		G728		8000		1
    16--23	unassigned
*/

#define MAX_RTP_AUDIO_TYPE  15

#define UNASGNED    8000 /* pick a number, any number, for unassigned payload
			  * type's sample rate
			  */
#define FRMENC	    400	 /* pick a reasonable number for frame-based audio
			  * encodings (versus sample-based audio encodings)
			  */
#define FRMENC2	    720

static ULONG32 g_pPayloadSamplesPerSec[MAX_RTP_AUDIO_TYPE+1] = {
	8000,	    8000,	8000,	    13000,
	UNASGNED,   8000,	16000,	    8000,
	8000,	    8000,	44100,	    44100,
	UNASGNED,   UNASGNED,	90000,	    8000 };

static ULONG32 g_pPayloadPacketSize[MAX_RTP_AUDIO_TYPE+1] = {
	320,	    FRMENC,	320,	    455, /*XXXEH- why 455? (pncont/sdpmdsc.cpp)*/
	UNASGNED,   160,	320,	    FRMENC,
	320,	    FRMENC,	441,	    441,
	UNASGNED,   UNASGNED,	FRMENC2,    FRMENC };


/****************************************************************************
 * Forward declarations
 */
_INTERFACE IRMACommonClassFactory;
class ExAudioEncoder;

/****************************************************************************
 *
 *  SendPacketCallback Class
 *
 */
class SendPacketCallback : public IRMACallback
{
    public:
    /****** Public Class Methods ******************************************/
    SendPacketCallback(ExAudioEncoder* obj);


    /************************************************************************
     *  IRMACallback Interface Methods                       ref:  rmaengin.h
     */
    STDMETHOD(Func)                     (THIS);


    /************************************************************************
     *  IUnknown COM Interface Methods                          ref:  pncom.h
     */
    STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
    STDMETHOD_(UINT32, AddRef ) (THIS);
    STDMETHOD_(UINT32, Release) (THIS);

    private:
    /****** Private Class Variables ****************************************/
    INT32                   m_lRefCount;
    ExAudioEncoder*	    m_pObj;

    /****** Private Class Methods ******************************************/
    ~SendPacketCallback();
    
    PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME       // Avoids GCC compiler warning
};

/****************************************************************************
 *
 *  ExAudioEncoder Class
 *
 */
class ExAudioEncoder : public IRMAEncoder,
		       public IRMAErrorSink,
		       public IRMAEncoderCompletion
{
    public:
    /****** Public Class Methods ******************************************/
     ExAudioEncoder(char* address, UINT16 port,
	    char* username, char* password,
	    ULONG32 ulRTPPayloadType,
	    ULONG32 ulNumPacketsToSend);
    ~ExAudioEncoder();
    void Encode(const char* filename);
    void SendAPacket();


    /************************************************************************
     *  IRMAEncoder Interface Methods                        ref:  rmaencod.h
     */
    STDMETHOD(InitEncoderResponseDone) (THIS_ PN_RESULT status);
    STDMETHOD(GetFileHeader)           (THIS);
    STDMETHOD(GetStreamHeader)         (THIS_ UINT16 stream);
    STDMETHOD(StartPackets)            (THIS_ UINT16 stream);
    STDMETHOD(StopPackets)             (THIS_ UINT16 stream);


    /************************************************************************
     *  IRMAErrorSink Interface Methods                      ref:  rmaerror.h
     */
   STDMETHOD(ErrorOccurred)
	(THIS_
	 const UINT8 unSeverity,
	 const UINT32 ulRMACode,
	 const UINT32 ulUserCode,
	 const char*  pUserString,
	 const char*  pMoreInfoURL
	);

    /************************************************************************
     * IRMAEncoderCompletion methods			    ref: rmaencod.h
     */
    STDMETHOD(EncoderDone)		(THIS_ 
					 PN_RESULT result);


    /************************************************************************
     *  IUnknown COM Interface Methods                          ref:  pncom.h
     */
    STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
    STDMETHOD_(UINT32, AddRef ) (THIS);
    STDMETHOD_(UINT32, Release) (THIS);


    private:
    /****** Private Class Variables ****************************************/
    INT32                          m_lRefCount;
    IRMACommonClassFactory*        m_pCommonClassFactory;
    IUnknown*                      m_pContext;
    IRMARequest*                   m_pRequest;
    IRMAEncoderResponse*           m_pEncoderResponse;
    IRMAEncoderResponseCompletion* m_pEncoderResponseCompletion;
    IRMAScheduler*		   m_pScheduler;
    BOOL                           m_EncoderDone;
    BOOL                           m_EncoderInitialized;
    BOOL                           m_bPacketsStarted;
    char*                          m_pAddress;
    UINT16                         m_port;
    char*                          m_pUsername;
    char*                          m_pPassword;
    UINT32                         m_starttime;
    ULONG32			   m_ulRTPPayloadType;
    ULONG32                        m_ulPacketsToSend;
    UINT32                         m_ulPacketsSent;
    UINT32			   m_ulPriorPacketTime;
    RMATimeval			   m_startTime;
    RMATimeval			   m_nextPacketTime;

    /****** Private Class Methods *****************************************/
    STDMETHOD(SetupRequest) (THIS_ const char* filename);
    STDMETHOD(HandleResponse) (THIS);
    void Cleanup();
};

#endif
