/****************************************************************************
 * 
 * $Id: main.cpp,v 1.2 1999/10/05 19:17:10 ehodge Exp $ 
 *
 *  Copyright (C) 1999 RealNetworks, Inc.
 *  All rights reserved.
 *
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 * 
 *  main.cpp
 *
 *  ExAudio encoder class for the SuperEncoder library.
 *
 */


/****************************************************************************
 * Includes
 */
#include <stdio.h>
#include <stdlib.h>

#ifdef __MWERKS__
    #include <console.h>
#endif

#include "pntypes.h"
#include "pncom.h"
#include "rmafiles.h"
#include "rmaencod.h"
#include "rmaerror.h"

#include "exencaud.h"


/****************************************************************************
 *  main
 *
 *  This tiny main program just creates an ExAudioEncoder object and starts
 *  it running.  See exencaud.cpp for the heart of the encoder.
 */
int
main(int argc, char* argv[])
{
    #ifdef __MWERKS__
    	/* Emulate command line on Macintosh: */
	argc = ccommand( &argv );
    #endif

    ExAudioEncoder* pTester;
    if(argc < 4)
    {
	fprintf(stdout, "Usage: %s <host> <encoder port> <username> <password> <stream name> \n"
		"    [RTP Payload type (e.g., 0 for PCMU)] [num packets]\n",
		argv[0]);
    }
    else
    {
	ULONG32 ulRTPPayloadType = 11; /* default to L16_1CH. */
	if(argc >= 7)
	{
	    ulRTPPayloadType = atoi(argv[6]);
	    if(ulRTPPayloadType > MAX_RTP_AUDIO_TYPE) /* in exencaud.h */
	    {
		fprintf(stdout, "%lu is an an invalid audio RTP payload "
			"type; switching to 11 (L16, 1-channel)\n\n",
			ulRTPPayloadType);
		ulRTPPayloadType = 11;
	    }
	    else
	    {
		/* NOTE: some of these may not yet be supported in the
		 * RealNetworks G2 system.  Those that are supported and
		 * have been tested with this encoder as of 3/1999 are
		 * 0, 5, 6, 8, 10, and 11.
		 */
		switch(ulRTPPayloadType)
		{
		    case 0:
		    case 5:
		    case 6:
		    case 8:
		    case 10:
		    case 11:
			break;

		    default:
			fprintf(stdout, "%lu is an untested RTP Payload "
				"type; switching to 11 (L16, 1-channel)\n\n",
				ulRTPPayloadType);
			ulRTPPayloadType = 11; /* L16_1CH */
			break;
		}
	    }
	}
	ULONG32 ulNumPackets = 0;
	if(argc >= 8)
	{
	    ulNumPackets = atoi(argv[7]);
	}
	pTester = new ExAudioEncoder(argv[1], atoi(argv[2]),
		argv[3], argv[4],
		ulRTPPayloadType, ulNumPackets);
	pTester->AddRef();
	pTester->Encode(argv[5]);
	pTester->Release();
    }
    return 0;
}
