/****************************************************************************
 * 
 * $Id: exencode.cpp,v 1.28 1999/10/16 02:04:54 paulm Exp $
 *
 *  Copyright (C) 1997 RealNetworks, Inc.
 *  All rights reserved.
 *
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 * 
 *  exencode.cpp
 *
 *  Example encoder class for the SuperEncoder library.
 *
 */


/****************************************************************************
 * Defines
 */
#define INITGUID


/****************************************************************************
 * Includes
 */
#include <stdio.h>

#if defined (_WINDOWS)
#include <windows.h>
#elif defined (_UNIX)
#include "unix/os.h"
#elif defined (_MACINTOSH)
#include "os.h"
#endif

#include "pntypes.h"
#include "pncom.h"
#include "rmapckts.h"
#include "rmacomm.h"
#include "rmafiles.h"
#include "rmaencod.h"
#include "rmaerror.h"

#include "exencode.h"

#define BITRATE 800
#define PACKETSIZE 100

typedef IUnknown* (PNEXPORT_PTR FPCREATE_CONTEXT) ();
typedef PN_RESULT (PNEXPORT_PTR FPRMSETDLLACCESSPATH) (const char*);

/****************************************************************************
 *  ExampleEncoder::ExampleEncoder                           ref:  exencode.h
 *
 *  Constructor. Address and port are passed in from main().
 */
ExampleEncoder::ExampleEncoder(char* address, UINT16 port,
			       UINT32 ulNumPacketsToSend) :
    m_lRefCount(0),
    m_pContext(NULL),
    m_pRequest(NULL),
    m_pEncoderResponse(NULL),
    m_pEncoderResponseCompletion(NULL),
    m_EncoderDone(FALSE),
    m_EncoderInitialized(FALSE),
    m_bPacketsStarted(FALSE),
    m_address(address),
    m_port(port),
    m_ulPacketsToSend(ulNumPacketsToSend),
    m_ulPacketsSent(0)
{
}


/****************************************************************************
 *  ExampleEncoder::~ExampleEncoder                          ref:  exencode.h
 *
 *  Destructor.
 */
ExampleEncoder::~ExampleEncoder()
{
    Cleanup();
}


/****************************************************************************
 *  ExampleEncoder::Encode                                   ref:  exencode.h
 *
 *  The main loop for this encoder.  main creates an ExampleEncoder, then
 *  calls this.
 */
void
ExampleEncoder::Encode(const char* pFilename)
{
    BOOL done = FALSE;
    UINT32 lasttime = 0;

    /* First let's get ourselves a context, which will give us access to
     * IRMACommonClassFactory in particular, and several other interfaces
     * this example does not use.
     */

#if defined (_WINDOWS) || (_WIN32)
    HINSTANCE hDll = LoadLibrary("encn3260.dll");
#elif defined (_UNIX)
    HINSTANCE hDll = LoadLibrary("encnet.so.6.0");
#elif defined (_MACINTOSH)
    HINSTANCE hDll = LoadLibrary("encnet60.dll");
#else
#error "No LoadLibrary for this platform"
#endif

    if (!hDll)
    {
	fprintf(stdout, "Unable to LoadLibrary\n");
	return;
    }
	
    IUnknown* (PNEXPORT_PTR fpCreateContext) ();
    FPRMSETDLLACCESSPATH fpSetDLLAccessPath;
 
    fpCreateContext =
	(FPCREATE_CONTEXT)GetProcAddress(hDll, "CreateContext");
    fpSetDLLAccessPath = 
	(FPRMSETDLLACCESSPATH)GetProcAddress(hDll, "SetDLLAccessPath");

    if (NULL != fpSetDLLAccessPath)
    {
	// Create a null delimited, double-null terminated string containing
	// the paths to the encnet library (DT_Common) and the sdpplin library
	// (DT_Plugins)...
	char pPaths[256];
	UINT32 ulPathLen = 0;
	char pNextPath[256];
#if defined(_MACINTOSH)
	const char* pCurrentDir = ".:";
#elif defined(_UNIX)
	const char* pCurrentDir = "./";
#else
	const char* pCurrentDir = ".\\";
#endif

	memset(pPaths, 0, 256);

	sprintf(pNextPath, "DT_Common=%s", pCurrentDir);
	memcpy(pPaths, pNextPath, strlen(pNextPath) + 1);
	ulPathLen = strlen(pNextPath) + 1;

	sprintf(pNextPath, "DT_Plugins=%s", pCurrentDir);
	memcpy(pPaths + ulPathLen, pNextPath, strlen(pNextPath) + 1);
	ulPathLen = strlen(pNextPath) + 1;

	fpSetDLLAccessPath((char*)pPaths);
    }

    if (!fpCreateContext)
    {
	fprintf(stdout, "Unable to GetProcAddress\n");
	return;
    }
	
    m_pContext = fpCreateContext();

    /* Now that we got a context object, get a CommonClassFactory object
     */
    if(PNR_OK != m_pContext->QueryInterface(IID_IRMACommonClassFactory,
					  (void**)&m_pCommonClassFactory))
    {
	fprintf(stdout, "No common class factory\n");
	return;
    }

    /*
     * Create and initialize the IRMARequest object
     */
    if(PNR_OK != SetupRequest(pFilename))
    {
	return;
    }

    /* Now we need an EncoderResponse object.  Once we have it, we will
     * use it for all further communication with the library.
     */
    if(PNR_OK != m_pContext->QueryInterface(IID_IRMAEncoderResponse,
	(void**)&m_pEncoderResponse))
    {
	fprintf(stdout, "No encoder response\n");
	return;
    }

    if(PNR_OK != m_pContext->QueryInterface(IID_IRMAEncoderResponseCompletion,
	(void**)&m_pEncoderResponseCompletion))
    {
	fprintf(stdout, "No encoder response completion\n");
	return;
    }

    /* We've created the EncoderResponse object, now give it the info
     * it needs to operate: the server and port to connect to, the filename
     * we are encoding, and a pointer back to this object
     */
    m_pEncoderResponse->InitEncoderResponse(m_address,
					    m_port,
					    m_pRequest,
					    "", // Username
					    "", // Password
					    (IRMAEncoder*)this);
    fprintf(stdout, "Entering mainloop\n");
    /* The encoder will remain in this loop until the EncoderResponse
     * object tells us to stop
     */
    UINT32 ulPPS = (UINT32)((double)BITRATE / (double)PACKETSIZE * 8.0);
    UINT32 ulSleepTime = 1000;
    if (ulPPS > ulSleepTime)
    {
	ulSleepTime = 0;
    }
    else
    {
	ulSleepTime /= ulPPS;
    }
    while(!m_EncoderDone)
    {
	/* If we haven't gotten the InitEncoderResponseDone callback yet,
	 * then don't do anything but Process().
	 */
	if(m_EncoderInitialized)
	{
	    /*
	     * Send one packet per second...Note that we will slowly
	     * drift using a sleep of 1 second
	     */

	    if(m_bPacketsStarted)
	    {
		SendAPacket();
#if defined (_WINDOWS) || defined (WIN32)
		Sleep(ulSleepTime);
#elif defined (_UNIX)
		usleep(ulSleepTime);
#elif defined (_MACINTOSH)
		unsigned long ticks;
		Delay(60, &ticks);
#else
#error "No sleep for this platform"
#endif
	    }
	}
	/* Every user of the superencoder library must call 
	 * IRMAEncoderResponse::Process() regularly.
	 */
	if(PNR_OK != m_pEncoderResponse->Process())
	    m_EncoderDone = TRUE;
    }
}


/****************************************************************************
 *  ExampleEncoder::SendAPacket                              ref:  exencode.h
 *
 *  Send a packet!
 */
void
ExampleEncoder::SendAPacket()
{
    IRMAPacket* pPacket;
    IRMABuffer* pData;

    /* First create an IRMAPacket to contain our data */
    if(PNR_OK != m_pCommonClassFactory->CreateInstance(IID_IRMAPacket,
						     (void**)&pPacket))
    {
	return;
    }

    /* And a buffer for the data itself */
    if(PNR_OK != m_pCommonClassFactory->CreateInstance(IID_IRMABuffer,
						     (void**)&pData))
    {
	return;
    }
    pData->SetSize(PACKETSIZE);

    UINT32 timenow;
    timenow = m_pEncoderResponse->GetTime();
    /* Fill in the raw data with a string containing the time.
     * The example renderer will display this string in it's window.
     */
    sprintf((char*)pData->GetBuffer(), "Time: %ld", timenow - m_starttime);

    /* Set all the info in the IRMAPacket */
    pPacket->Set(pData,
		 timenow - m_starttime,
		 0,
		 RMA_ASM_SWITCH_ON,
		 0);

    fprintf(stdout, "Packet: %s\n", pData->GetBuffer());

    /* Give the packet to the EncoderResponse */
    m_pEncoderResponse->PacketReady(PNR_OK, pPacket);

    /* Now we're done with these objects, release them to free the
     * memory
     */
    pPacket->Release();
    pData->Release();

    /*
     * If we have sent as many packets as the user wanted, then
     * tell the encoder response that we are done.  We do this
     * by calling StreamDone on all streams.
     */
    if(m_ulPacketsToSend)
    {
	m_ulPacketsSent++;
	if(m_ulPacketsSent == m_ulPacketsToSend)
	{
	    m_pEncoderResponse->StreamDone(0);
	    /*
	     * Don't send out any more packets, just call process.
	     */
	    m_EncoderInitialized = FALSE;
	}
    }
}


/****************************************************************************
 *  ExampleEncoder::SetupRequest                             ref:  exencode.h
 *
 *  Sets up the request information for the IRMARequest object
 */
STDMETHODIMP
ExampleEncoder::SetupRequest(const char* pFilename)
{
    PN_RESULT result = PNR_OK;

    if(PNR_OK != m_pCommonClassFactory->CreateInstance(CLSID_IRMARequest,
                                                       (void**)&m_pRequest))
    {
	fprintf(stdout, "Unable to create request\n");
	return PNR_FAIL;
    }

    if (PNR_OK != m_pRequest->SetURL(pFilename))
    {
	fprintf(stdout, "Unable to set URL\n");
	return PNR_FAIL;
    }

    IRMABuffer* pBuffer  = NULL;
    IRMAValues* pHeaders = NULL;

    result = m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,
                                                   (void**)&pBuffer);

    if (PNR_OK != result)
    {
	fprintf(stdout, "Unable to create Buffer\n");
	goto setupDone;
    }

    /*
     * Must include the '\0' in the buffer string
     */

    result = pBuffer->Set((BYTE*)"buzz; buzz=zzub", 16);

    if(PNR_OK != result)
    {
	fprintf(stdout, "Unable to set Buffer\n");
	goto setupDone;
    }

    result = m_pCommonClassFactory->CreateInstance(CLSID_IRMAValues,
                                                   (void**)&pHeaders);

    if(PNR_OK != result)
    {
	fprintf(stdout, "Unable to create Header\n");
	goto setupDone;
    }

    /*
     * Setting RFC822 header...foo:buzz; buzz=zzub
     */

    result = pHeaders->SetPropertyCString("foo", pBuffer);

    if(PNR_OK != result)
    {
	fprintf(stdout, "Unable to set Property\n");
	goto setupDone;
    }

    if(m_pRequest->SetRequestHeaders(pHeaders))
    {
	fprintf(stdout, "Unable to set Headers\n");
    }

setupDone:

    if (pBuffer)
    {
	pBuffer->Release();
    }

    if (pHeaders)
    {
	pHeaders->Release();
    }

    return result;
}


/****************************************************************************
 *  ExampleEncoder::SetupRequest                             ref:  exencode.h
 *
 *  Handle the response information contained in the IRMARequest object
 */
STDMETHODIMP
ExampleEncoder::HandleResponse()
{
    IRMAValues* pHeaders;

    if (PNR_OK == m_pRequest->GetResponseHeaders(pHeaders))
    {
	const char* name;
	IRMABuffer* value;
	PN_RESULT result;

	fprintf(stdout, "\n\nHEADERS...\n");

	result = pHeaders->GetFirstPropertyCString(name, value);

	while (PNR_OK == result)
	{
	    fprintf(stdout, "\t%s: %s\n", name, (char*)value->GetBuffer());
	    value->Release();
	    result = pHeaders->GetNextPropertyCString(name, value);
	}

	fprintf(stdout, "\n");

	pHeaders->Release();
    }

    return PNR_OK;
}

// IRMAEncoderCompletion methods

/****************************************************************************
 *  IRMAEncoderCompletion::EncoderDone			    ref: rmaencod.h
 *
 *  Called by the encoder response when it gets a response from the server
 *  that the connection has been shut down.
 */

STDMETHODIMP
ExampleEncoder::EncoderDone(PN_RESULT result)
{
    m_EncoderDone = TRUE;
    fprintf(stdout, "Got the EncoderDone.\n");
    Cleanup();
    return PNR_OK;
}


// IRMAEncoder Interface Methods

/****************************************************************************
 *  IRMAEncoder::InitEncoderResponseDone                     ref:  rmaencod.h
 *
 *  InitEncoderResponseDone is called when InitEncoderResponse has
 *  completed.  It may return PNR_OK or an error result.  An error
 *  should be considered critical, and the encoder should exit (or
 *  prompt for new parameters and start over or whatever)
 */
STDMETHODIMP
ExampleEncoder::InitEncoderResponseDone(PN_RESULT status)
{
    fprintf(stdout, "InitEncoderResponseDone(%ld)\n", status);
    if(PNR_OK == status)
    {
	m_EncoderInitialized = TRUE;
	m_starttime = m_pEncoderResponse->GetTime();
    }
    else
    {
	/* Failed, might as well exit.
	 */
	m_EncoderDone = TRUE;
    }

    return PNR_OK;
}


/****************************************************************************
 *  IRMAEncoder::GetFileHeader                               ref:  rmaencod.h
 *
 *  GetFileHeader is called to retrieve the header for the entire
 *  presentation.  Most importantly, it should contain the number
 *  of streams present.  This is very similar to the GetFileHeader
 *  method of IRMAFileFormat.
 */
STDMETHODIMP
ExampleEncoder::GetFileHeader()
{
    IRMAValues* pHeader;

    /* Create an IRMAValues object to hold the header */
    if(PNR_OK != m_pCommonClassFactory->CreateInstance(CLSID_IRMAValues,
						     (void**)&pHeader))
    {
	return PNR_FAIL;
    }

    /* The only property in a file header is the number of streams the
     * encoder is encoding.  In this case, always 1.
     */
    pHeader->SetPropertyULONG32("StreamCount", 1);

    /* Give the header to the IRMAEncoderResponse */
    m_pEncoderResponse->FileHeaderReady(PNR_OK, pHeader);

    /* And clean up our reference on the header */
    pHeader->Release();
    return PNR_OK;
}


/****************************************************************************
 *  IRMAEncoder::GetStreamHeader                             ref:  rmaencod.h
 *
 *  GetStreamHeader is called to retrieve a header for each stream
 *  in the presentation.  It is similar to GetStreamHeader in
 *  IRMAFileFormat.
 */
STDMETHODIMP
ExampleEncoder::GetStreamHeader(UINT16 stream)
{
    IRMAValues* pHeader;
    IRMABuffer* pOpaqueData;
    IRMABuffer* pStreamName;
    IRMABuffer* pMimeType;

    /* Create an IRMAValues object to hold the header */
    if(PNR_OK != m_pCommonClassFactory->CreateInstance(CLSID_IRMAValues,
						     (void**)&pHeader))
    {
	return PNR_FAIL;
    }

    /* A buffer for the opaque (type specific) data */
    if(PNR_OK != m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,
						     (void**)&pOpaqueData))
    {
	return PNR_FAIL;
    }

    if(PNR_OK != m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,
						       (void**)&pStreamName))
	return PNR_FAIL;

    if(PNR_OK != m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,
						       (void**)&pMimeType))
	return PNR_FAIL;

    char szMimeType[] = "application/x-pn-examplestream";
    pOpaqueData->Set((BYTE*)"ExampleOpaque", strlen("ExampleOpaque"));
    pStreamName->Set((BYTE*)"Example Stream", strlen("Example Stream") + 1);
    pMimeType->Set((BYTE*)szMimeType, strlen(szMimeType) + 1);

    /* All these properties should be filled in for stream headers. */
    pHeader->SetPropertyBuffer ("OpaqueData", pOpaqueData);
    pHeader->SetPropertyULONG32("StreamNumber",  0);
    pHeader->SetPropertyULONG32("MaxBitRate", BITRATE);
    pHeader->SetPropertyULONG32("AvgBitRate", BITRATE);
    pHeader->SetPropertyULONG32("MaxPacketSize", PACKETSIZE);
    pHeader->SetPropertyULONG32("AvgPacketSize", PACKETSIZE);
    pHeader->SetPropertyULONG32("Duration", 0xffffffff); // Make exrender happy
    pHeader->SetPropertyULONG32("StartTime", 0);
    pHeader->SetPropertyULONG32("Preroll", 2000);
    pHeader->SetPropertyCString("StreamName", pStreamName);
    pHeader->SetPropertyCString("MimeType", pMimeType);

    /*
     * Now set the ASM rule book
     */

    IRMABuffer* pBuffer = 0;
    unsigned char szEventRuleBk[128];
    sprintf((char*)szEventRuleBk, 
	    "priority=9,averagebandwidth=%d,PNMNonKeyFrameRule=T;",
	    BITRATE);
    
    if (PNR_OK == m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,
                                                        (void**)&pBuffer))
    {
	pBuffer->Set(szEventRuleBk, sizeof(szEventRuleBk));
	pHeader->SetPropertyCString("ASMRuleBook", pBuffer);
	pBuffer->Release();
    }

    /* Give the header to the EncoderResponse */
    m_pEncoderResponse->StreamHeaderReady(PNR_OK, pHeader);

    /* And clean up the objects we created */
    pOpaqueData->Release();
    pHeader->Release();
    pStreamName->Release();
    pMimeType->Release();

    return PNR_OK;
}


/****************************************************************************
 *  IRMAEncoder::StartPackets                                ref:  rmaencod.h
 *
 *  StartPackets is called to inform the encoder that it should start
 *  sending packets for a particular stream.
 */
STDMETHODIMP
ExampleEncoder::StartPackets(UINT16 stream)
{
    /*
     * The response headers should be available by now
     */
    HandleResponse();

    m_bPacketsStarted = TRUE;
    return PNR_OK;
}


/****************************************************************************
 *  IRMAEncoder::StopPackets                                 ref:  rmaencod.h
 *
 *  StopPackets is called when the encoder should stop sending packets
 *  for a stream.  This will not be called often, as it is generally
 *  up to the encoder application to determine when a stream is done.
 */
STDMETHODIMP
ExampleEncoder::StopPackets(UINT16 stream)
{
    m_bPacketsStarted = FALSE;
    return PNR_OK;
}


// IRMAErrorSink Interface Methods

/****************************************************************************
 *  IRMAEncoder::ErrorOccurred                               ref:  rmaerror.h
 *
 *  ErrorOccurred is called when an error occurs!
 */
STDMETHODIMP
ExampleEncoder::ErrorOccurred(const UINT8  unSeverity,
			      const UINT32 ulRMACode,
			      const UINT32 ulUserCode,
			      const char*  pUserString,
			      const char*  pMoreInfoURL)
{
    fprintf(stdout, "*** An error occurred:\n*** %s\n*** See %s for details\n",
	    pUserString, pMoreInfoURL);
    Cleanup();
    m_EncoderDone = TRUE;
    
    return PNR_OK;
}


// IUnknown COM Interface Methods
 
/****************************************************************************
 *  IUnknown::QueryInterface                                    ref:  pncom.h
 *
 *  This routine indicates which interfaces this object supports. If a given
 *  interface is supported, the object's reference count is incremented, and
 *  a reference to that interface is returned. Otherwise a NULL object and
 *  error code are returned. This method is called by other objects to
 *  discover the functionality of this object.
 */
STDMETHODIMP
ExampleEncoder::QueryInterface(REFIID riid, void** ppvObj)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
	AddRef();
	*ppvObj = (IUnknown*)(IRMAEncoder*)this;
	return PNR_OK;
    }
    else if(IsEqualIID(riid, IID_IRMAEncoder))
    {
	AddRef();
	*ppvObj = (IRMAEncoder*)this;
	return PNR_OK;
    }
    else if(IsEqualIID(riid, IID_IRMAErrorSink))
    {
	AddRef();
	*ppvObj = (IRMAErrorSink*)this;
	return PNR_OK;
    }
    else if(IsEqualIID(riid, IID_IRMAEncoderCompletion))
    {
	AddRef();
	*ppvObj = (IRMAEncoderCompletion*)this;
	return PNR_OK;
    }
    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}

/****************************************************************************
 *  IUnknown::AddRef                                            ref:  pncom.h
 *
 *  This routine increases the object reference count in a thread safe
 *  manner. The reference count is used to manage the lifetime of an object.
 *  This method must be explicitly called by the user whenever a new
 *  reference to an object is used.
 */
STDMETHODIMP_(ULONG32)
ExampleEncoder::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}

/****************************************************************************
 *  IUnknown::Release                                           ref:  pncom.h
 *
 *  This routine decreases the object reference count in a thread safe
 *  manner, and deletes the object if no more references to it exist. It must
 *  be called explicitly by the user whenever an object is no longer needed.
 */
STDMETHODIMP_(ULONG32)
ExampleEncoder::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
	return m_lRefCount;
    }

    delete this;
    return 0;
}

void
ExampleEncoder::Cleanup()
{
    if (m_pCommonClassFactory)
    {
	m_pCommonClassFactory->Release();
	m_pCommonClassFactory = 0;
    }

    if (m_pContext)
    {
	m_pContext->Release();
	m_pContext = 0;
    }

    if (m_pEncoderResponse)
    {
	/*
	 * Must allow EncoderResponse to free up protocol before calling
	 * Release()
	 */

	if (m_pEncoderResponseCompletion)
	{
	    m_pEncoderResponseCompletion->EncoderResponseDone();
	    m_pEncoderResponseCompletion->Release();
	    m_pEncoderResponseCompletion = 0;
	}

	m_pEncoderResponse->Release();
	m_pEncoderResponse = 0;
    }

    if (m_pRequest)
    {
	m_pRequest->Release();
	m_pRequest = 0;
    }
}

