/****************************************************************************
 * 
 *  $Id: exffplin.h,v 1.19 1999/11/10 05:03:35 jhug Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 * 
 *  exffplin.h
 *
 *  Example File Format plug-in for the RealMedia Architecture (RMA).
 */
#ifndef _EXFFPLIN_H_
#define _EXFFPLIN_H_


/****************************************************************************
 * Constants
 */
#define FILE_HEADER_OFFSET  0
#define INFO_SIZE	    50
#define FILE_HEADER_SIZE    (INFO_SIZE*3)
#define STREAM_HEADER_SIZE  20
#define PACKET_SIZE	    20
#define TIME_PER_PACKET	    1000
#define DATA_OFFSET	    (FILE_HEADER_OFFSET+FILE_HEADER_SIZE+STREAM_HEADER_SIZE)



/****************************************************************************
 *
 *  CMyExampleFileFormat Class
 *
 *  Example implementation of a basic file format.
 */
class CExampleFileFormat :	public IRMAPlugin, 
				public IRMAFileFormatObject, 
				public IRMAFileResponse,
				public IRMAPendingStatus,
				public IRMABackChannel,
				public IRMAASMSource,
				public IRMAFileViewSource,
				public IRMAFileStatResponse
{
    public:
    /****** Public Class Methods ******************************************/
    CExampleFileFormat()
	    : m_lRefCount(0)
	    , m_pContext(NULL)
	    , m_pFileObject(NULL)
	    , m_pFFResponse(NULL)
	    , m_pCommonClassFactory(NULL)
	    , m_bHeaderSent(FALSE)
	    , m_ulCurrentTime(0)
	    , m_state(Ready)
	    , m_pRequest(NULL)
	    , m_bSubStream0Active(TRUE)
	    , m_bSubStream1Active(TRUE)
	    , m_bSubStream2Active(TRUE)
	    , m_pViewSrcResp(NULL)
	    , m_ulFileSize(NULL)
	    , m_ulModTime(NULL)
	    {
	    };
    void SetResponseHeaders    (THIS);
    STDMETHOD(FileObjectReady) (THIS_ PN_RESULT status, IRMAFileObject* pFileObject);


    /************************************************************************
     *  IRMAFileFormatObject Interface Methods               ref:  rmaformt.h
     */
    STDMETHOD(GetFileFormatInfo)
	(THIS_
	 REF(const char**) pFileMimeTypes,
	 REF(const char**) pFileExtensions,
	 REF(const char**) pFileOpenNames
	);

    STDMETHOD(InitFileFormat)
	(THIS_
	 IRMARequest*        pRequest,
	 IRMAFormatResponse* pFormatResponse,
	 IRMAFileObject*     pFileObject
	);

    STDMETHOD(GetFileHeader  ) (THIS);
    STDMETHOD(GetStreamHeader) (THIS_ UINT16 streamNo);
    STDMETHOD(GetPacket      ) (THIS_ UINT16 streamNo);
    STDMETHOD(Seek           ) (THIS_ UINT32 requestedTime);
    STDMETHOD(Close          ) (THIS);

    /************************************************************************
     *  IRMAFileViewSource Interface Methods                 ref:  rmaformt.h
     */
    STDMETHOD(InitViewSource)   (
	IRMAFileObject*		    /*IN*/ pFileObject,
	IRMAFileViewSourceResponse* /*IN*/ pResp,
	SOURCE_TYPE		    /*IN*/ sourceType,
	IRMAValues*		    /*IN*/ pOptions);
    STDMETHOD(GetSource) 	(THIS);
    
    /* 
     * see IRMAFileFormatObject::Close 
     * STDMETHOD(Close)		(THIS);
     */
    
    /************************************************************************
    *  IRMAStatResponse Interface Methods                   ref:  rmafiles.h
    */
    STDMETHOD(StatDone) (THIS_ 
			 PN_RESULT,
			 UINT32 ulSize,
			 UINT32 ulCreationTime,
			 UINT32 ulAccessTime,
			 UINT32 ulModificationTime,
			 UINT32 ulFlags);

    /************************************************************************
    *  IRMAFileResponse Interface Methods                   ref:  rmafiles.h
    */
    STDMETHOD(InitDone ) (THIS_ PN_RESULT status);
    STDMETHOD(SeekDone ) (THIS_ PN_RESULT status);
    STDMETHOD(ReadDone ) (THIS_ PN_RESULT status, IRMABuffer* pBuffer);
    STDMETHOD(WriteDone) (THIS_ PN_RESULT status);
    STDMETHOD(CloseDone) (THIS_ PN_RESULT status);


    /************************************************************************
     *  IRMAPendingStatus Interface Methods                  ref:  rmapends.h
     */
    STDMETHOD(GetStatus)
	(THIS_
	 REF(UINT16)      uStatusCode, 
	 REF(IRMABuffer*) pStatusDesc, 
	 REF(UINT16)      ulPercentDone
	);


    /************************************************************************
     *  IRMABackChannel Interface Methods                      ref:  rmaasm.h
     */
    STDMETHOD(PacketReady) (THIS_ IRMAPacket* pPacket);


    /************************************************************************
     *  IRMAASMSource Interface Methods                        ref:  rmaasm.h
     */
    STDMETHOD (Subscribe)   (THIS_ UINT16 uStreamNumber, UINT16 uRuleNumber);
    STDMETHOD (Unsubscribe) (THIS_ UINT16 uStreamNumber, UINT16	uRuleNumber);


    /************************************************************************
     *  IRMAPlugin Interface Methods                         ref:  rmaplugn.h
     */
    STDMETHOD(GetPluginInfo)
	(THIS_ 
	 REF(BOOL)        bLoadMultiple,
	 REF(const char*) pDescription,
	 REF(const char*) pCopyright,
	 REF(const char*) pMoreInfoURL,
	 REF(UINT32)      versionNumber
	);
    STDMETHOD(InitPlugin) (THIS_ IUnknown* pRMACore);
    
    /************************************************************************
    *  IUnknown COM Interface Methods                          ref:  pncom.h
    */
    STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
    STDMETHOD_(UINT32, AddRef ) (THIS);
    STDMETHOD_(UINT32, Release) (THIS);


    private:
    /****** Private Class Variables ****************************************/
    INT32			m_lRefCount;
    IUnknown*			m_pContext;
    IRMAFileObject*		m_pFileObject;
    IRMAFormatResponse*		m_pFFResponse;
    IRMACommonClassFactory*	m_pCommonClassFactory;
    BOOL			m_bHeaderSent;
    UINT32			m_ulCurrentTime;
    IRMARequest*		m_pRequest;
    BOOL			m_bSubStream0Active;
    BOOL			m_bSubStream1Active;
    BOOL			m_bSubStream2Active;

    /***** View Source members *********************************************/
    IRMAFileViewSourceResponse* m_pViewSrcResp;
    UINT32			m_ulFileSize;
    UINT32			m_ulModTime;

    typedef enum
    {
	Ready, InitPending,
	GetFileHeaderSeekPending, GetFileHeaderReadPending, 
	GetStreamHeaderSeekPending, GetStreamHeaderReadPending, 
	GetPacketSeekPending, GetPacketReadPending,
	SeekSeekPending, ViewSourceInitPending, ViewSourceStatPending,
	ViewSourceReadPending, ViewSourceReady
    }
    ExampleState;

    ExampleState		m_state;

    /****** Private Static Class Variables *********************************/
    static const char*		zm_pDescription;
    static const char*		zm_pCopyright;
    static const char*		zm_pMoreInfoURL;
    static const char*		zm_pFileMimeTypes[];
    static const char*		zm_pFileExtensions[];
    static const char*		zm_pFileOpenNames[];

    /****** Private Class Methods ******************************************/
    ~CExampleFileFormat();
    
    PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME      // Avoids GCC compiler warning
};


#endif // ndef _EXFFPLIN_H_

