#define INITGUID

// system
#include <stdio.h>
#if defined(_WINDOWS)
#include <windows.h>
#include <conio.h>
#include <sys\types.h>
#include <sys\stat.h>
#elif defined(_UNIX)
#include "unix/os.h"
#elif defined(_MACINTOSH)
#include "os.h"
#include <console.h>
#endif

// include
#include "pntypes.h"
#include "pncom.h"
#include "pnresult.h"
#include "pxresult.h"
#include "rmalvpix.h"

// exlivess
#include "exlivess.h"

typedef PN_RESULT (PNEXPORT_PTR FPRMSETDLLACCESSPATH) (const char*);

// Constructor
CExampleLiveRealPixSlideshow::CExampleLiveRealPixSlideshow()
{
    m_lRefCount        = 0;
    m_ulState          = kStateConstructed;
    m_ulNumImages      = 0;
    m_ppSlideInfo      = NULL;
    m_ulCurIndex       = 0;
    m_ulNextIndex      = 1;
    m_ulInputNextIndex = 1;
}

// Destructor
CExampleLiveRealPixSlideshow::~CExampleLiveRealPixSlideshow()
{
    DeleteAllSlides();
}

PN_RESULT CExampleLiveRealPixSlideshow::Encode(const char *pszServer,
                                               UINT32      ulPort,
                                               const char *pszFilename,
                                               const char *pszFileList,
                                               UINT32      ulWidth,
                                               UINT32      ulHeight,
                                               UINT32      ulBitrate,
                                               const char *pszUsername,
                                               const char *pszPassword)
{
    PN_RESULT              retVal              = PNR_OK;
    FILE*                  fp                  = NULL;
    UINT32                 i                   = 0;
    FILE*                  imagefp             = NULL;
    HINSTANCE              hRPLiveDLL          = 0;
    IRMALiveRealPix*       pLiveRealPix        = NULL;
    IRMALiveRealPixResend* pLiveRealPixResend  = NULL;
    UINT32                 ulInputNextIndex    = 0;
    BOOL                   bDoAdvance          = TRUE;
    BOOL                   bFirstImageSend     = TRUE;
    FPRMCREATELIVEREALPIX  fpCreateLiveRealPix = NULL;
    FPRMSETDLLACCESSPATH   fpSetDLLAccessPath  = NULL;
    char                   szInput[256];
    UINT32                 ulNumChars          = 0;
    char*                  ppszMimeType[]      = {"image/vndr.rn-realpix.jpeg",
                                                  "image/vnd.rn-realpix.gif",
                                                  "image/vnd.rn-realpix.png"};
#if defined (_WINDOWS) || (_WIN32)
    const char*            pszDLLName          = "pxli3260.dll";
#elif defined (_UNIX)
    const char*            pszDLLName          = "pxlive.so.6.0";
#elif defined (_MACINTOSH)
    const char*            pszDLLName          = "pxlive60.dll";
#else
#error "No LoadLibrary for this platform"
#endif

    if (m_ulState != kStateConstructed)
    {
        retVal = PNR_UNEXPECTED;
        goto cleanup;
    }

    // Load the image list file
    //
    // The image list file has a very simple format:
    // First line contains integer with number of
    // images (let's say that number is N). Then there
    // should follow N lines with one image file name
    // (full path) per line. For example, it might look like:
    //
    // 4
    // d:\files\images\slides\slide0.gif
    // d:\files\images\slides\slide1.gif
    // d:\files\images\slides\slide2.gif
    // d:\files\images\slides\slide3.gif
    // 
    // Open the file
    fp = fopen(pszFileList, "r");
    if (!fp)
    {
        fprintf(stderr, "Cannot find %s.\n", pszFileList);
        retVal = PNR_FAIL;
        goto cleanup;
    }
    // Read the first line
    char szLine[256];
    fgets(szLine, 256, fp);
    m_ulNumImages = (UINT32) atoi(szLine);
    if (m_ulNumImages < 2)
    {
        fprintf(stderr, "Must be at least two images in the file.\n");
        retVal = PNR_FAIL;
        goto cleanup;
    }
    // Allocate array of SlideInfo pointers
    m_ppSlideInfo = new SlideInfo* [m_ulNumImages];
    if (!m_ppSlideInfo)
    {
        fprintf(stderr, "Cannot allocate %lu SlideInfo pointers.\n", m_ulNumImages);
        retVal = PNR_FAIL;
        goto cleanup;
    }
    // Null out the pointers
    memset((void*) m_ppSlideInfo, 0, m_ulNumImages * sizeof(SlideInfo*));
    // Loop through, reading the image file name and
    // then reading the image
    for (i = 0; i < m_ulNumImages; i++)
    {
        // Allocate a SlideInfo struct
        m_ppSlideInfo[i] = new SlideInfo;
        if (!m_ppSlideInfo[i])
        {
            fprintf(stderr, "Cannot allocate SlideInfo struct %lu\n", i);
            retVal = PNR_FAIL;
            goto cleanup;
        }
        // Zero out the struct
        memset((void*) m_ppSlideInfo[i], 0, sizeof(SlideInfo));
        // Get the file name
        fgets(szLine, 256, fp);
        // Get the string length
        INT32 lLen = strlen(szLine);
        // Null out the newline that fgets() leaves
        szLine[lLen - 1] = '\0';
        // Allocate a string
        m_ppSlideInfo[i]->m_pszFileName = new char [lLen];
        if (!m_ppSlideInfo[i]->m_pszFileName)
        {
            fprintf(stderr, "Cannot allocate string %lu: %s.\n", i, szLine);
            retVal = PNR_FAIL;
            goto cleanup;
        }
        // Copy the string
        strcpy(m_ppSlideInfo[i]->m_pszFileName, szLine);
    }
    fclose(fp);
    fp = NULL;

    // Load the Live RealPix helper DLL
    hRPLiveDLL = LoadLibrary(pszDLLName);
    if (!hRPLiveDLL)
    {
        fprintf(stderr, "Could not load %s. Make sure %s is in ");
        fprintf(stderr, "your path (or in the same directory).\n");
        goto cleanup;
    }

    // Get the proc address of CreateLiveRealPix - the lone entry point
    fpCreateLiveRealPix = (FPRMCREATELIVEREALPIX) GetProcAddress(hRPLiveDLL, "CreateLiveRealPix");
    fpSetDLLAccessPath  = (FPRMSETDLLACCESSPATH)  GetProcAddress(hRPLiveDLL, "SetDLLAccessPath");

    if(fpSetDLLAccessPath)
    {
        // Create a null delimited, double-null terminated string containing
        // the paths to the encnet library (DT_Common) and the sdpplin library
        // (DT_Plugins)...
        char        pPaths[256];
        UINT32      ulPathLen = 0;
        char        pNextPath[256];
#if defined(_MACINTOSH)
        const char* pCurrentDir = ".:";
#elif defined(_UNIX)
        const char* pCurrentDir = "./";
#else
        const char* pCurrentDir = ".\\";
#endif

        memset(pPaths, 0, 256);

        sprintf(pNextPath, "DT_Common=%s", pCurrentDir);
        memcpy(pPaths, pNextPath, strlen(pNextPath) + 1);
        ulPathLen = strlen(pNextPath) + 1;

        sprintf(pNextPath, "DT_Plugins=%s", pCurrentDir);
        memcpy(pPaths + ulPathLen, pNextPath, strlen(pNextPath) + 1);
        ulPathLen = strlen(pNextPath) + 1;

        fpSetDLLAccessPath((char*) pPaths);
    }

    if (!fpCreateLiveRealPix)
    {
        fprintf(stderr, "Could not get proc address of CreateLiveRealPix().\n");
        goto cleanup;
    }

    // Now we can create an instance of the IRMALiveRealPix interface
    retVal = fpCreateLiveRealPix(&pLiveRealPix);
    if (retVal != PNR_OK)
    {
        fprintf(stderr, "Failed to create instance of IRMALiveRealPix.\n");
        goto cleanup;
    }

    // QI for IRMALiveRealPixResend
    retVal = pLiveRealPix->QueryInterface(IID_IRMALiveRealPixResend,
                                          (void**) &pLiveRealPixResend);
    if (retVal != PNR_OK)
    {
        fprintf(stderr, "Could not QI for IRMALiveRealPixResend.\n");
        goto cleanup;
    }
 
    //
    // Call IRMALiveRealPixResend::InitResend() interface to tell the encoder
    // that we want to be able to resend images. You must call this interface
    // BEFORE calling IRMALiveRealPix::StartEncoder() if you want to use
    // the resend capability.
    //
    retVal = pLiveRealPixResend->InitResend(NULL);
    if (retVal != PNR_OK)
    {
        fprintf(stderr, "IRMALiveRealPixResend::InitResend() failed: 0x%08x\n", retVal);
        goto shutdown;
    }

    // Set up the structure which will initialize the library
    PixInitInfo cInitInfo;
    cInitInfo.m_ulStructLength      = sizeof(PixInitInfo);
    cInitInfo.m_pServerAddress      = (char *) pszServer;
    cInitInfo.m_ulServerPort        = ulPort;
    cInitInfo.m_pUsername           = (char *) pszUsername;
    cInitInfo.m_pPassword           = (char *) pszPassword;
    cInitInfo.m_pFilename           = (char *) pszFilename;
    cInitInfo.m_pTitle              = NULL;
    cInitInfo.m_pAuthor             = NULL;
    cInitInfo.m_pCopyright          = NULL;
    cInitInfo.m_ulBitrate           = ulBitrate;
    cInitInfo.m_ulMaxFps            = 0;
    cInitInfo.m_ulDisplayWidth      = ulWidth;
    cInitInfo.m_ulDisplayHeight     = ulHeight;
    cInitInfo.m_bPreserveAspect     = TRUE;
    cInitInfo.m_pDefaultURL         = NULL;
    cInitInfo.m_ulNumImageCodecs    = 3;
    cInitInfo.m_ppImageCodec        = ppszMimeType;
    cInitInfo.m_ulNumEffectPackages = 0;
    cInitInfo.m_ppEffectPackage     = NULL;

    // Set the new state
    m_ulState = kStateInitPending;

    // Now we can initialize the library
    retVal = pLiveRealPix->StartEncoder(&cInitInfo, this);
    if (retVal != PNR_OK)
    {
        fprintf(stderr, "IRMALiveRealPix::StartEncoder() failed: 0x%08x\n", retVal);
        goto cleanup;
    }

    // Wait for initialization
    while (m_ulState == kStateInitPending)
    {
        retVal = pLiveRealPix->Process();
        if (retVal != PNR_OK)
        {
            fprintf(stderr, "IRMALiveRealPix::Process() failed while waiting "
                    "for initialization: 0x%08x\n", retVal);
            goto cleanup;
        }
        MilliSleep(kSleepDuringInit);
    }

    // Check if something went wrong
    if (m_ulState != kStateInitialized)
    {
        fprintf(stderr, "IRMALiveRealPix::StartEncoder() failed: 0x%08x\n", retVal);
        goto cleanup;
    }

    // Initialize the "current image" index
    // and "next image" index
    m_ulCurIndex  = 0;
    m_ulNextIndex = 1;
    // Init the "current" image
    retVal = LoadAndInitImage(m_ppSlideInfo[m_ulCurIndex]->m_pszFileName,
                              pLiveRealPix,
                              m_ppSlideInfo[m_ulCurIndex]->m_ulImageHandle);
    if (retVal != PNR_OK)
    {
        goto shutdown;
    }
    // Init the "next" image
    retVal = LoadAndInitImage(m_ppSlideInfo[m_ulNextIndex]->m_pszFileName,
                              pLiveRealPix,
                              m_ppSlideInfo[m_ulNextIndex]->m_ulImageHandle);
    if (retVal != PNR_OK)
    {
        goto shutdown;
    }

    // Print out a message
    fprintf(stdout, "Currently on slide %lu of %lu (%s).\n", m_ulCurIndex,
            m_ulNumImages, m_ppSlideInfo[m_ulCurIndex]->m_pszFileName);

    // Now we enter our main processing loop. We send the current image
    // and the next image over and over again. Whenever
    // we detect a keyboard hit, we send an effect saying to display
    // the next image
    //
    // Set the first send flag
    bFirstImageSend = TRUE;
    // Clear the input key buffer
    szInput[0] = '\0';
    ulNumChars = 0;
    // Set the state
    m_ulState = kStateSendCurrentImage;
    while(1)
    {
        switch (m_ulState)
        {
            case kStateSendCurrentImage:
                {
                    // Clear the slide sent flag
                    m_ppSlideInfo[m_ulCurIndex]->m_bImageSent = FALSE;
                    // Set the new state
                    m_ulState = kStateSendCurrentImagePending;
                    // Send the image
                    retVal = pLiveRealPix->SendImage(m_ppSlideInfo[m_ulCurIndex]->m_ulImageHandle);
                    if (retVal != PNR_OK)
                    {
                        fprintf(stderr, "IRMALiveRealPix::SendImage() returned error: 0x%08x\n", retVal);
                        goto shutdown;
                    }
                }
                break;

            case kStateSendCurrentImagePending:
                {
                    // Sleep for a bit
                    MilliSleep(kSleepDuringSend);
                    // Check to see if the image has been sent
                    if (m_ppSlideInfo[m_ulCurIndex]->m_bImageSent)
                    {
                        m_ulState = kStateSendCurrentEffect;
                    }
                }
                break;

            case kStateSendCurrentEffect:
                {
                    // Zero out an effect struct
                    PixEffectInfo cEffectInfo;
                    memset((void*) &cEffectInfo, 0, sizeof(PixEffectInfo));
                    // Fill in the necessary values
                    cEffectInfo.m_ulStructLength      = sizeof(PixEffectInfo);
                    cEffectInfo.m_ucEffectType        = EFFECT_FADEIN;
                    cEffectInfo.m_ulStart             = 0;
                    cEffectInfo.m_ulDuration          = (bFirstImageSend ? 1000 : 0);
                    cEffectInfo.m_ulTarget            = m_ppSlideInfo[m_ulCurIndex]->m_ulImageHandle;
                    cEffectInfo.m_bAspectFlag         = TRUE;
                    cEffectInfo.m_bDiscardImage       = FALSE; // make sure we don't discard the image
                    cEffectInfo.m_bDisplayImmediately = TRUE;
                    // Clear the first image send flag
                    bFirstImageSend = FALSE;
                    // Clear the effect sent flag
                    m_ppSlideInfo[m_ulCurIndex]->m_bEffectSent = FALSE;
                    // Set the new state
                    m_ulState = kStateSendCurrentEffectPending;
                    // Send the effect
                    retVal = pLiveRealPix->SendEffect(&cEffectInfo);
                    if (retVal != PNR_OK)
                    {
                        fprintf(stderr, "IRMALiveRealPix::SendEffect() returned error: 0x%08x\n", retVal);
                        goto shutdown;
                    }
                    m_ppSlideInfo[m_ulCurIndex]->m_ulEffectHandle = cEffectInfo.m_ulHandle;
                }
                break;

            case kStateSendCurrentEffectPending:
                {
                    // Sleep a bit
                    MilliSleep(kSleepDuringSend);
                    // Check if we've been sent
                    if (m_ppSlideInfo[m_ulCurIndex]->m_bEffectSent)
                    {
                        m_ulState = kStateSendNextImage;
                    }
                }
                break;

            case kStateSendNextImage:
                {
                    // Clear the image sent flag
                    m_ppSlideInfo[m_ulNextIndex]->m_bImageSent = FALSE;
                    // Set the new state
                    m_ulState = kStateSendNextImagePending;
                    // Send the image
                    retVal = pLiveRealPix->SendImage(m_ppSlideInfo[m_ulNextIndex]->m_ulImageHandle);
                    if (retVal != PNR_OK)
                    {
                        fprintf(stderr, "IRMALiveRealPix::SendImage() returned error: 0x%08x\n", retVal);
                        goto shutdown;
                    }
                }
                break;

            case kStateSendNextImagePending:
                {
                    // Sleep for a bit
                    MilliSleep(kSleepDuringSend);
                    // Check if we've sent the image
                    if (m_ppSlideInfo[m_ulNextIndex]->m_bImageSent)
                    {
                        m_ulState = kStateSendCurrentImage;
                    }
                }
                break;

            case kStateSendLastCurrentEffect:
                {
                    // Zero out an effect struct
                    PixEffectInfo cEffectInfo;
                    memset((void*) &cEffectInfo, 0, sizeof(PixEffectInfo));
                    // Fill in the necessary values
                    cEffectInfo.m_ulStructLength      = sizeof(PixEffectInfo);
                    cEffectInfo.m_ucEffectType        = EFFECT_FADEIN;
                    cEffectInfo.m_ulStart             = 0;
                    cEffectInfo.m_ulDuration          = 0;
                    cEffectInfo.m_ulTarget            = m_ppSlideInfo[m_ulCurIndex]->m_ulImageHandle;
                    cEffectInfo.m_bAspectFlag         = TRUE;
                    cEffectInfo.m_bDiscardImage       = TRUE; // this time we DO discard the image
                    cEffectInfo.m_bDisplayImmediately = TRUE;
                    // Clear the effect sent flag
                    m_ppSlideInfo[m_ulCurIndex]->m_bEffectSent = FALSE;
                    // Set the new state
                    m_ulState = kStateSendLastCurrentEffectPending;
                    // Send the effect
                    retVal = pLiveRealPix->SendEffect(&cEffectInfo);
                    if (retVal != PNR_OK)
                    {
                        fprintf(stderr, "IRMALiveRealPix::SendEffect() returned error: 0x%08x\n", retVal);
                        goto shutdown;
                    }
                    m_ppSlideInfo[m_ulCurIndex]->m_ulEffectHandle = cEffectInfo.m_ulHandle;
                }
                break;

            case kStateSendLastCurrentEffectPending:
                {
                    // Sleep a bit
                    MilliSleep(kSleepDuringSend);
                    // Check if we've been sent
                    if (m_ppSlideInfo[m_ulCurIndex]->m_bEffectSent)
                    {
                        // Now, we have to decide whether we can send an immediate next image
                        // effect. If we guessed right and the next image we have been sending
                        // is indeed the next image, then we can. If not, then we will simply
                        // go back to sending the current image
                        if (m_ulInputNextIndex == m_ulNextIndex)
                        {
                            // We guessed right - the image we were pre-sending was indeed chosen
                            // to be the next image.
                            m_ulState = kStateSendNextEffect;
                        }
                        else
                        {
                            // We guessed wrong - the image we were pre-sending was NOT the next image
                            // so all we can do is update the indices and start sending the current image.
                            //
                            // Update the indices
                            m_ulCurIndex  = m_ulInputNextIndex;
                            m_ulNextIndex = m_ulCurIndex + 1;
                            if (m_ulNextIndex >= m_ulNumImages)
                            {
                                m_ulNextIndex = 0;
                            }
                            // Set the first image send flag
                            bFirstImageSend = TRUE;
                            // Set the state to start sending the current image
                            m_ulState = kStateSendCurrentImage;
                        }
                    }
                }
                break;

            case kStateSendNextEffect:
                {
                    // Zero out an effect struct
                    PixEffectInfo cEffectInfo;
                    memset((void*) &cEffectInfo, 0, sizeof(PixEffectInfo));
                    // Fill in the necessary values
                    cEffectInfo.m_ulStructLength      = sizeof(PixEffectInfo);
                    cEffectInfo.m_ucEffectType        = EFFECT_FADEIN;
                    cEffectInfo.m_ulStart             = 0;
                    cEffectInfo.m_ulDuration          = 1000;
                    cEffectInfo.m_ulTarget            = m_ppSlideInfo[m_ulNextIndex]->m_ulImageHandle;
                    cEffectInfo.m_bAspectFlag         = TRUE;
                    cEffectInfo.m_bDiscardImage       = FALSE; // make sure we don't discard the image
                    cEffectInfo.m_bDisplayImmediately = TRUE;
                    // Clear the effect sent flag
                    m_ppSlideInfo[m_ulNextIndex]->m_bEffectSent = FALSE;
                    // Set the new state
                    m_ulState = kStateSendNextEffectPending;
                    // Send the effect
                    retVal = pLiveRealPix->SendEffect(&cEffectInfo);
                    if (retVal != PNR_OK)
                    {
                        fprintf(stderr, "IRMALiveRealPix::SendEffect() returned error: 0x%08x\n", retVal);
                        goto shutdown;
                    }
                    m_ppSlideInfo[m_ulNextIndex]->m_ulEffectHandle = cEffectInfo.m_ulHandle;
                }
                break;

            case kStateSendNextEffectPending:
                {
                    // Sleep for a bit
                    MilliSleep(kSleepDuringSend);
                    // Check if we've sent the effect
                    if (m_ppSlideInfo[m_ulNextIndex]->m_bEffectSent)
                    {
                        // Now we can update the indices
                        m_ulCurIndex  = m_ulInputNextIndex;
                        m_ulNextIndex = m_ulCurIndex + 1;
                        if (m_ulNextIndex >= m_ulNumImages)
                        {
                            m_ulNextIndex = 0;
                        }
                        // Keep the first image send flag cleared
                        bFirstImageSend = FALSE;
                        // So now we go back to sending the current image
                        m_ulState = kStateSendCurrentImage;
                    }
                }
                break;
        }

        if (m_ulState == kStateError)
        {
            fprintf(stderr, "In Error state - exiting loop.\n");
            break;
        }

        char cKey;
        if (WasKeyboardHit(cKey))
        {
            if (cKey == 'q')
            {
                // Someone hit q for quit
                fprintf(stdout, "Quit key pressed.\n");
                break;
            }
            else if (cKey == 13 || cKey == 'n' || cKey == 'p')
            {
                m_ulInputNextIndex = 0;
                bDoAdvance         = TRUE;
                if (cKey == 13)
                {
                    // Get the index which was input
                    m_ulInputNextIndex = (UINT32) atoi(szInput);
                    // Clear the input buffer
                    ulNumChars       = 0;
                    szInput[0]       = '\0';
                    // Determine if this is a valid index
                    if (m_ulInputNextIndex >= m_ulNumImages ||
                        m_ulInputNextIndex == m_ulCurIndex)
                    {
                        bDoAdvance = FALSE;
                    }
                    else
                    {
                        fprintf(stdout, "Changing to slide %lu of %lu (%s).\n",
                                m_ulInputNextIndex, m_ulNumImages,
                                m_ppSlideInfo[m_ulInputNextIndex]->m_pszFileName);
                    }
                }
                else if (cKey == 'n')
                {
                    m_ulInputNextIndex = m_ulCurIndex + 1;
                    if (m_ulInputNextIndex >= m_ulNumImages)
                    {
                        bDoAdvance = FALSE;
                    }
                    else
                    {
                        fprintf(stdout, "Changing to NEXT slide %lu of %lu (%s).\n",
                                m_ulInputNextIndex, m_ulNumImages,
                                m_ppSlideInfo[m_ulInputNextIndex]->m_pszFileName);
                    }
                }
                else if (cKey == 'p')
                {
                    if (m_ulCurIndex > 0)
                    {
                        m_ulInputNextIndex = m_ulCurIndex - 1;
                        fprintf(stdout, "Changing to PREVIOUS slide %lu of %lu (%s).\n",
                                m_ulInputNextIndex, m_ulNumImages,
                                m_ppSlideInfo[m_ulInputNextIndex]->m_pszFileName);
                    }
                    else
                    {
                        bDoAdvance = FALSE;
                    }
                }

                // Do we have a valid next index
                if (bDoAdvance)
                {
                    // Set the state so we know we're changing images when we
                    // get the ImageSent() and EffectSent() callbacks
                    m_ulState = kStateChangingSlides;
                    // Dump all packets we've sent with SendImage() or SendEffect()
                    pLiveRealPixResend->DumpAllQueuedPackets();
                    // Release the "current" image
                    pLiveRealPixResend->ReleaseImage(m_ppSlideInfo[m_ulCurIndex]->m_ulImageHandle);
                    // If the actual next image (m_ulInputNextIndex) is NOT same as
                    // the pre-sent next image (m_ulNextIndex), then we need to release
                    // the pre-sent next image AND init the actual next image
                    if (m_ulInputNextIndex != m_ulNextIndex)
                    {
                        // We can release the pre-sent next index cause we
                        // guessed wrong.
                        pLiveRealPixResend->ReleaseImage(m_ppSlideInfo[m_ulNextIndex]->m_ulImageHandle);
                        // Init an image for the new current image
                        retVal = LoadAndInitImage(m_ppSlideInfo[m_ulInputNextIndex]->m_pszFileName,
                                                  pLiveRealPix,
                                                  m_ppSlideInfo[m_ulInputNextIndex]->m_ulImageHandle);
                        if (retVal != PNR_OK)
                        {
                            goto shutdown;
                        }
                    }
                    // Now we make a new guess as to what our pre-sent image will
                    // be. We always choose the next image (m_ulInputNextIndex + 1).
                    UINT32 ulNewNext = m_ulInputNextIndex + 1;
                    if (ulNewNext >= m_ulNumImages)
                    {
                        ulNewNext = 0;
                    }
                    // Init the new pre-sent image
                    retVal = LoadAndInitImage(m_ppSlideInfo[ulNewNext]->m_pszFileName,
                                              pLiveRealPix,
                                              m_ppSlideInfo[ulNewNext]->m_ulImageHandle);
                    if (retVal != PNR_OK)
                    {
                        goto shutdown;
                    }
                    // Set the new state
                    m_ulState = kStateSendLastCurrentEffect;
                }
            }
            else if (cKey >= '0' && cKey <= '9')
            {
                szInput[ulNumChars]     = cKey;
                szInput[ulNumChars + 1] = '\0';
                ulNumChars++;
            }
        }

        // Call process to give some CPU to the DLL
        retVal = pLiveRealPix->Process();
        if (retVal != PNR_OK)
        {
            fprintf(stderr, "IRMALivePix::Process() failed: 0x%08x\n", retVal);
            break;
        }
    }

shutdown:
    // Release all the images
    for (i = 0; i < m_ulNumImages; i++)
    {
        pLiveRealPixResend->ReleaseImage(m_ppSlideInfo[i]->m_ulImageHandle);
    }
    // Set the state
    m_ulState = kStateStopPending;
    // Tell the encoder to shutdown
    fprintf(stdout, "Stopping encoder.\n");
    pLiveRealPix->StopEncoder();
    while (m_ulState == kStateStopPending)
    {
        pLiveRealPix->Process();
        MilliSleep(kSleepDuringShutdown);
    }
    fprintf(stdout, "Encoder stopped.\n");
cleanup:
    if (fp)
    {
        fclose(fp);
    }
    DeleteAllSlides();
    PN_RELEASE(pLiveRealPix);
    PN_RELEASE(pLiveRealPixResend);
    if (hRPLiveDLL)
    {
        FreeLibrary(hRPLiveDLL);
    }

    return retVal;
}

STDMETHODIMP CExampleLiveRealPixSlideshow::QueryInterface(REFIID riid, void **ppvObj)
{
    PN_RESULT retVal = PNR_OK;

    if (IsEqualIID(riid, IID_IUnknown))
    {
        AddRef();
        *ppvObj = (IUnknown*) (IRMALiveRealPixResponse*) this;
    }
    else if(IsEqualIID(riid, IID_IRMALiveRealPixResponse))
    {
        AddRef();
        *ppvObj = (IRMALiveRealPixResponse*) this;
    }
    else
    {
        *ppvObj = NULL;
        retVal  = PNR_NOINTERFACE;
    }

    return retVal;
}

STDMETHODIMP_(ULONG32) CExampleLiveRealPixSlideshow::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}

STDMETHODIMP_(ULONG32) CExampleLiveRealPixSlideshow::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
        return m_lRefCount;
    }

    delete this;

    return 0;
}

STDMETHODIMP CExampleLiveRealPixSlideshow::EncoderStarted(PN_RESULT status, const char *pszHeaderString)
{
    // Check state
    if (m_ulState != kStateInitPending)
    {
        return PNR_UNEXPECTED;
    }

    if (status == PNR_OK)
    {
        fprintf(stdout, "%s\n", pszHeaderString);
        m_ulState = kStateInitialized;
    }
    else
    {
        fprintf(stderr, "Error occurred in initialization.\n", pszHeaderString);
        m_ulState = kStateError;
    }

    return PNR_OK;
}

STDMETHODIMP CExampleLiveRealPixSlideshow::ImageSent(PN_RESULT status, UINT32 ulImageHandle)
{
    PN_RESULT retVal = PNR_OK;

    if (status == PNR_OK ||
        status == PNR_SENDIMAGE_ABORTED)
    {
        // Run through our list and look for the handle
        for (UINT32 i = 0; i < m_ulNumImages; i++)
        {
            if (m_ppSlideInfo[i]->m_ulImageHandle == ulImageHandle)
            {
                // We found it - mark it as sent
                m_ppSlideInfo[i]->m_bImageSent = TRUE;
                break;
            }
        }
    }
    else
    {
        // Print out a message
        fprintf(stderr, "ImageSent(0x%08x, %lu): send failed.\n", status, ulImageHandle);
        // Set the state
        m_ulState = kStateError;
    }

    return retVal;
}

STDMETHODIMP CExampleLiveRealPixSlideshow::EffectSent(PN_RESULT status, UINT32 ulEffectHandle)
{
    PN_RESULT retVal = PNR_OK;

    if (status == PNR_OK ||
        status == PNR_SENDEFFECT_ABORTED)
    {
        // Run through our list and look for the handle
        for (UINT32 i = 0; i < m_ulNumImages; i++)
        {
            if (m_ppSlideInfo[i]->m_ulEffectHandle == ulEffectHandle)
            {
                // We found it - mark it as sent
                m_ppSlideInfo[i]->m_bEffectSent = TRUE;
                break;
            }
        }
    }
    else
    {
        // Print out a message
        fprintf(stdout, "EffectSent(0x%08x, %lu): send failed.\n", status, ulEffectHandle);
        // Set the state
        m_ulState = kStateError;
    }

    return retVal;
}

STDMETHODIMP CExampleLiveRealPixSlideshow::EncoderStopped(PN_RESULT status)
{
    PN_RESULT retVal = PNR_OK;

    if (m_ulState == kStateStopPending) // app-initiated shutdown
    {
        m_ulState = kStateStopped;
    }
    else if (m_ulState == kStateEncoderError)
    {
        m_ulState = kStateError;
    }
    else
    {
        m_ulState = kStateError;
        retVal    = PNR_UNEXPECTED;
    }

    return retVal;
}

STDMETHODIMP CExampleLiveRealPixSlideshow::ErrorOccurred(const UINT8   unSeverity,
                                                         const ULONG32 ulRMACode,
                                                         const ULONG32 ulUserCode,
                                                         const char*   pszUserString,
                                                         const char*   pszMoreInfoURL)
{
    fprintf(stderr, "Error Occurred:\n");
    fprintf(stderr, "    RMA Code:     0x%08x\n", ulRMACode);
    fprintf(stderr, "    User Code:    0x%08x\n", ulUserCode);
    fprintf(stderr, "    User String:  %s\n",     pszUserString);
    fprintf(stderr, "    pMoreInfoURL: %s\n",     pszMoreInfoURL);

    m_ulState = kStateEncoderError;

    return PNR_OK;
}

void CExampleLiveRealPixSlideshow::DeleteAllSlides()
{
    for (UINT32 i = 0; i < m_ulNumImages; i++)
    {
        PN_VECTOR_DELETE(m_ppSlideInfo[i]->m_pszFileName);
        PN_DELETE(m_ppSlideInfo[i]);
    }
    PN_VECTOR_DELETE(m_ppSlideInfo);
    m_ulNumImages = 0;
}

void CExampleLiveRealPixSlideshow::MilliSleep(UINT32 ulTime)
{
#if defined(_WINDOWS)
    Sleep(ulTime);
#elif defined(_MACINTOSH)
    EventRecord evt;
    WaitNextEvent(0, &evt, kSleepDuringActiveSend, 0);
#elif defined(_LINUX) || defined(_FREEBSD) || defined (_SCO) || defined(_SOLARIS)
    usleep(ulTime * 1000);
#endif
}

BOOL CExampleLiveRealPixSlideshow::WasKeyboardHit(char& rcKey)
{
    BOOL bRet = FALSE;
#if defined(_WINDOWS)
    if (_kbhit())
    {
        rcKey = (char) _getch();
        bRet  = TRUE;
    }
#elif defined(_MACINTOSH)
    if (kbhit())
    {
        rcKey = (char) getch();
        bRet  = TRUE;
    }
#elif defined(_UNIX)
#endif
    return bRet;
}

PN_RESULT CExampleLiveRealPixSlideshow::LoadAndInitImage(char*            pszFileName,
                                                         IRMALiveRealPix* pLiveRealPix,
                                                         REF(UINT32)      rulHandle)
{
    PN_RESULT retVal = PNR_FAIL;

    if (pszFileName && pLiveRealPix)
    {
        // Set the default
        rulHandle = 0;
        // Open the file
        FILE* fp = fopen(pszFileName, "rb");
        if (fp)
        {
            // Get the file size
            fseek(fp, 0, SEEK_END);
            UINT32 ulSize = (UINT32) ftell(fp);
            fseek(fp, 0, SEEK_SET);
            // Allocate a buffer
            BYTE* pBuf = new BYTE [ulSize];
            if (pBuf)
            {
                // Read the file
                UINT32 ulBytesRead = (UINT32) fread(pBuf, 1, ulSize, fp);
                if (ulBytesRead == ulSize)
                {
                    // We successfully read the file, now we need to call
                    // InitImage() on it.
                    //
                    // First, we set up the PixImageInfo struct
                    PixImageInfo cImageInfo;
                    cImageInfo.m_ulStructLength    = sizeof(PixImageInfo);
                    cImageInfo.m_pImageBuffer      = pBuf;
                    cImageInfo.m_ulImageBufferSize = ulSize;
                    cImageInfo.m_pImageCodec       = NULL; // If set to NULL, pxlive will auto-detect
                    // Now we call InitImage()
                    retVal = pLiveRealPix->InitImage(&cImageInfo);
                    if (SUCCEEDED(retVal))
                    {
                        // Copy the handle to the out parameter
                        rulHandle = cImageInfo.m_ulHandle;
                    }
                    else
                    {
                        fprintf(stderr, "IRMALiveRealPix::InitImage() failed on image file %s: 0x%08x\n",
                                pszFileName, retVal);
                    }
                    
                }
                else
                {
                    // Print out a message
                    fprintf(stderr, "Did not read enough bytes of %s (read %lu, asked for %lu).\n",
                            pszFileName, ulBytesRead, ulSize);
                }
            }
            else
            {
                fprintf(stderr, "Cannot allocate %lu bytes for %s.\n", ulSize, pszFileName);
            }
            PN_VECTOR_DELETE(pBuf);
            // Close the file
            fclose(fp);
        }
        else
        {
            fprintf(stderr, "Cannot open image file %s.\n", pszFileName);
        }
    }

    return retVal;
}
