#define INITGUID

// system
#include <stdio.h>
#if defined(_WINDOWS)

#include <windows.h>
#include <conio.h>
#include <sys\types.h>
#include <sys\stat.h>

#elif defined(_UNIX)

#include "unix/os.h"

#elif defined(_MACINTOSH)

#include "os.h"
#include <console.h>
#define _kbhit kbhit
#define _getch getch

#endif

// include
#include "pntypes.h"
#include "pncom.h"
#include "pnresult.h"
#include "pxresult.h"
#include "rmalvpix.h"

// exlivpix
#include "exlivpix.h"

typedef PN_RESULT (PNEXPORT_PTR FPRMSETDLLACCESSPATH) (const char*);

// Constructor
CExampleLiveRealPixEncode::CExampleLiveRealPixEncode()
{
    m_lRefCount      = 0;
    m_ulState        = kStateConstructed;
    m_ulLastFileSize = 0;
    m_ulLastCheckSum = 0;
}

// Destructor
CExampleLiveRealPixEncode::~CExampleLiveRealPixEncode()
{
}

//
// This is the main function which loops until either the user 
// hits 'q' on the keyboard or the server tells the encoder to
// stop, which could happen if an error occurred.
//
PN_RESULT CExampleLiveRealPixEncode::Encode(const char *pszServer,
                                            UINT32      ulPort,
                                            const char *pszFilename,
                                            const char *pszImageName,
                                            UINT32      ulWidth,
                                            UINT32      ulHeight,
                                            UINT32      ulBitrate,
                                            const char *pszUsername,
                                            const char *pszPassword)
{
    // Check state
    if (m_ulState != kStateConstructed)
    {
        return PNR_UNEXPECTED;
    }

    // Verify the image file exists
    FILE *fp = fopen(pszImageName, "rb");
    if (!fp)
    {
        fprintf(stdout, "Cannot find %s.\n", pszImageName);
        return PNR_FAIL;
    }
    fclose(fp);

    // Load the Live RealPix helper DLL
#if defined (_WINDOWS) || (_WIN32)
    const char* pszDLLName = "pxli3260.dll";
#elif defined (_UNIX)
    const char* pszDLLName = "pxlive.so.6.0";
#elif defined (_MACINTOSH)
    const char* pszDLLName = "pxlive60.dll";
#else
#error "No LoadLibrary for this platform"
#endif
    HINSTANCE hRPLiveDLL = LoadLibrary(pszDLLName);
    if (!hRPLiveDLL)
    {
        fprintf(stdout, "Could not load %s. Make sure %s is in ");
        fprintf(stdout, "your path (or in the same directory).\n");
        return PNR_FAIL;
    }

    // Get the proc address of CreateLiveRealPix - the lone entry point
    FPRMCREATELIVEREALPIX fpCreateLiveRealPix = 
                  (FPRMCREATELIVEREALPIX) GetProcAddress(hRPLiveDLL, "CreateLiveRealPix");
    FPRMSETDLLACCESSPATH fpSetDLLAccessPath = 
                  (FPRMSETDLLACCESSPATH) GetProcAddress(hRPLiveDLL, "SetDLLAccessPath");

    if(fpSetDLLAccessPath)
    {
        // Create a null delimited, double-null terminated string containing
        // the paths to the encnet library (DT_Common) and the sdpplin library
        // (DT_Plugins)...
        char        pPaths[256];
        UINT32      ulPathLen = 0;
        char        pNextPath[256];
#if defined(_MACINTOSH)
        const char* pCurrentDir = ".:";
#elif defined(_UNIX)
        const char* pCurrentDir = "./";
#else
        const char* pCurrentDir = ".\\";
#endif

        memset(pPaths, 0, 256);

        sprintf(pNextPath, "DT_Common=%s", pCurrentDir);
        memcpy(pPaths, pNextPath, strlen(pNextPath) + 1);
        ulPathLen = strlen(pNextPath) + 1;

        sprintf(pNextPath, "DT_Plugins=%s", pCurrentDir);
        memcpy(pPaths + ulPathLen, pNextPath, strlen(pNextPath) + 1);
        ulPathLen = strlen(pNextPath) + 1;

        fpSetDLLAccessPath((char*) pPaths);
    }

    if (!fpCreateLiveRealPix)
    {
        FreeLibrary(hRPLiveDLL);
        fprintf(stdout, "Could not get proc address of CreateLiveRealPix().\n");
        return PNR_FAIL;
    }

    // Now we can create an instance of the IRMALiveRealPix interface
    IRMALiveRealPix *pLiveRealPix = NULL;
    PN_RESULT        retVal       = fpCreateLiveRealPix(&pLiveRealPix);
    if (retVal != PNR_OK)
    {
        FreeLibrary(hRPLiveDLL);
        fprintf(stdout, "Failed to create instance of IRMALiveRealPix.\n");
        return PNR_FAIL;
    }
 
    // Set up the structure which will initialize the library
    PixInitInfo cInitInfo;
    cInitInfo.m_ulStructLength      = sizeof(PixInitInfo);
    cInitInfo.m_pServerAddress      = (char *) pszServer;
    cInitInfo.m_ulServerPort        = ulPort;
    cInitInfo.m_pUsername           = (char *) pszUsername;
    cInitInfo.m_pPassword           = (char *) pszPassword;
    cInitInfo.m_pFilename           = (char *) pszFilename;
    cInitInfo.m_pTitle              = NULL;
    cInitInfo.m_pAuthor             = NULL;
    cInitInfo.m_pCopyright          = NULL;
    cInitInfo.m_ulBitrate           = ulBitrate;
    cInitInfo.m_ulMaxFps            = 0;
    cInitInfo.m_ulDisplayWidth      = ulWidth;
    cInitInfo.m_ulDisplayHeight     = ulHeight;
    cInitInfo.m_bPreserveAspect     = TRUE;
    cInitInfo.m_pDefaultURL         = NULL;
    cInitInfo.m_ulNumImageCodecs    = 2;
    char*       ppszMimeType[]      = {"image/vndr.rn-realpix.jpeg",
                                       "image/vnd.rn-realpix.gif"};
    cInitInfo.m_ppImageCodec        = ppszMimeType;
    cInitInfo.m_ulNumEffectPackages = 0;
    cInitInfo.m_ppEffectPackage     = NULL;

    // Set the new state
    m_ulState = kStateInitPending;

    // Now we can initialize the library
    retVal = pLiveRealPix->StartEncoder(&cInitInfo, this);
    if (retVal != PNR_OK)
    {
        PN_RELEASE(pLiveRealPix);
        FreeLibrary(hRPLiveDLL);
        fprintf(stdout, "IRMALiveRealPix::StartEncoder() failed: %d\n", retVal);
        return PNR_FAIL;
    }

    // Wait for initialization
    while (m_ulState == kStateInitPending)
    {
        retVal = pLiveRealPix->Process();
        if (retVal != PNR_OK)
        {
            PN_RELEASE(pLiveRealPix);
            FreeLibrary(hRPLiveDLL);
            fprintf(stdout, "IRMALiveRealPix::Process() failed while waiting for initialization: %d\n", retVal);
            return 0;
        }
    }

    // Check if something went wrong
    if (m_ulState != kStateInitialized)
    {
        PN_RELEASE(pLiveRealPix);
        FreeLibrary(hRPLiveDLL);
        fprintf(stdout, "IRMALiveRealPix::StartEncoder() failed: %d\n", retVal);
        return PNR_FAIL;
    }

    // Now we enter a loop, looking for when the image watch file
    // has been modified. Whenever we detect it's been modified,
    // we load it in, send the image, send an effect, and go
    // back to watching again.
    UINT32 ulCurHandle  = 0;
    BYTE   ucEffectType = 1;
    UINT32 ulSize       = 0;
    m_ulState           = kStateIdle;
    while(m_ulState != kStateStopped)
    {
        switch (m_ulState)
        {
            case kStateIdle:
                {
#if defined(_WINDOWS)
                    Sleep(kMillisecBetweenStats);
#elif defined(_MACINTOSH)
                    EventRecord evt;
                    WaitNextEvent(0, &evt, kMillisecBetweenStats, 0);
#elif defined(_LINUX) || defined(_FREEBSD) || defined (_SCO) || defined(_SOLARIS)
                    usleep(kMillisecBetweenStats * 1000);
#endif
                    m_ulState = kStateCheckFile;
                }
                break;
            case kStateCheckFile:
                {
                    // Check whether the file has been modified
                    UINT32 ulCheckSum = 0;
                    UINT32 ulFileSize = 0;
                    retVal            = GetCheckSumAndFileSize(pszImageName, ulCheckSum, ulFileSize);
                    if (retVal == PNR_OK)
                    {
                        if (ulCheckSum != m_ulLastCheckSum ||
                            ulFileSize != m_ulLastFileSize)
                        {
                            m_ulState        = kStateImageChanged;
                            m_ulLastCheckSum = ulCheckSum;
                            m_ulLastFileSize = ulFileSize;
                            fprintf(stdout, "\n%s changed at %lu.\n", pszImageName, pLiveRealPix->GetTime());
                        }
                        else
                        {
                            m_ulState = kStateIdle;
                        }
                    }
                    else
                    {
                        fprintf(stdout, "\nCould not open %s.\n", pszImageName);
                        m_ulState = kStateError;
                    }
                }
                break;

            case kStateImageChanged:
                {
                    // Allocate a buffer the size of the file
                    BYTE *pBuffer = new BYTE [m_ulLastFileSize];
                    if (!pBuffer)
                    {
                        fprintf(stdout, "Could not allocate buffer for file.\n");
                        m_ulState = kStateError;
                        break;
                    }

                    // Read the file into the buffer
                    FILE *fp = fopen(pszImageName, "rb");
                    if (!fp)
                    {
                        fprintf(stdout, "Warning: Could not open file %s. Will try again.\n", pszImageName);
                        delete [] pBuffer;
                        m_ulState = kStateIdle;
                        break;
                    }
                    fread(pBuffer, 1, m_ulLastFileSize, fp);
                    fclose(fp);

                    // Set up the image info struct
                    PixImageInfo cImageInfo;
                    cImageInfo.m_ulStructLength    = sizeof(PixImageInfo);
                    cImageInfo.m_pImageBuffer      = pBuffer;
                    cImageInfo.m_ulImageBufferSize = m_ulLastFileSize;
                    cImageInfo.m_pImageCodec       = NULL; // If set to NULL, pxlive will auto-detect

                    // Initialize the image for sending
                    retVal = pLiveRealPix->InitImage(&cImageInfo);
                    if (retVal != PNR_OK)
                    {
                        fprintf(stdout, "IRMALivePix::InitImage() returned error: 0x%08x\n", retVal);
                        m_ulState = kStateError;
                        break;
                    }

                    // Now we can delete the buffer
                    delete [] pBuffer;

                    // Save the handle
                    ulCurHandle = cImageInfo.m_ulHandle;

                    // Set the new state
                    m_ulState = kStateImageSendPending;

                    // Send the image
                    retVal = pLiveRealPix->SendImage(ulCurHandle);
                    if (retVal != PNR_OK)
                    {
                        fprintf(stdout, "IRMALivePix::SendImage() returned error: %d\n", retVal);
                        m_ulState = kStateError;
                        break;
                    }
                }
                break;

            case kStateImageSendPending:
                {
#if defined(_WINDOWS)
                    Sleep(kSleepDuringActiveSend);
#elif defined(_MACINTOSH)
                    EventRecord evt;
                    WaitNextEvent(0, &evt, kSleepDuringActiveSend, 0);
#elif defined(_LINUX) || defined(_FREEBSD) || defined (_SCO) || defined(_SOLARIS)
                    usleep(kSleepDuringActiveSend * 1000);
#endif
                }
                break;

            case kStateImageSent:
                {
                    // Fill in the effect info struct
                    PixEffectInfo cEffectInfo;
                    cEffectInfo.m_ulStructLength      = sizeof(PixEffectInfo);
                    cEffectInfo.m_ucEffectType        = ucEffectType;
                    cEffectInfo.m_ulStart             = 0;
                    cEffectInfo.m_ulDuration          = 1000;
                    cEffectInfo.m_ulTarget            = ulCurHandle;
                    cEffectInfo.m_ulSrcX              = 0;
                    cEffectInfo.m_ulSrcY              = 0;
                    cEffectInfo.m_ulSrcW              = 0;
                    cEffectInfo.m_ulSrcH              = 0;
                    cEffectInfo.m_ulDstX              = 0;
                    cEffectInfo.m_ulDstY              = 0;
                    cEffectInfo.m_ulDstW              = 0;
                    cEffectInfo.m_ulDstH              = 0;
                    cEffectInfo.m_ulMaxFps            = 0;
                    cEffectInfo.m_bAspectFlag         = TRUE;
                    cEffectInfo.m_ucRed               = (BYTE) 0;
                    cEffectInfo.m_ucGreen             = (BYTE) 0;
                    cEffectInfo.m_ucBlue              = (BYTE) 0;
                    cEffectInfo.m_ucWipeDirection     = WIPE_DIRECTION_LEFT;
                    cEffectInfo.m_ucWipeType          = WIPE_TYPE_NORMAL;
                    cEffectInfo.m_pURL                = NULL;
                    cEffectInfo.m_pExtPackage         = NULL;
                    cEffectInfo.m_pExtName            = NULL;
                    cEffectInfo.m_pExtData            = NULL;
                    cEffectInfo.m_pExtFile            = NULL;
                    cEffectInfo.m_bDiscardImage       = TRUE;
                    cEffectInfo.m_bDisplayImmediately = TRUE;

                    // Send the effect
                    retVal = pLiveRealPix->SendEffect(&cEffectInfo);
                    if (retVal != PNR_OK)
                    {
                        fprintf(stdout, "IRMALivePix::SendEffect() returned error: 0x%08x\n", retVal);
                        m_ulState = kStateError;
                        break;
                    }

                    if (ucEffectType == 1)
                    {
                        fprintf(stdout, "FadeIn ");
                    }
                    else if (ucEffectType == 4)
                    {
                        fprintf(stdout, "Wipe ");
                    }
                    fprintf(stdout, "effect sent scheduled for approximately %lu\n", cEffectInfo.m_ulStart);

                    // Cycle the effect type
                    if (ucEffectType == 1)
                    {
                        ucEffectType = 4;
                    }
                    else
                    {
                        ucEffectType = 1;
                    }

                    // Set the new handle
                    ulCurHandle = cEffectInfo.m_ulHandle;

                    // Set the new state
                    m_ulState = kStateEffectSendPending;
                }
                break;

            case kStateEffectSendPending:
                {
#if defined(_WINDOWS)
                    Sleep(kSleepDuringActiveSend);
#elif defined(_MACINTOSH)
                    EventRecord evt;
                    WaitNextEvent(0, &evt, kSleepDuringActiveSend, 0);
#elif defined(_LINUX) || defined(_FREEBSD) || defined (_SCO) || defined(_SOLARIS)
                    usleep(kSleepDuringActiveSend * 1000);
#endif
                }
                break;

            case kStateEffectSent:
                m_ulState = kStateCheckFile;
                break;

            case kStateError:
                pLiveRealPix->StopEncoder();
                m_ulState = kStateStopPending;
                break;

            case kStateStopPending:
                break;
        }

        // Check for keyboard hit (Win32 and Mac)
#if defined(_WINDOWS) || defined(_MACINTOSH)
        if (_kbhit())
        {
            int ch = _getch();
            fprintf(stdout, "Keyboard hit detected: %c\n", ch);
            if (ch == 'q' && m_ulState != kStateStopPending)
            {
                fprintf(stdout, "Calling IRMALiveRealPix::StopEncoder()\n");
                pLiveRealPix->StopEncoder();
                m_ulState = kStateStopPending;
            }
        }
#endif

        // Call process to give some CPU to the DLL
        retVal = pLiveRealPix->Process();
        if (retVal != PNR_OK)
        {
            fprintf(stdout, "IRMALivePix::Process() failed: %d\n", retVal);
            break;
        }
    }

    PN_RELEASE(pLiveRealPix);
    FreeLibrary(hRPLiveDLL);

    return PNR_OK;
}

STDMETHODIMP CExampleLiveRealPixEncode::QueryInterface(REFIID riid, void **ppvObj)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
        AddRef();
        *ppvObj = (IUnknown *) (IRMALiveRealPixResponse *) this;
        return PNR_OK;
    }
    else if(IsEqualIID(riid, IID_IRMALiveRealPixResponse))
    {
        AddRef();
        *ppvObj = (IRMALiveRealPixResponse *) this;
        return PNR_OK;
    }

    *ppvObj = NULL;

    return PNR_NOINTERFACE;
}

STDMETHODIMP_(ULONG32) CExampleLiveRealPixEncode::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}

STDMETHODIMP_(ULONG32) CExampleLiveRealPixEncode::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
        return m_lRefCount;
    }

    delete this;

    return 0;
}

STDMETHODIMP CExampleLiveRealPixEncode::EncoderStarted(PN_RESULT status, const char *pszHeaderString)
{
    // Check state
    if (m_ulState != kStateInitPending)
    {
        return PNR_UNEXPECTED;
    }

    if (status == PNR_OK)
    {
        fprintf(stdout, "%s\n", pszHeaderString);
        m_ulState = kStateInitialized;
    }
    else
    {
        fprintf(stdout, "Error occurred in initialization.\n", pszHeaderString);
        m_ulState = kStateError;
    }

    return PNR_OK;
}

STDMETHODIMP CExampleLiveRealPixEncode::ImageSent(PN_RESULT status, UINT32 ulImageHandle)
{
    // Check state
    if (m_ulState != kStateImageSendPending)
    {
        return PNR_UNEXPECTED;
    }

    if (status == PNR_OK)
    {
        fprintf(stdout, "Image sent successfully.\n");
        m_ulState = kStateImageSent;
    }
    else
    {
        fprintf(stdout, "Error occurred sending image.\n");
        m_ulState = kStateError;
    }

    return PNR_OK;
}

STDMETHODIMP CExampleLiveRealPixEncode::EffectSent(PN_RESULT status, UINT32 ulEffectHandle)
{
    // Check state
    if (m_ulState != kStateEffectSendPending)
    {
        return PNR_UNEXPECTED;
    }

    if (status == PNR_OK)
    {
        fprintf(stdout, "Effect sent successfully.\n");
        m_ulState = kStateEffectSent;
    }
    else
    {
        fprintf(stdout, "Error occurred sending effect.\n");
        m_ulState = kStateError;
    }

    return PNR_OK;
}

STDMETHODIMP CExampleLiveRealPixEncode::EncoderStopped(PN_RESULT status)
{
    if (m_ulState != kStateStopPending && // app-initiated shutdown
        m_ulState != kStateEncoderError)  // encoder-initiated shutdown
    {
        return PNR_UNEXPECTED;
    }

    if (status == PNR_OK)
    {
        fprintf(stdout, "Encoder stopped successfully.\n");
    }
    else
    {
        fprintf(stdout, "Error stopping encoder.\n");
    }

    m_ulState = kStateStopped;

    return PNR_OK;
}

STDMETHODIMP CExampleLiveRealPixEncode::ErrorOccurred(const UINT8   unSeverity,
                                                  const ULONG32 ulRMACode,
                                                  const ULONG32 ulUserCode,
                                                  const char   *pszUserString,
                                                  const char   *pszMoreInfoURL)
{
    fprintf(stderr, "Error Occurred:\n");
    fprintf(stderr, "    RMA Code:     0x%08x\n", ulRMACode);
    fprintf(stderr, "    User Code:    0x%08x\n", ulUserCode);
    fprintf(stderr, "    User String:  %s\n",     pszUserString);
    fprintf(stderr, "    pMoreInfoURL: %s\n",     pszMoreInfoURL);

    m_ulState = kStateEncoderError;

    return PNR_OK;
}

PN_RESULT CExampleLiveRealPixEncode::GetCheckSumAndFileSize(const char* pszFileName,
                                                            UINT32&     rulCheckSum,
                                                            UINT32&     rulFileSize)
{
    PN_RESULT retVal = PNR_OK;

    FILE *fp = fopen(pszFileName, "rb");
    if (fp)
    {
        BYTE   ucReadBuf[2048];
        UINT32 ulSizeSum  = 0;
        UINT32 ulCheckSum = 0;
        while (!feof(fp))
        {
            UINT32 ulBytesRead = (UINT32) fread(ucReadBuf, 1, 2048, fp);
            if (ulBytesRead)
            {
                ulSizeSum += ulBytesRead;
                for (UINT32 i = 0; i < ulBytesRead; i++)
                {
                    ulCheckSum += ucReadBuf[i];
                }
            }
        }
        fclose(fp);

        // Set out parameters
        rulCheckSum = ulCheckSum;
        rulFileSize = ulSizeSum;
    }
    else
    {
        retVal = PNR_FAIL;
    }

    return retVal;
}

