/****************************************************************************
 * 
 *  $Id: exlivobj.cpp,v 1.24 1999/04/02 22:43:10 glennh Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 * 
 *  exlivobj.cpp
 *
 *  An example renderer which shows the use of a broadcast plug-in.
 */
 

/****************************************************************************
 * Constants
 */

// Forward declare some flags which shouldn't be here!
#define PN_RELIABLE_FLAG                        0x0001
#define PN_KEYFRAME_FLAG                        0x0002
#define PN_LASTFRAME_FLAG                       0x0004

// This size is totally arbitrary
#define EXAMPLE_PACKET_SIZE 256
// As is this time per packet (in milliseconds)
#define EXAMPLE_TIME_PER_PACKET 100


/****************************************************************************
 * Includes
 */
#include <string.h>
#include <stdio.h>

#include "pntypes.h"
#include "pncom.h"
#include "rmapckts.h"
#include "rmaformt.h"
#include "rmaengin.h"
#include "rmacomm.h"

#include "exlivpln.h"
#include "exlivobj.h"
#include "exlivpln.ver"


/****************************************************************************
 *  ExampleBroadcastObject static variables                  ref:  exlivobj.h
 *
 *  These variables are passed to the RMA core to provide information about
 *  this plug-in. They are required to be static in order to remain valid
 *  for the lifetime of the plug-in.
 */
const IID   ExampleBroadcastObject::zm_myIID           = IID_IRMABroadcastFormatObject;
const char* ExampleBroadcastObject::zm_pDescription    = "Example Broadcast Plugin";
const char* ExampleBroadcastObject::zm_pCopyright      = "(c) 1995,1996,1997 RealNetworks, All rights reserved.";
const char* ExampleBroadcastObject::zm_pMoreInfoURL    = "http://www.real.com";


/****************************************************************************
 *  ExampleBroadcastObject::ExampleBroadcastObject           ref:  exlivobj.h
 *
 *  Constructor
 */
ExampleBroadcastObject::ExampleBroadcastObject() :
    m_lRefCount(0),
    m_pURL(NULL),
    m_pFFResponse(NULL),
    m_bPacketsStarted(FALSE),
    m_pCommonClassFactory(NULL),
    m_pScheduler(NULL),
    m_sequence(0)
{
    if(g_example_broadcast_format)
    {
	m_pContext = g_example_broadcast_format->m_pContext;
	m_pContext->AddRef();
    }
    else
    {
	// This should mean we're the first instantiation of an
	// ExampleBroadcastObject since this plugin was loaded.
	// We expect that InitPlugin will be called before anything else.
	m_pContext = NULL;
    }
}


/****************************************************************************
 *  IRMAPlugin::GetPluginInfo                                ref:  rmaplugn.h
 *
 *  This routine returns descriptive information about the plug-in, most
 *  of which is used in the About box of the user interface. It is called
 *  when the RMA core application is launched.
 */
STDMETHODIMP
ExampleBroadcastObject::GetPluginInfo
(
    REF(BOOL)	    /*OUT*/ bLoadMultiple,
    REF(const char*)/*OUT*/ pDescription,
    REF(const char*)/*OUT*/ pCopyright,
    REF(const char*)/*OUT*/ pMoreInfoURL,
    REF(UINT32)     /*OUT*/ ulVersionNumber
)
{
    bLoadMultiple = FALSE;   // Must be FALSE for broadcast plugins

    pDescription    = zm_pDescription;
    pCopyright	    = zm_pCopyright;
    pMoreInfoURL    = zm_pMoreInfoURL;
    ulVersionNumber = TARVER_ULONG32_VERSION;

    return PNR_OK;
}


/****************************************************************************
 *  IRMAPlugin::InitPlugin                                   ref:  rmaplugn.h
 *
 *  This routine performs initialization steps such as determining whether
 *  required interfaces are available. It is called when the RMA core 
 *  application is launched, and whenever an URL associated with this
 *  plug-in is opened.
 */
STDMETHODIMP
ExampleBroadcastObject::InitPlugin(IUnknown* pContext)
{
    // We only need this to be called once to get the context to use for all
    // future instances.
    if(!g_example_broadcast_format)
    {
	// Initialize the global object
	g_example_broadcast_format = new ExampleBroadcastFormat(pContext);

	// Set our local pointer to the context in case this instance gets
	// used for anything else
	m_pContext = pContext;
	m_pContext->AddRef();
    }    
    return PNR_OK;
}


/****************************************************************************
 *  ExampleBroadcastObject::~ExampleBroadcastObject          ref:  exlivobj.h
 *
 *  Destructor
 */
ExampleBroadcastObject::~ExampleBroadcastObject()
{
    if(m_pContext)
	m_pContext->Release();

    if(m_pFFResponse)
	m_pFFResponse->Release();

    if(m_pCommonClassFactory)
	m_pCommonClassFactory->Release();

    if(m_pScheduler)
	m_pScheduler->Release();

    if(m_pURL)
	delete [] m_pURL;
}


/****************************************************************************
 *  ExampleBroadcastObject::NextPacket                       ref:  exlivobj.h
 *
 */
void
ExampleBroadcastObject::NextPacket()
{
    IRMABuffer* pBuffer;
    IRMAPacket* pPacket;
    RMATimeval  now;
    SendPacketCallback* cb;
    UINT32 timeStamp;

    /*
     * If the stream has been stopped, just exit
     */

    if (!m_bPacketsStarted)
    {
	return;
    }

    if(PNR_OK != m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,
						     (void**)&pBuffer))
	m_pFFResponse->StreamDone(0);

    if(PNR_OK != m_pCommonClassFactory->CreateInstance(CLSID_IRMAPacket,
						     (void**)&pPacket))
	m_pFFResponse->StreamDone(0);

    now = m_pScheduler->GetCurrentSchedulerTime();

    /* This buffer represents the actual type specific data in the packet.
     * For this example, just fill it in with a string containing a squence
     * and the current time according to the scheduler object
     */
    pBuffer->SetSize(EXAMPLE_PACKET_SIZE);
    sprintf((char *)pBuffer->GetBuffer(), "P:%ld/T:%ld.%ld", 
	    m_sequence,
	    now.tv_sec,
	    now.tv_usec);

    m_sequence++;

    /* Calculate the timestamp for this packet.  Should work out to
     * the time the last packet was sent plus EXAMPLE_TIME_PER_PACKET
     */
    timeStamp= ((m_nextPacket.tv_sec * 1000) + (m_nextPacket.tv_usec / 1000)) -
	((m_startTime.tv_sec * 1000) + (m_startTime.tv_usec / 1000));

    /* Now fill in the actual IRMAPacket and pass it back to the server
     */
    pPacket->Set(pBuffer,
		 timeStamp,
		 0, // Stream number
		 RMA_ASM_SWITCH_ON,
		 0);
    m_pFFResponse->PacketReady(PNR_OK, pPacket);

    pPacket->Release();
    pBuffer->Release();
    /* Calculate the time for the next packet */
    m_nextPacket.tv_usec += EXAMPLE_TIME_PER_PACKET * 1000;
    m_nextPacket.tv_sec += m_nextPacket.tv_usec / 1000000;
    m_nextPacket.tv_usec = m_nextPacket.tv_usec % 1000000;

    /* And create a callback object for the scheduler to call at the
     * time we want to send the next packet
     */
    cb = new SendPacketCallback(this);
    cb->AddRef();
    m_pScheduler->AbsoluteEnter(cb, m_nextPacket);
    cb->Release();
}


// IUnknown COM Interface Methods

/****************************************************************************
 *  IUnknown::AddRef                                            ref:  pncom.h
 *
 *  This routine increases the object reference count in a thread safe
 *  manner. The reference count is used to manage the lifetime of an object.
 *  This method must be explicitly called by the user whenever a new
 *  reference to an object is used.
 */
STDMETHODIMP_(ULONG32) ExampleBroadcastObject::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}


/****************************************************************************
 *  IUnknown::Release                                           ref:  pncom.h
 *
 *  This routine decreases the object reference count in a thread safe
 *  manner, and deletes the object if no more references to it exist. It must
 *  be called explicitly by the user whenever an object is no longer needed.
 */
STDMETHODIMP_(ULONG32) ExampleBroadcastObject::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
        return m_lRefCount;
    }

    delete this;
    return 0;
}


/****************************************************************************
 *  IUnknown::QueryInterface                                    ref:  pncom.h
 *
 *  This routine indicates which interfaces this object supports. If a given
 *  interface is supported, the object's reference count is incremented, and
 *  a reference to that interface is returned. Otherwise a NULL object and
 *  error code are returned. This method is called by other objects to
 *  discover the functionality of this object.
 */
STDMETHODIMP ExampleBroadcastObject::QueryInterface(
    REFIID riid, void** ppvObj)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
	AddRef();
	*ppvObj = (IUnknown*)(IRMAPlugin*)this;
	return PNR_OK;
    } 
    else if(IsEqualIID(riid, IID_IRMABroadcastFormatObject))
    {
	AddRef();
	*ppvObj = (IRMABroadcastFormatObject*)this;
	return PNR_OK;
    }
    else if(IsEqualIID(riid, IID_IRMAPlugin))
    {
	AddRef();
	*ppvObj = (IRMAPlugin*)this;
	return PNR_OK;
    }
    else if(IsEqualIID(riid, IID_IRMAFileSystemObject))
    {
	AddRef();
	*ppvObj = (IRMAFileSystemObject*)this;
	return PNR_OK;
    }
    else if(IsEqualIID(riid, IID_IRMAFileExists))
    {
	AddRef();
	*ppvObj = (IRMAFileExists*)this;
	return PNR_OK;
    }
    else if(IsEqualIID(riid, IID_IRMABroadcastMapper))
    {
	AddRef();
	*ppvObj = (IRMABroadcastMapper*)this;
	return PNR_OK;
    }

    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}


// IRMABroadcastFormatObject Interface Methods

/****************************************************************************
 *  IRMABroadcastFormatObject::GetBroadcastFormatInfo        ref:  rmaformt.h
 *
 *  Returns a string that uniquely identifies this broadcast format
 */
STDMETHODIMP
ExampleBroadcastObject::GetBroadcastFormatInfo(REF(const char*) pToken)
{
    pToken = "pn-exbroad";
    return PNR_OK;
}


/****************************************************************************
 *  IRMABroadcastFormatObject::InitBroadcastFormat           ref:  rmaformt.h
 *
 */
STDMETHODIMP
ExampleBroadcastObject::InitBroadcastFormat(
    const char* pURL,
    IRMAFormatResponse* pFileFormatResponse)
{
    if (m_pURL)
    {
	delete m_pURL;
	m_pURL = NULL;
    }
    if (pURL)
    {
	m_pURL = new char[strlen(pURL)+1];
	if (!m_pURL) return PNR_OUTOFMEMORY;
        strcpy(m_pURL,pURL);
    }

    m_pFFResponse = pFileFormatResponse;

    m_pFFResponse->AddRef();

    // Grab a reference to the common class factory for later use
    if(PNR_OK != m_pContext->QueryInterface(IID_IRMACommonClassFactory, 
					  (void**)&m_pCommonClassFactory))
    {
	m_pFFResponse->Release();
	m_pFFResponse = NULL;
	return PNR_OUTOFMEMORY;
    }

    // Same with the scheduler
    if(PNR_OK != m_pContext->QueryInterface(IID_IRMAScheduler,
					  (void**)&m_pScheduler))
    {
	m_pFFResponse->Release();
	m_pFFResponse = NULL;
	m_pCommonClassFactory->Release();
	m_pCommonClassFactory = NULL;
	return PNR_OUTOFMEMORY;
    }

    // Got all our stuff in order, tell the server we're ready to go
    m_pFFResponse->InitDone(PNR_OK);
    return PNR_OK;
}


/****************************************************************************
 *  IRMABroadcastFormatObject::GetFileHeader                 ref:  rmaformt.h
 *
 *  Called by controller to ask the file format for the header for file.
 */
STDMETHODIMP 
ExampleBroadcastObject::GetFileHeader()
{
    IRMAValues* pHeader;

    /* This example always has one stream.  A more complicated
     * Broadcast format plugin might have several
     */
    if(PNR_OK != m_pCommonClassFactory->CreateInstance(CLSID_IRMAValues,
						     (void**)&pHeader))
	return PNR_OUTOFMEMORY;

    pHeader->SetPropertyULONG32("StreamCount", 1);
    m_pFFResponse->FileHeaderReady(PNR_OK, pHeader);
    pHeader->Release();

    return PNR_OK;
}


/****************************************************************************
 *  IRMABroadcastFormatObject::GetStreamHeader               ref:  rmaformt.h
 *
 *	Called by controller to ask the file format for the header for
 *	a particular stream in the file. The file format should call 
 *	IRMABroadcastFormatSession::HeaderReady() for the 
 *	IRMABroadcastFormatSession object that was passed in during 
 *	initialization, when the header is available.
 */
PN_RESULT
ExampleBroadcastObject::GetStreamHeader(UINT16 unStreamNumber)
{
    IRMAValues* pHeader;
    IRMABuffer* pOpaqueData;
    IRMABuffer* pStreamName;
    IRMABuffer* pMimeType;
    UINT32 bitRate;

    /* Create an IRMAValues */
    if(PNR_OK != m_pCommonClassFactory->CreateInstance(CLSID_IRMAValues,
						     (void**)&pHeader))
	return PNR_OUTOFMEMORY;

    /* Create IRMABuffers for each of the string properties that we're going to
     * pass back to the server, and for the type specific data
     */
    if((PNR_OK != m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,
						      (void**)&pOpaqueData)))
	return PNR_OUTOFMEMORY;
    
    if((PNR_OK != m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,
							(void**)&pStreamName)))
	return PNR_OUTOFMEMORY;
    if(PNR_OK != m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,
						       (void**)&pMimeType))
	return PNR_OUTOFMEMORY;

    /* opaque represents the type specific data for the stream.  In this
     * example, it doesn't really matter what we put here.
     */
    char opaque[] = "Example Opaque Data";
    char streamName[] = "Example Stream";
    char mimeType[] = "application/x-pn-examplestream";

    /* Set up the IRMABuffers for each of these strings. */
    pOpaqueData->Set((const UCHAR*)opaque, strlen(opaque));
    pStreamName->Set((const UCHAR*)streamName, strlen(streamName) + 1);
    pMimeType->Set((const UCHAR*)mimeType, strlen(mimeType) + 1);

    /* Calculate bits per second for the stream */
    bitRate = (EXAMPLE_PACKET_SIZE * 8) * (1000 / EXAMPLE_TIME_PER_PACKET);

    /* Now fill in all the standard properties for this stream in the
     * stream header
     */
    pHeader->SetPropertyBuffer ("OpaqueData", pOpaqueData);
    pHeader->SetPropertyULONG32("StreamNumber",  0);
    pHeader->SetPropertyULONG32("MaxBitRate", bitRate);
    pHeader->SetPropertyULONG32("AvgBitRate", bitRate);
    pHeader->SetPropertyULONG32("MaxPacketSize", EXAMPLE_PACKET_SIZE);
    pHeader->SetPropertyULONG32("AvgPacketSize", EXAMPLE_PACKET_SIZE);
    pHeader->SetPropertyULONG32("StartTime", 0);
    pHeader->SetPropertyULONG32("Preroll", 3000);
    pHeader->SetPropertyULONG32("Duration", 60000);

    pHeader->SetPropertyCString("StreamName", pStreamName);
    pHeader->SetPropertyCString("MimeType", pMimeType);

    /* Pass the stream header back to the server */
    m_pFFResponse->StreamHeaderReady(PNR_OK, pHeader);

    /* Release our references to everything we got from the common
     * class factory above
     */
    pOpaqueData->Release();
    pHeader->Release();
    pStreamName->Release();
    pMimeType->Release();

    return PNR_OK;
}


/****************************************************************************
 *  IRMABroadcastFormatObject::StartPackets                  ref:  rmaformt.h
 *
 *  Called by controller to ask the file format for the next packet
 *  for a particular stream in the file. The file format should call 
 *  IRMABroadcastFormatSession::PacketReady() for the IRMABroadcastFormatSession
 *  object that was passed in during initialization, when the packet
 *  is available.
 */
STDMETHODIMP
ExampleBroadcastObject::StartPackets(UINT16 unStreamNumber)
{
    SendPacketCallback* cb;

    m_bPacketsStarted = TRUE;
    m_sequence = 0;
    m_startTime = m_pScheduler->GetCurrentSchedulerTime();
    m_nextPacket = m_startTime;

    // Create a callback object for the first packet
    cb = new SendPacketCallback(this);
    cb->AddRef();
    // Schedule the first packet callback for right now.
    m_pScheduler->AbsoluteEnter(cb, m_startTime);
    cb->Release();

    return PNR_OK;
}


/****************************************************************************
 *  IRMABroadcastFormatObject::StopPackets                   ref:  rmaformt.h
 *
 *  Stop sending packets to the IRMABroadcastFormatSession object.
 */
STDMETHODIMP
ExampleBroadcastObject::StopPackets(UINT16 unStreamNumber)
{
    m_bPacketsStarted = FALSE;

    if(m_pFFResponse)
    {
	m_pFFResponse->Release();
	m_pFFResponse = 0;
    }

    return PNR_OK;
}


// IRMAFileSystemObject Interface Methods

/****************************************************************************
 *  IRMAFileSystemObject::GetFileSystemInfo                  ref:  rmafiles.h
 *
 *  Return some basic information amount the filesystem implemented herein.
 */
STDMETHODIMP
ExampleBroadcastObject::GetFileSystemInfo(
    REF(const char*) /*OUT*/ pShortName,
    REF(const char*) /*OUT*/ pProtocol
)
{
    pShortName = "pn-exbroad";
    pProtocol  = "pnexamplebroadcast";

    return PNR_OK;
}


/****************************************************************************
 *  IRMAFileSystemObject::InitFileSystem                     ref:  rmafiles.h
 *
 */
STDMETHODIMP
ExampleBroadcastObject::InitFileSystem(IRMAValues* options)
{
    return PNR_OK;
}


/****************************************************************************
 *  IRMAFileSystemObject::CreateFile                         ref:  rmafiles.h
 *
 *  Create and return a FileObject for this FileSystem.
 */
STDMETHODIMP
ExampleBroadcastObject::CreateFile(
    IUnknown**    /*OUT*/	ppFileObject
)
{
    *ppFileObject = (IUnknown*)(IRMAFileObject*)this;
    AddRef();
    return PNR_OK;
}


/****************************************************************************
 *  IRMAFileSystemObject::CreateDir                          ref:  rmafiles.h
 *
 */
STDMETHODIMP
ExampleBroadcastObject::CreateDir(
    IUnknown**     /*OUT*/     ppDirObject
)
{
    return PNR_NOTIMPL;
}


// IRMAFileExists Interface Methods

/****************************************************************************
 *  IRMAFileExists::DoesExist                                ref:  rmafiles.h
 *
 */
STDMETHODIMP
ExampleBroadcastObject::DoesExist(const char* pPath,
				  IRMAFileExistsResponse* pResponse)
{
    /* This example serves a stream under any file name passed to it,
     * So this can always return TRUE.
     *
     * Normally a broadcast format would search the list of "files"
     * it is currently ready to broadcast and return TRUE or FALSE
     * depending on whether pPath is one of them or not.
     */
    pResponse->DoesExistDone(TRUE);
    return PNR_OK;
}


// IRMABroadcastMapper Interface Methods

/****************************************************************************
 *  IRMABroadcastMapper::FindBroadcastType                   ref:  rmafiles.h
 *
 */
STDMETHODIMP
ExampleBroadcastObject::FindBroadcastType(const char* pURL,
					IRMABroadcastMapperResponse* pResponse)
{
    /* Match the URL to a broadcast token
     * In most broadcast formats the token should always be the same.
     */
    pResponse->BroadcastTypeFound(PNR_OK, "pn-exbroad");
    return PNR_OK;
}


// SendPacketCallback Class 

/****************************************************************************
 *  SendPacketCallback::SendPacketCallback                   ref:  exlivobj.h
 *
 */
SendPacketCallback::SendPacketCallback(ExampleBroadcastObject* obj)
{
    m_lRefCount = 0;
    m_pObj = obj;
    m_pObj->AddRef();
}


/****************************************************************************
 *  SendPacketCallback::~SendPacketCallback                  ref:  exlivobj.h
 *
 */
SendPacketCallback::~SendPacketCallback()
{
    m_pObj->Release();
}

/****************************************************************************
 *  IRMACallback::Func                                       ref:  rmaengin.h
 *
 *  Called by the scheduler object when it's time to send a new packet.
 */
STDMETHODIMP
SendPacketCallback::Func()
{
    /* The real work is done by the ExampleBroadcastObject that created
     * this callback.
     */
    m_pObj->NextPacket();
    return PNR_OK;
}


/****************************************************************************
 *  IUnknown::AddRef                                            ref:  pncom.h
 *
 */
STDMETHODIMP_(ULONG32) SendPacketCallback::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}


/****************************************************************************
 *  IUnknown::Release                                           ref:  pncom.h
 *
 */
STDMETHODIMP_(ULONG32) SendPacketCallback::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
        return m_lRefCount;
    }

    delete this;
    return 0;
}


/****************************************************************************
 *  IUnknown::QueryInterface                                    ref:  pncom.h
 *
 */
STDMETHODIMP SendPacketCallback::QueryInterface(
    REFIID riid, void** ppvObj)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
	AddRef();
	*ppvObj = (IUnknown*)(IRMABroadcastFormatObject*)this;
	return PNR_OK;
    } 
    else if(IsEqualIID(riid, IID_IRMACallback))
    {
	AddRef();
	*ppvObj = (IRMABroadcastFormatObject*)this;
	return PNR_OK;
    }

    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}
