/****************************************************************************
 * 
 *  $Id: exlivobj.h,v 1.15 1998/05/21 20:32:05 smpatel Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 * 
 *  exlivobj.h
 *
 *  An example which shows the use of a broadcast plug-in.
 */
#ifndef _EXLIVOBJ_H_
#define _EXLIVOBJ_H_


/****************************************************************************
 * Includes
 */
#include "rmaformt.h"
#include "rmaengin.h"


/****************************************************************************
 * Forward declarations
 */
class  ExampleBroadcastObject;
struct IRMACommonClassFactory;


/****************************************************************************
 *
 *  SendPacketCallback Class
 *
 */
class SendPacketCallback : public IRMACallback
{
    public:
    /****** Public Class Methods ******************************************/
    SendPacketCallback(ExampleBroadcastObject* obj);


    /************************************************************************
     *  IRMACallback Interface Methods                       ref:  rmaengin.h
     */
    STDMETHOD(Func)                     (THIS);


    /************************************************************************
     *  IUnknown COM Interface Methods                          ref:  pncom.h
     */
    STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
    STDMETHOD_(UINT32, AddRef ) (THIS);
    STDMETHOD_(UINT32, Release) (THIS);

    private:
    /****** Private Class Variables ****************************************/
    INT32                   m_lRefCount;
    ExampleBroadcastObject* m_pObj;

    /****** Private Class Methods ******************************************/
    ~SendPacketCallback();
    
    PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME       // Avoids GCC compiler warning
};


/****************************************************************************
 *
 *  ExampleBroadcastObject Class
 *
 */
class ExampleBroadcastObject : public IRMABroadcastFormatObject,
			       public IRMAFileSystemObject,
			       public IRMAPlugin,
			       public IRMAFileExists,
			       public IRMABroadcastMapper
{
    public:
    /****** Public Class Methods ******************************************/
    ExampleBroadcastObject();
    void NextPacket();


    /************************************************************************
     *  IRMABroadcastFormatObject Interface Methods          ref:  rmaformt.h
     */
    STDMETHOD(GetBroadcastFormatInfo) (THIS_ REF(const char*) pToken);
    STDMETHOD(InitBroadcastFormat)
	(THIS_
	 const char*		/*IN*/	pURL, 
	 IRMAFormatResponse*	/*IN*/	pFormatResponse
	);
    STDMETHOD(GetFileHeader)	(THIS);
    STDMETHOD(GetStreamHeader)	(THIS_ UINT16 unStreamNumber);
    STDMETHOD(StartPackets)	(THIS_ UINT16 unStreamNumber);
    STDMETHOD(StopPackets)	(THIS_ UINT16 unStreamNumber);


    /************************************************************************
     *  IRMAFileSystemObject Interface Methods               ref:  rmafiles.h
     */
    STDMETHOD(GetFileSystemInfo)
	(THIS_
	 REF(const char*) /*OUT*/ pShortName,
	 REF(const char*) /*OUT*/ pProtocol
	);
    STDMETHOD(InitFileSystem) (THIS_ IRMAValues* options);
    STDMETHOD(CreateFile)     (THIS_ IUnknown**  ppFileObject);
    STDMETHOD(CreateDir)      (THIS_ IUnknown** ppDirObject);


    /* File Objects returned by broadcast formats need to implement
     * IRMAFileExists and IRMABroadcastMapper.  They do NOT need to
     * implement IRMAFileObject!
     */
 
     
    /************************************************************************
     *  IRMAFileExists Interface Methods                     ref:  rmafiles.h
     */
    STDMETHOD(DoesExist)
	(THIS_
	 const char* pPath,
	 IRMAFileExistsResponse* pResponse
	);


    /************************************************************************
     *  IRMABroadcastMapper Interface Methods                ref:  rmafiles.h
     */
    STDMETHOD(FindBroadcastType)
	(THIS_
	 const char* pURL,
	 IRMABroadcastMapperResponse* pResponse
	);

    

    /************************************************************************
     *  IRMAPlugin Interface Methods                         ref:  rmaplugn.h
     */
    STDMETHOD(GetPluginInfo)
	(THIS_ 
	 REF(BOOL)        bLoadMultiple,
	 REF(const char*) pDescription,
	 REF(const char*) pCopyright,
	 REF(const char*) pMoreInfoURL,
	 REF(UINT32)      versionNumber
	);
    STDMETHOD(InitPlugin) (THIS_ IUnknown* pRMACore);


    /************************************************************************
    *  IUnknown COM Interface Methods                          ref:  pncom.h
    */
    STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
    STDMETHOD_(UINT32, AddRef ) (THIS);
    STDMETHOD_(UINT32, Release) (THIS);


    private:
    /****** Private Class Variables ****************************************/
    INT32				m_lRefCount;
    IUnknown*				m_pContext;
    IRMACommonClassFactory*             m_pCommonClassFactory;
    IRMAFormatResponse*                 m_pFFResponse;
    IRMAScheduler*                      m_pScheduler;
    char*                               m_pURL;
    UINT32                              m_sequence;
    BOOL                                m_bPacketsStarted;
    RMATimeval                          m_startTime;
    RMATimeval                          m_nextPacket;

    /****** Private Static Class Variables *********************************/
    static const IID		        zm_myIID;
    static const char*		        zm_pDescription;
    static const char*		        zm_pCopyright;
    static const char*		        zm_pMoreInfoURL;

    /****** Private Class Methods ******************************************/
    ~ExampleBroadcastObject();

    PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME      // Avoids GCC compiler warning
};

#endif
