/****************************************************************************
 * 
 *  $Id: exlogpln.h,v 1.9 1997/10/21 21:27:31 christof Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 * 
 *  exlogpln.h
 *
 *  Example Logger plug-in for the RealMedia Architecture (RMA).
 */

/****************************************************************************
 * 
 *  EXAMPLE LOGGER PLUGIN for RMA Server
 *
 *  This is a very simple logger plugin, that it receives the log information
 *  (i.e. Warnings, Errors) from the server and saves them into a file.
 */
#ifndef _EXLOGPLN_H_
#define _EXLOGPLN_H_


/****************************************************************************
 * Forward declarations
 */
struct IUnknown;
struct IRMAPlugin;


/****************************************************************************
 *
 *  Logger Class
 *
 *  Example implementation of a logger plug-in.
 */
class Logger  : public IRMAPlugin, 
		public IRMAErrorSink
{
    public:
    /****** Public Class Methods ******************************************/
    Logger();


    /************************************************************************
     *  IRMAErrorSink Interface Methods                      ref:  rmaerror.h
     */
    STDMETHOD(ErrorOccurred)
	(THIS_
	 const UINT8	unSeverity,  
	 const UINT32	ulRMACode,
	 const UINT32	ulUserCode,
	 const char*	pUserString,
 	 const char*	pMoreInfoURL
	);


    /************************************************************************
     *  IRMAPlugin Interface Methods                         ref:  rmaplugn.h
     */
    STDMETHOD(GetPluginInfo)
        (THIS_ 
          REF(BOOL)        bLoadMultiple,
          REF(const char*) pDescription,
          REF(const char*) pCopyright,
          REF(const char*) pMoreInfoURL,
          REF(UINT32)      versionNumber
        );
    STDMETHOD(InitPlugin) (THIS_ IUnknown* pContext);


    /************************************************************************
     *  IUnknown COM Interface Methods                          ref:  pncom.h
     */
    STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
    STDMETHOD_(UINT32, AddRef ) (THIS);
    STDMETHOD_(UINT32, Release) (THIS);


    private:
    /****** Private Static Class Variables *********************************/
    INT32    m_lRefCount;
    FILE*    m_hLogFile;

    /****** Private Class Methods ******************************************/
    ~Logger();

    PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME       // Avoids GCC compiler warning
};


#endif // _EXLOGPLN_H_