/****************************************************************************
 * 
 * $Id: main.cpp,v 1.32 1999/02/19 23:35:39 nhart Exp $
 *
 *  Copyright (C) 1997 RealNetworks, Inc.
 *  All rights reserved.
 *
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 * 
 *  main.cpp
 *
 *  Example Text "encoder" class
 *
 */

#define INITGUID


/****************************************************************************
 * Constants
 */
#define NUM_MSEC_BETWEEN_POLLS 5500
#define PBUF_SIZE              2048


/****************************************************************************
 * Includes
 */
#include <stdio.h>
#include <stdlib.h>

#include "pntypes.h"
#include "pncom.h"
#include "rmafiles.h"
#include "rmaencod.h"
#include "rmalvtxt.h"

#ifdef _WIN16
    #include <windows.h>
#endif

#ifdef _MACINTOSH
    #ifdef __MWERKS__
	#include <console.h>
    #endif
    #include "os.h"  /* for Mac version of LoadLibrary() */
#endif

#ifdef _UNIX
    #include "unix/os.h" /* for Unix version of LoadLibrary() */
#endif

typedef PN_RESULT (PNEXPORT_PTR FPRMSETDLLACCESSPATH) (const char*);

/****************************************************************************
 * Globals
 */
FPRMCREATELIVETEXT m_fpCreateLiveText;
FPRMSETDLLACCESSPATH m_fpSetDLLAccessPath;



/****************************************************************************
 *  main
 *
 */
int
main(int argc, char* argv[])
{
    #ifdef __MWERKS__
	// Emulate command line on Macintosh
	argc = ccommand( &argv );
    #endif

    int numExlvtxtArgs = 1; /* "exlvtext.exe" is argv[0] */
    char szHost[256]; numExlvtxtArgs++; /* argv[1] */
    char szPort[256]; numExlvtxtArgs++; /* argv[2] */
    char szUsername[256]; numExlvtxtArgs++; /* argv[3] */
    char szPassword[256]; numExlvtxtArgs++; /* argv[4] */
    char szStreamFilename[512];	numExlvtxtArgs++; /* argv[5] */
    char szAvgBitrate[256]; numExlvtxtArgs++; /* argv[6] */

    IRMALiveText* pLiveTextEncoder = NULL;
    IRMALiveText2* pLiveText2 = NULL;

    /* prepare/load the RMALiveText module */
    HINSTANCE hDllRtlive;


#ifndef _WIN16
    if(argc != numExlvtxtArgs)
    {
	fprintf(stdout,
		"Usage: %s <host> <port> <username> <password> "
		"<stream filename> <expected avg bitrate>\n",
		argv[0]);
	
	fprintf(stdout, "\n(enter q to quit:)\n"
		"  host (RM server machine): ");
	gets(szHost);
	if(!strlen(szHost)  ||  !strcmp(szHost,"q"))
	{
	    return 0;
	}
	
	fprintf(stdout, "port (listed in FSMount's /encoder/ section\n"
		"  of server config file): ");
	gets(szPort);
	if(!strlen(szPort)  ||  !strcmp(szPort,"q"))
	{
	    return 0;
	}
	
	fprintf(stdout, "username: ");
	gets(szUsername);
	if(!strcmp(szUsername,"q"))
	{
	    return 0;
	}
		
	fprintf(stdout, "password: ");
	gets(szPassword);
	if(!strcmp(szPassword,"q"))
	{
	    return 0;
	}
	
	fprintf(stdout, "stream filename (the name RM Players will \n"
		"  use to view the stream): ");
	gets(szStreamFilename);
	if(!strlen(szStreamFilename)  ||  !strcmp(szStreamFilename,"q"))
	{
	    return 0;
	}

	fprintf(stdout,"Expected average bit rate (bits per sec) of this stream\n"
		"  (hit 'enter' to default to %lubps):", 2048L);
	gets(szAvgBitrate);
	if(!strcmp(szAvgBitrate,"q"))
	{
	    return 0;
	}
	if(!strlen(szAvgBitrate))
	{
	    strcpy(szAvgBitrate, "2048");
	}


	fprintf(stdout, "\n");
    }
    else  /* argc==6: */
#endif /* end: ifndef _WIN16 */
    {
	strcpy(szHost, argv[1]);
	strcpy(szPort, argv[2]);
	strcpy(szUsername, argv[3]);
	strcpy(szPassword, argv[4]);
	strcpy(szStreamFilename, argv[5]);
    }


    if (!(hDllRtlive = LoadLibrary("rtli3260.dll")))
    {
#if defined(_MACINTOSH)
        if (!(hDllRtlive = LoadLibrary("rtlive60.dll")))
#else
        if (!(hDllRtlive = LoadLibrary("rtlive.dll")))
#endif
	{
#if defined(_MACINTOSH)  ||  defined(_UNIX)
	    fprintf(stdout, "Failed to load the 'rtlive' and/or 'encnet'\n"
			    "shared library\n");
# if defined(_UNIX)
	    fprintf(stdout, dlerror());
# endif
#else /* Windows (not Mac or Unix): */
	    fprintf(stdout, "Failed to load the 'rtli3260' DLL\n"
			    "Make sure both it and encn3260.dll are\n"
			    "located in a directory that is in your path");
#endif
	    return PNR_FAILED;
	}
    }

    /* retrieve the proc addresses from the module: */
    m_fpCreateLiveText = (FPRMCREATELIVETEXT) GetProcAddress(hDllRtlive,
	    "CreateLiveText");
    m_fpSetDLLAccessPath = (FPRMSETDLLACCESSPATH) GetProcAddress(hDllRtlive,
	    "SetDLLAccessPath");

    if(NULL != m_fpSetDLLAccessPath)
    {
	// Create a null delimited, double-null terminated string containing
	// the paths to the encnet library (DT_Common) and the sdpplin library
	// (DT_Plugins)...
	char pPaths[256];
	UINT32 ulPathLen = 0;
	char pNextPath[256];
#if defined(_MACINTOSH)
	const char* pCurrentDir = ".:";
#elif defined(_UNIX)
	const char* pCurrentDir = "./";
#else
	const char* pCurrentDir = ".\\";
#endif

	memset(pPaths, 0, 256);

	sprintf(pNextPath, "DT_Common=%s", pCurrentDir);
	memcpy(pPaths, pNextPath, strlen(pNextPath) + 1);
	ulPathLen = strlen(pNextPath) + 1;

	sprintf(pNextPath, "DT_Plugins=%s", pCurrentDir);
	memcpy(pPaths + ulPathLen, pNextPath, strlen(pNextPath) + 1);
	ulPathLen = strlen(pNextPath) + 1;

	m_fpSetDLLAccessPath((char*)pPaths);
    }

    if(NULL != m_fpCreateLiveText)
    {
	/*Creates the LiveText Encoder: */
	if(PNR_OK == m_fpCreateLiveText((IRMALiveText**) &pLiveTextEncoder))
	{
	    pLiveTextEncoder->SetType("tickertape");

	    /* By setting this to TRUE, the player, in tickertape mode, will
	     * "loop" the text it has received if and when the stream
	     * "runs dry", i.e., when no new text has arrived by the time all
	     * the old text has just crawled out of view.  The old text will
	     * then crawl in from the right edge of the window and continue
	     * loop in this fashion until a <CLEAR> tag is sent.  All text
	     * that arrives after a <CLEAR> will loop when the stream runs
	     * dry (and the old text prior to the <CLEAR> will never be
	     * redisplayed).
	     */
	    pLiveTextEncoder->SetDoLooping(TRUE);

	    pLiveTextEncoder->SetHyperlinkInfo(
		    /* Whether or not to underline hyperlinked text: */
		    TRUE,
		    /* the color of the text inside <A ..>...</A>: */
		    "");  /* NOTE: this color is ignored in tickertapes */
	    

	    if(PNR_OK == pLiveTextEncoder->QueryInterface(IID_IRMALiveText2,
		    (void**)&pLiveText2))
	    {
		/* We've got the newer interface that allows us to set the
		 * average bit rate and other new attributes:
		 */
		pLiveText2->DeclareAverageBitrate(atol(szAvgBitrate));
	    }

	    /* Here is where you can set the other values related to the
	     * overall presentation of the text in the player (client):
	     */
	    

	    /* Now that the text window attributes are set, call
	     * InitLiveText() which initializes the encoder so it
	     * can communicate with the server:
	     */
	    pLiveTextEncoder->InitLiveText(
		    szHost,
		    atoi(szPort),
		    szUsername,
		    szPassword,
		    szStreamFilename);

	    fprintf(stdout, "Entering mainloop\n");

	    ULONG32 ulLastTime = 0L;
	    ULONG32 curStreamTime = 0L;

	    /* The encoder will remain in this loop until the EncoderResponse
	     * object tells us to stop
	     */
	    while(!pLiveTextEncoder->EncoderIsDone())
	    {
		/* If we haven't gotten the InitEncoderResponseDone callback
		 * yet, then don't do anything but Process().
		 */
		if(pLiveTextEncoder->EncoderIsInitialized()  && 
			pLiveTextEncoder->PacketsHaveStarted())
		{
		    curStreamTime = pLiveTextEncoder->GetTime();
		    
/* *********** THIS IS SAMPLE CODE; ****************************** */
/* *********** DELETE THIS AND REPLACE WITH YOUR DATA ************ */
		    /* temporarily holds what's input in gets(): */
		    char pBuf[PBUF_SIZE+1];

		    /* send data every n milliseconds (I chose 5500 because
		     * it is about the right rate to send this data to keep
		     * in pace with the default crawlRate (20 pixels/second))
		     */
		    if(curStreamTime - ulLastTime > NUM_MSEC_BETWEEN_POLLS ||
			    ulLastTime == 0L) 
		    {
			ulLastTime = curStreamTime;
			pBuf[0] = '\0';

			sprintf(pBuf,
				"%lu.%03lu",
				(curStreamTime/1000), (curStreamTime%1000) );
			INT16 foo=strlen(pBuf);
			if(foo > 0)
			{
			    if(foo > MAX_PACKET_SIZE)
			    {
				foo = MAX_PACKET_SIZE;
				pBuf[foo] = '\0';
			    }
			    if(EOF == pBuf[foo-1]  ||
				    curStreamTime > 240000)/* dies @4 min. */
			    {
				pLiveTextEncoder->SetEncoderDone();
			    }
			    char pTckrUpperBuf[MAX_PACKET_SIZE+1];

			    /* This is thrown in to demonstrate how you can
			     * change the color of the "ticker upper" item.
			     * In this example, the ticker upper color
			     * changes "randomly" (every time curStreamTime
			     * variable is divisible by 4).  All subsequent
			     * "ticker upper" items will then be renderer in
			     * a "random" color (based on the first three
			     * bytes of curStreamTime). AddTickerItem() takes
			     * the first buffer it gets and prepends a "<TU>"
			     * to it to force it to be a "ticker upper" item.
			     * It then prepends a "<TL>" to the second buffer
			     * and then combines the two buffers into one and
			     * adds them to the buffer in the queue to be
			     * streamed.  So, to change a ticker upper or
			     * lower item's color to, say, red, you must
			     * prepend a "<TU color=red>" to the first buffer
			     * (or a "<TL color=red>" to the second) before
			     * passing them to AddTickerItem().  All
			     * subsequent ticker upper (or lower) items will
			     * be red until you send a new <TU color=...> (or
			     * <TL color=...> for "ticker lower" items).
			     * See realtext.htm in the SDK documentation for
			     * further information on text formatting.
			     */
			    char pColorBuf[64];
			    pColorBuf[0]='\0';
			    /* 
			     * Create a "random" color at "random" times:
			     */
			    if(!(curStreamTime & 0x00000003))
			    {
				ULONG32 tuc = curStreamTime & 0x00FFFFFF;
				UCHAR* ptuc = (UCHAR *)&tuc;
				sprintf(pColorBuf,
				"<TU color=#%02X%02X%02X>",
				 /* "|" blue with 0xB0 to keep 
				  * the color from being too dark to show
				  * up on the black background of the
				  * tickertape.  "&" green with 0x0F so
				  * it isn't too prominent:
				  */
				ptuc[0], ptuc[1]&0x0F, ptuc[2]|0xB0);
			    }
			    else
			    {
				pColorBuf[0] = '\0';
			    }

			    /* Make the "ticker upper" text be bold and in
			     * the "courier" font (which is "courier new" in
			     * Windows), and make the word "time" be linked
			     * to the US government's atomic clock web site
			     * in Colorado; see realtext.htm in the SDK docs
			     * for more details on formating and hyperlinking
			     * text. Also: change the first parameter in the
			     * call to pLiveTextEncoder->SetHyperlinkInfo()
			     * to FALSE (above in this file) if you
			     * don't want hyperlinked text to be underlined:
			     */
			    sprintf(pTckrUpperBuf,
				"%s<B><FONT face=\"courier\">The "
    "<A HREF=\"http://www.bldrdoc.gov/doc-tour/atomic_clock.html\">time</A> "
				"is: </B></FONT>", pColorBuf); 

			    pLiveTextEncoder->AddTickerItem(pTckrUpperBuf,
				    pBuf, /* ticker "lower" item */
				    TRUE); /* force packet to go out now */

			    /* A non-tickertape version of this program would
			     * do the following to send what's in pBuf:
			    
			    pLiveTextEncoder->AddData(pBuf, TRUE);
			    
			    */
			}
		    }

/* *********** END OF SAMPLE CODE ******************************** */
/* *********** YOUR CODE GOES HERE:******************************* */
/* gather your data here into a NULL-terminated buffer (or 2 buffers
 * if you are running a tickertape) and call AddData() with your
 * buffer (or call AddTickerItem() with your upper and lower item
 * buffers).  Note: you can call AddData() with tickertape data, but
 * you'll have to put your own "<TU>" and "<TL>"s in it.
 */
		    /* Use TRUE in the last parameter of the following function
		     * to stream the contents of all unsent text immediately
		     * (regardless of how small the amount of text is), or use
		     * FALSE to let the encoder decide when to send it (based on
		     * elapsed time since last packet and/or size of buffer):
		     * NOTE: szBuffer is a char* buffer that you must create,
		     * initialize, maintain, and fill with data.  Also, it MUST
		     * be NULL-terminated before being sent to AddData().
		     * bSendImmediately is a BOOL value that you must create
		     * and initialize, but is an optional parameter and defaults
		     * to FALSE.
		     */

		    /* 
		    pLiveTextEncoder->AddData(szBuffer,
			    bSendImmediately);
		     */

		}

		
		
		/* Every user of the superencoder library must call 
		 * IRMAEncoderResponse::Process() regularly.
		 */
		if(PNR_OK != pLiveTextEncoder->Process())
		{
		    pLiveTextEncoder->SetEncoderDone();
		}
	    }		    
	}
    }

    if(pLiveTextEncoder)
    {
	pLiveTextEncoder->Release();
    }

    if(pLiveText2)
    {
	pLiveText2->Release();
    }

    if(hDllRtlive)
    {
	FreeLibrary(hDllRtlive);
	hDllRtlive = NULL;
    }

    return 0;
}
