/****************************************************************************
 * 
 * $Id: main2.cpp,v 1.22 2000/03/08 22:06:21 ehodge Exp $
 *
 *  Copyright (C) 1997 RealNetworks, Inc.
 *  All rights reserved.
 *
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 * 
 *  main2.cpp
 *
 *  Example Text "encoder" class
 *
 */

#define INITGUID



/****************************************************************************
 * Constants
 */
#define END_OF_STREAM_REQUEST_CHAR     '$'
#define DEFAULT_NUM_MSEC_BETWEEN_POLLS 1000  //1000msec == 1 sec.
#define MIN_POLLING_PERIOD	       500
#define MAX_POLLING_PERIOD	       60000 //60000msec == 1 minute.
#define PBUF_SIZE                      2048
#if !defined(ENABLE_RTLIVE_STDOUT_MESSAGES)
#define	ENABLE_RTLIVE_STDOUT_MESSAGES	    0x40000000
#endif
#if !defined(DISABLE_RTLIVE_STDOUT_MESSAGES)
#define	DISABLE_RTLIVE_STDOUT_MESSAGES	    0x0
#endif


/****************************************************************************
 * Includes
 */
#include <stdio.h>
#include <stdlib.h>
#include <time.h>   /* for ctime(). */
#include <fcntl.h>  /* for _O_TEXT and _O_RDONLY. */
// #include <io.h>  /* for _open() & _close() */

#include "pntypes.h"
#include "pncom.h"
#include "rmafiles.h"
#include "rmaencod.h"
#include "rmalvtxt.h"

#ifdef _WIN16
    #include <windows.h>
#endif

#ifdef _MACINTOSH
    #ifdef __MWERKS__
	#include <console.h>
    #endif
    #include "os.h"  /* for Mac version of LoadLibrary() */
#endif

#ifdef _UNIX
    #include "unix/os.h" /* for Unix version of LoadLibrary() */
#endif

#ifdef __MWERKS__
    #include <unix.h>
    #include <stat.h>     /* for stat() */
#else
    #include <sys/types.h>
    #include <sys/stat.h> /* for stat() */
#endif

typedef PN_RESULT (PNEXPORT_PTR FPRMSETDLLACCESSPATH) (const char*);

/****************************************************************************
 * Globals
 */
FPRMCREATELIVETEXT m_fpCreateLiveText;
FPRMSETDLLACCESSPATH m_fpSetDLLAccessPath;

/****************************************************************************
 *  main
 *
 */
int
main(int argc, char* argv[])
{
    #ifdef _MACINTOSH
    argc = ccommand( &argv );
    #endif

    int numExlvtxtArgs = 1; /* "exlvtext.exe" is argv[0] */
    char szHost[256]; numExlvtxtArgs++; /* argv[1] */
    char szPort[256]; numExlvtxtArgs++; /* argv[2] */
    char szUsername[256]; numExlvtxtArgs++; /* argv[3] */
    char szPassword[256]; numExlvtxtArgs++; /* argv[4] */
    char szStreamFilename[512];	numExlvtxtArgs++; /* argv[5] */
    char szTextInputFilename[512]; numExlvtxtArgs++; /* argv[6] */
    char szPollingPeriodInSeconds[256]; numExlvtxtArgs++; /* argv[7] */
    char szWindowType[256]; numExlvtxtArgs++; /* argv[8] */
    char szWindowWidth[256]; numExlvtxtArgs++; /* argv[9] */
    char szWindowHeight[256]; numExlvtxtArgs++; /* argv[10] */
    char szWindowBGColor[256]; numExlvtxtArgs++; /* argv[11] */
    char szWindowDoLoop[256]; numExlvtxtArgs++; /* argv[12] */
    char szWindowExtraSpaces[256]; numExlvtxtArgs++; /* argv[13] */
    char szWindowCrawlrate[256]; numExlvtxtArgs++; /* argv[14] */
    char szWindowScrollrate[256]; numExlvtxtArgs++; /* argv[15] */
    char szWindowDoWordwrap[256]; numExlvtxtArgs++; /* argv[16] */
    char szWindowUnderlineHyperlinks[256]; numExlvtxtArgs++; /* argv[17] */
    char szWindowLinkColor[256]; numExlvtxtArgs++; /* argv[18] */
    char szAvgBitrate[256]; numExlvtxtArgs++; /* argv[19] */

    char szTmp[128];

    szHost[0] = szPort[0] = szUsername[0] = szPassword[0] =
	    szStreamFilename[0] = szTextInputFilename[0] =
	    szPollingPeriodInSeconds[0] = szWindowType[0] =
	    szWindowWidth[0] = szWindowHeight[0] = szWindowBGColor[0] =
	    szWindowDoLoop[0] = szWindowExtraSpaces[0] =
	    szWindowCrawlrate[0] = szWindowScrollrate[0] =
	    szWindowDoWordwrap[0] = szWindowUnderlineHyperlinks[0] =
	    szWindowLinkColor[0] = szAvgBitrate[0] = szTmp[0] = '\0';
    
    ULONG32 ulPollingPeriodInMsec = DEFAULT_NUM_MSEC_BETWEEN_POLLS;

    IRMALiveText* pLiveTextEncoder = NULL;
    IRMALiveText2* pLiveText2 = NULL;

    /* prepare/load the RMALiveText module */
    HINSTANCE hDllRtlive;


#ifndef _WIN16
    if(argc != numExlvtxtArgs)
    {
	fprintf(stdout,
		"Usage: %s <host> <port> <username> <password> "
		"<streamfilename>"
		"\n  <sourcefilename> <file-polling period> "
		"<RealText window type>"
		"\n  <window width> <window height> <window bgcolor>"
		"\n  <loopText (true or false)> <extra spaces (ignore or use)>",
		argv[0]);
	fprintf(stdout,
		"\n  <crawlrate> <scrollrate> <do wordwrap>"
		"\n  <underline hyperlinks (true or false)> <link color>"
		"\n  <expected avg bitrate>\n");
	
	fprintf(stdout, "\n(enter q to quit:)\n"
		"  host (RM server machine): ");
	gets(szHost);
	if(!strlen(szHost)  ||  !strcmp(szHost,"q"))
	{
	    return 0;
	}
	
	fprintf(stdout, "port (listed in FSMount's /encoder/ section\n"
		"  of server config file): ");
	gets(szPort);
	if(!strlen(szPort)  ||  !strcmp(szPort,"q"))
	{
	    return 0;
	}
	
	fprintf(stdout, "username: ");
	gets(szUsername);
	if(!strcmp(szUsername,"q"))
	{
	    return 0;
	}
		
	fprintf(stdout, "password: ");
	gets(szPassword);
	if(!strcmp(szPassword,"q"))
	{
	    return 0;
	}
	
	fprintf(stdout, "stream filename (the name RealPlayers will \n"
		"  use to view the stream): ");
	gets(szStreamFilename);
	if(!strlen(szStreamFilename)  ||  !strcmp(szStreamFilename,"q"))
	{
	    return 0;
	}

	fprintf(stdout, "Input file (the name of the file where RealText\n"
		"  can be read by this encoder and then streamed whenever\n"
		"  the timestamp of the file changes):");
	gets(szTextInputFilename);
	if(!strlen(szTextInputFilename)  || !strcmp(szTextInputFilename,"q"))
	{
	    return 0;
	}

	fprintf(stdout, "Frequency in seconds that this application checks\n"
		"  for an update to the input file "
		"(hit 'enter' to default to 1):");
	gets(szPollingPeriodInSeconds);
	if(!strcmp(szPollingPeriodInSeconds,"q"))
	{
	    return 0;
	}
	if(!strlen(szPollingPeriodInSeconds))
	{
	    strcpy(szPollingPeriodInSeconds, "1");
	}

	//Now, get the attributes of the presentation window:
	fprintf(stdout,"Window type (e.g., tickertape.\n"
		"  Hit 'enter' or 'return' key for generic):");
	gets(szWindowType);
	if(!strcmp(szWindowType,"q"))
	{
	    return 0;
	}
	if(!strlen(szWindowType))
	{
	    strcpy(szWindowType, "generic");
	}
	BOOL bIsTickerTape = !strcmp(szWindowType, "tickertape");
	BOOL bIsMarquee = !strcmp(szWindowType, "marquee");
	BOOL bIsScrollingNews = !strcmp(szWindowType, "scrollingnews");

	fprintf(stdout,"Do you want to use the default width, height, "
		"\n  bgcolor, loop handling, extra-spaces handling,"
		"\n  crawlrate, scrollrate, word-wrap handling, "
		"\n  hyperlink underlining, and hyperlink color "
		"\n  for the chosen window type? (Y or N)");
	gets(szTmp);
	if(!strcmp(szTmp,"q"))
	{
	    return 0;
	}
	if(strlen(szTmp)>0  &&  (szTmp[0] == 'N'  ||  szTmp[0] == 'n'))
	{
	    fprintf(stdout,"Window width (hit 'enter' to default to %d):",
		    (bIsTickerTape||bIsMarquee?500:320));
	    gets(szWindowWidth);
	    if(!strcmp(szWindowWidth,"q"))
	    {
		return 0;
	    }

	    fprintf(stdout,"Window height (hit 'enter' to default to %d):",
		    (bIsTickerTape||bIsMarquee?30:180));
	    gets(szWindowHeight);
	    if(!strcmp(szWindowHeight,"q"))
	    {
		return 0;
	    }

	    fprintf(stdout,"Window background color\n"
		    "  (hit 'enter' to default to %s):",
		    (bIsTickerTape?"black":"white"));
	    gets(szWindowBGColor);
	    if(!strcmp(szWindowBGColor,"q"))
	    {
		return 0;
	    }

	    if(bIsTickerTape  ||  bIsMarquee)
	    {
		fprintf(stdout,"looping enabled (true or false)?\n"
			"  (hit 'enter' to default to true):");
		gets(szWindowDoLoop);
		if(!strcmp(szWindowDoLoop,"q"))
		{
		    return 0;
		}
	    }

	    fprintf(stdout,"Extra space handling (use or ignore)?\n"
		    "  (hit 'enter' to default to use):");
	    gets(szWindowExtraSpaces);
	    if(!strcmp(szWindowExtraSpaces,"q"))
	    {
		return 0;
	    }
	    
	    if(!bIsScrollingNews)
	    {
		fprintf(stdout,"crawl rate\n"
			"  (hit 'enter' to default to %d):",
			(bIsTickerTape||bIsMarquee?20:0));
		gets(szWindowCrawlrate);
		if(!strcmp(szWindowCrawlrate,"q"))
		{
		    return 0;
		}
	    }
	    
	    if(!bIsTickerTape  &&  !bIsMarquee)
	    {
		fprintf(stdout,"scroll rate\n"
			"  (hit 'enter' to default to %d):",
			(bIsScrollingNews?10:0));
		gets(szWindowScrollrate);
		if(!strcmp(szWindowScrollrate,"q"))
		{
		    return 0;
		}

		fprintf(stdout,"wordwrap enabled (true or false)?\n"
			"  (hit 'enter' to default to %s):",
			(!bIsTickerTape  &&  !bIsMarquee?"true":"false"));
		gets(szWindowDoWordwrap);
		if(!strcmp(szWindowDoWordwrap,"q"))
		{
		    return 0;
		}
	    }

	    fprintf(stdout,"Hyperlinks underlined (true or false)?\n"
		    "  (hit 'enter' to default to true):");
	    gets(szWindowUnderlineHyperlinks);
	    if(!strcmp(szWindowUnderlineHyperlinks,"q"))
	    {
		return 0;
	    }
	    
	    fprintf(stdout,"Hyperlink color?\n"
		    "  (hit 'enter' to default to blue):");
	    gets(szWindowLinkColor);
	    if(!strcmp(szWindowLinkColor,"q"))
	    {
		return 0;
	    }
	}

	if(!strlen(szWindowWidth))
	{
	    strcpy(szWindowWidth, (bIsTickerTape||bIsMarquee?"500":"320"));
	}
	if(!strlen(szWindowHeight))
	{
	    strcpy(szWindowHeight, (bIsTickerTape||bIsMarquee?"30":"180"));
	}
	if(!strlen(szWindowBGColor))
	{
	    strcpy(szWindowBGColor, (bIsTickerTape?"black":"white"));
	}
	if(!strlen(szWindowDoLoop))
	{
	    strcpy(szWindowDoLoop,
		    (bIsTickerTape||bIsMarquee?"true":"false"));
	}
	if(!strlen(szWindowExtraSpaces))
	{
	    strcpy(szWindowExtraSpaces, "use");
	}
	if(!strlen(szWindowCrawlrate))
	{
	    strcpy(szWindowCrawlrate,
		    (bIsTickerTape||bIsMarquee?"20":"0"));
	}
	if(!strlen(szWindowScrollrate))
	{
	    strcpy(szWindowScrollrate, (bIsScrollingNews?"10":"0"));
	}
	if(!strlen(szWindowDoWordwrap))
	{
	    strcpy(szWindowDoWordwrap,
		    (!bIsTickerTape  &&  !bIsMarquee?"true":"false"));
	}
	if(!strlen(szWindowUnderlineHyperlinks))
	{
	    strcpy(szWindowUnderlineHyperlinks, "true");
	}
	if(!strlen(szWindowLinkColor))
	{
	    strcpy(szWindowLinkColor, "blue");
	}

	fprintf(stdout,"Expected average bit rate (bits per sec) of"
			" this stream\n"
		"  (hit 'enter' to default to %lubps):", 1000L);
	gets(szAvgBitrate);
	if(!strcmp(szAvgBitrate,"q"))
	{
	    return 0;
	}
	if(!strlen(szAvgBitrate))
	{
	    strcpy(szAvgBitrate, "1000");
	}

	fprintf(stdout, "\n");
    }
    else  /* argc==18 */
#endif /* end: ifndef _WIN16 */
    {
	int a = 0;
	strcpy(szHost, argv[++a]);
	strcpy(szPort, argv[++a]);
	strcpy(szUsername, argv[++a]);
	strcpy(szPassword, argv[++a]);
	strcpy(szStreamFilename, argv[++a]);
	strcpy(szTextInputFilename, argv[++a]);
	strcpy(szPollingPeriodInSeconds, argv[++a]);
	strcpy(szWindowType, argv[++a]);
	strcpy(szWindowWidth, argv[++a]);
	strcpy(szWindowHeight, argv[++a]);
	strcpy(szWindowBGColor, argv[++a]);
	strcpy(szWindowDoLoop, argv[++a]);
	strcpy(szWindowExtraSpaces, argv[++a]);
	strcpy(szWindowCrawlrate, argv[++a]);
	strcpy(szWindowScrollrate, argv[++a]);
	strcpy(szWindowDoWordwrap, argv[++a]);
	strcpy(szWindowUnderlineHyperlinks, argv[++a]);
	strcpy(szWindowLinkColor, argv[++a]);
	strcpy(szAvgBitrate, argv[++a]);
    }

    FILE* pTextInputFile = 0;
    struct stat statBuf;
    int statResult;

    /* Get data associated with input file: */
    statResult = stat(szTextInputFilename, &statBuf);
    /* Check if statistics are valid: */
    if(statResult != 0)
    {
	fprintf(stdout, "Problem getting information from input file.");
	return 0;
    }
    else
    {
	/* Output some of the statistics: */
	fprintf(stdout, "File size            : %ld\n", statBuf.st_size );
	fprintf(stdout, "Time modified        : %s\n",
		ctime( &statBuf.st_mtime ) );
    }


    if (!(hDllRtlive = LoadLibrary("rtli3260.dll")))
    {
#if defined(_MACINTOSH)
        if (!(hDllRtlive = LoadLibrary("rtlive60.dll")))
#else
        if (!(hDllRtlive = LoadLibrary("rtlive.dll")))
#endif
	{
#if defined(_MACINTOSH)  ||  defined(_UNIX)
	    fprintf(stdout, "Failed to load the 'rtlive' and/or 'encnet'\n"
			    "shared library\n");
#if !defined(_MACINTOSH)
	    fprintf(stdout, dlerror());
#endif
#else /* Windows (not Mac or Unix): */
	    fprintf(stdout, "Failed to load the 'rtli3260' DLL\n"
			    "Make sure both it and encn3260.dll are\n"
			    "located in a directory that is in you path");
#endif
	    return PNR_FAILED;
	}
    }

    /* retrieve the proc addresses from the module: */
    m_fpCreateLiveText = (FPRMCREATELIVETEXT) GetProcAddress(hDllRtlive,
	    "CreateLiveText");
    m_fpSetDLLAccessPath = (FPRMSETDLLACCESSPATH) GetProcAddress(hDllRtlive,
	    "SetDLLAccessPath");

    if(NULL != m_fpSetDLLAccessPath)
    {
	// Create a null delimited, double-null terminated string containing
	// the paths to the encnet library (DT_Common) and the sdpplin library
	// (DT_Plugins)...
	char pPaths[256];
	UINT32 ulPathLen = 0;
	char pNextPath[256];
#if defined(_MACINTOSH)
	const char* pCurrentDir = ".:";
#elif defined(_UNIX)
	const char* pCurrentDir = "./";
#else
	const char* pCurrentDir = ".\\";
#endif

	memset(pPaths, 0, 256);

	sprintf(pNextPath, "DT_Common=%s", pCurrentDir);
	memcpy(pPaths, pNextPath, strlen(pNextPath) + 1);
	ulPathLen = strlen(pNextPath) + 1;

	sprintf(pNextPath, "DT_Plugins=%s", pCurrentDir);
	memcpy(pPaths + ulPathLen, pNextPath, strlen(pNextPath) + 1);
	ulPathLen = strlen(pNextPath) + 1;

	m_fpSetDLLAccessPath((char*)pPaths);
    }

    if(NULL != m_fpCreateLiveText)
    {

	/* Creates the LiveText Encoder: */
	if(PNR_OK == m_fpCreateLiveText((IRMALiveText**) &pLiveTextEncoder))
	{
	    BOOL bNoPacketsSentYet = TRUE;

	    if((pTextInputFile = fopen(szTextInputFilename, "rt")) ==NULL)
	    {
		fprintf(stdout, "The file \"%s\" could not be opened\n",
			szTextInputFilename);
		pLiveTextEncoder->Release();
		FreeLibrary(hDllRtlive);
		hDllRtlive = NULL;
		return 0;
	    }
	    if(pTextInputFile)
	    {
		fclose(pTextInputFile);
	    }

	    ulPollingPeriodInMsec = atol(szPollingPeriodInSeconds) * 1000;
	    if(ulPollingPeriodInMsec < MIN_POLLING_PERIOD)
	    {
		ulPollingPeriodInMsec = MIN_POLLING_PERIOD;
	    }
	    if(ulPollingPeriodInMsec > MAX_POLLING_PERIOD)
	    {
		ulPollingPeriodInMsec = MAX_POLLING_PERIOD;
	    }

	    /* Let's set all the attributes related to the presentation: */
	    pLiveTextEncoder->SetType(szWindowType);
	    pLiveTextEncoder->SetWindowDimensions(
			atol(szWindowWidth), atol(szWindowHeight));
	    pLiveTextEncoder->SetBackgroundColor(szWindowBGColor);
	    pLiveTextEncoder->SetHyperlinkInfo(
		    /* Whether or not to underline hyperlinked text: */
		    't'==szWindowUnderlineHyperlinks[0]  ||
			    'T'==szWindowUnderlineHyperlinks[0]?TRUE:FALSE,
		    /* the color of the text inside <A ..>...</A>: */
		    szWindowLinkColor);
	    pLiveTextEncoder->SetTextMotion((ULONG32)atol(szWindowScrollrate),
		    (ULONG32)atol(szWindowCrawlrate));
	    pLiveTextEncoder->UseWordwrap(
		    't'==szWindowDoWordwrap[0]  ||
		    'T'==szWindowDoWordwrap[0]?TRUE:FALSE);
	    pLiveTextEncoder->SetDoLooping(
		    't'==szWindowDoLoop[0]  ||
		    'T'==szWindowDoLoop[0]?TRUE:FALSE);

	    if(PNR_OK == pLiveTextEncoder->QueryInterface(IID_IRMALiveText2,
		    (void**)&pLiveText2))
	    {
		/* We've got the newer interface that allows us to set the
		 * average bit rate and other new attributes:
		 */
		pLiveText2->DeclareAverageBitrate(atol(szAvgBitrate));
		pLiveText2->IgnoreExtraSpaces(
			'i'==szWindowExtraSpaces[0]  ||
			'I'==szWindowExtraSpaces[0]?TRUE:FALSE);

		/* The following can be used to cause run-time details NOT
		 * to be printed to stdout while this app is running.
		 * A command-line parameter could be added to this app to
		 * query the user for the value to set here.  If you want
		 * to enable messages, to which rtlive defaults, you can
		 * either comment out the following or call:
		 *   pLiveText2->SetFlags(ENABLE_RTLIVE_STDOUT_MESSAGES);
		 */
		/*
		pLiveText2->SetFlags(DISABLE_RTLIVE_STDOUT_MESSAGES);
		*/
	    }
	    
	    /* Now that the text window attributes are set, call
	     * InitLiveText() which initializes the encoder so it
	     * can communicate with the server:
	     */
	    PN_RESULT result;	    
	    if(PNR_OK != (result = pLiveTextEncoder->InitLiveText(
		    szHost,
		    atoi(szPort),
		    szUsername,
		    szPassword,
		    szStreamFilename)))
	    {
		fprintf(stdout, "Error: InitLiveText() returned %lx",
			(ULONG32)result);
		PN_RELEASE(pLiveTextEncoder);
		PN_RELEASE(pLiveText2);
		if(hDllRtlive)
		{
		    FreeLibrary(hDllRtlive);
		    hDllRtlive = NULL;
		}

		if(pTextInputFile)
		{
		    fclose(pTextInputFile);
		}
		return -1;
	    }

	    fprintf(stdout, "Entering mainloop\n");

	    ULONG32 ulLastTime = 0L;
	    ULONG32 curStreamTime = 0L;

	    /* The encoder will remain in this loop until the EncoderResponse
	     * object tells us to stop
	     */
	    while(!pLiveTextEncoder->EncoderIsDone())
	    {
		/* If we haven't gotten the InitEncoderResponseDone callback
		 * yet, then don't do anything but Process().
		 */
		if(pLiveTextEncoder->EncoderIsInitialized()  && 
			pLiveTextEncoder->PacketsHaveStarted())
		{
		    curStreamTime = pLiveTextEncoder->GetTime();
		    
/* *********** THIS IS SAMPLE CODE; ****************************** */
/* *********** DELETE THIS AND REPLACE WITH YOUR DATA ************ */
		    /* temporarily holds what's input in gets(): */
		    char pBuf[PBUF_SIZE+1];

		    /* send data every n milliseconds, where n is
		       DEFAULT_NUM_MSEC_BETWEEN_POLLS, #defined above: 
		     */
		    if(curStreamTime - ulLastTime > 
			    ulPollingPeriodInMsec ||
			    ulLastTime == 0L) 
		    {
			ulLastTime = curStreamTime;
			pBuf[0] = '\0';

			LONG32 prevFileTimeStamp = statBuf.st_mtime;
			LONG32 prevFileSize = statBuf.st_size;
			/* Get data associated with input file: */
			statResult = stat(szTextInputFilename, &statBuf);

			/* Check if statistics are valid: */
			if(statResult != 0)
			{
			    fprintf(stdout,"Problem getting file info\n");
			}

			if(prevFileTimeStamp != statBuf.st_mtime
				||  prevFileSize != statBuf.st_size
				||  bNoPacketsSentYet)
			{
			    if((pTextInputFile = fopen(szTextInputFilename,
				    "rt")) ==NULL)
			    {
				fprintf(stdout, 
					"The file \"%s\" can no longer"
					" be opened\n",
					szTextInputFilename);
			    }
			    else
			    {
				INT32 numread;
				BOOL bIsFirstTimeThroughDoLoop = TRUE;
				do
				{
				    numread = (INT32)fread(pBuf,sizeof(char),
					    PBUF_SIZE, pTextInputFile);
				    pBuf[numread]='\0';
				    if(bIsFirstTimeThroughDoLoop  &&
					    numread  &&  ulLastTime > 0L  &&
					    !bNoPacketsSentYet  &&
					    pBuf[0]==
					    END_OF_STREAM_REQUEST_CHAR)
				    {
					fprintf(stdout, "\nGoodbye!\n");
					pLiveTextEncoder->SetEncoderDone();
					break;
				    }
				    if(numread)
				    {
					pLiveTextEncoder->AddData(pBuf,
						TRUE);

					bNoPacketsSentYet = FALSE;
				    }
				    bIsFirstTimeThroughDoLoop = FALSE;
				}while(numread==PBUF_SIZE);

				fclose(pTextInputFile);
				pTextInputFile = NULL;
			    }
			}
			else
			{
			    fprintf(stdout,
				    "No file changes detected at %lu msec\n",
				    curStreamTime);
			}
		    }

/* *********** END OF SAMPLE CODE ******************************** */
/* *********** YOUR CODE GOES HERE:******************************* */
/* gather your data here into a NULL-terminated buffer (or 2 buffers
 * if you are running a tickertape) and call AddData() with your
 * buffer (or call AddTickerItem() with your upper and lower item
 * buffers).  Note: you can call AddData() with tickertape data, but
 * you'll have to put your own "<TU>" and "<TL>"s in it.
 */
		    /* Use TRUE in the last parameter of the following function
		     * to stream the contents of all unsent text immediately
		     * (regardless of how small the amount of text is), or use
		     * FALSE to let the encoder decide when to send it (based on
		     * elapsed time since last packet and/or size of buffer):
		     * NOTE: szBuffer is a char* buffer that you must create,
		     * initialize, maintain, and fill with data.  Also, it MUST
		     * be NULL-terminated before being sent to AddData().
		     * bSendImmediately is a BOOL value that you must create
		     * and initialize.
		     */

		    /* 
		    pLiveTextEncoder->AddData(szBuffer,
			    bSendImmediately);
		     */

		}

		
		
		/* Every user of the superencoder library must call 
		 * IRMAEncoderResponse::Process() regularly.
		 */
		if(PNR_OK != pLiveTextEncoder->Process())
		{
		    pLiveTextEncoder->SetEncoderDone();
		}
	    }		    
	}
    }
    else
    {
	fprintf(stdout, "Failed getting m_fpCreateLiveText()\n");
    } 

    if(pLiveTextEncoder)
    {
	pLiveTextEncoder->Release();
    }

    if(pLiveText2)
    {
	pLiveText2->Release();
    }

    if(hDllRtlive)
    {
	FreeLibrary(hDllRtlive);
	hDllRtlive = NULL;
    }

    if(pTextInputFile)
    {
	fclose(pTextInputFile);
    }

    return 0;
}
