/****************************************************************************
 * 
 *  $Id: exmonpln.h,v 1.15 1998/12/09 22:27:12 darrens Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 *
 *  exmonpln.cpp
 *
 *  Example Monitor plug-in for RealMedia Architecture (RMA).
 *
 */
#ifndef _EXMON_PLUGIN_H_
#define _EXMON_PLUGIN_H_


/****************************************************************************
 * Constants
 */
#ifndef MAX_DISPLAY_NAME
    #define MAX_DISPLAY_NAME    80
#endif
#define MAX_PROP_LEN        32767


/****************************************************************************
 * Forward declarations
 */
struct IUnknown;
struct IRMAPlugin;
struct IRMAPNRegistry;
struct IRMABuffer;
struct IRMAValues;

struct WatchProp
{
    char	    pName[MAX_DISPLAY_NAME];
    UINT32	    id;
    IRMAValues*     elem;
    INT32	    t_int;
    char*	    t_str;
};

STDAPI RMACreateInstance(IUnknown** /*OUT*/);  


/****************************************************************************
 *
 *  CExampleMonitor Class
 *
 *  Example implementation of a Monitor plug-in.
 */
class CExampleMonitor : public IRMAPropWatchResponse,
			public IRMAPlugin
{
    private:
    /****** Private Class Variables ****************************************/
    INT32			m_lRefCount;
    INT32			m_nPlayers;
    IUnknown*			m_pContext;
    IRMAPNRegistry*		m_pRegistry;
    IRMAPropWatch*		m_pPropWatch;
    IRMAScheduler*		m_pScheduler;
    IRMAErrorMessages*		m_pLogger;
    WatchProp			m_ServerPropList[2];
    WatchProp			m_ClientPropList[1];
   
    /****** Private Static Class Variables *********************************/
    static const char*		zm_pDescription;
    static const char*		zm_pCopyright;
    static const char*		zm_pMoreInfoURL;
  

    /****** Private Classes  ***********************************************/
    class ConnNode
    {
        public:
        ConnNode(ConnNode*  prev,
                 UINT32     id)
        {        
            m_id = id;
            m_pszIPAddr = NULL;
            m_ulUDPResendReqs = 0;
	    m_hCallback = 0;
            m_pNext = NULL;
        
            if (prev)
            {
                prev->m_pNext = this;
            }
        };

        ~ConnNode()
        {
            if (m_pszIPAddr) delete m_pszIPAddr;
        }

        UINT32		m_id;
        char*		m_pszIPAddr;
        UINT32		m_ulUDPResendReqs;
	CallbackHandle	m_hCallback;
        ConnNode*	m_pNext;
    };

    class ConnList
    {
        public:
        ConnList()
        {
            m_pHead    = NULL;
            m_pCursor  = NULL; 
            m_pTail    = NULL;
        };

        ~ConnList()
        {
            while (m_pHead)
            {
                m_pCursor = m_pHead->m_pNext;
                delete m_pHead;
                m_pHead = m_pCursor;
            }
        }

        void RemoveNode(UINT32 id)
        {
            ConnNode* pPrev = NULL;
    
            pPrev = m_pCursor = m_pHead;
            while (m_pCursor)
            {
                if (m_pCursor->m_id == id)
                {
                    if (m_pCursor == m_pHead)
                    {
                        m_pHead = m_pCursor->m_pNext;
                
                        if (!m_pHead)   m_pTail = NULL;
                    }
                    else if (m_pCursor == m_pTail)
                    {         
                       m_pTail = pPrev;
                       m_pTail->m_pNext = NULL;
                    }
                    else
                    {
                       pPrev->m_pNext = m_pCursor->m_pNext;
                    }

		    delete m_pCursor;
                
                    break;
                }

                pPrev = m_pCursor;
                m_pCursor = m_pCursor->m_pNext;
            }
        }

        ConnNode* AddNode(UINT32 id)
        {
            ConnNode* pNode = new ConnNode(m_pTail, id);

            m_pTail = pNode;
            if (!m_pHead)  m_pHead = pNode;
            if (!m_pCursor) m_pCursor = m_pHead;

	    return pNode;
        }

        ConnNode* FindNode(UINT32 id)
        {
            ConnNode* pNode = NULL;

            m_pCursor = m_pHead;
            while (m_pCursor)
            {
                if (m_pCursor->m_id == id)
                {
                    pNode = m_pCursor;
                    break;
                }

                m_pCursor = m_pCursor->m_pNext;
            }

            return pNode;
        }
                          
        ConnNode* GetNextNode()
        {
            ConnNode* pNode = m_pCursor;
         
            if (m_pCursor)
            {
                m_pCursor = m_pCursor->m_pNext;
            }
            return pNode;
        }

	ConnNode* Cursor()
	{
	    return m_pCursor;
	}

        void ResetCursor()
        {      
            m_pCursor = m_pHead;
        }
            
        private:
        ConnNode* m_pHead;
        ConnNode* m_pTail;
        ConnNode* m_pCursor;
    };
 

    /****** Private Class Methods ******************************************/
    ~CExampleMonitor() { };

    PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME       // Avoids GCC compiler warning

    STDMETHOD(Close) 		(THIS);
    STDMETHOD(SetWatches)	(THIS);

    public:
    /****** Public Class Variables ****************************************/
    ConnList*                   m_pConnList;


    /****** Public Class Methods ******************************************/
    CExampleMonitor(IRMAPNRegistry* registry = 0)
		: m_lRefCount(0)
		, m_pRegistry(registry)
		, m_pPropWatch(NULL)
		, m_pScheduler(NULL)
		, m_pLogger(NULL)
		, m_pConnList(NULL)
    {
    };
    STDMETHOD(UpdateDisplay)	(THIS);

				
    /************************************************************************
     *  IRMAPropWatchResponse Interface Methods                ref:  rmamon.h
     */
    STDMETHOD(AddedProp)
	(THIS_
	 const UINT32		id,
	 const RMAPropType	propType,
	 const UINT32		ulParentID
	);
    STDMETHOD(ModifiedProp)
	(THIS_
	 const UINT32		id,
	 const RMAPropType	propType,
	 const UINT32		ulParentID
	);
    STDMETHOD(DeletedProp)
	(THIS_
	 const UINT32		id,
	 const UINT32		ulParentID
	);


    /************************************************************************
     *  IRMAPlugin Interface Methods                         ref:  rmaplugn.h
     */
    STDMETHOD(GetPluginInfo)
        ( THIS_ 
          REF(BOOL)        bLoadMultiple,
          REF(const char*) pDescription,
          REF(const char*) pCopyright,
          REF(const char*) pMoreInfoURL,
          REF(UINT32)      versionNumber
        );
    STDMETHOD(InitPlugin) (THIS_ IUnknown* pContext);


    /************************************************************************
     *  IUnknown COM Interface Methods                          ref:  pncom.h
     */
    STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
    STDMETHOD_(UINT32, AddRef ) (THIS);
    STDMETHOD_(UINT32, Release) (THIS);
};



/****************************************************************************
 *
 *  Poll Class
 *
 */
class Poll : public IRMACallback
{
    public:
    /****** Public Class Methods ******************************************/
    Poll(CExampleMonitor*   pMonitor,
	 IRMAPNRegistry*    pRegistry,
	 IRMAScheduler*	    pScheduler,
	 char*		    pszConnName);


   /************************************************************************
    *  IRMACallback Interface Methods                       ref:  rmaengin.h
    */
    STDMETHOD(Func) (THIS);


   /************************************************************************
    *  IUnknown COM Interface Methods                          ref:  pncom.h
    */
    STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
    STDMETHOD_(UINT32, AddRef ) (THIS);
    STDMETHOD_(UINT32, Release) (THIS);


    private:
    /****** Private Class Variables ****************************************/
    INT32		m_lRefCount;
    CExampleMonitor*	m_pMonitor;
    IRMAPNRegistry*	m_pRegistry;
    IRMAScheduler*	m_pScheduler;
    char*		m_pszConnName;
    UINT32		m_id;

    /****** Private Class Methods ******************************************/
    ~Poll();

    PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME       // Avoids GCC compiler warning
};


#endif // _EXMON_PLUGIN_H_
