#include <stdio.h>
#include <stdlib.h>
#include "pntypes.h"
#include "pnwintyp.h"
#include "pncom.h"
#include "rmaslta.h"
#include "os.h"





typedef PN_RESULT (PNEXPORT_PTR FPRMSETDLLACCESSPATH) (const char*);
typedef IRMASLTA* (PNEXPORT_PTR FPCREATE_SLTA)();







void ShowSyntax()
{
    printf("USAGE:\nexslta <host> <port> <user> <pass> <livefile> <localfile> [target bandwith] [title] [author] [copyright]\n");
}





int main (int argv, char* argc[])
{
    HINSTANCE            hSLTADll = NULL;
    FPCREATE_SLTA        fpCreateSLTA = NULL;
    IRMASLTA*            pSlta = NULL;
    FPRMSETDLLACCESSPATH fpSetDLLAccessPath = NULL;
#ifdef _UNIX
    char*		 rphome = NULL;
#endif

    // check number of params
    if (argv < 7 || argv > 11)
    {
        ShowSyntax();
        return 0;
    }

    const char* pHost       = argc[1];
    int         nPort       = atoi(argc[2]);
    const char* pUser       = argc[3];
    const char* pPass       = argc[4];
    const char* pLive       = argc[5];
    const char* pFile       = argc[6];
    int         nBps        = 0;
    const char* pAuthor     = NULL;
    const char* pTitle      = NULL;
    const char* pCopyright  = NULL;

    // optional params
    if (argv > 7)
        nBps = atoi(argc[7]);
    if (argv > 8)
        pAuthor = argc[8];
    if (argv > 9)
        pTitle = argc[9];
    if (argv > 10)
        pCopyright = argc[10];

#ifdef _WINDOWS
    char* pSltaName = "slta3260.dll";
#else
    char* pSltaName = "sltalib.so.6.0";
#endif

    const char* pSupPath = getenv("EXSLTA_SUPPORT_PATH");
    const char* pPlugPath = getenv("EXSLTA_PLUGIN_PATH");
    char pPath[_MAX_PATH + 32];

    if (!pSupPath || pSupPath[0] == 0)
    {
	pSupPath = "./";
    }
    if (pPath[sprintf(pPath, "%s", pSupPath) - 1] != '/')
    {
	strcat(pPath, "/");
    }
    strcat(pPath, pSltaName);


    // Load the SLTA library    
    if (!(hSLTADll = LoadLibrary(pPath))) 
    {
 	printf("Failed to load the %s shared library\n", pSltaName);
        return 0;
    }

    // Setup the dll paths for the SLTA library
    fpSetDLLAccessPath = (FPRMSETDLLACCESSPATH) GetProcAddress(hSLTADll, "SetDLLAccessPath");
    if (fpSetDLLAccessPath)
    {
        sprintf(pPath, "DT_Plugins=%s", pPlugPath ? pPlugPath : ".");
	fpSetDLLAccessPath(pPath);

        sprintf(pPath, "DT_Common=%s", pSupPath ? pSupPath : ".");
	fpSetDLLAccessPath(pPath);
    }

    // Create our SLTA object
    fpCreateSLTA = (FPCREATE_SLTA)GetProcAddress(hSLTADll, "CreateSLTA");
    if (fpCreateSLTA)
    {
        pSlta = fpCreateSLTA();

        if (pSlta)
        {
            // Set the TAC
            if (pTitle || pAuthor || pCopyright)
            {
	        pSlta->SetTAC(pTitle, pAuthor, pCopyright);
            }

            // Set the bandwidth
            pSlta->SetTargetBandwidth(nBps);

            // Connect and encode the file
            if(pSlta->Connect(pHost, nPort, pUser, pPass, pLive) == PNR_OK)
            {
                PN_RESULT res = pSlta->Encode(pFile);
                switch (res)
                {
                case PNR_FILE_NOT_FOUND:
		    printf("file not found: %s.\n", pFile);
                    break;

                case PNR_SERVER_DISCONNECTED:
		    printf("Server disconnected.\n");
                    break;

                case PNR_NO_FILESYSTEM:
		    printf("Missing filesystem plugin.\n");
                    break;

                case PNR_NO_FILEFORMAT:
		    printf("Missing fileformat plugin.\n");
		    break;

                case PNR_OK:
                    printf("Successfully encoded file.\n");
                    break;

                default:
		    printf("Error while encoding %s.\n", pFile);
                    break;
                }
            }
            else
            {
                printf("could not connect to server\n");
            }

	    pSlta->Disconnect();
            pSlta->Release();
        }
        else
        {
            printf("unknown error in %s\n", pSltaName);
        }
    }
    else
    {
        printf("unknown error in %s\n", pSltaName);
    }

    if (hSLTADll)
    {
        FreeLibrary(hSLTADll);
    }

    return 0;
}

