/****************************************************************************
 * 
 *  $Id: exvsrndr.cpp,v 1.4 1998/11/13 00:53:51 jfarr Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 *
 *  exvsrndr.cpp
 *
 *  Example Renderer for RealMedia Architecture (RMA).
 *
 */


/****************************************************************************
 * Defines
 */
#define INITGUID     /* Interface ID's */
#define STRICT


/****************************************************************************
 * Includes
 */
#include "pntypes.h"

#include <stdio.h>

#ifdef _WINDOWS
    #include <windows.h>
#endif

#ifdef _UNIX
    #include <stdlib.h>
    #include <unistd.h>
    #include <X11/Intrinsic.h>
    #include <X11/StringDefs.h>
    #include <X11/Shell.h>
    #include <X11/Xaw/Label.h>
#endif

#include "pnwintyp.h"
#include "pncom.h"
#include "rmacomm.h"
#include "rmapckts.h"
#include "rmafiles.h"
#include "rmacore.h"
#include "rmarendr.h"
#include "rmahyper.h"
#include "rmaplugn.h"
#include "rmawin.h"
#include "rmaasm.h"
#include "rmaevent.h"
#include "rmavsurf.h"
#include "rmasite2.h"

#include "exvsrndr.h"
#include "exvsrndr.ver"



/****************************************************************************
 *  RMACreateInstance                                        ref:  rmaplugn.h
 *
 *  This routine creates a new instance of the CExampleRenderer class.
 *  It is called when the RMA core application is launched, and whenever
 *  a stream associated with this plug-in needs to be rendered.
 */
STDAPI
RMACreateInstance(IUnknown** ppIUnknown)
{
    *ppIUnknown = (IUnknown*)(IRMAPlugin*)new CExampleRenderer();
    if (*ppIUnknown)
    {
	(*ppIUnknown)->AddRef();
	return PNR_OK;
    }
    return PNR_OUTOFMEMORY;
}


/****************************************************************************
 *  CExampleRenderer static variables                       ref:  exvsrndr.h
 *
 *  These variables are passed to the RMA core to provide information about
 *  this plug-in. They are required to be static in order to remain valid
 *  for the lifetime of the plug-in.
 */
const char* CExampleRenderer::zm_pDescription    = "Example Renderer Plugin";
const char* CExampleRenderer::zm_pCopyright      = "(c) 1995,1996,1997 Real Networks, All rights reserved.";
const char* CExampleRenderer::zm_pMoreInfoURL    = "http://www.real.com";
const char* CExampleRenderer::zm_pStreamMimeTypes[] = {"application/x-pn-examplestream", NULL};


/****************************************************************************
 *  IRMAPlugin::GetPluginInfo                                ref:  rmaplugn.h
 *
 *  This routine returns descriptive information about the plug-in, most
 *  of which is used in the About box of the user interface. It is called
 *  when the RMA core application is launched.
 */
STDMETHODIMP
CExampleRenderer::GetPluginInfo
(
    REF(BOOL)        /*OUT*/ bLoadMultiple,
    REF(const char*) /*OUT*/ pDescription,
    REF(const char*) /*OUT*/ pCopyright,
    REF(const char*) /*OUT*/ pMoreInfoURL,
    REF(UINT32)      /*OUT*/ ulVersionNumber
)
{
    bLoadMultiple   = TRUE;
    pDescription    = zm_pDescription;
    pCopyright	    = zm_pCopyright;
    pMoreInfoURL    = zm_pMoreInfoURL;
    ulVersionNumber = TARVER_ULONG32_VERSION;

    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::GetRendererInfo                            ref:  rmarendr.h
 *
 *  This routine returns crucial information required to associate this
 *  plug-in with a given stream MIME type. This information tells the RMA
 *  core which Renderer to use to display a particular type of stream. This
 *  method is called when the RMA core application is launched.
 */
STDMETHODIMP
CExampleRenderer::GetRendererInfo
(
    REF(const char**) /*OUT*/ pStreamMimeTypes,
    REF(UINT32)       /*OUT*/ unInitialGranularity
)
{
    pStreamMimeTypes = zm_pStreamMimeTypes;
    unInitialGranularity = 100; // OnTimeSync() every 100ms

    return PNR_OK;
}


/****************************************************************************
 *  IRMAPlugin::InitPlugin                                   ref:  rmaplugn.h
 *
 *  This routine performs initialization steps such as determining if
 *  required interfaces are available. It is called when the RMA core 
 *  application is launched, and whenever a stream associated with this
 *  plug-in needs to be rendered.
 */
STDMETHODIMP
CExampleRenderer::InitPlugin(IUnknown* /*IN*/ pContext)
{
    m_pContext = pContext;
    m_pContext->AddRef();

    m_pContext->QueryInterface(IID_IRMAHyperNavigate, (void**) &m_pHyperNavigate);
    m_pContext->QueryInterface(IID_IRMACommonClassFactory,(void**)&m_pCommonClassFactory);

    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::StartStream                                ref:  rmarendr.h
 *
 *  The RMA core calls this routine to provide access to the stream and
 *  player. It is called when the plug-in is being initialized.
 */
STDMETHODIMP
CExampleRenderer::StartStream
(
    IRMAStream*	    pStream,
    IRMAPlayer*	    pPlayer
)
{
    m_pStream  = pStream;
    m_pPlayer  = pPlayer;

    if (m_pStream ) m_pStream->AddRef();
    if (m_pPlayer ) m_pPlayer->AddRef();

    m_pCommonClassFactory->CreateInstance(CLSID_IRMAMultiInstanceSiteUserSupplier,
	    							(void**)&m_pMISUS);

    m_pMISUS->SetSingleSiteUser((IUnknown*)(IRMASiteUser*)this);

    if (m_pStream)
    {
	IRMAStreamSource* pSource = 0;
	if (m_pStream->GetSource(pSource) == PNR_OK)
	{
	    /* It is OK if the source does not support backchannel. Reasons:
	     *
	     * 1. This stream may not be coming from example fileformat.
	     *	  It may instead be merged into a container fileformat which
	     *	  may be does not support BackChannel.
	     *
	     * 2. The protocol used to serve this stream may not support 
	     *	  BackChannel.
	     */
	    pSource->QueryInterface(IID_IRMABackChannel, (void**) &m_pBackChannel);
	    
	    pSource->Release();
	}

	/* Is ASM supported on this stream? */
	m_pStream->QueryInterface(IID_IRMAASMStream, (void**) &m_pASMStream);
    }

    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnHeader                                   ref:  rmarendr.h
 *
 *  This routine is passed the stream header object created by the associated
 *  file format plug-in. Use the methods defined in the IRMAValues interface
 *  to access the stream header data. This method is called from the RMA
 *  core when the plug-in is being initialized.
 */
STDMETHODIMP
CExampleRenderer::OnHeader(IRMAValues* pHeader)
{
    // Keep this for later use...
    m_pHeader = pHeader;
    m_pHeader->AddRef();

    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnBegin                                   ref:  rmarendr.h
 *
 *  This routine is called by the RMA core to inform the Renderer that 
 *  playback has just begun or has been resumed. The stream's time value just
 *  after resuming the playback is provided.
 */
STDMETHODIMP
CExampleRenderer::OnBegin(UINT32 ulTime)
{
    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::GetDisplayType                             ref:  rmarendr.h
 *
 *  This routine returns the preferred display type of the Renderer. Any
 *  other additional information necessary for the display can also be
 *  provided. This method is called by the RMA core when the plug-in is being
 *  initialized.
 */
STDMETHODIMP
CExampleRenderer::GetDisplayType
(
    REF(RMA_DISPLAY_TYPE)   ulFlags,
    REF(IRMABuffer*)	    pBuffer
)
{
    ulFlags = RMA_DISPLAY_WINDOW;

    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnPacket                                   ref:  rmarendr.h
 *
 *  This routine is passed the packet object streamed by the associated
 *  file format plug-in. It is called by the RMA core when a packet is due
 *  to be delivered. In most cases, the actual rendering of the packet is
 *  done in this method. The stream's time offset with respect to the master
 *  timeline is provided.
 */
STDMETHODIMP
CExampleRenderer::OnPacket
(
    IRMAPacket* pPacket,
    INT32 lTimeOffset
)
{
    /* Ignore pre-seek packets. Some renderers may want to use them though */
    if (m_bInSeekMode)
    {
	return PNR_OK;
    }

    // Release the last packet if we had one...
    if (m_pLastPacket)
    {
        m_pLastPacket->Release();
    }

    // Keep this one for later use...
    m_pLastPacket = pPacket;

    if (m_pLastPacket)
    {
	m_pLastPacket->AddRef();
    }

    // NOTE: If your stream has been offset in time due to start, end, or
    // groupstart properties on the stream, you may be getting packets which
    // need to be adjusted before writing into the audio services, or before
    // comparing against the time line. Use the following code to determine
    // to offset of this packet's delivery time in the real timeline.
    UINT32 ulPacketDeliveryTime = pPacket->GetTime(); 
    UINT32 ulRealDeliveryTime;
 
    if ((UINT32)lTimeOffset > ulPacketDeliveryTime)
    {
        ulRealDeliveryTime = 0;
    }
    else
    {
	ulRealDeliveryTime = ulPacketDeliveryTime - lTimeOffset;
    }

    // For example purposes we remember this information
    m_ulLastPacketTime = ulPacketDeliveryTime;
    m_ulLastRealPacketTime = ulRealDeliveryTime;

    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnTimeSync                                 ref:  rmarendr.h
 *
 *  This routine is called by the RMA core periodically passing the current
 *  playback time. The Renderer should use this time value to synchronize the
 *  playback of various streams.
 */
STDMETHODIMP
CExampleRenderer::OnTimeSync(UINT32 ulTime)
{
    // Here's a good time to actually render the data!

    // Remember the time we are!
    m_ulLastTime = ulTime;

    // Redraw our data by damaging the entire area of our data
    PNxRect damageRect = {0, 0, m_size.cx, m_size.cy };
    m_pMISUSSite->DamageRect(damageRect);
    m_pMISUSSite->ForceRedraw();

    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnPreSeek                                  ref:  rmarendr.h
 *
 *  This routine is called by the RMA core before a seek is about to occur.
 *  The stream's time value just before the seek, and the time value just
 *  after the seek are provided.
 */
STDMETHODIMP
CExampleRenderer::OnPreSeek(UINT32 ulOldTime, UINT32 ulNewTime)
{
    m_bInSeekMode   = TRUE;
    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnPostSeek                                 ref:  rmarendr.h
 *
 *  This routine is called by the RMA core just after a seek has occured.
 *  The stream's time value just before the seek, and the time value just
 *  after the seek are provided.
 */
STDMETHODIMP
CExampleRenderer::OnPostSeek(UINT32 ulOldTime, UINT32 ulNewTime)
{
    m_bInSeekMode   = FALSE;
    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnPause                                    ref:  rmarendr.h
 *
 *  This routine is called by the RMA core just after a pause has occured.
 *  The stream's time value just before pausing is provided.
 */
STDMETHODIMP
CExampleRenderer::OnPause(UINT32 ulTime)
{
    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnBuffering                                ref:  rmarendr.h
 *
 *  This routine is called by the RMA core to inform the Renderer that
 *  buffering of data is occuring. The reason for buffering (e.g. start-up
 *  of stream, seek has occured, network congestion, etc.), as well as the
 *  percentage of the buffering process complete are provided.
 */
STDMETHODIMP
CExampleRenderer::OnBuffering(UINT32 ulFlags, UINT16 unPercentComplete)
{
    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnEndofPackets                             ref:  rmarendr.h
 *
 *  Called by client engine to inform the renderer that all the
 *  packets have been delivered. However, if the user seeks before
 *  EndStream() is called, renderer may start getting packets again
 *  and the client engine will eventually call this function again.
 */
STDMETHODIMP
CExampleRenderer::OnEndofPackets(void)
{
    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::EndStream                                  ref:  rmarendr.h
 *
 *  This routine is called by the RMA core when the rendering of the stream
 *  has completed. Deallocation of any resources should be done here.
 */
STDMETHODIMP
CExampleRenderer::EndStream()
{
    /* Send report of a mouse click to the fileformat if backchannel 
     * is supported
     */
    if (m_pBackChannel && m_pCommonClassFactory && m_pStream)
    {
	IRMAPacket* pPacket = 0;
	IRMABuffer* pBuffer = 0;
	char tmpStr[100];
	if ((PNR_OK == m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,
					       (void**)&pBuffer)) &&
	    (PNR_OK == m_pCommonClassFactory->CreateInstance(CLSID_IRMAPacket,
					       (void**)&pPacket)))
	{
	    ::sprintf(tmpStr, "Number of clicks = %lu\n", m_ulNumberOfClicks);

	    pBuffer->Set((UCHAR*) tmpStr, ::strlen(tmpStr) + 1);
	    pPacket->Set(pBuffer,0,m_pStream->GetStreamNumber(),
			 RMA_ASM_SWITCH_ON, 0);
	    pBuffer->Release();

	    m_pBackChannel->PacketReady(pPacket);

	    pPacket->Release();
	}

    }

    // You can't use IRMAStream, IRMASourceStream, IRMABackChannel, or
    // IRMAASMStream after EndStream has been called.
    PN_RELEASE(m_pStream);
    PN_RELEASE(m_pBackChannel);
    PN_RELEASE(m_pASMStream);

    // We are done drawing now, so we can call EndOptimizedBlt
    CleanupVideoSurface();

    return PNR_OK;
}

/****************************************************************************
 *  CExampleRenderer::CExampleRenderer                      ref:  exvsrndr.h
 *
 *  Constructor
 */
CExampleRenderer::CExampleRenderer()
    : m_lRefCount(0)
    , m_pContext(NULL)
    , m_pStream(NULL)
    , m_pPlayer(NULL)
    , m_pHeader(NULL)
    , m_pLastPacket(NULL)
    , m_ulLastTime(0)
    , m_pHyperNavigate(0)
    , m_pBackChannel(0)
    , m_pASMStream(0)
    , m_pCommonClassFactory(0)
    , m_ulNumberOfClicks(0)
    , m_bIsSubscribed(TRUE)
    , m_pMISUS(NULL)
    , m_pMISUSSite(NULL)
    , m_bInSeekMode(FALSE)
    #ifdef _UNIX
    , m_widget(NULL)
    #endif
    , m_pEvent(NULL)
    , m_nRed(0)
    , m_nGreen(0)
    , m_nBlue(0)
{
    // Initialize Bitmap Info Header
    m_BitmapInfoHeader.biSize = sizeof(RMABitmapInfoHeader) - (sizeof(UINT32)*3);
    m_BitmapInfoHeader.biWidth		= 0;
    m_BitmapInfoHeader.biHeight		= 0;
    m_BitmapInfoHeader.biPlanes		= 1;
    m_BitmapInfoHeader.biBitCount	= 0;
    m_BitmapInfoHeader.biBitCount	= 24;
    m_BitmapInfoHeader.biCompression	= RMA_RGB;
    m_BitmapInfoHeader.biSizeImage	= 0;
    m_BitmapInfoHeader.biXPelsPerMeter	= 0;
    m_BitmapInfoHeader.biYPelsPerMeter	= 0;
    m_BitmapInfoHeader.biClrUsed	= 0;
    m_BitmapInfoHeader.biClrImportant	= 0;
}

/****************************************************************************
 *  CExampleRenderer::~CExampleRenderer                      ref:  exvsrndr.h
 *
 *  Destructor
 */
CExampleRenderer::~CExampleRenderer()
{
    // NOTE: You should do your renderer cleanup here, instead of
    // in EndStream(), because your renderer is still left around
    // after the stream is ended in case it is a display renderer
    // and it needs to "paint" it's display area.

    PN_RELEASE(m_pHeader);
    PN_RELEASE(m_pLastPacket);
    PN_RELEASE(m_pContext);
    PN_RELEASE(m_pPlayer);
    PN_RELEASE(m_pHyperNavigate);
    PN_RELEASE(m_pCommonClassFactory);
}


/****************************************************************************
 *  CExampleRenderer::Draw                                   ref:  exvsrndr.h
 *
 */
#define MAX_PACKET_SIZE	    400 
void CExampleRenderer::Draw()
{
    // Add draw code here!
    UINT32	i = 0;
    UCHAR*	pImageData = NULL;
    PNxSize	rSize;
    PNxRect	rSrcRect, rDestRect;
    IRMAVideoSurface* pVideoSurface = NULL;

    // Get the size of the site
    if (m_pMISUSSite)
    {
	m_pMISUSSite->GetSize(rSize);
    }
    rSrcRect.left = 0;
    rSrcRect.top = 0;
    rSrcRect.right = rSize.cx;
    rSrcRect.bottom = rSize.cy;
    
    rDestRect.left = 0;
    rDestRect.top = 0;
    rDestRect.right = rSize.cx;
    rDestRect.bottom = rSize.cy;
    
    // Allocate our image buffer and fill it with color
    pImageData = new UCHAR[m_BitmapInfoHeader.biSizeImage];
    
    // Set the red pixel values
    for (i = 0; i < m_BitmapInfoHeader.biSizeImage; i += 3)
    {
	pImageData[i] = m_nRed;
    }
    // Set the blue pixel values
    for (i = 1; i < m_BitmapInfoHeader.biSizeImage; i += 3)
    {
	pImageData[i] = m_nGreen;
    }
    // set the green pixel values
    for (i = 2; i < m_BitmapInfoHeader.biSizeImage; i += 3)
    {
	pImageData[i] = m_nBlue;
    }
    m_nRed += 5;
    m_nGreen += 11;
    m_nBlue += 2;

    // Obtain a pointer to the video surface
    pVideoSurface = (IRMAVideoSurface*)m_pEvent->param1;
    pVideoSurface->AddRef();

    // Pass our image data to the video surface to blt
    m_pEvent->result = 
	pVideoSurface->OptimizedBlt(pImageData, rDestRect, rSrcRect);
    
    PN_RELEASE(pVideoSurface);
    delete[] pImageData;
    pImageData = NULL;
}


/****************************************************************************
 *  CExampleRenderer::HandleClick                            ref:  exvsrndr.h
 *
 */
void CExampleRenderer::HandleClick()
{
    m_ulNumberOfClicks++;

    if (m_pHyperNavigate)
    {
	m_pHyperNavigate->GoToURL("http://www.real.com", NULL);
    }

    if (m_pASMStream)
    {
	/* We know there are only three rules. On each mouse click, we toggle
	 * on subscription of rule 2. Rule 0/1 are automatic (based on
 	 * the current packet loss).
	 */
	if (m_bIsSubscribed)
	{
	    m_bIsSubscribed = FALSE;
	    m_pASMStream->Unsubscribe(2);
	}
	else
	{
	    m_bIsSubscribed = TRUE;
	    m_pASMStream->Subscribe(2);
	}
    }
}


// IRMASiteUser Interface Methods

/****************************************************************************
 *  IRMASiteUser::AttachSite                                   ref:  rmawin.h
 *
 */
STDMETHODIMP
CExampleRenderer::AttachSite(IRMASite* /*IN*/ pSite)
{
    if (m_pMISUSSite) return PNR_UNEXPECTED;
    m_pMISUSSite = pSite;
    m_pMISUSSite->AddRef();

    /*
     * This is the best time to set the size of the site, we
     * know for example the the header has already been received
     * at this point.
     *
     * In most display renderers, you will store size information
     * in your stream header. In this example, we assume a specific
     * size, but we will initialize that size here as if it had
     * come from our header.
     *
     */
    m_size.cx = 400;
    m_size.cy = 100;

    // Set up the bitmap info header for the size and bit depth
    // we will be drawing
    m_BitmapInfoHeader.biWidth	= m_size.cx;
    m_BitmapInfoHeader.biHeight = m_size.cy;
#ifdef _MACINTOSH
    m_BitmapInfoHeader.biBitCount = 32;
#else
    m_BitmapInfoHeader.biBitCount = 24;
#endif
    m_BitmapInfoHeader.biSizeImage = m_BitmapInfoHeader.biWidth *
				     m_BitmapInfoHeader.biHeight *
				     m_BitmapInfoHeader.biBitCount / 8;

    // Now is a good time to call BeginOptimizedBlt and set
    // up our video surface for drawing
    NegotiateImageFormat();

    m_pMISUSSite->SetSize(m_size);
    
    return PNR_OK;
}

void
CExampleRenderer::NegotiateImageFormat()
{
    PN_RESULT	      hResult	    = PNR_OK;
    IRMASite2*	      pMISUSSite2   = NULL;
    IRMAVideoSurface* pVideoSurface = NULL;

    m_pMISUSSite->QueryInterface(IID_IRMASite2, (void**)&pMISUSSite2);
    
    if (pMISUSSite2)
    {
	pMISUSSite2->GetVideoSurface(pVideoSurface);
	if (pVideoSurface)
	{
	    // Find out if video surface supports our desired format
	    hResult = pVideoSurface->BeginOptimizedBlt(&m_BitmapInfoHeader);
	    if (PNR_OK != hResult)
	    {
		// If not, find out what format the surface would prefer
		RMA_COMPRESSION_TYPE ulPreferredFormat = RMA_RGB;
		hResult = pVideoSurface->GetPreferredFormat(ulPreferredFormat);

		// We can call BeginOptimizedBlt again with the preferred
		// format, and then remember to convert our data before 
		// calling OptimizedBlt()...
	    }
	}
    }

    PN_RELEASE(pMISUSSite2);
    PN_RELEASE(pVideoSurface);
}

void 
CExampleRenderer::CleanupVideoSurface()
{
    IRMASite2* pMISUSSite2 = NULL;
    IRMAVideoSurface* pVideoSurface = NULL;

    if (m_pMISUSSite)
    {
	m_pMISUSSite->QueryInterface(IID_IRMASite2, (void**)&pMISUSSite2);
    
	if (pMISUSSite2)
	{
	    pMISUSSite2->GetVideoSurface(pVideoSurface);
	    if (pVideoSurface)
	    {
		pVideoSurface->EndOptimizedBlt();
		PN_RELEASE(pVideoSurface);
	    }
	}
    }

    PN_RELEASE(pMISUSSite2);
}


/****************************************************************************
 *  IRMASiteUser::DetachSite                                   ref:  rmawin.h
 *
 */
STDMETHODIMP
CExampleRenderer::DetachSite()
{
    PN_RELEASE(m_pMISUSSite);

    // We're done with these...
    if (m_pMISUS) m_pMISUS->ReleaseSingleSiteUser();
    PN_RELEASE(m_pMISUS);

    return PNR_OK;
}


/****************************************************************************
 *  IRMASiteUser::NeedsWindowedSites                           ref:  rmawin.h
 *
 */
STDMETHODIMP_(BOOL)
CExampleRenderer::NeedsWindowedSites() 
{ 
    return FALSE; 
};


/****************************************************************************
 *  IRMASiteUser::HandleEvent                                  ref:  rmawin.h
 *
 */
STDMETHODIMP
CExampleRenderer::HandleEvent(PNxEvent* /*IN*/ pEvent)
{
    pEvent->handled = FALSE;
    pEvent->result  = 0;

    #ifdef _UNIX
    XEvent* xevent = (XEvent*)pEvent->param1;
    Widget oldWidget = m_widget;
    m_widget = (Widget)pEvent->window;
    if(!oldWidget)
    {
	XGCValues values;
	Font font;
	XFontStruct* fontinfo;
	fontinfo = XLoadQueryFont(XtDisplay((Widget)m_widget), "6x10");
	values.font = fontinfo->fid;
	m_gc = XtGetGC((Widget)m_widget,
		       GCFont,
		       &values);
    }
    #endif

    switch (pEvent->event)
    {
	#ifdef _WINDOWS
	    case WM_LBUTTONDOWN:
	    {
		HandleClick();
	    }
	    break;
	#endif
	
	#ifdef _MACINTOSH
	    case mouseDown:
	    {
		HandleClick();
	    }
	    break;
	#endif
	
	#ifdef _UNIX
	    case ButtonPress:
		HandleClick();
	    break;
	    default:
	    {
		break;
	    }
	#endif

	    case RMA_SURFACE_UPDATE:
	    {
		// we store the event struct and tell ourselves to draw
		m_pEvent = pEvent;
		Draw();
		pEvent->handled = TRUE;
		m_pEvent = NULL;
	    }
	    break;
    }
    return PNR_OK;
}


// IUnknown COM Interface Methods

/****************************************************************************
 *  IUnknown::AddRef                                            ref:  pncom.h
 *
 *  This routine increases the object reference count in a thread safe
 *  manner. The reference count is used to manage the lifetime of an object.
 *  This method must be explicitly called by the user whenever a new
 *  reference to an object is used.
 */
STDMETHODIMP_(UINT32) CExampleRenderer::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}


/****************************************************************************
 *  IUnknown::Release                                           ref:  pncom.h
 *
 *  This routine decreases the object reference count in a thread safe
 *  manner, and deletes the object if no more references to it exist. It must
 *  be called explicitly by the user whenever an object is no longer needed.
 */
STDMETHODIMP_(UINT32) CExampleRenderer::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
        return m_lRefCount;
    }

    delete this;
    return 0;
}


/****************************************************************************
 *  IUnknown::QueryInterface                                    ref:  pncom.h
 *
 *  This routine indicates which interfaces this object supports. If a given
 *  interface is supported, the object's reference count is incremented, and
 *  a reference to that interface is returned. Otherwise a NULL object and
 *  error code are returned. This method is called by other objects to
 *  discover the functionality of this object.
 */
STDMETHODIMP CExampleRenderer::QueryInterface(REFIID riid, void** ppvObj)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
	AddRef();
	*ppvObj = (IUnknown*)(IRMAPlugin*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMAPlugin))
    {
	AddRef();
	*ppvObj = (IRMAPlugin*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMARenderer))
    {
	AddRef();
	*ppvObj = (IRMARenderer*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMASiteUser))
    {
        AddRef();
        *ppvObj = (IRMASiteUser*)this;
        return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMASiteUserSupplier))
    {
	if (m_pMISUS)
	{
	    return m_pMISUS->QueryInterface(IID_IRMASiteUserSupplier,ppvObj);
	}
	else
	{
	    *ppvObj = NULL;
	    return PNR_UNEXPECTED;
	}
    }

    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}
