/****************************************************************************
 * 
 *  $Id: exvsrndr.h,v 1.1 1998/04/08 23:29:56 darrens Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 * 
 *  exvsrndr.h
 *
 *  Example Renderer plug-in for the RealMedia Architecture (RMA).
 */
#ifndef _EXVSRNDR_H_
#define _EXVSRNDR_H_


/****************************************************************************
 *
 *  CExampleRenderer Class
 *
 *  Example implementation of a basic renderer.
 */
class CExampleRenderer : public IRMAPlugin, 
                         public IRMARenderer, 
                         public IRMASiteUser
{
    public:
    /****** Public Class Methods ******************************************/
    CExampleRenderer();

    void HandleClick();
    void Draw();


    /************************************************************************
     *  IRMARenderer Interface Methods                       ref:  rmarendr.h
     */
    STDMETHOD(StartStream) (THIS_ IRMAStream* pStream, IRMAPlayer* pPlayer);
    STDMETHOD(EndStream )  (THIS);
    STDMETHOD(OnHeader  )  (THIS_ IRMAValues* pStreamHeaderObj);
    STDMETHOD(OnPacket  )  (THIS_ IRMAPacket* pPacket, INT32 timeOffset);
    STDMETHOD(OnTimeSync)  (THIS_ UINT32 currentPlayBackTime);
    STDMETHOD(OnPreSeek )  (THIS_ UINT32 timeBefore, UINT32 timeAfter);
    STDMETHOD(OnPostSeek)  (THIS_ UINT32 timeBefore, UINT32 timeAfter);
    STDMETHOD(OnPause   )  (THIS_ UINT32 timeBeforePause);
    STDMETHOD(OnBegin   )  (THIS_ UINT32 timeAfterBegin);
    STDMETHOD(OnBuffering) (THIS_ UINT32 reason, UINT16 percentComplete);
    STDMETHOD(GetRendererInfo)
        (THIS_
         REF(const char**) pStreamMimeTypes,
         REF(UINT32)       initialGranularity
        );
    STDMETHOD(GetDisplayType)
        (THIS_
         REF(RMA_DISPLAY_TYPE) displayType,
         REF(IRMABuffer*)      pDisplayInfo
        );
    STDMETHOD(OnEndofPackets) (THIS);


    /************************************************************************
     *  IRMASiteUser Interface Methods                         ref:  rmawin.h
     */
    STDMETHOD(AttachSite)  (THIS_ IRMASite* pSite);
    STDMETHOD(DetachSite)  (THIS);
    STDMETHOD(HandleEvent) (THIS_ PNxEvent* pEvent);
    STDMETHOD_(BOOL,NeedsWindowedSites)	(THIS);
    
    
    /************************************************************************
     *  IRMAPlugin Interface Methods                         ref:  rmaplugn.h
     */
    STDMETHOD(GetPluginInfo)
        (THIS_ 
          REF(BOOL)        bLoadMultiple,
          REF(const char*) pDescription,
          REF(const char*) pCopyright,
          REF(const char*) pMoreInfoURL,
          REF(UINT32)      versionNumber
        );
    STDMETHOD(InitPlugin) (THIS_ IUnknown* pContext);


    /************************************************************************
     *  IUnknown COM Interface Methods                          ref:  pncom.h
     */
    STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
    STDMETHOD_(UINT32, AddRef ) (THIS);
    STDMETHOD_(UINT32, Release) (THIS);


    private:
    /****** Private Class Variables ****************************************/
    INT32		m_lRefCount;
    IUnknown*		m_pContext;
    IRMAStream*		m_pStream;
    IRMAPlayer*		m_pPlayer;
    IRMAValues*		m_pHeader;
    IRMAPacket*		m_pLastPacket;
    UINT32		m_ulLastTime;
    UINT32		m_ulLastPacketTime;
    UINT32		m_ulLastRealPacketTime;
    UINT32		m_ulNumberOfClicks;
    BOOL		m_bIsSubscribed;
    IRMAHyperNavigate*	m_pHyperNavigate;
    IRMABackChannel*	m_pBackChannel;
    IRMAASMStream*	m_pASMStream;
    PNxSize		m_size;
    RMABitmapInfoHeader m_BitmapInfoHeader;
    UCHAR		m_nRed, m_nGreen, m_nBlue;
    IRMAMultiInstanceSiteUserSupplier*	m_pMISUS;
    IRMASite*				m_pMISUSSite;
    IRMACommonClassFactory*		m_pCommonClassFactory;
    BOOL		m_bInSeekMode;

    PNxEvent* m_pEvent;

    #ifdef _UNIX
	Widget  m_widget;
	GC      m_gc;
    #endif

    /****** Private Static Class Variables *********************************/
    static const char*	zm_pDescription;
    static const char*	zm_pCopyright;
    static const char*	zm_pMoreInfoURL;
    static const char*	zm_pStreamMimeTypes[];

    /****** Private Class Methods ******************************************/
    ~CExampleRenderer();

    PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME       // Avoids GCC compiler warning
    
    void    NegotiateImageFormat();
    void    CleanupVideoSurface();

};

#endif /* ndef _EXVSRNDR_H_ */

