/****************************************************************************
 * 
 *  $Id: exwnrndr.cpp,v 1.12 1997/10/21 21:27:34 christof Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 *
 *  Example Renderer for RealMedia Architecture (RMA).
 *
 */


/****************************************************************************
 * Defines
 */
#define INITGUID     /* Interface ID's */
#define STRICT


/****************************************************************************
 * Includes
 */
#include <stdio.h>

#include "pntypes.h"
#ifdef _WINDOWS
     #include <windows.h>
#endif

#ifdef _UNIX
    #include <stdlib.h>
    #include <unistd.h>
    #include <X11/Intrinsic.h>
    #include <X11/StringDefs.h>
    #include <X11/Shell.h>
    #include <X11/Xaw/Label.h>
    #include <X11/Xaw/Command.h>
    #include <X11/Xaw/Box.h>
    #include <X11/Xaw/Toggle.h>
    #include <X11/Xaw/Form.h>
#endif

#include "pnwintyp.h"
#include "pncom.h"
#include "rmacomm.h"
#include "rmapckts.h"
#include "rmafiles.h"
#include "rmacore.h"
#include "rmarendr.h"
#include "rmahyper.h"
#include "rmaplugn.h"
#include "rmawin.h"
#include "rmaasm.h"

#include "exwnrndr.h"
#include "exwnrndr.ver"


/****************************************************************************
 * Globals
 */
#ifdef _UNIX
    Widget g_Label1;
    Widget g_Label2;
#endif



/****************************************************************************
 *  RMACreateInstance                                        ref:  rmaplugn.h
 *
 *  This routine creates a new instance of the CExampleRenderer class.
 *  It is called when the RMA core application is launched, and whenever
 *  a stream associated with this plug-in needs to be rendered.
 */
STDAPI
RMACreateInstance(IUnknown** ppIUnknown)
{
    *ppIUnknown = (IUnknown*)(IRMAPlugin*)new CExampleRenderer();
    if (*ppIUnknown)
    {
	(*ppIUnknown)->AddRef();
	return PNR_OK;
    }
    return PNR_OUTOFMEMORY;
}


/****************************************************************************
 *  CExampleRenderer static variables                        ref:  exwnrndr.h
 *
 *  These variables are passed to the RMA core to provide information about
 *  this plug-in. They are required to be static in order to remain valid
 *  for the lifetime of the plug-in.
 */
const char* CExampleRenderer::zm_pDescription    = "Example Renderer Plugin";
const char* CExampleRenderer::zm_pCopyright      = "(c) 1995,1996,1997 RealNetworks, All rights reserved.";
const char* CExampleRenderer::zm_pMoreInfoURL    = "http://www.real.com";
const char* CExampleRenderer::zm_pStreamMimeTypes[] = {"application/x-pn-examplestream", NULL};


/****************************************************************************
 *  IRMAPlugin::GetPluginInfo                                ref:  rmaplugn.h
 *
 *  This routine returns descriptive information about the plug-in, most
 *  of which is used in the About box of the user interface. It is called
 *  when the RMA core application is launched.
 */
STDMETHODIMP
CExampleRenderer::GetPluginInfo
(
    REF(BOOL)        /*OUT*/ bLoadMultiple,
    REF(const char*) /*OUT*/ pDescription,
    REF(const char*) /*OUT*/ pCopyright,
    REF(const char*) /*OUT*/ pMoreInfoURL,
    REF(ULONG32)     /*OUT*/ ulVersionNumber
)
{
    bLoadMultiple = TRUE;   // Must be true for file formats.

    pDescription    = zm_pDescription;
    pCopyright	    = zm_pCopyright;
    pMoreInfoURL    = zm_pMoreInfoURL;
    ulVersionNumber = TARVER_ULONG32_VERSION;

    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::GetRendererInfo                            ref:  rmarendr.h
 *
 *  This routine returns crucial information required to associate this
 *  plug-in with a given stream MIME type. This information tells the RMA
 *  core which Renderer to use to display a particular type of stream. This
 *  method is called when the RMA core application is launched.
 */
STDMETHODIMP
CExampleRenderer::GetRendererInfo
(
    REF(const char**) /*OUT*/ pStreamMimeTypes,
    REF(UINT32)      /*OUT*/ unInitialGranularity
)
{
    pStreamMimeTypes = zm_pStreamMimeTypes;
    unInitialGranularity = 100; // OnTimeSync() every 100ms

    return PNR_OK;
}


/****************************************************************************
 *  IRMAPlugin::InitPlugin                                   ref:  rmaplugn.h
 *
 *  This routine performs initialization steps such as determining if
 *  required interfaces are available. It is called when the RMA core 
 *  application is launched, and whenever a stream associated with this
 *  plug-in needs to be rendered.
 */
STDMETHODIMP
CExampleRenderer::InitPlugin(IUnknown* /*IN*/ pContext)
{
    m_pContext = pContext;
    m_pContext->AddRef();

    m_pContext->QueryInterface(IID_IRMAHyperNavigate, (void**) &m_pHyperNavigate);
    m_pContext->QueryInterface(IID_IRMACommonClassFactory,(void**)&m_pCommonClassFactory);

    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::StartStream                                ref:  rmarendr.h
 *
 *  The RMA core calls this routine to provide access to the stream and
 *  player. It is called when the plug-in is being initialized.
 */
STDMETHODIMP
CExampleRenderer::StartStream
(
    IRMAStream*	    pStream,
    IRMAPlayer*	    pPlayer
)
{
    m_pStream  = pStream;
    m_pPlayer  = pPlayer;

    if (m_pStream ) m_pStream->AddRef();
    if (m_pPlayer ) m_pPlayer->AddRef();

    if (m_pStream)
    {
	IRMAStreamSource* pSource = 0;
	if (m_pStream->GetSource(pSource) == PNR_OK)
	{
	    /* It is OK if the source does not support backchannel. Reasons:
	     *
	     * 1. This stream may not be coming from example fileformat.
	     *	  It may instead be merged into a container fileformat which
	     *	  may be does not support BackChannel.
	     *
	     * 2. The protocol used to serve this stream may not support 
	     *	  BackChannel.
	     */
	    pSource->QueryInterface(IID_IRMABackChannel, (void**) &m_pBackChannel);
	    
	    pSource->Release();
	}

	/* Is ASM supported on this stream? */
	m_pStream->QueryInterface(IID_IRMAASMStream, (void**) &m_pASMStream);
    }

    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnHeader                                   ref:  rmarendr.h
 *
 *  This routine is passed the stream header object created by the associated
 *  file format plug-in. Use the methods defined in the IRMAValues interface
 *  to access the stream header data. This method is called from the RMA
 *  core when the plug-in is being initialized.
 */
STDMETHODIMP
CExampleRenderer::OnHeader(IRMAValues* pHeader)
{
    // Keep this for later use...
    m_pHeader = pHeader;
    m_pHeader->AddRef();

    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnBegin                                   ref:  rmarendr.h
 *
 *  This routine is called by the RMA core to inform the Renderer that 
 *  playback has just begun or has been resumed. The stream's time value just
 *  after resuming the playback is provided.
 */
STDMETHODIMP
CExampleRenderer::OnBegin(ULONG32 ulTime)
{
    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::GetDisplayType                             ref:  rmarendr.h
 *
 *  This routine returns the preferred display type of the Renderer. Any
 *  other additional information necessary for the display can also be
 *  provided. This method is called by the RMA core when the plug-in is being
 *  initialized.
 */
STDMETHODIMP
CExampleRenderer::GetDisplayType
(
    REF(RMA_DISPLAY_TYPE)   ulFlags,
    REF(IRMABuffer*)	    pBuffer
)
{
    ulFlags = RMA_DISPLAY_WINDOW;

    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnPacket                                   ref:  rmarendr.h
 *
 *  This routine is passed the packet object streamed by the associated
 *  file format plug-in. It is called by the RMA core when a packet is due
 *  to be delivered. In most cases, the actual rendering of the packet is
 *  done in this method. The stream's time offset with respect to the master
 *  timeline is provided.
 */
STDMETHODIMP
CExampleRenderer::OnPacket(IRMAPacket* pPacket, LONG32 lTimeOffset)
{
    /* Ignore pre-seek packets. Some renderers may want to use them though */
    if (m_bInSeekMode)
    {
	return PNR_OK;
    }

    // Release the last packet if we had one...
    if (m_pLastPacket)
    {
        m_pLastPacket->Release();
    }

    // Keep this one for later use...
    m_pLastPacket = pPacket;

    if (m_pLastPacket)
    {
	m_pLastPacket->AddRef();
    }

    // NOTE: If your stream has been offset in time due to start, end, or
    // groupstart properties on the stream, you may be getting packets which
    // need to be adjusted before writing into the audio services, or before
    // comparing against the time line. Use the following code to determine
    // to offset of this packet's delivery time in the real timeline.
    ULONG32 ulPacketDeliveryTime = pPacket->GetTime(); 
    ULONG32 ulRealDeliveryTime;
 
    if ((ULONG32)lTimeOffset > ulPacketDeliveryTime)
    {
        ulRealDeliveryTime = 0;
    }
    else
    {
	ulRealDeliveryTime = ulPacketDeliveryTime - lTimeOffset;
    }

    // For example purposes we remember this information
    m_ulLastPacketTime = ulPacketDeliveryTime;
    m_ulLastRealPacketTime = ulRealDeliveryTime;

    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnTimeSync                                 ref:  rmarendr.h
 *
 *  This routine is called by the RMA core periodically passing the current
 *  playback time. The Renderer should use this time value to synchronize the
 *  playback of various streams.
 */
STDMETHODIMP CExampleRenderer::OnTimeSync(ULONG32 ulTime)
{
    // Here's a good time to actually render the data!

    // Remember the time we are!
    m_ulLastTime = ulTime;

    // Redraw our data by damaging the entire area of our data
    PNxRect damageRect = {0, 0, m_size.cx, m_size.cy };
    m_pSite->DamageRect(damageRect);
    m_pSite->ForceRedraw();
	
    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnPreSeek                                  ref:  rmarendr.h
 *
 *  This routine is called by the RMA core before a seek is about to occur.
 *  The stream's time value just before the seek, and the time value just
 *  after the seek are provided.
 */
STDMETHODIMP
CExampleRenderer::OnPreSeek(ULONG32 ulOldTime, ULONG32 ulNewTime)
{
    m_bInSeekMode   = TRUE;
    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnPostSeek                                 ref:  rmarendr.h
 *
 *  This routine is called by the RMA core just after a seek has occured.
 *  The stream's time value just before the seek, and the time value just
 *  after the seek are provided.
 */
STDMETHODIMP
CExampleRenderer::OnPostSeek(ULONG32 ulOldTime, ULONG32 ulNewTime)
{
    m_bInSeekMode   = FALSE;
    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnPause                                    ref:  rmarendr.h
 *
 *  This routine is called by the RMA core just after a pause has occured.
 *  The stream's time value just before pausing is provided.
 */
STDMETHODIMP
CExampleRenderer::OnPause(ULONG32 ulTime)
{
    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnBuffering                                ref:  rmarendr.h
 *
 *  This routine is called by the RMA core to inform the Renderer that
 *  buffering of data is occuring. The reason for buffering (e.g. start-up
 *  of stream, seek has occured, network congestion, etc.), as well as the
 *  percentage of the buffering process complete are provided.
 */
STDMETHODIMP
CExampleRenderer::OnBuffering(ULONG32 ulFlags, UINT16 unPercentComplete)
{
    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::OnEndofPackets                             ref:  rmarendr.h
 *
 *  Called by client engine to inform the renderer that all the
 *  packets have been delivered. However, if the user seeks before
 *  EndStream() is called, renderer may start getting packets again
 *  and the client engine will eventually call this function again.
 */
STDMETHODIMP
CExampleRenderer::OnEndofPackets(void)
{
    return PNR_OK;
}


/****************************************************************************
 *  IRMARenderer::EndStream                                  ref:  rmarendr.h
 *
 *  This routine is called by the RMA core when the rendering of the stream
 *  has completed. Deallocation of any resources should be done here.
 */
STDMETHODIMP
CExampleRenderer::EndStream()
{
    /* Send report of a mouse click to the fileformat if backchannel 
     * is supported
     */
    if (m_pBackChannel && m_pCommonClassFactory && m_pStream)
    {
	IRMAPacket* pPacket = 0;
	IRMABuffer* pBuffer = 0;
	char tmpStr[100];
	if ((PNR_OK == m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,
					       (void**)&pBuffer)) &&
	    (PNR_OK == m_pCommonClassFactory->CreateInstance(CLSID_IRMAPacket,
					       (void**)&pPacket)))
	{
	    ::sprintf(tmpStr, "Number of clicks = %lu\n", m_ulNumberOfClicks);

	    pBuffer->Set((UCHAR*) tmpStr, ::strlen(tmpStr) + 1);
	    pPacket->Set(pBuffer,0,m_pStream->GetStreamNumber(),
			 RMA_ASM_SWITCH_ON, 0);
	    pBuffer->Release();

	    m_pBackChannel->PacketReady(pPacket);

	    pPacket->Release();
	}

    }

    // You can't use IRMAStream, IRMASourceStream, IRMABackChannel, or
    // IRMAASMStream after EndStream has been called.
    PN_RELEASE(m_pStream);
    PN_RELEASE(m_pBackChannel);
    PN_RELEASE(m_pASMStream);

    return PNR_OK;
}


/****************************************************************************
 *  CExampleRenderer::~CExampleRenderer                      ref:  exwnrndr.h
 *
 *  Destructor
 */
CExampleRenderer::~CExampleRenderer()
{
    // NOTE: You should do your renderer cleanup here, instead of
    // in EndStream(), because your renderer is still left around
    // after the stream is ended in case it is a display renderer
    // and it needs to "paint" it's display area.

    PN_RELEASE(m_pHeader);
    PN_RELEASE(m_pLastPacket);
    PN_RELEASE(m_pContext);
    PN_RELEASE(m_pPlayer);
    PN_RELEASE(m_pHyperNavigate);
    PN_RELEASE(m_pCommonClassFactory);
}


/****************************************************************************
 *  CExampleRenderer::Draw                                   ref:  exwnrndr.h
 *
 */
#define MAX_PACKET_SIZE	    400 
void CExampleRenderer::Draw()
{
    // Add draw code here!
    char	szText1[128];
    char	szText2[128];
    char	szText3[128];
    char	szText4[MAX_PACKET_SIZE + 50];
    char	szPacketFormat[40];
    int		nLength1,nLength2,nLength3,nLength4;
    int		time,hours,min,secs,tenths;
    IRMABuffer*	pBuffer = NULL;

    time    = m_ulLastTime;
    tenths  = time % 1000;
    time   /= 1000;
    secs    = time % 60;
    time   /= 60;
    min	    = time % 60;
    hours   = time / 60;

    nLength1 = sprintf(szText1,"Last Time Sync: %d:%02d:%02d.%03d",
		hours,min,secs,tenths);

    time    = m_ulLastPacketTime;
    tenths  = time % 1000;
    time   /= 1000;
    secs    = time % 60;
    time   /= 60;
    min	    = time % 60;
    hours   = time / 60;

    nLength2 = sprintf(szText2,"Last Packet Time: %d:%02d:%02d.%03d",
		hours,min,secs,tenths);

    time    = m_ulLastRealPacketTime;
    tenths  = time % 1000;
    time   /= 1000;
    secs    = time % 60;
    time   /= 60;
    min	    = time % 60;
    hours   = time / 60;

    nLength3 = sprintf(szText3,"Last Offset Packet Time: %d:%02d:%02d.%03d",
		hours,min,secs,tenths);

    nLength4 = sprintf(szText4,"Last Packet was NULL");

    if (m_pLastPacket)
    {
        pBuffer = m_pLastPacket->GetBuffer();
	if (pBuffer)
	{
	    sprintf(szPacketFormat,"Last Packet Contained: %%.%is",
		(pBuffer->GetSize() < MAX_PACKET_SIZE ? 
		    pBuffer->GetSize() : MAX_PACKET_SIZE));
	    nLength4 = sprintf(szText4,szPacketFormat,pBuffer->GetBuffer());
	    pBuffer->Release();
	}
    }

#if _WINDOWS
    ::Rectangle(m_hDC,115,5,595,95);
    ::TextOut(m_hDC,120,10,szText1,nLength1);
    ::TextOut(m_hDC,120,30,szText2,nLength2);
    ::TextOut(m_hDC,120,50,szText3,nLength3);
    ::TextOut(m_hDC,120,70,szText4,nLength4);
#endif

#if _UNIX
    if(m_widget && m_gc)
    {
	XClearWindow(XtDisplay((Widget)m_wDrawable),
		     XtWindow((Widget)m_wDrawable));
	XDrawString(XtDisplay((Widget)m_wDrawable),
		    XtWindow((Widget)m_wDrawable),
		    m_gc,
		    0, 10, szText1, strlen(szText1));
	XDrawString(XtDisplay((Widget)m_wDrawable),
		    XtWindow((Widget)m_wDrawable),
		    m_gc,
		    0, 30, szText2, strlen(szText2));
	XDrawString(XtDisplay((Widget)m_wDrawable),
		    XtWindow((Widget)m_wDrawable),
		    m_gc,
		    0, 50, szText3, strlen(szText3));
	XDrawString(XtDisplay((Widget)m_wDrawable),
		    XtWindow((Widget)m_wDrawable),
		    m_gc,
		    0, 70, szText4, strlen(szText4));
	XFlush(XtDisplay((Widget)m_widget));
    }
#endif

#ifdef _MACINTOSH
    PNxRect bounds = {0, 0, m_size.cx, m_size.cy };
    
    Rect rErase;
    ::SetRect(&rErase, bounds.left, bounds.top, bounds.right, bounds.bottom);
    ::EraseRect(&rErase);
    
    ::TextSize(9);
    ::MoveTo(bounds.left + 20, bounds.top + 20);
    ::DrawText(szText1, 0, nLength1);
    ::MoveTo(bounds.left + 20, bounds.top + 50);
    ::DrawText(szText2, 0, nLength2);
#endif

}


/****************************************************************************
 *  CExampleRenderer::HandleClick                            ref:  exwnrndr.h
 *
 */
void CExampleRenderer::HandleClick()
{
    m_ulNumberOfClicks++;
}


#ifdef _UNIX
void rule_callback(Widget w, XtPointer client_data, XtPointer call_data)
{
    CExampleRenderer* rend = (CExampleRenderer*)client_data;
    rend->HandleRuleCallback(w);
}

void
CExampleRenderer::HandleRuleCallback(Widget w)
{
    int state, rule;
    
    XtVaGetValues(w, XtNstate, &state, NULL);
    state &= 1;

    if(w == m_wRule1)
	rule = 1;
    else
	rule = 2;

    if(state)
	m_pASMStream->Subscribe(rule);
    else
	m_pASMStream->Unsubscribe(rule);
}

void navigate_callback(Widget w, XtPointer client_data, XtPointer call_data)
{
    CExampleRenderer* rend = (CExampleRenderer*)client_data;
    rend->HandleNavigateCallback();
}

void
CExampleRenderer::HandleNavigateCallback()
{
    m_pHyperNavigate->GoToURL("http://www.real.com", NULL);
}

#endif /* _UNIX */


// IRMASiteUser Interface Methods

/****************************************************************************
 *  IRMASiteUser::AttachSite                                   ref:  rmawin.h
 *
 */
STDMETHODIMP
CExampleRenderer::AttachSite(IRMASite* /*IN*/ pSite)
{
    if (m_pSite) return PNR_UNEXPECTED;

    m_size.cx = 600;
    m_size.cy = 100;
    pSite->SetSize(m_size); //set the site size

    /*
     * In this method we can gain access to the "window" of the 
     * site. Since we returned TRUE from NeedsWindowedSites, we 
     * should be being associated with an IRMASiteWindowed.
     */
    IRMASiteWindowed* pSiteWindowed = NULL;
    if (PNR_OK != pSite->QueryInterface(IID_IRMASiteWindowed, 
		    (void**)&pSiteWindowed))
    {
	return PNR_UNEXPECTED;
    }

    PNxWindow* pWindow = pSiteWindowed->GetWindow();

#ifdef _WINDOWS
    HWND    hWnd = (HWND)pWindow->window;

    m_hNavigate = CreateWindow(
		    "BUTTON", "Navigate", WS_CHILD | WS_VISIBLE,
		    10, 10, 100, 20, hWnd, (HMENU)ID_NAVIGATE, NULL, 0);

    m_hRule1 = CreateWindow(
		    "BUTTON", "ASM Rule 1", 
		    BS_AUTOCHECKBOX | WS_CHILD | WS_VISIBLE,
		    10, 40, 100, 20, hWnd, (HMENU)ID_RULE1, NULL, 0);

#ifdef _WIN16                        
    #define BST_CHECKED	0x0001
#endif	    
    SendMessage(m_hRule1,BM_SETCHECK,BST_CHECKED,0);

    m_hRule2 = CreateWindow(
		    "BUTTON", "ASM Rule 2", 
		    BS_AUTOCHECKBOX | WS_CHILD | WS_VISIBLE,
		    10, 70, 100, 20, hWnd, (HMENU)ID_RULE2, NULL, 0);

    SendMessage(m_hRule2,BM_SETCHECK,BST_CHECKED,0);

#endif
#ifdef _UNIX
    m_widget = pWindow->window;

    m_wDrawable = XtVaCreateManagedWidget("",
					  coreWidgetClass,
					  m_widget,
					  XtNx, 120,
					  XtNy, 0,
					  XtNwidth, 300,
					  XtNheight, 200,
					  XtNborderWidth, 0,
					  NULL);
    m_wNavigate = XtVaCreateManagedWidget("Navigate",
					  commandWidgetClass,
					  (Widget)m_widget,
					  XtNlabel, "Navigate",
					  XtNx, 10,
					  XtNy, 10,
					  XtNwidth, 100,
					  XtNheight, 20,
					  NULL);
    m_wRule1 = XtVaCreateManagedWidget("Rule1",
				       toggleWidgetClass,
				       (Widget)m_widget,
				       XtNlabel, "ASM Rule 1",
				       XtNwidth, 100,
				       XtNheight, 20,
				       XtNstate, True,
				       XtNx, 10,
				       XtNy, 40,
				       NULL);
    m_wRule2 = XtVaCreateManagedWidget("Rule2",
				       toggleWidgetClass,
				       (Widget)m_widget,
				       XtNlabel, "ASM Rule 2",
				       XtNwidth, 100,
				       XtNheight, 20,
				       XtNx, 10,
				       XtNy, 70,
				       XtNstate, True,
				       NULL);
    XtAddCallback(m_wRule1, XtNcallback, rule_callback, (void*)this);
    XtAddCallback(m_wRule2, XtNcallback, rule_callback, (void*)this);
    XtAddCallback(m_wNavigate, XtNcallback, navigate_callback, (void*)this);

    XtRealizeWidget(m_wNavigate);
    XtRealizeWidget(m_wRule1);
    XtRealizeWidget(m_wRule2);
    pSite->ForceRedraw();
#endif
    PN_RELEASE(pSiteWindowed);

    m_pSite = pSite;
    m_pSite->AddRef();

    return PNR_OK;
}


/****************************************************************************
 *  IRMASiteUser::DetachSite                                   ref:  rmawin.h
 *
 */
STDMETHODIMP
CExampleRenderer::DetachSite()
{
    PN_RELEASE(m_pSite);

#ifdef _WINDOWS
    DestroyWindow(m_hNavigate);
    DestroyWindow(m_hRule1);
    DestroyWindow(m_hRule2);
#endif

    return PNR_OK;
}


/****************************************************************************
 *  IRMASiteUser::NeedsWindowedSites                           ref:  rmawin.h
 *
 */
STDMETHODIMP_(BOOL)
CExampleRenderer::NeedsWindowedSites() 
{ 
    return TRUE; 
};


/****************************************************************************
 *  IRMASiteUser::HandleEvent                                  ref:  rmawin.h
 *
 */
STDMETHODIMP
CExampleRenderer::HandleEvent(PNxEvent* /*IN*/ pEvent)
{
    pEvent->handled = FALSE;
    pEvent->result  = 0;

#ifdef _UNIX
    if(!m_gc && m_widget)
    {
	XGCValues values;
	Font font;
	XFontStruct* fontinfo;
	fontinfo = XLoadQueryFont(XtDisplay((Widget)m_widget), "6x10");
	values.font = fontinfo->fid;
	m_gc = XtGetGC((Widget)m_widget,
		       GCFont,
		       &values);
    }
#endif

    switch (pEvent->event)
    {
#ifdef _WINDOWS
	case WM_PAINT:
	{
	    HWND hWnd = (HWND)pEvent->window;
	    PAINTSTRUCT ps;
	    m_hDC = ::BeginPaint(hWnd,&ps);
	    Draw();
	    ::EndPaint(hWnd,&ps);
	    pEvent->handled = TRUE;
	}
	break;
	
	case WM_LBUTTONDOWN:
	{
	    HandleClick();
	}
	break;

	case WM_COMMAND:
	{
	    WPARAM  wParam = (WPARAM)pEvent->param1;
	    LPARAM  lParam = (LPARAM)pEvent->param2;
	    WORD    wNotifyCode = HIWORD(wParam); // notification code 
	    WORD    wID = LOWORD(wParam);         // control id
	    HWND    hwndCtl = (HWND) lParam;      // handle of control 

	    switch(wID)
	    {
		case ID_NAVIGATE:
		{
		    m_pHyperNavigate->GoToURL("http://www.real.com", NULL);
		}
		break;

		case ID_RULE1:
		{
		    BOOL bIsChecked = (BST_CHECKED == 
					SendMessage(hwndCtl,BM_GETCHECK,0,0));
		    if (m_pASMStream)
		    {
			if (bIsChecked)
			{
			    m_pASMStream->Subscribe(0);
			}
			else
			{
			    m_pASMStream->Unsubscribe(0);
			}
		    }
		}
		break;

		case ID_RULE2:
		{
		    BOOL bIsChecked = (BST_CHECKED == 
					SendMessage(hwndCtl,BM_GETCHECK,0,0));
		    if (m_pASMStream)
		    {
			if (bIsChecked)
			{
			    m_pASMStream->Subscribe(1);
			}
			else
			{
			    m_pASMStream->Unsubscribe(1);
			}
		    }
		}
		break;

	    }
    	}
	break;

#endif
#ifdef _MACINTOSH

	case updateEvt:
	case activateEvt:
	{
	    Draw();
	}
	break;

	case mouseDown:
	{
	    HandleClick();
	}
	break;
#endif
#ifdef _UNIX
	case ButtonPress:
	    HandleClick();
	    break;
	case Expose:
	    Draw();
	    break;
	default:
	    break;
#endif    
    }
    return PNR_OK;
}


// IUnknown COM Interface Methods

/****************************************************************************
 *  IUnknown::AddRef                                            ref:  pncom.h
 *
 *  This routine increases the object reference count in a thread safe
 *  manner. The reference count is used to manage the lifetime of an object.
 *  This method must be explicitly called by the user whenever a new
 *  reference to an object is used.
 */
STDMETHODIMP_(ULONG32) CExampleRenderer::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}


/****************************************************************************
 *  IUnknown::Release                                           ref:  pncom.h
 *
 *  This routine decreases the object reference count in a thread safe
 *  manner, and deletes the object if no more references to it exist. It must
 *  be called explicitly by the user whenever an object is no longer needed.
 */
STDMETHODIMP_(ULONG32) CExampleRenderer::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
        return m_lRefCount;
    }

    delete this;
    return 0;
}


/****************************************************************************
 *  IUnknown::QueryInterface                                    ref:  pncom.h
 *
 *  This routine indicates which interfaces this object supports. If a given
 *  interface is supported, the object's reference count is incremented, and
 *  a reference to that interface is returned. Otherwise a NULL object and
 *  error code are returned. This method is called by other objects to
 *  discover the functionality of this object.
 */
STDMETHODIMP CExampleRenderer::QueryInterface(REFIID riid, void** ppvObj)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
	AddRef();
	*ppvObj = (IUnknown*)(IRMAPlugin*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMAPlugin))
    {
	AddRef();
	*ppvObj = (IRMAPlugin*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMARenderer))
    {
	AddRef();
	*ppvObj = (IRMARenderer*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMASiteUser))
    {
        AddRef();
        *ppvObj = (IRMASiteUser*)this;
        return PNR_OK;
    }
    /*
     * NOTE: This plugin only supports single instances and
     * as such does not expose the IRMASiteUserSupplier interface.
     */

    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}
