/****************************************************************************
 * 
 *  $Id: exwnrndr.h,v 1.11 1997/10/21 21:27:34 christof Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 * 
 *  exwnrndr.h
 *
 *  Example Renderer for RealMedia Architecture (RMA).
 */
#ifndef _EXWNRNDR_H_
#define _EXWNRNDR_H_


/****************************************************************************
 * Constants
 */
#define ID_NAVIGATE	100
#define ID_RULE1	101
#define ID_RULE2	102



/****************************************************************************
 *
 *  CExampleRenderer Class
 *
 *  Example implementation of a basic renderer.
 */
class CExampleRenderer :    public IRMAPlugin, 
			    public IRMARenderer, 
			    public IRMASiteUser
{
    public:
    /****** Public Class Methods ******************************************/
    CExampleRenderer()
	: m_lRefCount(0)
	, m_pContext(NULL)
	, m_pStream(NULL)
	, m_pPlayer(NULL)
	, m_pHeader(NULL)
	, m_pLastPacket(NULL)
	, m_ulLastTime(0)
	, m_pHyperNavigate(0)
	, m_pBackChannel(0)
	, m_pASMStream(0)
	, m_pCommonClassFactory(0)
	, m_ulNumberOfClicks(0)
	, m_bIsSubscribed(TRUE)
	, m_pSite(NULL)
	, m_bInSeekMode(FALSE)
	#ifdef _UNIX
        , m_widget(NULL)
        , m_gc(NULL)
	#endif
	{ };
	
    void HandleClick();
    void Draw();


    /************************************************************************
     *  IRMARenderer Interface Methods                       ref:  rmarendr.h
     */
    STDMETHOD(StartStream) (THIS_ IRMAStream* pStream, IRMAPlayer* pPlayer);
    STDMETHOD(EndStream )  (THIS);
    STDMETHOD(OnHeader  )  (THIS_ IRMAValues* pStreamHeaderObj);
    STDMETHOD(OnPacket  )  (THIS_ IRMAPacket* pPacket, INT32 timeOffset);
    STDMETHOD(OnTimeSync)  (THIS_ UINT32 currentPlayBackTime);
    STDMETHOD(OnPreSeek )  (THIS_ UINT32 timeBefore, UINT32 timeAfter);
    STDMETHOD(OnPostSeek)  (THIS_ UINT32 timeBefore, UINT32 timeAfter);
    STDMETHOD(OnPause   )  (THIS_ UINT32 timeBeforePause);
    STDMETHOD(OnBegin   )  (THIS_ UINT32 timeAfterBegin);
    STDMETHOD(OnBuffering) (THIS_ UINT32 reason, UINT16 percentComplete);
    STDMETHOD(GetRendererInfo)
        (THIS_
         REF(const char**) pStreamMimeTypes,
         REF(UINT32)       initialGranularity
        );
    STDMETHOD(GetDisplayType)
        (THIS_
         REF(RMA_DISPLAY_TYPE) displayType,
         REF(IRMABuffer*)      pDisplayInfo
        );
    STDMETHOD(OnEndofPackets) (THIS);


    /************************************************************************
     *  IRMAPlugin Interface Methods                         ref:  rmaplugn.h
     */
    STDMETHOD(GetPluginInfo)
        (THIS_ 
          REF(BOOL)        bLoadMultiple,
          REF(const char*) pDescription,
          REF(const char*) pCopyright,
          REF(const char*) pMoreInfoURL,
          REF(UINT32)      versionNumber
        );
    STDMETHOD(InitPlugin) (THIS_ IUnknown* pContext);


    /************************************************************************
     *  IRMASiteUser Interface Methods                         ref:  rmawin.h
     */
    STDMETHOD(AttachSite)  (THIS_ IRMASite* pSite);
    STDMETHOD(DetachSite)  (THIS);
    STDMETHOD(HandleEvent) (THIS_ PNxEvent* pEvent);
    STDMETHOD_(BOOL,NeedsWindowedSites)	(THIS);


    /************************************************************************
     *  IUnknown COM Interface Methods                          ref:  pncom.h
     */
    STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
    STDMETHOD_(UINT32, AddRef ) (THIS);
    STDMETHOD_(UINT32, Release) (THIS);

  
    private:
    /****** Private Class Variables ****************************************/
    INT32			m_lRefCount;
    IUnknown*			m_pContext;
    IRMAStream*			m_pStream;
    IRMAPlayer*			m_pPlayer;
    IRMAValues*			m_pHeader;
    IRMAPacket*			m_pLastPacket;
    UINT32			m_ulLastTime;
    UINT32			m_ulLastPacketTime;
    UINT32			m_ulLastRealPacketTime;
    UINT32			m_ulNumberOfClicks;
    BOOL			m_bIsSubscribed;
    IRMAHyperNavigate*		m_pHyperNavigate;
    IRMABackChannel*		m_pBackChannel;
    IRMAASMStream*		m_pASMStream;
    PNxSize			m_size;
    IRMASite*			m_pSite;
    IRMACommonClassFactory*	m_pCommonClassFactory;
    BOOL			m_bInSeekMode;

    #ifdef _WINDOWS
	HDC			m_hDC;
	HWND			m_hNavigate;
	HWND			m_hRule1;
	HWND			m_hRule2;
    #endif
    
    #ifdef _UNIX
public:
	void                    HandleRuleCallback(Widget);
	void                    HandleNavigateCallback();
private:
	void*                   m_widget;
	GC                      m_gc;
	Widget                  m_wDrawable;
	Widget                  m_wNavigate;
	Widget                  m_wRuleBox;
	Widget                  m_wRule1;
	Widget                  m_wRule2;
    #endif


    /****** Private Static Class Variables *********************************/
    static const char*	zm_pDescription;
    static const char*	zm_pCopyright;
    static const char*	zm_pMoreInfoURL;
    static const char*	zm_pStreamMimeTypes[];

    /****** Private Class Methods ******************************************/
    ~CExampleRenderer();

    PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME;
};


#endif // ndef _EXWNRNDR_H_

