/****************************************************************************
 * 
 *  $Id: exadvsnk.cpp,v 1.2 1998/12/10 02:55:01 jordanb Exp $
 *
 *  Copyright (C) 1995,1996,1997,1998 RealNetworks, Inc.
 *  All rights reserved.
 *
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary 
 *  information of RealNetworks, Inc, and is licensed
 *  subject to restrictions on use and distribution.
 *
 *
 *  Sample Implementation of Client Advise Sink Interfaces
 *
 */

#include "pntypes.h"

#include "os.h"
#include "pncom.h"
#include "rmacomm.h"
#include "rmamon.h"
#include "rmacore.h"
#include "rmaengin.h"

#include "rmaclsnk.h"
#include "exadvsnk.h"

extern BOOL     bEnableAdviceSink;
extern BOOL     bEnableVerboseMode;


ExampleClientAdviceSink::ExampleClientAdviceSink(IUnknown* pUnknown)
    : m_lRefCount (0)
    , m_pUnknown (NULL)
    , m_pRegistry (NULL)
    , m_pScheduler (NULL)
{
    if (pUnknown)
    {
	m_pUnknown = pUnknown;
	m_pUnknown->AddRef();

	if (PNR_OK != m_pUnknown->QueryInterface(IID_IRMAPNRegistry, (void**)&m_pRegistry))
	{
	    m_pRegistry = NULL;
	}

	if (PNR_OK != m_pUnknown->QueryInterface(IID_IRMAScheduler, (void**)&m_pScheduler))
	{
	    m_pScheduler = NULL;
	}

	if(PNR_OK == m_pUnknown->QueryInterface(IID_IRMAPlayer,
						(void**)&m_pPlayer))
	{
	    m_pPlayer->AddAdviseSink(this);
	}
    }
}

ExampleClientAdviceSink::~ExampleClientAdviceSink(void)
{
    if (m_pScheduler)
    {
        m_pScheduler->Release();
        m_pScheduler = NULL;
    }

    if (m_pRegistry)
    {
	m_pRegistry->Release();
	m_pRegistry = NULL;
    }

    if (m_pUnknown)
    {
	m_pUnknown->Release();
	m_pUnknown = NULL;
    }

    PN_RELEASE(m_pPlayer);
}


// *** IUnknown methods ***

/////////////////////////////////////////////////////////////////////////
//  Method:
//	IUnknown::QueryInterface
//  Purpose:
//	Implement this to export the interfaces supported by your 
//	object.
//
STDMETHODIMP ExampleClientAdviceSink::QueryInterface(REFIID riid, void** ppvObj)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
	AddRef();
	*ppvObj = (IUnknown*)(IRMAClientAdviseSink*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMAClientAdviseSink))
    {
	AddRef();
	*ppvObj = (IRMAClientAdviseSink*)this;
	return PNR_OK;
    }

    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}

/////////////////////////////////////////////////////////////////////////
//  Method:
//	IUnknown::AddRef
//  Purpose:
//	Everyone usually implements this the same... feel free to use
//	this implementation.
//
STDMETHODIMP_(ULONG32) ExampleClientAdviceSink::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}

/////////////////////////////////////////////////////////////////////////
//  Method:
//	IUnknown::Release
//  Purpose:
//	Everyone usually implements this the same... feel free to use
//	this implementation.
//
STDMETHODIMP_(ULONG32) ExampleClientAdviceSink::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
        return m_lRefCount;
    }

    delete this;
    return 0;
}

/*
 *	IRMAClientAdviseSink methods
 */

/************************************************************************
 *	Method:
 *	    IRMAClientAdviseSink::OnPosLength
 *	Purpose:
 *	    Called to advise the client that the position or length of the
 *	    current playback context has changed.
 */
STDMETHODIMP
ExampleClientAdviceSink::OnPosLength(UINT32	  ulPosition,
				   UINT32	  ulLength)
{
    if (bEnableAdviceSink)
    {
        fprintf(stdout, "OnPostLength(%ld, %ld)\n", ulPosition, ulLength);
    }

    return PNR_OK;
}

/************************************************************************
 *	Method:
 *	    IRMAClientAdviseSink::OnPresentationOpened
 *	Purpose:
 *	    Called to advise the client a presentation has been opened.
 */
STDMETHODIMP ExampleClientAdviceSink::OnPresentationOpened()
{
    if (bEnableAdviceSink)
    {
        fprintf(stdout, "OnPresentationOpened()\n");
    }

    return PNR_OK;
}


/************************************************************************
 *	Method:
 *	    IRMAClientAdviseSink::OnPresentationClosed
 *	Purpose:
 *	    Called to advise the client a presentation has been closed.
 */
STDMETHODIMP ExampleClientAdviceSink::OnPresentationClosed()
{
    if (bEnableAdviceSink)
    {
        fprintf(stdout, "OnPresentationClosed()\n");
    }

    return PNR_OK;
}

void ExampleClientAdviceSink::ShowMeTheStatistics (char* pszRegistryKey)
{
    char    szRegistryValue[MAX_DISPLAY_NAME] = {0};
    INT32   lValue = 0;
    INT32   i = 0;
    INT32   lStatistics = 8;
    
    if (!bEnableAdviceSink)
    {
	return;
    }

    // collect statistic
    for (i = 0; i < lStatistics; i++)
    {
	switch (i)
	{
	case 0:
	    sprintf(szRegistryValue, "%s.Normal", pszRegistryKey);
	    break;
	case 1:
	    sprintf(szRegistryValue, "%s.Recovered", pszRegistryKey);
	    break;
	case 2:
	    sprintf(szRegistryValue, "%s.Received", pszRegistryKey);
	    break;
	case 3:
	    sprintf(szRegistryValue, "%s.Lost", pszRegistryKey);
	    break;
	case 4:
	    sprintf(szRegistryValue, "%s.Late", pszRegistryKey);
	    break;
	case 5:
	    sprintf(szRegistryValue, "%s.ClipBandwidth", pszRegistryKey);
	    break;
	case 6:
	    sprintf(szRegistryValue, "%s.AverageBandwidth", pszRegistryKey);
	    break;
	case 7:
	    sprintf(szRegistryValue, "%s.CurrentBandwidth", pszRegistryKey);
	    break;
	default:
	    break;
	}

	m_pRegistry->GetIntByName(szRegistryValue, lValue);
	fprintf(stdout, "%s = %ld\n", szRegistryValue, lValue);
    }
}

/************************************************************************
 *	Method:
 *	    IRMAClientAdviseSink::OnStatisticsChanged
 *	Purpose:
 *	    Called to advise the client that the presentation statistics
 *	    have changed. 
 */
STDMETHODIMP ExampleClientAdviceSink::OnStatisticsChanged(void)
{
    if (bEnableAdviceSink)
    {
        fprintf(stdout, "OnStatisticsChanged():\n");

        UINT32  unPlayerIndex = 0;
        UINT32  unSourceIndex = 0;
        UINT32  unStreamIndex = 0;

        INT32   nRegistryValue = 0;

        char*   pszRegistryPrefix = "Statistics";
        char    szRegistryName[MAX_DISPLAY_NAME] = {0};
   
        // display the content of whole statistic registry
        if (m_pRegistry)
        {
	    // ok, let's start from the top (player)
	    sprintf(szRegistryName, "%s.Player%ld", pszRegistryPrefix, unPlayerIndex);
	    while (PT_COMPOSITE == m_pRegistry->GetTypeByName(szRegistryName))
	    {
	        // display player statistic
	        ShowMeTheStatistics(szRegistryName);

	        sprintf(szRegistryName, "%s.Source%ld", szRegistryName, unSourceIndex);
	        while (PT_COMPOSITE == m_pRegistry->GetTypeByName(szRegistryName))
	        {
		    // display source statistic
		    ShowMeTheStatistics(szRegistryName);

		    sprintf(szRegistryName, "%s.Stream%ld", szRegistryName, unStreamIndex);
		    while (PT_COMPOSITE == m_pRegistry->GetTypeByName(szRegistryName))
		    {
		        // display stream statistic
		        ShowMeTheStatistics(szRegistryName);

		        unStreamIndex++;

		        sprintf(szRegistryName, "%s.Player%ld.Source%ld.Stream%ld", 
			        pszRegistryPrefix, unPlayerIndex, unSourceIndex, unStreamIndex);
		    }

		    unSourceIndex++;

		    sprintf(szRegistryName, "%s.Player%ld.Source%ld",
			    pszRegistryPrefix, unPlayerIndex, unSourceIndex);
	        }

	        unPlayerIndex++;

	        sprintf(szRegistryName, "%s.Player%ld", pszRegistryPrefix, unPlayerIndex);
	    }
        }
    }

    return PNR_OK;
}


/************************************************************************
 *	Method:
 *	    IRMAClientAdviseSink::OnPreSeek
 *	Purpose:
 *	    Called by client engine to inform the client that a seek is
 *	    about to occur. The render is informed the last time for the 
 *	    stream's time line before the seek, as well as the first new
 *	    time for the stream's time line after the seek will be completed.
 *
 */
STDMETHODIMP ExampleClientAdviceSink::OnPreSeek(	ULONG32	ulOldTime,
						ULONG32	ulNewTime)
{
    if (bEnableAdviceSink)
    {
        fprintf(stdout, "OnPreSeek(%ld, %ld)\n", ulOldTime, ulNewTime);
    }

    return PNR_OK;
}


/************************************************************************
 *	Method:
 *	    IRMAClientAdviseSink::OnPostSeek
 *	Purpose:
 *	    Called by client engine to inform the client that a seek has
 *	    just occured. The render is informed the last time for the 
 *	    stream's time line before the seek, as well as the first new
 *	    time for the stream's time line after the seek.
 *
 */
STDMETHODIMP ExampleClientAdviceSink::OnPostSeek(	ULONG32	ulOldTime,
						ULONG32	ulNewTime)
{
    if (bEnableAdviceSink)
    {
        fprintf(stdout, "OnPostSeek(%ld, %ld)\n", ulOldTime, ulNewTime);
    }

    return PNR_OK;
}


/************************************************************************
 *	Method:
 *	    IRMAClientAdviseSink::OnStop
 *	Purpose:
 *	    Called by client engine to inform the client that a stop has
 *	    just occured. 
 *
 */
STDMETHODIMP ExampleClientAdviceSink::OnStop(void)
{
    RMATimeval now;

    if (bEnableAdviceSink)
    {
        fprintf(stdout, "OnStop()\n");
    }

    // Find out the current time and subtract the beginning time to
    // figure out how many seconds we played
    now = m_pScheduler->GetCurrentSchedulerTime();
    m_ulStopTime = now.tv_sec;

    return PNR_OK;
}

/************************************************************************
 *	Method:
 *	    IRMAClientAdviseSink::OnPause
 *	Purpose:
 *	    Called by client engine to inform the client that a pause has
 *	    just occured. The render is informed the last time for the 
 *	    stream's time line before the pause.
 *
 */
STDMETHODIMP ExampleClientAdviceSink::OnPause(ULONG32 ulTime)
{
    if (bEnableAdviceSink)
    {
        fprintf(stdout, "OnPause(%ld)\n", ulTime);
    }

    return PNR_OK;
}


/************************************************************************
 *	Method:
 *	    IRMAClientAdviseSink::OnBegin
 *	Purpose:
 *	    Called by client engine to inform the client that a begin or
 *	    resume has just occured. The render is informed the first time 
 *	    for the stream's time line after the resume.
 *
 */
STDMETHODIMP ExampleClientAdviceSink::OnBegin(ULONG32 ulTime)
{
    RMATimeval now;

    if (bEnableAdviceSink)
    {
        fprintf(stdout, "OnBegin(%ld)\n", ulTime);
    }

    // Record the current time, so we can figure out many seconds we played
    now = m_pScheduler->GetCurrentSchedulerTime();
    m_ulStartTime = now.tv_sec;

    return PNR_OK;
}


/************************************************************************
 *	Method:
 *	    IRMAClientAdviseSink::OnBuffering
 *	Purpose:
 *	    Called by client engine to inform the client that buffering
 *	    of data is occuring. The render is informed of the reason for
 *	    the buffering (start-up of stream, seek has occured, network
 *	    congestion, etc.), as well as percentage complete of the 
 *	    buffering process.
 *
 */
STDMETHODIMP ExampleClientAdviceSink::OnBuffering(ULONG32	ulFlags,
						UINT16	unPercentComplete)
{
    if (bEnableAdviceSink)
    {
        fprintf(stdout, "OnBuffering(%ld, %d)\n", ulFlags, unPercentComplete);
    }

    return PNR_OK;
}


/************************************************************************
 *	Method:
 *	    IRMAClientAdviseSink::OnContacting
 *	Purpose:
 *	    Called by client engine to inform the client is contacting
 *	    hosts(s).
 *
 */
STDMETHODIMP ExampleClientAdviceSink::OnContacting(const char* pHostName)
{
    if (bEnableAdviceSink)
    {
        fprintf(stdout, "OnContacting(\"%s\")\n", pHostName);
    }

    return PNR_OK;
}

