/****************************************************************************
 * 
 *  $Id: excontxt.cpp,v 1.3 1999/11/17 01:41:47 jfarr Exp $
 *
 *  Copyright (C) 1995,1996,1997,1998 RealNetworks, Inc.
 *  All rights reserved.
 *
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary 
 *  information of RealNetworks, Inc, and is licensed
 *  subject to restrictions on use and distribution.
 *
 *
 *  Sample Implementation of Client Context
 *
 */

#include "pntypes.h"
#include "os.h"

#include "pnwintyp.h"
#include "pncom.h"
#include "rmacomm.h"
#include "rmawin.h"
#include "fivemmap.h"

//#include "pnbuffer.h"
//#include "pnmangle.h"

#include "rmaclsnk.h"
#include "rmaerror.h"
//#include "rmaprefs.h"

#include "exadvsnk.h"
#include "exerror.h"
#include "exsitsup.h"
#include "exaumgr.h"
#include "excontxt.h"

extern BOOL bEnableAdviceSink;


ExampleClientContext::ExampleClientContext(PNxWindow* pWindow)
    : m_lRefCount(0)
    , m_pPNxWindow(pWindow)
    , m_pClientSink(NULL)
    , m_pErrorSink(NULL)
    , m_pAuthMgr(NULL)
    , m_pSiteSupplier(NULL)
{
}


ExampleClientContext::~ExampleClientContext()
{
    Close();
};

void ExampleClientContext::Init(IUnknown*	 /*IN*/ pUnknown)
{
    char* pszCipher = NULL;

    m_pClientSink	= new ExampleClientAdviceSink(pUnknown);
    m_pErrorSink	= new ExampleErrorSink(pUnknown);
    m_pAuthMgr          = new ExampleAuthenticationManager();
    m_pSiteSupplier	= new ExampleSiteSupplier(pUnknown, m_pPNxWindow);

    if (m_pClientSink)
    {
	m_pClientSink->AddRef();
    }
    
    if (m_pErrorSink)
    {
	m_pErrorSink->AddRef();
    }

    if (m_pAuthMgr)
    {
	m_pAuthMgr->AddRef();
    }

    if (m_pSiteSupplier)
    {
	m_pSiteSupplier->AddRef();
    }
}

void ExampleClientContext::Close()
{
    PN_RELEASE(m_pClientSink);
    PN_RELEASE(m_pErrorSink);
    PN_RELEASE(m_pAuthMgr);
    PN_RELEASE(m_pSiteSupplier);
}



// *** IUnknown methods ***

/////////////////////////////////////////////////////////////////////////
//  Method:
//	IUnknown::QueryInterface
//  Purpose:
//	Implement this to export the interfaces supported by your 
//	object.
//
STDMETHODIMP ExampleClientContext::QueryInterface(REFIID riid, void** ppvObj)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
	AddRef();
	*ppvObj = this;
	return PNR_OK;
    }
    else if (m_pClientSink && 
	     m_pClientSink->QueryInterface(riid, ppvObj) == PNR_OK)
    {
	return PNR_OK;
    }
    else if (m_pErrorSink && 
	     m_pErrorSink->QueryInterface(riid, ppvObj) == PNR_OK)
    {
	return PNR_OK;
    }
    else if(m_pAuthMgr &&
	    m_pAuthMgr->QueryInterface(riid, ppvObj) == PNR_OK)
    {
	return PNR_OK;
    }
    else if(m_pSiteSupplier &&
	    m_pSiteSupplier->QueryInterface(riid, ppvObj) == PNR_OK)
    {
	return PNR_OK;
    }
    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}

/////////////////////////////////////////////////////////////////////////
//  Method:
//	IUnknown::AddRef
//  Purpose:
//	Everyone usually implements this the same... feel free to use
//	this implementation.
//
STDMETHODIMP_(ULONG32) ExampleClientContext::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}

/////////////////////////////////////////////////////////////////////////
//  Method:
//	IUnknown::Release
//  Purpose:
//	Everyone usually implements this the same... feel free to use
//	this implementation.
//
STDMETHODIMP_(ULONG32) ExampleClientContext::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
        return m_lRefCount;
    }

    delete this;
    return 0;
}
