/****************************************************************************
 * 
 *  $Id: exerror.cpp,v 1.2 1999/11/17 01:41:47 jfarr Exp $
 *
 *  Copyright (C) 1995,1996,1997,1998 RealNetworks, Inc.
 *  All rights reserved.
 *
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary 
 *  information of RealNetworks, Inc, and is licensed
 *  subject to restrictions on use and distribution.
 *
 *
 *  Sample Implementation of IRMAErrorSink Interface
 *
 */

#include "pntypes.h"

#include "os.h"
#include "pncom.h"
#include "rmacomm.h"
#include "rmaerror.h"
#include "rmapckts.h"

#include "exerror.h"



ExampleErrorSink::ExampleErrorSink(IUnknown *pUnknown) 
    :m_lRefCount(0)
    ,m_pUnknown(NULL)
{
    if (pUnknown)
    {
	m_pUnknown = pUnknown;
	m_pUnknown->AddRef();
    }
}

ExampleErrorSink::~ExampleErrorSink()
{
    if (m_pUnknown)
    {
	m_pUnknown->Release();
	m_pUnknown = NULL;
    }
}

// *** IUnknown methods ***

/////////////////////////////////////////////////////////////////////////
//  Method:
//	IUnknown::QueryInterface
//  Purpose:
//	Implement this to export the interfaces supported by your 
//	object.
//
STDMETHODIMP ExampleErrorSink::QueryInterface(REFIID riid, void** ppvObj)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
	AddRef();
	*ppvObj = (IUnknown*)(IRMAErrorSink*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMAErrorSink))
    {
	AddRef();
	*ppvObj = (IRMAErrorSink*) this;
	return PNR_OK;
    }

    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}

/////////////////////////////////////////////////////////////////////////
//  Method:
//	IUnknown::AddRef
//  Purpose:
//	Everyone usually implements this the same... feel free to use
//	this implementation.
//
STDMETHODIMP_(ULONG32) ExampleErrorSink::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}

/////////////////////////////////////////////////////////////////////////
//  Method:
//	IUnknown::Release
//  Purpose:
//	Everyone usually implements this the same... feel free to use
//	this implementation.
//
STDMETHODIMP_(ULONG32) ExampleErrorSink::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
        return m_lRefCount;
    }

    delete this;
    return 0;
}

/*
 *  IRMAErrorSink methods
 */

STDMETHODIMP 
ExampleErrorSink::ErrorOccurred(const UINT8	unSeverity,  
				const ULONG32	ulRMACode,
				const ULONG32	ulUserCode,
				const char*	pUserString,
				const char*	pMoreInfoURL
				)
{
    IRMABuffer* pBuffer = GetErrorText(ulRMACode);

    fprintf(stdout, "Report(%d, 0x%x, \"%s\", %ld, \"%s\", \"%s\")\n",
		    unSeverity,
		    ulRMACode,
		    (pUserString && *pUserString) ? pUserString : "(NULL)",
		    ulUserCode,
		    (pMoreInfoURL && *pMoreInfoURL) ? pMoreInfoURL : "(NULL)",
                    pBuffer ? (char *)pBuffer->GetBuffer() : "(unknown error)");
    if (pBuffer != NULL)
    {
	pBuffer->Release();
    }
    return PNR_OK;
}

IRMABuffer*
ExampleErrorSink::GetErrorText(const ULONG32 ulRMACode)
{
    IRMAErrorMessages* pErrorMessages;
    IRMABuffer*        pBuffer = NULL;

    if (m_pUnknown && SUCCEEDED(m_pUnknown->QueryInterface(
	IID_IRMAErrorMessages, (void**) &pErrorMessages)))
    {
	pBuffer = pErrorMessages->GetErrorText(ulRMACode);
	pErrorMessages->Release();
    }
    return pBuffer;
}

