/****************************************************************************
 * 
 *  $Id: exsitsup.cpp,v 1.3 1999/03/18 00:22:31 jordanb Exp $
 *  
 *  Copyright (C) 1995,1996,1997,1998 RealNetworks.
 *  All rights reserved.
 *  
 *  This program contains proprietary 
 *  information of RealNetworks, Inc, and is licensed
 *  subject to restrictions on use and distribution.
 *
 */

#include <stdio.h>

#include "pncom.h"
#include "pntypes.h"
#include "pnwintyp.h"
#include "rmawin.h"
#include "rmapckts.h"
#include "rmacomm.h"
#include "fivemmap.h"
#include "exsitsup.h"

#include <Xm/Xm.h>
#include <Xm/DrawingA.h>

// global vars defined in main.cpp
extern Widget  container;
extern Widget  toplevel;
extern Widget  visualComponent;
extern Widget  playButton;
extern Widget  pauseButton;
extern Widget  stopButton;

#define VERT_OFFSET   40
#define HORIZ_OFFSET  30


/************************************************************************
 *  Method:
 *    Constructor
 */
ExampleSiteSupplier::ExampleSiteSupplier(IUnknown* pUnkPlayer, PNxWindow* pWindow)
    : m_lRefCount(0)
    , m_pPNxWindow(pWindow)
    , m_pSiteManager(NULL)
    , m_pCCF(NULL)
    , m_pUnkPlayer(pUnkPlayer)
    , m_pSiteWatcherSite(NULL)
    , m_bFirstTime(TRUE)
{
    if (m_pUnkPlayer)
    {
	m_pUnkPlayer->QueryInterface(IID_IRMASiteManager,
			(void**)&m_pSiteManager);

	m_pUnkPlayer->QueryInterface(IID_IRMACommonClassFactory,
			(void**)&m_pCCF);

	m_pUnkPlayer->AddRef();
    }
};

/************************************************************************
 *  Method:
 *    Destructor
 */
ExampleSiteSupplier::~ExampleSiteSupplier()
{
    PN_RELEASE(m_pSiteManager);
    PN_RELEASE(m_pCCF);
    PN_RELEASE(m_pUnkPlayer);
}

/************************************************************************
 *  Method:
 *    IUnknown::QueryInterface
 */
STDMETHODIMP 
ExampleSiteSupplier::QueryInterface(REFIID riid, void** ppvObj)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
	AddRef();
	*ppvObj = (IUnknown*)(IRMASiteSupplier*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMASiteSupplier))
    {
	AddRef();
	*ppvObj = (IRMASiteSupplier*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMASiteWatcher))
    {
	AddRef();
	*ppvObj = (IRMASiteWatcher*)this;
	return PNR_OK;
    }

    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}

/************************************************************************
 *  Method:
 *    IUnknown::AddRef
 */
STDMETHODIMP_(ULONG32) 
ExampleSiteSupplier::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}

/************************************************************************
 *  Method:
 *    IUnknown::Release
 */
STDMETHODIMP_(ULONG32) 
ExampleSiteSupplier::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
        return m_lRefCount;
    }

    delete this;
    return 0;
}

/************************************************************************
 *  Method:
 *    IRMASiteSupplier::SitesNeeded
 *  Purpose:
 *    Called to inform the site supplier that a site with a particular
 *    set of characteristics is needed. If the site supplier can 
 *    fulfill the request it should call the site manager and add one
 *    or more new sites.
 *    Note that the request for sites is associated with a Request ID
 *    the client core will inform the site supplier when this requested
 *    site is no longer needed.
 */
STDMETHODIMP 
ExampleSiteSupplier::SitesNeeded
(
    UINT32	uRequestID,
    IRMAValues*	pProps
)
{
    /*
     * If there are no properties, then we can't really create a
     * site, because we have no idea what type of site is desired!
     */
    if (!pProps)
    {
	return PNR_INVALID_PARAMETER;
    }

    HRESULT		hres		= PNR_OK;
    IRMAValues*		pSiteProps	= NULL;
    IRMASiteWindowed*	pSiteWindowed	= NULL;
    IRMABuffer*		pValue		= NULL;
    UINT32		style		= 0;
    IRMASite*		pSite;

    // Just let the RMA client core create a windowed site for us.
    hres = m_pCCF->CreateInstance(CLSID_IRMASiteWindowed,(void**)&pSiteWindowed);
    if (PNR_OK != hres)
    {
	goto exit;
    }

    hres = pSiteWindowed->QueryInterface(IID_IRMASite,(void**)&pSite);
    if (PNR_OK != hres)
    {
	goto exit;
    }

    hres = pSiteWindowed->QueryInterface(IID_IRMAValues,(void**)&pSiteProps);
    if (PNR_OK != hres)
    {
	goto exit;
    }

    /*
     * We need to figure out what type of site we are supposed to
     * to create. We need to "switch" between site user and site
     * properties. So look for the well known site user properties
     * that are mapped onto sites...
     */
    hres = pProps->GetPropertyCString("playto",pValue);
    if (PNR_OK == hres)
    {
	pSiteProps->SetPropertyCString("channel",pValue);
	PN_RELEASE(pValue);
    }
    else
    {
	hres = pProps->GetPropertyCString("name",pValue);
	if (PNR_OK == hres)
	{
	    pSiteProps->SetPropertyCString("LayoutGroup",pValue);
    	    PN_RELEASE(pValue);
	}
    }

    hres = pSiteWindowed->AttachWindow(m_pPNxWindow);
    if (PNR_OK != hres)
    {
	goto exit;
    }

    /*
     * We need to wait until we have set all the properties before
     * we add the site.
     */
    hres = m_pSiteManager->AddSite(pSite);
    if (PNR_OK != hres)
    {
	goto exit;
    }

    pSite->AttachWatcher(this);

    m_CreatedSites.SetAt((void*)uRequestID,pSite);
    pSite->AddRef();

exit:

    PN_RELEASE(pSiteProps);
    PN_RELEASE(pSiteWindowed);
    PN_RELEASE(pSite);

    return hres;
}

/************************************************************************
 *  Method:
 *    IRMASiteSupplier::SitesNotNeeded
 *  Purpose:
 *    Called to inform the site supplier that all sites from a previos
 *    site request are no longer needed. If the site supplier had 
 *    previously created non-persistant sites (like popup windows)
 *    to fulfill a request for sites it should call the site manager 
 *    and remove those sites.
 */
STDMETHODIMP 
ExampleSiteSupplier::SitesNotNeeded(UINT32 uRequestID)
{
    printf("SitesNotNeeded.\n");

    IRMASite*		pSite = NULL;
    IRMASiteWindowed*	pSiteWindowed = NULL;
    void*		pVoid = NULL;

    if (!m_CreatedSites.Lookup((void*)uRequestID,pVoid))
    {
	return PNR_INVALID_PARAMETER;
    }
    pSite = (IRMASite*)pVoid;

    m_pSiteManager->RemoveSite(pSite);

    // get the IRMASiteWindowed ptr
    pSite->QueryInterface(IID_IRMASiteWindowed,(void**)&pSiteWindowed);
    pSiteWindowed->DetachWindow();
    PN_RELEASE(pSiteWindowed);

    m_CreatedSites.RemoveKey((void*)uRequestID);

    // ref count = 1; deleted from this object's view!
    pSite->Release();

    return PNR_OK;
}

/************************************************************************
 *  Method:
 *    IRMASiteSupplier::BeginChangeLayout
 *  Purpose:
 *    Called to inform the site supplier a layout change has beginning
 *    it can expect to recieve SitesNeeded() and SitesNotNeeded() calls
 *    while a layout change is in progress,
 */
STDMETHODIMP 
ExampleSiteSupplier::BeginChangeLayout()
{
    return PNR_OK;
}

/************************************************************************
 *  Method:
 *    IRMASiteSupplier::DoneChangeLayout
 *  Purpose:
 *    Called to inform the site supplier the layout change has been
 *    completed.
 */
STDMETHODIMP 
ExampleSiteSupplier::DoneChangeLayout()
{
    return PNR_OK;
}


/*
 * IRMASiteWatcher methods called by the site when a watcher 
 * is attached to or detached from it.
 */
/************************************************************************
 *  Method:
 *    IRMASite::AttachSite
 */
STDMETHODIMP 
ExampleSiteSupplier::AttachSite(IRMASite* /*IN*/ pSite)
{
    if (m_pSiteWatcherSite) return PNR_UNEXPECTED;
    m_pSiteWatcherSite = pSite;
    if (m_pSiteWatcherSite) m_pSiteWatcherSite->AddRef();
    return PNR_OK;
}

/************************************************************************
 *  Method:
 *    IRMASite::DetachSite
 */
STDMETHODIMP 
ExampleSiteSupplier::DetachSite()
{
    if (!m_pSiteWatcherSite) return PNR_UNEXPECTED;
    PN_RELEASE(m_pSiteWatcherSite);
    return PNR_OK;
}

/*
 * IRMASiteWatcher methods called by the site an attempt is
 * made to change it's position or size. The site watcher must
 * return PNR_OK for the change to occur. If the site watcher
 * returns any value other than PNR_OK then the size or position
 * will not change. The site watcher can also modify the new
 * size of position.
 */

/************************************************************************
 *  Method:
 *    IRMASite::ChangingPosition
 */
STDMETHODIMP 
ExampleSiteSupplier::ChangingPosition
(
    PNxPoint			posOld,
    REF(PNxPoint)   /*IN-OUT*/	posNew
)
{
    return PNR_OK;
}

/************************************************************************
 *  Method:
 *    IRMASite::ChangingSize
 */
STDMETHODIMP 
ExampleSiteSupplier::ChangingSize
(
    PNxSize			sizeOld,
    REF(PNxSize) /*IN-OUT*/	sizeNew
)
{
#ifdef _DEBUG
  //printf("SiteWatcher: ChangingSize - new Width: %ld Height: %ld\n",
  // 	    sizeNew.cx, sizeNew.cy);
#endif

    // see if we really have a new size 
    if (m_width != sizeNew.cx || m_height != sizeNew.cy || m_bFirstTime)
    {
        m_bFirstTime = FALSE; 

        // save the new values 
	m_width = sizeNew.cx;
	m_height = sizeNew.cy;

	int topLevelH = m_height + (VERT_OFFSET * 2);
	int topLevelW = m_width + (HORIZ_OFFSET * 2);

	// layout  our window(s)
        XtVaSetValues(toplevel,
	   	      XmNwidth, topLevelW,
		      XmNheight, topLevelH,
		      NULL);

        XtVaSetValues(container,
	   	      XmNwidth,  topLevelW,
		      XmNheight, topLevelH,
		      NULL);

        XtVaSetValues(visualComponent,
	   	      XmNwidth,  m_width,
		      XmNheight, m_height,
		      NULL);

        XtVaSetValues(visualComponent,
		      XmNx, HORIZ_OFFSET,
		      XmNy, VERT_OFFSET,
		      NULL);

	Dimension buttonH = 0;
	Dimension buttonW = 0;
        XtVaGetValues(playButton,
	   	      XmNwidth,  &buttonW,
		      XmNheight, &buttonH,
		      NULL);

	// set button position/dimensions
        XtVaSetValues(playButton,
		      XmNx, 5,
		      NULL);

        XtVaSetValues(pauseButton,
		      XmNx, 5 + (int) buttonW + 5,
		      NULL);

        XtVaSetValues(stopButton,
		      XmNx, 5 + (int) buttonW + 5 + (int) buttonW + 5,
		      NULL);
    }

    // Allow the change!   
    return PNR_OK;
}
