/****************************************************************************
 * 
 *  $Id: exsitsup.h,v 1.2 1998/12/10 02:55:01 jordanb Exp $
 *  
 *  Copyright (C) 1995,1996,1997,1998 RealNetworks.
 *  All rights reserved.
 *
 *  This program contains proprietary 
 *  information of RealNetworks, Inc, and is licensed
 *  subject to restrictions on use and distribution.
 *
 */

#ifndef _EXSITSUP_H_
#define _EXSITSUP_H_

/****************************************************************************
 * 
 *  Class:
 *
 *	ExampleSiteSupplier
 *
 *  Purpose:
 *
 *	Implementation for ragui's IRMASiteSupplier
 *
 */
class ExampleSiteSupplier : 
	public IRMASiteSupplier, IRMASiteWatcher
{
private:
    LONG32		    m_lRefCount;
    IRMASiteManager*	    m_pSiteManager;
    IRMACommonClassFactory* m_pCCF;
    IUnknown*		    m_pUnkPlayer;
    FiveMinuteMap	    m_CreatedSites;
    PNxWindow*              m_pPNxWindow;
    INT32		    m_width;
    INT32		    m_height;
    IRMASite*               m_pSiteWatcherSite;
    BOOL                    m_bFirstTime;

    ~ExampleSiteSupplier();
    PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME;

public:
    ExampleSiteSupplier(IUnknown* pUnkPlayer, PNxWindow* pWindow);
    
    /*
     * IUnknown methods
     */
    STDMETHOD(QueryInterface)	(THIS_
				REFIID riid,
				void** ppvObj);

    STDMETHOD_(ULONG32,AddRef)	(THIS);

    STDMETHOD_(ULONG32,Release)	(THIS);

    /*
     * IRMASiteSupplier methods
     */

    /************************************************************************
     *	Method:
     *	    IRMASiteSupplier::SitesNeeded
     *	Purpose:
     *	  Called to inform the site supplier that a site with a particular
     *	  set of characteristics is needed. If the site supplier can 
     *	  fulfill the request it should call the site manager and add one
     *	  or more new sites.
     *    Note that the request for sites is associated with a Request ID
     *    the client core will inform the site supplier when this requested
     *    site is no longer needed.
     */
    STDMETHOD(SitesNeeded)	(THIS_
				UINT32			uRequestID,
				IRMAValues*		pSiteProps);

    /************************************************************************
     *  Method:
     *    IRMASiteSupplier::SitesNotNeeded
     *  Purpose:
     *    Called to inform the site supplier that all sites from a previos
     *	  site request are no longer needed. If the site supplier had 
     *	  previously created non-persistant sites (like popup windows)
     *    to fulfill a request for sites it should call the site manager 
     *    and remove those sites.
     */
    STDMETHOD(SitesNotNeeded)
				(THIS_
				UINT32			uRequestID);


    /************************************************************************
     *  Method:
     *    IRMASiteSupplier::BeginChangeLayout
     *  Purpose:
     *    Called to inform the site supplier a layout change has beginning
     *	  it can expect to recieve SitesNeeded() and SitesNotNeeded() calls
     *	  while a layout change is in progress,
     */
    STDMETHOD(BeginChangeLayout) (THIS);

    /************************************************************************
     *  Method:
     *    IRMASiteSupplier::DoneChangeLayout
     *  Purpose:
     *    Called to inform the site supplier the layout change has been
     *	  completed.
     */
    STDMETHOD(DoneChangeLayout) (THIS);


    /*
     * IRMASiteWatcher methods
     */


    /*
     * IRMASiteWatcher methods called by the site when a watcher 
     * is attached to or detached from it.
     */
    STDMETHOD(AttachSite)	(THIS_
				IRMASite*	/*IN*/	pSite);

    STDMETHOD(DetachSite)	(THIS);

    /*
     * IRMASiteWatcher methods called by the site an attempt is
     * made to change it's position or size. The site watcher must
     * return PNR_OK for the change to occur. If the site watcher
     * returns any value other than PNR_OK then the size or position
     * will not change. The site watcher can also modify the new
     * size of position.
     */
    STDMETHOD(ChangingPosition)	(THIS_
				PNxPoint		posOld,
				REF(PNxPoint)/*IN-OUT*/	posNew);

    STDMETHOD(ChangingSize)	(THIS_
				PNxSize			sizeOld,
				REF(PNxSize) /*IN-OUT*/	sizeNew);
};

#endif // _EXSITSUP_H_

