/****************************************************************************
 * 
 *  $Id: main.cpp,v 1.5 1999/04/29 18:06:27 jordanb Exp $
 *
 *  Copyright (C) 1995,1996,1997,1998 RealNetworks, Inc.
 *  All rights reserved.
 *
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary 
 *  information of RealNetworks, Inc, and is licensed
 *  subject to restrictions on use and distribution.
 *
 *
 *  Test client for RealMedia Architecture.
 *
 */

/****************************************************************************
 * 
 *  Test Client:
 *  
 *
 *  This is a Motif based test client running on Unix only.
 *
 */ 

#ifndef _UNIX
#error "This module is platform specific and will run only on Motif-based UNIX systems."
#endif


#define INITGUID

#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <Xm/PushB.h>

#include "pntypes.h"

#include <stdlib.h>
#include "os.h"
#include "pnwintyp.h"

#include "pncom.h"
#include "rmapckts.h"
#include "rmacomm.h"
#include "rmamon.h"
#include "rmafiles.h"
#include "rmaengin.h"
#include "rmacore.h"
#include "rmaclsnk.h"
#include "rmaerror.h"
#include "rmaauth.h"
#include "rmawin.h"
#include "rmaprefs.h"

#include "fivemmap.h"

#include "exadvsnk.h"
#include "exerror.h"
#include "exsitsup.h"
#include "exaumgr.h"
#include "excontxt.h"


// typedef for SetDLLAccessPath
typedef PN_RESULT (PNEXPORT_PTR FPRMSETDLLACCESSPATH) (const char*);


// global  data 
XtAppContext       appcontext;
Widget             container;
Widget             toplevel;
Widget             visualComponent;
Widget             playButton, pauseButton, stopButton;


FPRMCREATEENGINE        fpCreateEngine       = NULL;
FPRMCLOSEENGINE	        fpCloseEngine        = NULL;
FPRMSETDLLACCESSPATH	fpSetDLLAccessPath   = NULL;

Display*           xDisplay             = NULL;
Window             xWindow              = 0;
BOOL               bEnableAdviceSink    = FALSE;
BOOL               bEnableVerboseMode   = FALSE;
HINSTANCE          hDll		        = NULL;
IRMAClientEngine*  pEngine		= NULL;
IRMAClientEngineSelector* pSelector     = NULL;
IRMAPlayer*        pPlayer              = NULL;
ExampleClientContext*   pContext            = NULL;
BOOL               isPaused             = FALSE;
BOOL               isStopped            = FALSE;




// local static functions 
static void PrintUsage();
static void CreateWidgets(int argc, char *argv[]);
static PN_RESULT InitCore();

// callback functions 
static BOOL ProcessIdle(XtPointer data);
static void HandleXEvents(Widget widget, XtPointer pDummy, XmDrawingAreaCallbackStruct* pCBStruct);
static void PlayButtonClicked(Widget w, XtPointer clientData, XtPointer callData);
static void PauseButtonClicked(Widget w, XtPointer clientData, XtPointer callData);
static void StopButtonClicked(Widget w, XtPointer clientData, XtPointer callData);
static void HandleMotionNotify(Widget w, XEvent* event, String* params, Cardinal* num_params);


// const data
const int WIDTH  = 252;
const int HEIGHT = 224;


// prep actions table
static XtActionsRec actions[] = { 
                                   {"HandleMotionNotify", HandleMotionNotify },
				};
// prep translation table
static char translations[] = "<Motion>: HandleMotionNotify()";
    

// main entry point
int main(int argc, char *argv[])
{
    PN_RESULT		    theErr		= PNR_OK;

    IRMAErrorSink*	    pErrorSink		= NULL;
    IRMAErrorSinkControl*   pErrorSinkControl	= NULL;
    BOOL                    bURLFound           = FALSE;
    char*		    pszURL		= NULL;
    struct _PNxEvent*       pEvent              = NULL;


    // examine arguments
    for (int i = 1; i < argc; i++)
    {
        if (0 == stricmp(argv[i], "-a"))
        {
            bEnableAdviceSink = TRUE;
        }
        else if (0 == stricmp(argv[i], "-s"))
        {
            bEnableVerboseMode = TRUE;
        }
        else if (!bURLFound)
        {
            pszURL = argv[i];
            bURLFound = TRUE;
        }
        else
        {
            PrintUsage();
	    return -1;
        }
    }

    if (!bURLFound)
    {
        PrintUsage();
        return -1;
    }

    // create windows 
    CreateWidgets(argc, argv);

    // creat PN X Window
    PNxWindow pnWindow;
    memset(&pnWindow, 0, sizeof(PNxWindow));
    pnWindow.display = xDisplay;
    pnWindow.window = (void*) xWindow;

    printf("X Window = %d.\n", (int) pnWindow.window);
    printf("X Display = %d.\n", (int) pnWindow.display);

    // initialize G2 Core 
    if (InitCore() != PNR_OK)
    {
        printf("mtclient: Unable to initialize G2 Client Core.\n");
        exit(-2);
    }

    pContext = new ExampleClientContext(&pnWindow);
    if (PNR_OK != pEngine->CreatePlayer(pPlayer))
    {
        theErr = PNR_FAILED;
	printf("mtclient: Unable to create player.\n");
    }
    else
    {
        if (bEnableVerboseMode)
        {
            fprintf(stdout, "Initializing  player...\n");
        }

        // initialize context and player
        pContext->Init(pPlayer);
        pPlayer->SetClientContext(pContext);

	// setup error sink
	pPlayer->QueryInterface(IID_IRMAErrorSinkControl, 
				     (void**) &pErrorSinkControl);
	if (pErrorSinkControl)
	{	
    	    pContext->QueryInterface(IID_IRMAErrorSink, 
    					    (void**) &pErrorSink);
    	    if (pErrorSink)
    	    {
    		pErrorSinkControl->AddErrorSink(pErrorSink, PNLOG_EMERG, PNLOG_INFO);
    	    }

	    pErrorSink->Release();
	}

	pErrorSinkControl->Release();
	pErrorSinkControl = NULL;

	// get selector
        pEngine->QueryInterface(IID_IRMAClientEngineSelector, (void**) &pSelector);
        if (!pSelector)
        {
            fprintf(stdout, "mtclient: Unable to get IRMAClientEngineSelector interface.\n");
        }

        if (bEnableVerboseMode)
        {
            fprintf(stdout, "Starting player...\n");
        }

        if (PNR_OK == pPlayer->OpenURL(pszURL))
	{
	    pPlayer->Begin();
	}
    }

    // set up idle processing
    XtAppAddWorkProc(appcontext, (XtWorkProc) ProcessIdle, toplevel);

    // main Xt app lop 
    XtAppMainLoop(appcontext);
}

BOOL ProcessIdle(XtPointer data)
{
    static struct timeval mtime;
    static PNxEvent* nullEvent = NULL;

    // give client core time to work
    mtime.tv_sec  = 0; 
    mtime.tv_usec = 20;  // 20 ms is minimum

    pSelector->Select(0, 
  	             (fd_set *) 0, 
		     (fd_set *) 0,
		     (fd_set *) 0,
		     &mtime);

    pEngine->EventOccurred(nullEvent);

    return FALSE;
}

void PlayButtonClicked(Widget w, XtPointer clientData, XtPointer callData)
{
    //printf("PlayButtonClicked: entered.\n");

    if (isPaused || isStopped)
       pPlayer->Begin();

    isStopped = FALSE;
    isPaused = FALSE;
}

void PauseButtonClicked(Widget w, XtPointer clientData, XtPointer callData)
{
    //printf("PauseButtonClicked: entered.\n");

    if (!isPaused && !isStopped)
       pPlayer->Pause();

    isStopped = FALSE;
    isPaused = TRUE;
}

void StopButtonClicked(Widget w, XtPointer clientData, XtPointer callData)
{
    //printf("StopButtonClicked: entered.\n");

    if (!isStopped)
       pPlayer->Stop();

    isStopped = TRUE;
    isPaused = FALSE;
}

PN_RESULT InitCore()
{
    PN_RESULT res = PNR_FAILED;

    if (bEnableVerboseMode)
    {
        fprintf(stdout, "Initializing  G2 Client Core...\n");
    }

    char* rphome = getenv("REALPLAYER_HOME");
    if (!rphome)
    {
	fprintf(stderr, "mtclient: can't locate install directory\nPlease set REALPLAYER_HOME environment variable!\n");
	exit(1);
    }
    char szDllName[512];
    *szDllName = '\0';
    sprintf(szDllName, "%s/Common/rmacore.so.6.0", rphome);

    if (hDll = LoadLibrary(szDllName))
    {
        // retrieve the proc addresses from the module
        fpCreateEngine = (FPRMCREATEENGINE) GetProcAddress(hDll, "CreateEngine");
        fpCloseEngine  = (FPRMCLOSEENGINE)  GetProcAddress(hDll, "CloseEngine");
	fpSetDLLAccessPath = (FPRMSETDLLACCESSPATH) GetProcAddress(hDll, "SetDLLAccessPath");


	if (fpCreateEngine && fpCloseEngine && fpSetDLLAccessPath)
	{
	    /*Create a null delimited, double-null terminated string containing
	     * the paths to the encnet library (DT_Common) and the sdpplin library
	     * (DT_Plugins)... */
	    char pPaths[256];
	    char* pPathNextPosition = pPaths;
	    memset(pPaths, 0, 256);

	    char pNextPath[256];
	    memset(pNextPath, 0, 256);

	    const char* pCurrentDir = "./";

	    sprintf(pNextPath, "DT_Common=%s/Common", rphome);
	    memcpy(pPathNextPosition, pNextPath, strlen(pNextPath) + 1);
	    pPathNextPosition += strlen(pNextPath) + 1;

	    sprintf(pNextPath, "DT_Plugins=%s/Plugins", rphome);
	    memcpy(pPathNextPosition, pNextPath, strlen(pNextPath) + 1);
	    pPathNextPosition += strlen(pNextPath) + 1;

	    sprintf(pNextPath, "DT_Codecs=%s/Codecs", rphome);
	    memcpy(pPathNextPosition, pNextPath, strlen(pNextPath) + 1);

	    fpSetDLLAccessPath((char*)pPaths);

	    if (PNR_OK == fpCreateEngine((IRMAClientEngine**) &pEngine))
            {
	        res = PNR_OK;

		if (bEnableVerboseMode)
		{
		    fprintf(stdout, "G2 Client Core ready...\n");
		}
	    }
	}
    }
    else
    {
 	fprintf(stdout, "Failed to load the 'rmacore.so.6.0' shared library\n");
	fprintf(stdout, dlerror());
    }
 
    return res;
}


void CreateWidgets(int argc, char *argv[])
{
    // create top level window, container widget and video display window
    toplevel = XtVaAppInitialize(&appcontext, "MotifWin", NULL, 0, &argc, argv, NULL, NULL);


    container = XtVaCreateManagedWidget("container", xmDrawingAreaWidgetClass, toplevel,
                                          XmNwidth,  WIDTH,
                                          XmNheight, HEIGHT,
                                          NULL);

    visualComponent = XtVaCreateManagedWidget("drawingarea", xmDrawingAreaWidgetClass, container,
                                          XmNwidth,  192,
                                          XmNheight, 144,
					  XmNx, 30,
					  XmNy, 40,
					  XmNbackground, 0,
                                          NULL);

    
    
    playButton = XtVaCreateManagedWidget("playButton", xmPushButtonWidgetClass, container, NULL);
    pauseButton = XtVaCreateManagedWidget("pauseButton", xmPushButtonWidgetClass, container, NULL);
    stopButton = XtVaCreateManagedWidget("stopButton", xmPushButtonWidgetClass, container, NULL);

    XmString label;
    label = XmStringCreateLocalized("Play");
    XtVaSetValues(playButton, XmNlabelString, label, NULL);
    XmStringFree(label);
					 
    label = XmStringCreateLocalized("Pause");
    XtVaSetValues(pauseButton, XmNlabelString, label, NULL);
    XmStringFree(label);

    label = XmStringCreateLocalized("Stop");
    XtVaSetValues(stopButton, XmNlabelString, label, NULL);
    XmStringFree(label);

    XtRealizeWidget(toplevel);

    // get the X Display and X Window
    xDisplay = XtDisplay(toplevel);
    xWindow = XtWindow(visualComponent);

    // add actions table to widget
    XtAppAddActions(appcontext, actions, XtNumber(actions));

    // parse and install translation table
    XtTranslations parsedTranslations = XtParseTranslationTable(translations);
    XtOverrideTranslations(visualComponent, parsedTranslations);
    
    // hook events
    XtAddCallback(visualComponent, XmNexposeCallback, (XtCallbackProc) HandleXEvents, 0);
    XtAddCallback(visualComponent, XmNinputCallback, (XtCallbackProc) HandleXEvents, 0);

    XtAddCallback(playButton, XmNactivateCallback, PlayButtonClicked, 0);
    XtAddCallback(pauseButton, XmNactivateCallback, PauseButtonClicked, 0);
    XtAddCallback(stopButton, XmNactivateCallback, StopButtonClicked, 0);
}

void HandleXEvents(Widget widget, XtPointer pDummy, XmDrawingAreaCallbackStruct* pCBStruct)
{
    static PNxEvent event;

    //printf("HandleXEvents: Event = %d.\n", pCBStruct->event->type);

    event.event = pCBStruct->event->type;
    event.window = pCBStruct->event->xany.window;
    event.param1 = pCBStruct->event->xany.display;
    event.param2 = pCBStruct->event;

    // pass event to Client Engine
    pEngine->EventOccurred(&event);
}

static void HandleMotionNotify(Widget /*w*/, XEvent* xevent, String* /*params*/, Cardinal* /*num_params*/)
{
    static PNxEvent event;

    //printf("HandleMotionNotify: entered. Mouse Pos X == %d Y == %d.\n", 
    //	   ((XMotionEvent*)xevent)->x, ((XMotionEvent*)xevent)->y);

    event.event =  MotionNotify;
    event.window = (void*) xevent->xany.window;
    event.param1 = xevent->xany.display;
    event.param2 = xevent;

    // pass event to Client Engine
    pEngine->EventOccurred(&event);
}

void PrintUsage()
{
    fprintf(stdout,"\n");
    fprintf(stdout,"USAGE:\n%s [-as] <URL>\n", "mtclient");
    fprintf(stdout,"       -a : optional flag to show advise sink output\n");
    fprintf(stdout,"       -s : optional flag to output useful status messages\n");
    fprintf(stdout,"\n");
}





