/****************************************************************************
 * 
 *  $Id: pmixplin.cpp,v 1.21 1997/10/15 21:53:02 christof Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 *
 *  pmixplin.cpp
 *
 *  Audio Pre and Post Mix File Format Example for RealMedia Architecture.
 */


/****************************************************************************
 *
 *  Pre and Post Mix Hook Example FILE FORMAT	for (.PRE, audio/x-pn-pre)
 *						for (.PST, audio/x-pn-pst)
 *
 *  This is a very simple file format, that has a 150 byte file header,
 *  a 20 byte stream header, and then fixed packets of 100 bytes each 
 *  which represent 1 second chunks of data.
 */


/****************************************************************************
 * Defines
 */
#define   INITGUID     /* Interface ID's */

#ifdef _DEBUG
#undef PN_THIS_FILE		
static char PN_THIS_FILE[] = __FILE__;
#endif


/****************************************************************************
 * Includes
 */
#include <string.h>

#include "pntypes.h"
#include "pncom.h"
#include "rmacomm.h"
#include "rmapckts.h"
#include "rmafiles.h"
#include "rmaformt.h"
#include "rmaplugn.h"

#include "pmixplin.h"
#include "pmffplin.ver"


/****************************************************************************
 *  RMACreateInstance                                        ref:  rmaplugn.h
 *
 *  This routine creates a new instance of the CPostMixHookFF class.
 *  It is called when the RMA core application is launched, and whenever
 *  an URL associated with this plug-in is opened.
 */
STDAPI
RMACreateInstance(IUnknown** ppIUnknown)
{
    *ppIUnknown = (IUnknown*)(IRMAPlugin*)new CPostMixHookFF();
    if (*ppIUnknown)
    {
	(*ppIUnknown)->AddRef();
	return PNR_OK;
    }
    return PNR_OUTOFMEMORY;
}


/****************************************************************************
 *  CMyExampleFileFormat static variables                    ref:  pmixplin.h
 *
 *  These variables are passed to the RMA core to provide information about
 *  this plug-in. They are required to be static in order to remain valid
 *  for the lifetime of the plug-in.
 */
const char* CPostMixHookFF::zm_pDescription    = "Pre/Post Mix Example File Format Plugin";
const char* CPostMixHookFF::zm_pCopyright      = "(c) 1995,1996,1997 RealNetworks, All rights reserved.";
const char* CPostMixHookFF::zm_pMoreInfoURL    = "http://www.real.com";
const char* CPostMixHookFF::zm_pFileMimeTypes[]  = {"audio/x-pn-pre","audio/x-pn-pst",NULL};
const char* CPostMixHookFF::zm_pFileExtensions[] = {"pre","pst",NULL};
const char* CPostMixHookFF::zm_pFileOpenNames[]  = {"Pre/Post Mix Test File Format (*.pre,*.pst)", NULL};


/****************************************************************************
 *  IRMAPlugin::GetPluginInfo                                ref:  rmaplugn.h
 *
 *  This routine returns descriptive information about the plug-in, most
 *  of which is used in the About box of the user interface. It is called
 *  when the RMA core application is launched.
 */
STDMETHODIMP
CPostMixHookFF::GetPluginInfo
(
    REF(BOOL)	    /*OUT*/ bLoadMultiple,
    REF(const char*)/*OUT*/ pDescription,
    REF(const char*)/*OUT*/ pCopyright,
    REF(const char*)/*OUT*/ pMoreInfoURL,
    REF(UINT32)    /*OUT*/ ulVersionNumber
)
{
    bLoadMultiple = TRUE;   // Must be true for file formats.

    pDescription    = zm_pDescription;
    pCopyright	    = zm_pCopyright;
    pMoreInfoURL    = zm_pMoreInfoURL;
    ulVersionNumber = TARVER_ULONG32_VERSION;

    return PNR_OK;
}


/****************************************************************************
 *  IRMAFileFormatObject::GetFileFormatInfo                  ref:  rmaformt.h
 *
 *  This routine returns crucial information required to associate this
 *  plug-in with a given MIME type. This information tells the core which
 *  File Format plug-in to use for a particular URL. The routine is called
 *  when the RMA core application is launched.
 */
STDMETHODIMP
CPostMixHookFF::GetFileFormatInfo
(
    REF(const char**) /*OUT*/ pFileMimeTypes,
    REF(const char**) /*OUT*/ pFileExtensions,
    REF(const char**) /*OUT*/ pFileOpenNames
)
{
    pFileMimeTypes  = zm_pFileMimeTypes;
    pFileExtensions = zm_pFileExtensions;
    pFileOpenNames  = zm_pFileOpenNames;

    return PNR_OK;
}


/****************************************************************************
 *  IRMAPlugin::InitPlugin                                   ref:  rmaplugn.h
 *
 *  This routine performs initialization steps such as determining whether
 *  required interfaces are available. It is called when the RMA core 
 *  application is launched, and whenever an URL associated with this
 *  plug-in is opened.
 */
STDMETHODIMP
CPostMixHookFF::InitPlugin(IUnknown* /*IN*/ pContext)
{
    m_pContext = pContext;
    m_pContext->AddRef();
    m_pContext->QueryInterface(IID_IRMACommonClassFactory,(void**)&m_pCommonClassFactory);

    return PNR_OK;
}


/****************************************************************************
 *  IRMAFileFormatObject::InitFileFormat                     ref:  rmaformt.h
 *
 *  This routine initializes the file, and stores references to objects used
 *  throughout the example. It is called whenever an URL associated with this
 *  plug-in is opened.
 */
STDMETHODIMP
CPostMixHookFF::InitFileFormat
(
    IRMARequest*	    /*IN*/  pRequest, 
    IRMAFormatResponse*	    /*IN*/  pFormatResponse,
    IRMAFileObject*	    /*IN*/  pFileObject
)
{
    if (m_pURL)
    {
	delete [] m_pURL;
	m_pURL = NULL;
    }
    m_pRequest    = pRequest;
    m_pFFResponse = pFormatResponse;
    m_pFileObject = pFileObject;

    if (m_pRequest)	m_pRequest->AddRef();
    if (m_pFFResponse)	m_pFFResponse->AddRef();
    if (m_pFileObject)	m_pFileObject->AddRef();

    // we want to make sure the file object is initialized, we can't 
    if (m_pRequest)
    {
	char* pURL = 0;
	m_pRequest->GetURL(pURL);
	if (pURL)
	{
	    m_pURL = new char[strlen(pURL)+1];
	    if (!m_pURL) return PNR_OUTOFMEMORY;
	    strcpy(m_pURL,pURL);
	}
    }

    // actually return the header count until the file init is done... (See InitDone).
    m_state = InitPending;

    // Note, we need to pass ourself to the FileObject, because this is its
    // first oppurtunity to know that we implement the IRMAFileResponse
    // interface it will call for completed pending operations
    m_pFileObject->Init(PN_FILE_READ, this);

    return PNR_OK;
}	


/****************************************************************************
 *  IRMAFileResponse::InitDone                               ref:  rmafiles.h
 *
 *  This routine notifies the RMA core that the initialization of the file is
 *  done. It is called automatically when the initialization of the file
 *  has completed.
 */
STDMETHODIMP
CPostMixHookFF::InitDone
(
    PN_RESULT	status
)
{
    // If we are not ready then something has gone wrong
    if (m_state != InitPending) return PNR_UNEXPECTED;

    // This simple file format is not a container type, so it only supports one
    // stream and therefore one header, since we now know the file is initialized
    // we can return the header count to the controller...
    m_state = Ready;

    m_pFFResponse->InitDone(status);

    return PNR_OK;
}


/****************************************************************************
 *  IRMAFileFormatObject::GetFileHeader                      ref:  rmaformt.h
 *
 *  This routine returns to the RMA core an object containing the file
 *  header information. Several routines are actually required to complete
 *  the process due to the asynchronous nature of the RMA file system. This
 *  method is called by the RMA core after the file has been initialized.
 */
STDMETHODIMP
CPostMixHookFF::GetFileHeader()
{
    // If we are not ready then something has gone wrong
    if (m_state != Ready) return PNR_UNEXPECTED;

    // To give the file header we need to be make sure the file is 
    // positioned at the start of the file. We need to call the file 
    // object's seek method.

    // Since this is asyncronous we need to note our state so we can
    // correctly respond to the seek complete response from the file
    // object.
    m_state = GetFileHeaderSeekPending;

    // Actually seek...
    m_pFileObject->Seek(HEADER_OFFSET,FALSE);

    // See SeekDone() for next "step" of the GetStreamHeader process.

    return PNR_OK;
}


/****************************************************************************
 *  IRMAFileFormatObject::GetStreamHeader                    ref:  rmaformt.h
 *
 *  This routine returns to the RMA core an object containing the stream
 *  header information for a particular stream. Several routines are actually
 *  required to complete the process due to the asynchronous nature of the
 *  RMA file system. This method is called (after the file header has been
 *  read) by the RMA core for each stream in the file format.
 */
STDMETHODIMP CPostMixHookFF::GetStreamHeader(UINT16 unStreamNumber)
{
    // If we are not ready then something has gone wrong
    if (m_state != Ready) return PNR_UNEXPECTED;

    // To give the header we need to be make sure the file is positioned
    // at the start of the file. We need to call the file object's
    // seek method.

    // Since this is asyncronous we need to note our state so we can
    // correctly respond to the seek complete response from the file
    // object.
    m_state = GetStreamHeaderSeekPending;

    // Actually seek...
    m_pFileObject->Seek(HEADER_OFFSET, FALSE);

    // See SeekDone() for next "step" of the GetStreamHeader process.

    return PNR_OK;
}


/****************************************************************************
 *  IRMAFileFormatObject::GetPacket                          ref:  rmaformt.h
 *
 *  This routine returns to the RMA core an object containing the packet
 *  data for a particular stream. Several routines are actually required to
 *  complete the process due to the asynchronous nature of the RMA file
 *  system. This method is called by the RMA core each time it needs another
 *  packet.
 */
STDMETHODIMP
CPostMixHookFF::GetPacket(UINT16 unStreamNumber)
{
    PN_RESULT result = PNR_OK;

    // If we are not ready then something has gone wrong
    if (m_state != Ready) return PNR_UNEXPECTED;

    if (!m_bHeaderSent)
    {
        result = PNR_UNEXPECTED;
    }
    else
    {
	// We are being asked to get the next packet and we have sent
	// the header, so basically we know the next PACKET_SIZE bytes
	// should be the next packet... again, since read is asynchronous, 
	// we need to set up our state...
	m_state = GetPacketReadPending;

	// Actually read...
	m_pFileObject->Read(PACKET_SIZE);

	// See ReadDone() for next "step" of GetPacket()
    }

    return result;
}


/****************************************************************************
 *  IRMAFileResponse::SeekDone                               ref:  rmafiles.h
 *
 *  This routine is automatically called when the Seek() operation in the
 *  file is complete. Other actions are then taken dependent on the current
 *  processing state.
 */
STDMETHODIMP
CPostMixHookFF::SeekDone(PN_RESULT status)
{
    PN_RESULT result = PNR_OK;

    // Note, the seek may be done because we needed to seek to produce
    // a packet or because as a file format we were asked to seek to
    // a time position. We need to remember why we tried to seek and
    // respond accordingly...

    switch (m_state)
    {
	case GetFileHeaderSeekPending:
	{
	    // We are in the process of responding to a GetFileHeader()
	    // request and the seek has completed, so now we will try
	    // to read the first FILE_HEADER_SIZE bytes... again, since 
	    // this is asynchronous, we need to set up our state...
	    m_state = GetFileHeaderReadPending;

	    // Actually read...
	    m_pFileObject->Read(FILE_HEADER_SIZE);

	    // See ReadDone() for next "step" of GetFileHeader()
	}
	break;

	case GetStreamHeaderSeekPending:
	{
	    // We are in the process of responding to a GetStreamHeader()
	    // request and the seek has completed, so now we will try
	    // to read the first STREAM_HEADER_SIZE bytes... again, since 
	    // this is asynchronous, we need to set up our state...
	    m_state = GetStreamHeaderReadPending;

	    // Actually read...
	    m_pFileObject->Read(STREAM_HEADER_SIZE);

	    // See ReadDone() for next "step" of GetStreamHeader()
	}
	break;

	case SeekSeekPending:
	{
	    // We are in the process of responding to a Seek()
	    // request and the seek has completed, so we are really
	    // done with the seek, so let's inform the FormatResponse.
	    m_state = Ready;

	    // Tell the FormatResponse of our success in seeking.
	    m_pFFResponse->SeekDone(status);
	}
	break;

	default:
	{
	    result = PNR_UNEXPECTED;
	}
	break;
    }

    return result;
}


/****************************************************************************
 *  IRMAFileResponse::ReadDone                               ref:  rmafiles.h
 *
 *  This routine is automatically called when the Read() from the file is
 *  done. Other actions are then taken dependent on the current processing
 *  state.
 */
STDMETHODIMP
CPostMixHookFF::ReadDone
(
    PN_RESULT		status,
    IRMABuffer*		pBuffer
)
{
    PN_RESULT result = PNR_OK;

    // Note, the read may be done because we needed to read to produce
    // a header or a packet. We need to remember why we tried to read and
    // respond accordingly...

    switch (m_state)
    {
	case GetFileHeaderReadPending:
	{
	    // We are in the process of responding to a GetFileHeader()
	    // request and the read has completed, which means we
	    // are done and can go back to the ready state...
	    m_state = Ready;

	    // We now need to form a "header" object and pass it
	    // off to our controller. Notice since our header data
	    // comes straight out of the file as is, we don't need
	    // to copy data at all, the IRMABuffer returned by read
	    // can be placed into the header object and passed off.

	    // Create header object here, notice that we call the
	    // CreateInstance method of the controller, but we could
	    // have implemented our own object that exposed the IRMAValues
	    // interface.
	    IRMAValues* pValues;
	    if (PNR_OK == m_pCommonClassFactory->CreateInstance(CLSID_IRMAValues,
						(void**)&pValues))
	    {
		IRMABuffer* pTitle = NULL;
		IRMABuffer* pAuthor = NULL;
		IRMABuffer* pCopyright = NULL;
		IRMABuffer* pHookStreamInfo = NULL;
		char*	    pTitleData;
		char*	    pAuthorData;
		char*	    pCopyrightData;
		char*	    pHookStreamData;
		char*	    pBufferData;

		m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,(void**)&pTitle);
		m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,(void**)&pAuthor);
		m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,(void**)&pCopyright);
		m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,(void**)&pHookStreamInfo);

		pTitle->SetSize(INFO_SIZE+1);
		pAuthor->SetSize(INFO_SIZE+1);
		pCopyright->SetSize(INFO_SIZE+1);
		pHookStreamInfo->SetSize(INFO_SIZE+1);

		pTitleData	= (char*)pTitle->GetBuffer();
		pAuthorData	= (char*)pAuthor->GetBuffer();
		pCopyrightData	= (char*)pCopyright->GetBuffer();
		pHookStreamData	= (char*)pHookStreamInfo->GetBuffer();
		pBufferData	= (char*)pBuffer->GetBuffer();

		strncpy(pTitleData,pBufferData,INFO_SIZE);
		pTitleData[INFO_SIZE] = '\0';

		strncpy(pAuthorData,pBufferData+INFO_SIZE,INFO_SIZE);
		pAuthorData[INFO_SIZE] = '\0';

		strncpy(pCopyrightData,pBufferData+(2*INFO_SIZE),INFO_SIZE);
		pCopyrightData[INFO_SIZE] = '\0';

		strncpy(pHookStreamData,pBufferData+(3*INFO_SIZE),INFO_SIZE);
		pHookStreamData[INFO_SIZE] = '\0';

                // Fill in the Header with the relevant data...
                pValues->SetPropertyBuffer ("Title",        pTitle);
                pValues->SetPropertyBuffer ("Author",       pAuthor);
                pValues->SetPropertyBuffer ("Copyright",    pCopyright);
                pValues->SetPropertyBuffer ("HookInfo",     pHookStreamInfo);
		pValues->SetPropertyULONG32("StreamCount",  1);

		// Tell the FormatResponse of our success in 
		// getting the header.
		m_pFFResponse->FileHeaderReady(status, pValues);

		// Release our reference on the header!
		pValues->Release();
		pTitle->Release();
		pAuthor->Release();
		pCopyright->Release();
		pHookStreamInfo->Release();
	    }
	}
	break;

	case GetStreamHeaderReadPending:
	{
	    // We are in the process of responding to a GetStreamHeader()
	    // request and the read has completed, which means we
	    // are done and can go back to the ready state...
	    m_state = Ready;

	    // We now need to form a "header" object and pass it
	    // off to our controller. Notice since our header data
	    // comes straight out of the file as is, we don't need
	    // to copy data at all, the IRMABuffer returned by read
	    // can be placed into the header object and passed off.

	    // Create header object here, notice that we call the
	    // CreateInstance method of the controller, but we could
	    // have implemented our own object that exposed the IRMAValues
	    // interface.
	    IRMAValues* pValues;
	    if (PNR_OK == m_pCommonClassFactory->CreateInstance(CLSID_IRMAValues,
						(void**)&pValues))
	    {
		// Remember that we have sent the header and packets are
		// now valid to be sent... (just a little bit of
		// sanity checking).
		m_bHeaderSent = TRUE;

		UINT16	uStreamNumber = 0;
		UINT32	ulMaxBitRate = 8*PACKET_SIZE*(TIME_PER_PACKET/1000);
		UINT32	ulAvgBitRate = 8*PACKET_SIZE*(TIME_PER_PACKET/1000);
		UINT32	ulMaxPacketSize = PACKET_SIZE;
		UINT32	ulAvgPacketSize = PACKET_SIZE;
		UINT32	ulStartTime = 0;
		UINT32	ulPreroll = 0;
		UINT32	ulDuration = 20000; // Should be length of clip in ms.

		char sFileext[4];
		strncpy(sFileext, &m_pURL[strlen(m_pURL)-3],4);

		char	szMimeType[15];
		if (!strcmp( sFileext, "pre" )) 
			strcpy(szMimeType,"audio/x-pn-pre");
		else if (!strcmp( sFileext, "pst" )) 
			strcpy(szMimeType,"audio/x-pn-pst");

		IRMABuffer* pMimeType = 0;
		m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,(void**)&pMimeType);
		pMimeType->Set((UCHAR*) szMimeType, ::strlen(szMimeType) + 1);

		// Fill in the Header with the relevant data...
		pValues->SetPropertyBuffer ("OpaqueData",    pBuffer);
		pValues->SetPropertyULONG32("StreamNumber",  uStreamNumber);
		pValues->SetPropertyULONG32("MaxBitRate",    ulMaxBitRate);
		pValues->SetPropertyULONG32("AvgBitRate",    ulAvgBitRate);
		pValues->SetPropertyULONG32("MaxPacketSize", ulMaxPacketSize);
		pValues->SetPropertyULONG32("AvgPacketSize", ulAvgPacketSize);
		pValues->SetPropertyULONG32("StartTime",     ulStartTime);
		pValues->SetPropertyULONG32("Preroll",       ulPreroll);
		pValues->SetPropertyULONG32("Duration",      ulDuration);
 		pValues->SetPropertyCString("MimeType",      pMimeType);
		
		pMimeType->Release();

		IRMABuffer* pTitle = NULL;
		IRMABuffer* pAuthor = NULL;
		IRMABuffer* pCopyright = NULL;
		IRMABuffer* pHookStreamInfo = NULL;
		char*	    pTitleData;
		char*	    pAuthorData;
		char*	    pCopyrightData;
		char*	    pHookStreamData;
		char*	    pBufferData;

		m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,(void**)&pTitle);
		m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,(void**)&pAuthor);
		m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,(void**)&pCopyright);
		m_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,(void**)&pHookStreamInfo);

		pTitle->SetSize(INFO_SIZE+1);
		pAuthor->SetSize(INFO_SIZE+1);
		pCopyright->SetSize(INFO_SIZE+1);
		pHookStreamInfo->SetSize(INFO_SIZE+1);

		pTitleData	= (char*)pTitle->GetBuffer();
		pAuthorData	= (char*)pAuthor->GetBuffer();
		pCopyrightData	= (char*)pCopyright->GetBuffer();
		pHookStreamData	= (char*)pHookStreamInfo->GetBuffer();
		pBufferData	= (char*)pBuffer->GetBuffer();

		strncpy(pTitleData,pBufferData,INFO_SIZE);
		pTitleData[INFO_SIZE] = '\0';

		strncpy(pAuthorData,pBufferData+INFO_SIZE,INFO_SIZE);
		pAuthorData[INFO_SIZE] = '\0';

		strncpy(pCopyrightData,pBufferData+(2*INFO_SIZE),INFO_SIZE);
		pCopyrightData[INFO_SIZE] = '\0';

		strncpy(pHookStreamData,pBufferData+(3*INFO_SIZE),INFO_SIZE);
		pHookStreamData[14] = '\0';	// we only want the first 14 bytes for
						// the mime-type "audio/x-pn-wav"

        // Fill in the Header with the relevant data...
        pValues->SetPropertyBuffer ("Title",        pTitle);
        pValues->SetPropertyBuffer ("Author",       pAuthor);
        pValues->SetPropertyBuffer ("Copyright",    pCopyright);
        pValues->SetPropertyBuffer ("HookInfo",     pHookStreamInfo);
		pValues->SetPropertyULONG32("StreamCount",  1);

		// Tell the FormatResponse of our success in 
		// getting the header.
		m_pFFResponse->StreamHeaderReady(status, pValues);

		// Release our reference on the header!
		pValues->Release();
		pTitle->Release();
		pAuthor->Release();
		pCopyright->Release();
		pHookStreamInfo->Release();
	    }
	}
	break;

	case GetPacketReadPending:
	{
	    // We are in the process of responding to a GetPacket()
	    // request and the read has completed, which means we
	    // are done and can go back to the ready state...
	    m_state = Ready;

	    if (status == PNR_OK)
	    {
		// We now need to form a "packet" object and pass it
		// off to our controller. Notice since our packet data
		// comes straight out of the file as is, we don't need
		// to copy data at all, the IRMABuffer returned by read
		// can be placed into the packet object and passed off.

		// Create packet object here, notice that we call the
		// CreateInstance method of the controller, but we could
		// have implemented our own object that exposed the IRMAPacket
		// interface.
		IRMAPacket* pPacket;
		if (PNR_OK == m_pCommonClassFactory->CreateInstance(CLSID_IRMAPacket,
						       (void**)&pPacket))
		{
		    // Fill in the Packet with the relevant data...
		    pPacket->Set(pBuffer,m_ulCurrentTime,0,
				 RMA_ASM_SWITCH_ON, 0);

		    // In our simple file format all packets are 1 second apart.
		    m_ulCurrentTime += TIME_PER_PACKET;

		    // Tell the FormatResponse of our success in 
		    // getting the packet.
		    m_pFFResponse->PacketReady(status, pPacket);

		    // Release our reference on the packet!
		    pPacket->Release();
		}
	    }
	    // If the read failed then we will call PacketReady() with
	    // a NULL packet!
	    else
	    {
		// In our simple file format all packets are 1 second apart.
		m_ulCurrentTime = 0;

		// Tell the FormatResponse of our success in 
		// getting the packet.
		m_pFFResponse->StreamDone(0);
	    }
	}
	break;

	default:
	{
	    result = PNR_UNEXPECTED;
	}
	break;
    }

    return result;
}


/****************************************************************************
 *  IRMAFileFormatObject::Seek                               ref:  rmaformt.h
 *
 *  This routine moves to the packet nearest to the requested time in the
 *  file. It is called, for example, in response to the user moving the
 *  slider to a different location in the playback timeline.
 */
STDMETHODIMP
CPostMixHookFF::Seek(UINT32 ulOffset)
{
    // If we are not ready then something has gone wrong
    // if (m_state != Ready) return PNR_UNEXPECTED;
    /* This was wrong.  We can be waiting for a Read request to
     * complete when we get a Seek request.  In which case, we should
     * drop whatever we were waiting for, and assume the filesystem
     * will do the same when it gets our seek request.  
     */

    // Notice that the seek is passed as time in milliseconds, we
    // need to convert this to our packet number and its offset
    // in the file...
    UINT32 ulPacket = ulOffset/TIME_PER_PACKET;
    UINT32 ulPacketOffset = (ulPacket*PACKET_SIZE)+DATA_OFFSET;

    // To actually seek to the correct place in the time line,
    // we will seek in the file. We need to call the file object's
    // seek method.

    // Since this is asyncronous we need to note our state so we can
    // correctly respond to the seek complete response from the file
    // object.
    m_state = SeekSeekPending;

    // Note that the real time we are seeked to is not exactly what
    // was requested...
    m_ulCurrentTime = ulPacket*TIME_PER_PACKET;

    // Actually seek...
    m_pFileObject->Seek(ulPacketOffset,FALSE);

    // See SeekDone() for next "step" of the Seek() process.

    return PNR_OK;
}


/****************************************************************************
 *  IRMAFileFormatObject::Close                              ref:  rmaformt.h
 *
 *  This routine handles any cleanup required before closing the plug-in. All
 *  references to objects should be released and memory deallocated. This is
 *  called by the RMA core when the playback is finished or stopped.
 */
STDMETHODIMP
CPostMixHookFF::Close()
{
    
    if (m_pContext) 
    {
	m_pContext->Release(); 
	m_pContext = 0;
    }

    if (m_pFileObject) 
    {
	m_pFileObject->Release();
	m_pFileObject = 0;
    }

    if (m_pFFResponse) 
    {
	m_pFFResponse->Release();
	m_pFFResponse = 0;
    }

    if (m_pRequest)
    {
	m_pRequest->Release();
	m_pRequest = 0;
    }

    if (m_pCommonClassFactory) 
    {
 	m_pCommonClassFactory->Release();
	m_pCommonClassFactory = 0;
    }

    if (m_pURL)
    {
	delete [] m_pURL;	
	m_pURL = NULL;
    }

    return PNR_OK;
}


/****************************************************************************
 *  IRMAFileFormatObject::CloseDone                          ref:  rmaformt.h
 *
 */
STDMETHODIMP
CPostMixHookFF::CloseDone(PN_RESULT status)
{
    return PNR_OK;
}


/****************************************************************************
 *  IRMAFileFormatObject::WriteDone                          ref:  rmaformt.h
 *
 */
STDMETHODIMP
CPostMixHookFF::WriteDone(PN_RESULT status)
{
    // We don't ever write, so we don't expect to get this...
    return PNR_UNEXPECTED;
}


/****************************************************************************
 *  CPostMixHookFF::FileObjectReady                          ref:  pmixplin.h
 *
 */
STDMETHODIMP CPostMixHookFF::FileObjectReady
(
    PN_RESULT status,
    IRMAFileObject* pFileObject)
{
    return PNR_OK;
}


// IUnknown COM Interface Methods

/****************************************************************************
 *  IUnknown::AddRef                                            ref:  pncom.h
 *
 *  This routine increases the object reference count in a thread safe
 *  manner. The reference count is used to manage the lifetime of an object.
 *  This method must be explicitly called by the user whenever a new
 *  reference to an object is used.
 */
STDMETHODIMP_(UINT32) CPostMixHookFF::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}


/****************************************************************************
 *  IUnknown::Release                                           ref:  pncom.h
 *
 *  This routine decreases the object reference count in a thread safe
 *  manner, and deletes the object if no more references to it exist. It must
 *  be called explicitly by the user whenever an object is no longer needed.
 */
STDMETHODIMP_(UINT32) CPostMixHookFF::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
        return m_lRefCount;
    }

    delete this;
    return 0;
}


/****************************************************************************
 *  IUnknown::QueryInterface                                    ref:  pncom.h
 *
 *  This routine indicates which interfaces this object supports. If a given
 *  interface is supported, the object's reference count is incremented, and
 *  a reference to that interface is returned. Otherwise a NULL object and
 *  error code are returned. This method is called by other objects to
 *  discover the functionality of this object.
 */
STDMETHODIMP CPostMixHookFF::QueryInterface(REFIID riid, void** ppvObj)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
	AddRef();
	*ppvObj = (IUnknown*)(IRMAPlugin*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMAPlugin))
    {
	AddRef();
	*ppvObj = (IRMAPlugin*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMAFileFormatObject))
    {
	AddRef();
	*ppvObj = (IRMAFileFormatObject*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMAFileResponse))
    {
	AddRef();
	*ppvObj = (IRMAFileResponse*)this;
	return PNR_OK;
    }

    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}
