/****************************************************************************
 * 
 *  $Id: pmixplin.h,v 1.11 1997/10/21 21:27:35 christof Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 * 
 *  pmixplin.h
 *
 *  Example File Format for Pre/Post Mix Hook RealMedia Architecture (RMA).
 */
#ifndef _EXFFPLIN_H_
#define _EXFFPLIN_H_


/****************************************************************************
 * Constants
 */
#define HEADER_OFFSET	     0
#define INFO_SIZE	    50
#define FILE_HEADER_SIZE    (INFO_SIZE*4)
#define STREAM_HEADER_SIZE  (INFO_SIZE*4) 
#define PACKET_SIZE	    20
#define TIME_PER_PACKET	    1000
#define DATA_OFFSET	    (HEADER_OFFSET+FILE_HEADER_SIZE+STREAM_HEADER_SIZE)


/****************************************************************************
 *
 *  CPostMixHookFF Class
 *
 *  Example implementation of a basic file format.
 */
class CPostMixHookFF :	public IRMAPlugin, 
			public IRMAFileFormatObject, 
			public IRMAFileResponse
{
    public:
    /****** Public Class Methods ******************************************/
    CPostMixHookFF()
	    : m_lRefCount(0)
	    , m_pContext(NULL)
	    , m_pFileObject(NULL)
	    , m_pFileSystemObject(NULL)
	    , m_pFFResponse(NULL)
	    , m_pCommonClassFactory(NULL)
	    , m_bHeaderSent(FALSE)
	    , m_ulCurrentTime(0)
	    , m_state(Ready)
	    , m_pURL(NULL)
	    , m_pRequest(NULL)
	    {
	    };
    STDMETHOD(FileObjectReady)
	(THIS_ 
	 PN_RESULT       status,
	 IRMAFileObject* pFileObject
	);


    /************************************************************************
     *  IRMAFileFormatObject Interface Methods               ref:  rmaformt.h
     */
    STDMETHOD(GetFileFormatInfo)
	(THIS_
	 REF(const char**) pFileMimeTypes,
	 REF(const char**) pFileExtensions,
	 REF(const char**) pFileOpenNames
	);

    STDMETHOD(InitFileFormat)
	(THIS_
	 IRMARequest*        pRequest,
	 IRMAFormatResponse* pFormatResponse,
	 IRMAFileObject*     pFileObject
	);

    STDMETHOD(GetFileHeader  ) (THIS);
    STDMETHOD(GetStreamHeader) (THIS_ UINT16 streamNo);
    STDMETHOD(GetPacket      ) (THIS_ UINT16 streamNo);
    STDMETHOD(Seek           ) (THIS_ UINT32 requestedTime);
    STDMETHOD(Close          ) (THIS);


    /************************************************************************
    *  IRMAFileResponse Interface Methods                   ref:  rmafiles.h
    */
    STDMETHOD(InitDone ) (THIS_ PN_RESULT status);
    STDMETHOD(SeekDone ) (THIS_ PN_RESULT status);
    STDMETHOD(ReadDone ) (THIS_ PN_RESULT status, IRMABuffer* pBuffer);
    STDMETHOD(WriteDone) (THIS_ PN_RESULT status);
    STDMETHOD(CloseDone) (THIS_ PN_RESULT status);


    /************************************************************************
     *  IRMAPlugin Interface Methods                         ref:  rmaplugn.h
     */
    STDMETHOD(GetPluginInfo)
	(THIS_ 
	 REF(BOOL)        bLoadMultiple,
	 REF(const char*) pDescription,
	 REF(const char*) pCopyright,
	 REF(const char*) pMoreInfoURL,
	 REF(UINT32)      versionNumber
	);
    STDMETHOD(InitPlugin) (THIS_ IUnknown* pRMACore);


    /************************************************************************
    *  IUnknown COM Interface Methods                          ref:  pncom.h
    */
    STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
    STDMETHOD_(UINT32, AddRef ) (THIS);
    STDMETHOD_(UINT32, Release) (THIS);


    private:
    /****** Private Class Variables ****************************************/
    INT32			m_lRefCount;
    IUnknown*			m_pContext;
    IRMAFileObject*		m_pFileObject;
    IRMAFileSystemObject*	m_pFileSystemObject;
    IRMAFormatResponse*		m_pFFResponse;
    IRMACommonClassFactory*	m_pCommonClassFactory;
    IRMARequest*		m_pRequest;
    BOOL			m_bHeaderSent;
    UINT32			m_ulCurrentTime;
    char*			m_pURL;

    typedef enum
    {
	Ready, InitPending,
	GetFileHeaderSeekPending, GetFileHeaderReadPending, 
	GetStreamHeaderSeekPending, GetStreamHeaderReadPending, 
	GetPacketSeekPending, GetPacketReadPending,
	SeekSeekPending
    }
    ExampleState;
    ExampleState		m_state;

    /****** Private Static Class Variables *********************************/
    static const char*		zm_pDescription;
    static const char*		zm_pCopyright;
    static const char*		zm_pMoreInfoURL;
    static const char*		zm_pFileMimeTypes[];
    static const char*		zm_pFileExtensions[];
    static const char*		zm_pFileOpenNames[];

    /****** Private Class Methods ******************************************/
    ~CPostMixHookFF() { };

    PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME      // Avoids GCC compiler warning
};


#endif // ndef _EXFFPLIN_H_

