/****************************************************************************
 * 
 *  $Id: premixrd.h,v 1.11 1997/11/25 20:00:44 rahul Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 * 
 *  premixrd.h
 *
 *  Pre-mix hook example renderer plug-in for RealMedia Architecture (RMA).
 */
#ifndef _PRERENDR_H_
#define _PRERENDR_H_


/****************************************************************************
 * Forward declarations
 */
class CPNAudioInfoResp;


/****************************************************************************
 *
 *  CPreMixRenderer Class
 *
 *  Example implementation of a basic renderer.
 */
class CPreMixRenderer :	public IRMAPlugin, 
			public IRMARenderer
{
    public:
    /****** Public Class Methods ******************************************/
    CPreMixRenderer();


    /************************************************************************
     *  IRMARenderer Interface Methods                       ref:  rmarendr.h
     */
    STDMETHOD(StartStream) (THIS_ IRMAStream* pStream, IRMAPlayer* pPlayer);
    STDMETHOD(EndStream )  (THIS);
    STDMETHOD(OnHeader  )  (THIS_ IRMAValues* pStreamHeaderObj);
    STDMETHOD(OnPacket  )  (THIS_ IRMAPacket* pPacket, INT32 timeOffset);
    STDMETHOD(OnTimeSync)  (THIS_ UINT32 currentPlayBackTime);
    STDMETHOD(OnPreSeek )  (THIS_ UINT32 timeBefore, UINT32 timeAfter);
    STDMETHOD(OnPostSeek)  (THIS_ UINT32 timeBefore, UINT32 timeAfter);
    STDMETHOD(OnPause   )  (THIS_ UINT32 timeBeforePause);
    STDMETHOD(OnBegin   )  (THIS_ UINT32 timeAfterBegin);
    STDMETHOD(OnBuffering) (THIS_ UINT32 reason, UINT16 percentComplete);
    STDMETHOD(GetRendererInfo)
        (THIS_
         REF(const char**) pStreamMimeTypes,
         REF(UINT32)       initialGranularity
        );
    STDMETHOD(GetDisplayType)
        (THIS_
         REF(RMA_DISPLAY_TYPE) displayType,
         REF(IRMABuffer*)      pDisplayInfo
        );
    STDMETHOD(OnEndofPackets) (THIS);


    /************************************************************************
     *  IRMAPlugin Interface Methods                         ref:  rmaplugn.h
     */
    STDMETHOD(GetPluginInfo)
        (THIS_ 
          REF(BOOL)        bLoadMultiple,
          REF(const char*) pDescription,
          REF(const char*) pCopyright,
          REF(const char*) pMoreInfoURL,
          REF(UINT32)      versionNumber
        );
    STDMETHOD(InitPlugin) (THIS_ IUnknown* pContext);

    
    /************************************************************************
     *  IUnknown COM Interface Methods                          ref:  pncom.h
     */
    STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
    STDMETHOD_(UINT32, AddRef ) (THIS);
    STDMETHOD_(UINT32, Release) (THIS);


    /************************************************************************
     *  CPreMixHook Class
     *
     *  Implement a pre mix hook.
     */
    class CPreMixHook: public IRMAAudioHook
    {
        public:
	/****** Public Class Methods ***************************************/
        CPreMixHook();
	int SetIContext( IUnknown* pIU );
	
	/********************************************************************
	*  IRMAAudioHook Interface Methods                   ref:  rmaausvc.h
	*/
        STDMETHOD(OnInit) (THIS_ RMAAudioFormat* pFormat);
        STDMETHOD(OnBuffer)
            (THIS_
	     RMAAudioData* pInData,
	     RMAAudioData* pOutData
	    );

	/********************************************************************
	*  IUnknown COM Interface Methods                       ref:  pncom.h
	*/
	STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
	STDMETHOD_(UINT32, AddRef ) (THIS);
	STDMETHOD_(UINT32, Release) (THIS);

        private:
	/****** Private Class Methods **************************************/
        ~CPreMixHook();

	/****** Private Class Variables ************************************/
    INT32			m_lRefCount;
 	UINT16			m_uChannels;
	UINT16			m_uBitsPerSample;
	UINT32			m_ulSamplesPerSec;
	UINT16			m_uMaxBlockSize;
	IUnknown*		m_pContext;
	IRMACommonClassFactory* m_pCommonClassFactory;

	PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME   // Avoids GCC compiler warning
    };
    friend CPreMixHook;


    private:
    /****** Private Class Variables ****************************************/
    INT32               m_lRefCount;
    IUnknown*           m_pContext;
    IRMAStream*         m_pStream;
    IRMAPlayer*         m_pPlayer;
    IRMAValues*         m_pValues;
    IRMAAudioPlayer*    m_pAudioPlayer;
    IRMAAudioStream*    m_pAudioStream;
    UINT32              m_ulLastTime;
    UINT32              m_ulGranularity;
    IRMABuffer*		m_pHookStreamInfo;
    CPreMixHook*	m_pHook;
    CPNAudioInfoResp*	m_pResp;
    BOOL		m_bInSeekMode;

    /****** Private Static Class Variables *********************************/
    static const char*  zm_pDescription;
    static const char*  zm_pCopyright;
    static const char*  zm_pMoreInfoURL;
    static const char*  zm_pStreamMimeTypes[];

    /****** Private Class Methods ******************************************/
    ~CPreMixRenderer() 
	{ 
	    EndStream();
	};

    PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME       // Avoids GCC compiler warning
};



/****************************************************************************
 *
 *  CPNAudioInfoResp Class
 *
 */
class CPNAudioInfoResp: public IRMAAudioStreamInfoResponse
{
    public:
    /****** Public Class Methods ******************************************/
    CPNAudioInfoResp();
    int SetInfo( IRMAAudioPlayer* pPlayer, IRMAAudioHook* pAudioHook);
    int SetStreamHookName( IRMABuffer* pHookInfo );


    /************************************************************************
     *  IRMAAudioStreamInfoResponse Interface Methods        ref:  rmaausvc.h
     */
    STDMETHOD(OnStream) (THIS_ IRMAAudioStream* pAudioStream );


    /************************************************************************
     *  IUnknown COM Interface Methods                          ref:  pncom.h
     */
    STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
    STDMETHOD_(UINT32, AddRef ) (THIS);
    STDMETHOD_(UINT32, Release) (THIS);


    private:
    /****** Private Class Variables ****************************************/
    INT32  			m_lRefCount;
    IUnknown*		m_pContext;
    IRMAAudioPlayer*	m_pAudioPlayer;
    IRMAAudioHook*	m_pHook;
    IRMABuffer*		m_pHookStreamName;

    /****** Private Class Methods ******************************************/
    ~CPNAudioInfoResp();
    
    PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME       // Avoids GCC compiler warning
};


#endif // ndef _PRERENDR_H_
