/****************************************************************************
 * 
 *  $Id: pstmixrd.h,v 1.10 1997/10/15 00:33:28 rahul Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 * 
 *  pstmixrd.h
 *
 *  Post-mix hook example renderer plug-in for RealMedia Architecture (RMA).
 */
#ifndef _PSTRENDR_H_
#define _PSTRENDR_H_


/****************************************************************************
 *
 *  CPostMixRenderer Class
 *
 *  Example implementation of a basic renderer.
 */
class CPostMixRenderer :	public IRMAPlugin, 
				public IRMARenderer
{
    public:
    /****** Public Class Methods ******************************************/
    CPostMixRenderer();

    /************************************************************************
     *  IRMARenderer Interface Methods                       ref:  rmarendr.h
     */
    STDMETHOD(StartStream) (THIS_ IRMAStream* pStream, IRMAPlayer* pPlayer);
    STDMETHOD(EndStream )  (THIS);
    STDMETHOD(OnHeader  )  (THIS_ IRMAValues* pStreamHeaderObj);
    STDMETHOD(OnPacket  )  (THIS_ IRMAPacket* pPacket, INT32 timeOffset);
    STDMETHOD(OnTimeSync)  (THIS_ UINT32 currentPlayBackTime);
    STDMETHOD(OnPreSeek )  (THIS_ UINT32 timeBefore, UINT32 timeAfter);
    STDMETHOD(OnPostSeek)  (THIS_ UINT32 timeBefore, UINT32 timeAfter);
    STDMETHOD(OnPause   )  (THIS_ UINT32 timeBeforePause);
    STDMETHOD(OnBegin   )  (THIS_ UINT32 timeAfterBegin);
    STDMETHOD(OnBuffering) (THIS_ UINT32 reason, UINT16 percentComplete);
    STDMETHOD(GetRendererInfo)
        (THIS_
         REF(const char**) pStreamMimeTypes,
         REF(UINT32)       initialGranularity
        );
    STDMETHOD(GetDisplayType)
        (THIS_
         REF(RMA_DISPLAY_TYPE) displayType,
         REF(IRMABuffer*)      pDisplayInfo
        );
    STDMETHOD(OnEndofPackets) (THIS);


    /************************************************************************
     *  IRMAPlugin Interface Methods                         ref:  rmaplugn.h
     */
    STDMETHOD(GetPluginInfo)
        (THIS_ 
          REF(BOOL)        bLoadMultiple,
          REF(const char*) pDescription,
          REF(const char*) pCopyright,
          REF(const char*) pMoreInfoURL,
          REF(UINT32)      versionNumber
        );
    STDMETHOD(InitPlugin) (THIS_ IUnknown* pContext);

    
    /************************************************************************
     *  IUnknown COM Interface Methods                          ref:  pncom.h
     */
    STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
    STDMETHOD_(UINT32, AddRef ) (THIS);
    STDMETHOD_(UINT32, Release) (THIS);


    /************************************************************************
     *  CPostMixHook Class
     *
     *  Implement a post mix hook.
     */
    class CPostMixHook: public IRMAAudioHook
    {
	public:
	/****** Public Class Methods ***************************************/
        CPostMixHook();
	int SetIContext( IUnknown* pIU );

	/********************************************************************
	*  IRMAAudioHook Interface Methods                   ref:  rmaausvc.h
	*/
        STDMETHOD(OnInit) (THIS_ RMAAudioFormat* pFormat);
        STDMETHOD(OnBuffer)
            (THIS_
	     RMAAudioData* pInData,
	     RMAAudioData* pOutData
	    );

	/********************************************************************
	*  IUnknown COM Interface Methods                       ref:  pncom.h
	*/
	STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
	STDMETHOD_(UINT32, AddRef ) (THIS);
	STDMETHOD_(UINT32, Release) (THIS);

	private:
	/****** Private Class Methods **************************************/
        ~CPostMixHook();

	/****** Private Class Variables ************************************/
        LONG32			m_lRefCount;
        UINT16                  m_uChannels;
        UINT16                  m_uBitsPerSample;
        ULONG32                 m_ulSamplesPerSec;
        UINT16                  m_uMaxBlockSize;
	IUnknown*		m_pContext;
	IRMACommonClassFactory* m_pCommonClassFactory;

	PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME   // Avoids GCC compiler warning
    };
    friend CPostMixHook;


private:
    /****** Private Class Methods ******************************************/
    ~CPostMixRenderer() 
	{ 
	    EndStream();
	};

    /****** Private Class Variables ****************************************/
    LONG32		m_lRefCount;
    IUnknown*		m_pContext;
    IRMAStream*		m_pStream;
    IRMAPlayer*		m_pPlayer;
    IRMAValues*		m_pValues;
    IRMAAudioPlayer*    m_pAudioPlayer;
    ULONG32		m_ulLastTime;
    ULONG32		m_ulGranularity;
    CPostMixHook*	m_pHook;
    BOOL		m_bInSeekMode;

     /****** Private Static Class Variables *********************************/
    static const char*	zm_pDescription;
    static const char*	zm_pCopyright;
    static const char*	zm_pMoreInfoURL;
    static const char*	zm_pStreamMimeTypes[];

    PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME       // Avoids GCC compiler warning
};

#endif // endef _PSTRENDR_H_
