#include "pncom.h"
#include "pntypes.h"
#include "pnwintyp.h"
#include "rmawin.h"
#include "rmacomm.h"
#include "rmamon.h"
#include "extacmgr.h"
#include "rmaengin.h"
#include "rmacore.h"
#include "rmapckts.h"
#include "os.h"


// static tax array & constants
char *szTACNames[] = 
{
    "Title",
    "Author",
    "Copyright",
    "Abstract",
    "Keywords"
};


/****************************************************************************
 *  ExampleTACManager::ExampleTACManager
 *
 *  Purpose:
 *      Init the members and TAC prop IDs.
 */
ExampleTACManager::ExampleTACManager()
    : m_lRefCount(0)
    , m_pTitle(NULL)
    , m_pAuthor(NULL)
    , m_pCopyright(NULL)
    , m_pAbstract(NULL)
    , m_pKeywords(NULL)
    , m_pRegName(NULL)
    , m_uPlayerIndex(0)
    , m_pPropWatch(NULL)
    , m_pRegistry(NULL)
{
    // init tac prop ID array
    for (int n = 0; n < NUM_TAC_NAMES; n++)
    {
	m_uTACPropIDs[n] = 0;
    }

}


/****************************************************************************
 *  ExampleTACManager::~ExampleTACManager
 *
 *  Purpose:
 *      Cleanup.
 */
ExampleTACManager::~ExampleTACManager()
{
    if (m_pTitle)
    {
	delete[] m_pTitle;
	m_pTitle = NULL;
    }

    if (m_pAuthor)
    {
	delete[] m_pAuthor;
	m_pAuthor = NULL;
    }

    if (m_pCopyright)
    {
	delete[] m_pCopyright;
	m_pCopyright = NULL;
    }

    if (m_pAbstract)
    {
	delete[] m_pAbstract;
	m_pAbstract = NULL;
    }

    if (m_pKeywords)
    {
	delete[] m_pKeywords;
	m_pKeywords = NULL;
    }

    if (m_pRegName)
    {
	delete[] m_pRegName;
	m_pRegName = NULL;
    }
}


/****************************************************************************
 *  ExampleTACManager::Init
 *
 *  Purpose:
 *      Setup the property watcher for the TAC properties so we can receive
 *  Notifications when they change.
 */
BOOL
ExampleTACManager::Init(IRMAPlayer* pPlayer)
{
    BOOL bOk = FALSE;

    // We require a player to initialize
    if (pPlayer)
    {
	// We require a registry id to watch the TAC properties
	IRMARegistryID* pRegID = NULL;
	if (pPlayer->QueryInterface(IID_IRMARegistryID, 
				    (void**)&pRegID) == PNR_OK)
	{
	    pRegID->GetID(m_uPlayerIndex);
	    pRegID->Release();

	    // We require the player's registry...
	    if (pPlayer->QueryInterface(IID_IRMAPNRegistry, 
					(void**)&m_pRegistry) == PNR_OK)
	    {
		// ...and the name of our registry
		IRMABuffer* pBuffer = NULL;
		if (m_pRegistry->GetPropName(m_uPlayerIndex, 
		    pBuffer) == PNR_OK)
		{
		    ULONG32 uLen = pBuffer->GetSize();
		    m_pRegName = new char[uLen+1];
		    strcpy(m_pRegName, (const char*) pBuffer->GetBuffer());
		    pBuffer->Release();

		    // Now setup our prop watcher...
		    if(m_pRegistry->CreatePropWatch(m_pPropWatch) == PNR_OK)
		    {
			m_pPropWatch->Init((IRMAPropWatchResponse*)this);
			bOk = TRUE;

			// ..and watch our TAC properties
			char szName[MAX_DISPLAY_NAME] = "";
			if (m_pRegName && *m_pRegName != '\0')
			{
			    for (int n = 0; n < NUM_TAC_NAMES; n++)
			    {
				UINT32 uId = m_uTACPropIDs[n];

				sprintf(szName, "%s.%s", 
				        m_pRegName, szTACNames[n]);
				m_uTACPropIDs[n] = m_pRegistry->GetId(szName);

				if (uId > 0)
				{
				    m_pPropWatch->SetWatchById(uId);
				}
			    }
			}

			// Now update our TAC
			Update();
		    }
		}
	    }
	}
    }

    return bOk;
}


/****************************************************************************
 *  ExampleTACManager::Terminate
 *
 *  Purpose:
 *      Free the propwatch.
 */
void
ExampleTACManager::Terminate()
{
    if (m_pPropWatch)
    {
	for (int n = 0; n < NUM_TAC_NAMES; n++)
	{
	    if (m_uTACPropIDs[n] > 0)
	    {
		m_pPropWatch->ClearWatchById(m_uTACPropIDs[n]);
		m_uTACPropIDs[n] = 0;
	    }
	}

	m_pPropWatch->Release();
	m_pPropWatch = NULL;
    }

    if (m_pRegistry)
    {
	m_pRegistry->Release();
	m_pRegistry = NULL;
    }
}


const char* 
ExampleTACManager::GetTitle()
{
    return m_pTitle;
}


const char* 
ExampleTACManager::GetAuthor()
{
    return m_pAuthor;
}


const char* 
ExampleTACManager::GetCopyright()
{
    return m_pCopyright;
}


const char* 
ExampleTACManager::GetAbstract()
{
    return m_pAbstract;
}


const char* 
ExampleTACManager::GetKeywords()
{
    return m_pKeywords;
}


/****************************************************************************
 *  ExampleTACManager::Update
 *
 *  Purpose:
 *      Iterate through our TAC values in the registry and update our
 *  internal strings.
 */
void
ExampleTACManager::Update()
{
    IRMABuffer* pValue = NULL;
    if (m_pRegistry)
    {
	for (int n = 0; n < NUM_TAC_NAMES; n++)
	{
	    // the title
	    if (m_uTACPropIDs[n] > 0)
	    {
		m_pRegistry->GetStrById(m_uTACPropIDs[n], pValue);
		if (pValue)
		{
		    const char* pProp = (const char*)pValue->GetBuffer();
		    UINT32 uLen = pValue->GetSize();
		    switch (n)
		    {
			case TAC_TITLE:
			{
			    InternalSetStringProp(&m_pTitle, pProp, uLen);
			    break;
			}

			case TAC_AUTHOR:
			{
			    InternalSetStringProp(&m_pAuthor, pProp, uLen);
			    break;
			}

			case TAC_COPYRIGHT:
			{
			    InternalSetStringProp(&m_pCopyright, pProp, uLen);
			    break;
			}

			case TAC_ABSTRACT:
			{
			    InternalSetStringProp(&m_pAbstract, pProp, uLen);
			    break;
			}

			case TAC_KEYWORDS:
			{
			    InternalSetStringProp(&m_pKeywords, pProp, uLen);
			    break;
			}

			default:
			{
			    // uh oh!
			    break;
			}
		    }
		    pValue->Release();
		    pValue = NULL;
		}
	    }
	}
    }

}


/****************************************************************************
 *  IUnknown::AddRef                                            ref:  pncom.h
 *
 *  This routine increases the object reference count in a thread safe
 *  manner. The reference count is used to manage the lifetime of an object.
 *  This method must be explicitly called by the user whenever a new
 *  reference to an object is used.
 */
STDMETHODIMP_(UINT32) ExampleTACManager::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}


/****************************************************************************
 *  IUnknown::Release                                           ref:  pncom.h
 *
 *  This routine decreases the object reference count in a thread safe
 *  manner, and deletes the object if no more references to it exist. It must
 *  be called explicitly by the user whenever an object is no longer needed.
 */
STDMETHODIMP_(UINT32) ExampleTACManager::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
        return m_lRefCount;
    }

    delete this;
    return 0;
}


/****************************************************************************
 *  IUnknown::QueryInterface                                    ref:  pncom.h
 *
 *  This routine indicates which interfaces this object supports. If a given
 *  interface is supported, the object's reference count is incremented, and
 *  a reference to that interface is returned. Otherwise a NULL object and
 *  error code are returned. This method is called by other objects to
 *  discover the functionality of this object.
 */
STDMETHODIMP ExampleTACManager::QueryInterface(REFIID riid, void** ppvObj)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
	AddRef();
	*ppvObj = (IUnknown*)(IRMAPropWatchResponse*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMAPropWatchResponse))
    {
	AddRef();
	*ppvObj = (IRMAPropWatchResponse*)this;
	return PNR_OK;
    }

    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}




/****************************************************************************
 *  IRMAPropWatchResponse::AddedProp                           ref:  rmamon.h
 *
 *  Purpose:
 *      Gets called when a watched Property gets modified. It passes
 *  the id of the Property just modified, its datatype and the
 *  id of its immediate parent COMPOSITE property.
 */
STDMETHODIMP
ExampleTACManager::AddedProp(const UINT32 id,
			     const RMAPropType propType,
			     const UINT32 ulParentHash)
{
    // Here is where we could handle new properties if we wanted to
    // update our UI as soon as the property is added
    return PNR_NOTIMPL;
}


/****************************************************************************
 *  IRMAPropWatchResponse::ModifiedProp                        ref:  rmamon.h
 *
 *  Purpose:
 *      Gets called when a watched Property gets modified. It passes
 *  the id of the Property just modified, its datatype and the
 *  id of its immediate parent COMPOSITE property.
 */
STDMETHODIMP
ExampleTACManager::ModifiedProp(const UINT32 id,
			        const RMAPropType propType,
			        const UINT32 ulParentHash)
{
    // Here is where we could handle modified properties if we wanted to
    // update our UI as soon as the property changes
    return PNR_NOTIMPL;
}


/****************************************************************************
 *  IRMAPropWatchResponse::DeletedProp                         ref:  rmamon.h
 *
 *  Purpose:
 *      Gets called when a watched Property gets deleted. As can be
 *  seen, it returns the id of the Property just deleted and
 *  its immediate parent COMPOSITE property.
 */
STDMETHODIMP
ExampleTACManager::DeletedProp(const UINT32 id,
			       const UINT32 ulParentHash)
{
    for (int n = 0; n < NUM_TAC_NAMES; n++)
    {
	if (m_uTACPropIDs[n] == id)
	{
	    m_uTACPropIDs[n] = 0;
	}
    }

    return m_pPropWatch->ClearWatchById(id);
}



/****************************************************************************
 *  ExampleTACManager::InternalSetStringProp
 *
 *  Purpose:
 *      Helper function for setting our internal TAC strings.
 */
void
ExampleTACManager::InternalSetStringProp(char**		ppProp, 
					 const char*	pNew, 
					 UINT32		uLen)
{
    if (*ppProp)
    {
	if (strlen(*ppProp) < uLen+1)
	{
	    delete[] *ppProp;
	    *ppProp = NULL;
	}
    }

    if (!*ppProp)
    {
	*ppProp = new char[uLen+1];
    }

    if (*ppProp)
    {
	strncpy(*ppProp, pNew, uLen);
    }
}