#ifndef __EXTACMGR_H
#define __EXTACMGR_H


#define NUM_TAC_NAMES 5

#define TAC_TITLE      0
#define TAC_AUTHOR     1
#define TAC_COPYRIGHT  2
#define TAC_ABSTRACT   3
#define TAC_KEYWORDS   4


struct IRMAPlayer;
struct IRMAPNRegistry;
struct IRMAPropWatch;
struct IRMAPropWatchResponse;


class ExampleTACManager : public IRMAPropWatchResponse
{
public:
   /************************************************************************
    *  ExampleTACManager public constructor & destructor
    */
    ExampleTACManager();
    virtual ~ExampleTACManager();

   /************************************************************************
    *  ExampleTACManager helper methods
    */
    BOOL	Init(IRMAPlayer* pPlayer);
    void	Terminate();
    void	Update();
    const char* GetTitle();
    const char* GetAuthor();
    const char* GetCopyright();
    const char* GetAbstract();
    const char* GetKeywords();

   /************************************************************************
    *  IRMAPropWatchResponse Interface Methods                ref:  rmamon.h
    */
    STDMETHOD(AddedProp)	(THIS_
				const UINT32		id,
				const RMAPropType   	propType,
				const UINT32		ulParentHash);

    STDMETHOD(ModifiedProp)	(THIS_
				const UINT32		id,
				const RMAPropType   	propType,
				const UINT32		ulParentHash);

    STDMETHOD(DeletedProp)	(THIS_
				const UINT32		id,
				const UINT32		ulParentHash);

   /************************************************************************
    *  IUnknown COM Interface Methods                          ref:  pncom.h
    */
    STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
    STDMETHOD_(UINT32, AddRef ) (THIS);
    STDMETHOD_(UINT32, Release) (THIS);


   /************************************************************************
    *  ExampleTACManager protected data members
    */
protected:
    LONG32	    m_lRefCount;
    char*	    m_pTitle;
    char*	    m_pAuthor;
    char*	    m_pCopyright;
    char*	    m_pAbstract;
    char*	    m_pKeywords;
    char*	    m_pRegName;
    UINT32	    m_uPlayerIndex;
    IRMAPropWatch*  m_pPropWatch;
    IRMAPNRegistry* m_pRegistry;
    UINT32	    m_uTACPropIDs[NUM_TAC_NAMES];

    void	    InternalSetStringProp(char**	ppProp, 
					  const char*	pNew, 
					  UINT32	uLen);
};


#endif
