/****************************************************************************
 * 
 *  $Id: main.cpp,v 1.45 1999/05/15 00:36:20 jfarr Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 *
 *  main.cpp
 *
 *  Example client for RealMedia Architecture (RMA).
 *
 */

/****************************************************************************
 * 
 *  Example client:
 *  
 *  This is an example client running without a graphical user interface.
 *
 *  It could be used as:
 *    - Sample source to develop a customized RMA client.
 *    - A testing tool for plugins.
 *
 */ 


/****************************************************************************
 * Defines
 */
#define INITGUID     /* Interface ID's */

#ifdef _UNIX
#include <stdlib.h>  // needed for getenv()
#endif


/****************************************************************************
 * Includes
 */
#include "pntypes.h"
#include "pnwintyp.h"
#include "pncom.h"
#include "rmapckts.h"
#include "rmacomm.h"
#include "rmamon.h"
#include "rmafiles.h"
#include "rmaengin.h"
#include "rmacore.h"
#include "rmaclsnk.h"
#include "rmaerror.h"
#include "rmaauth.h"
#include "rmawin.h"

#include "os.h"
#include "fivemmap.h"
#include "exadvsnk.h"
#include "exerror.h"
#include "exsitsup.h"
#include "exaumgr.h"
#include "excontxt.h"


// XXXXXJEB: this should really be in rmacore.h
typedef PN_RESULT (PNEXPORT_PTR FPRMSETDLLACCESSPATH) (const char*);

/****************************************************************************
 * Globals
 */
FPRMCREATEENGINE	m_fpCreateEngine;
FPRMCLOSEENGINE	 	m_fpCloseEngine;

#ifdef _UNIX
FPRMSETDLLACCESSPATH	m_fpSetDLLAccessPath;
#endif

BOOL			bShowMeTheStatistics = FALSE;
BOOL			bShowMeTheTAC = FALSE;


/****************************************************************************
 *  main
 *
 */
int main( int argc, char *argv[] )
{
    PN_RESULT		    theErr		= PNR_OK;
    ExampleClientContext*   exContext		= NULL;
    IRMAPlayer*		    pPlayer		= NULL;
    IRMAClientEngine*	    pEngine		= NULL;
#ifdef _UNIX
    IRMAClientEngineSelector* pSelector         = NULL;
    char*		    rphome		= NULL;
#endif
    IRMAErrorSink*	    pErrorSink		= NULL;
    IRMAErrorSinkControl* pErrorSinkControl	= NULL;
    HINSTANCE		    hDll		= NULL;
    char*		    pszURL		= NULL;
    BOOL		    bUnknownOption	= FALSE;

#ifdef _MACINTOSH
    // Emulate command line on Macintosh
    argc = ccommand( &argv );
    SIOUXSettings.standalone     = FALSE;
    SIOUXSettings.setupmenus     = FALSE;
    SIOUXSettings.showstatusline = TRUE;
#endif

    // Parse params
    if (argc > 1)
    {
	for (int nArg = 1; nArg < argc && !bUnknownOption; nArg++)
	{
	    if (0 == stricmp(argv[nArg], "-s"))
	    {
		bShowMeTheStatistics = TRUE;
	    }
	    else if (0 == stricmp(argv[nArg], "-i"))
	    {
		bShowMeTheTAC = TRUE;
	    }
	    else if (*argv[nArg] == '-')
	    {
		bUnknownOption = TRUE;
	    }
	    else if (!pszURL)
	    {
		// assume that the first non-option we see is the URL
		pszURL = argv[nArg];
	    }
	    else
	    {
		// don't try to confuse me with extra parameters!
		bUnknownOption = TRUE;
	    }
	}
    }

    if (argc < 2 || bUnknownOption)
    {
	fprintf(stdout,"USAGE: %s [-s] [-i] file:yourfile.ext\n", argv[0]);
	fprintf(stdout,"       -s : optional flag to show the statistics\n");
	fprintf(stdout,"       -i : optional flag to show clip info\n");
	return -1;
    }

    if (!(exContext = new ExampleClientContext()))
    {
	theErr = PNR_UNEXPECTED;
	goto cleanup;
    }
    exContext->AddRef();		

    // initialize the globals
    m_fpCreateEngine	= NULL;
    m_fpCloseEngine	= NULL;
#ifdef _UNIX
    m_fpSetDLLAccessPath = NULL;
#endif

    // prepare/load the RMACore module
    char   szDllName[_MAX_PATH];
    
#if defined(_UNIX)
    rphome = getenv("REALPLAYER_HOME");
    if (!rphome)
    {
	fprintf(stderr, "testplay: can't locate install directory\nPlease set REALPLAYER_HOME environment variable!\n");
	exit(1);
    }
    sprintf(szDllName, "%s/Common/rmacore.so.6.0", rphome);
#elif defined(_MACINTOSH)
    // ASSUME rmacore is in System Folder:Application Support:Real:Common
    // SHOULD obtain value from preference file
    OSErr	macErr;

    // set szDllName to empty string
    szDllName[0] = '\0'; 

    // find the Extensions folder
    short	vRefNum;
    long	dirID;
    macErr = FindFolder (kOnSystemDisk, kApplicationSupportFolderType, kDontCreateFolder, &vRefNum, &dirID);

    // find the path of the Extensions folder
    DirInfo	block;
    Str63	directoryName;
    char	tmpName[_MAX_PATH];

    block.ioDrParID = dirID;
    block.ioNamePtr = directoryName;
    do
    {
	block.ioVRefNum = vRefNum;
	block.ioFDirIndex = -1;
	block.ioDrDirID = block.ioDrParID;

	macErr = PBGetCatInfoSync ((CInfoPBPtr)&block);
	if (macErr == noErr)
	{
	    strcpy (tmpName, szDllName);
	    szDllName[0] = '\0';
	    strncat (szDllName, (char*)&directoryName[1], directoryName[0]);
	    if (strlen(tmpName))
	    {
		strcat (szDllName, ":");
		strcat (szDllName, tmpName);
	    }
	}
    }
    while ((macErr == noErr) && (block.ioDrDirID != 2)); // 2 means root directory of a volume
    strcat (szDllName, ":Real:Common:rmacore60.dll");
//    strcpy(szDllName, "rmacore60.dll");

#elif defined(_WIN32) 
    // get location of rmacore from windows registry 
    DWORD bufSize;
    HKEY hKey; 
    PN_RESULT hRes;
    
    szDllName[0] = '\0'; 
    bufSize = sizeof(szDllName) - 1;

    if(ERROR_SUCCESS == RegOpenKey(HKEY_CLASSES_ROOT,
	"Software\\RealNetworks\\Preferences\\DT_Common", &hKey)) 
    { 
	// get the path to pnen 
	hRes = RegQueryValue(hKey, "", szDllName, (long *)&bufSize); 
	RegCloseKey(hKey); 
    }

    strcat(szDllName, "pnen3260.dll");

#elif defined(_WIN16) 
    strcpy(szDllName, "pnen1660.dll");
#endif
    
    if (!(hDll = LoadLibrary(szDllName))) 
    {
	theErr = PNR_FAILED;
#if defined(_UNIX)
 	fprintf(stdout, "Failed to load the 'rmacore.so.6.0' shared library\n");
	fprintf(stdout, dlerror());
#else
 	fprintf(stdout, "Failed to load the '%s' library.\n", szDllName);
#endif
	goto cleanup;
    }

    fprintf(stdout, "Loaded rmacore\n");
    

    // retrieve the proc addresses from the module
    m_fpCreateEngine = (FPRMCREATEENGINE) GetProcAddress(hDll, "CreateEngine");
    m_fpCloseEngine  = (FPRMCLOSEENGINE)  GetProcAddress(hDll, "CloseEngine");
#ifdef _UNIX
    m_fpSetDLLAccessPath = (FPRMSETDLLACCESSPATH) GetProcAddress(hDll, "SetDLLAccessPath");
#endif
 
    if (m_fpCreateEngine == NULL ||
 	m_fpCloseEngine == NULL 
#ifdef _UNIX
	|| m_fpSetDLLAccessPath == NULL
#endif
	)
    {
	theErr = PNR_FAILED;
	goto cleanup;
    }

    // NOTE: Although the SetDLLAccess API is platform independant, it is 
    // not required for non-unix platforms since the core on those platforms handle this issue.
#ifdef _UNIX
    if (NULL != m_fpSetDLLAccessPath)
    {
	/*Create a null delimited, double-null terminated string containing
	 * the paths to the encnet library (DT_Common) and the sdpplin library
	 * (DT_Plugins)... */
	char pPaths[256];
	char* pPathNextPosition = pPaths;
	memset(pPaths, 0, 256);

	char pNextPath[256];
	memset(pNextPath, 0, 256);

	sprintf(pNextPath, "DT_Common=%s/Common", rphome);
	memcpy(pPathNextPosition, pNextPath, strlen(pNextPath) + 1);
	pPathNextPosition += strlen(pNextPath) + 1;

	sprintf(pNextPath, "DT_Plugins=%s/Plugins", rphome);
	memcpy(pPathNextPosition, pNextPath, strlen(pNextPath) + 1);
	pPathNextPosition += strlen(pNextPath) + 1;

	sprintf(pNextPath, "DT_Codecs=%s/Codecs", rphome);
	memcpy(pPathNextPosition, pNextPath, strlen(pNextPath) + 1);

	m_fpSetDLLAccessPath((char*)pPaths);
    }
#endif

    // create client engine 
    if (PNR_OK != m_fpCreateEngine((IRMAClientEngine**) &pEngine))
    {
	theErr = PNR_FAILED;
	goto cleanup;
    }

    // create player
    if (PNR_OK != pEngine->CreatePlayer(pPlayer))
    {
	theErr = PNR_FAILED;
	goto cleanup;
    }

    // initialize the example context
    exContext->Init(pPlayer);

    pPlayer->SetClientContext(exContext);

    pPlayer->QueryInterface(  IID_IRMAErrorSinkControl, 
					(void**) &pErrorSinkControl);
    if ( pErrorSinkControl )
    {	
	exContext->QueryInterface(  IID_IRMAErrorSink, 
					(void**) &pErrorSink);
		
	if (pErrorSink)
	{
	    pErrorSinkControl->AddErrorSink(pErrorSink, PNLOG_EMERG, PNLOG_INFO);
	}
    }	

    // There is only one URL for this presentation
    if (PNR_OK == pPlayer->OpenURL(pszURL))
    {
	pPlayer->Begin();

#if defined( _WIN32 ) || defined( _WINDOWS )
	MSG msg;
	while (GetMessage(&msg, NULL, 0, 0) && !pPlayer->IsDone())
	{
	    DispatchMessage(&msg);
	}

#elif defined (_UNIX)
        struct _PNxEvent* pNothing = 0;
        int     ret = 0;
        struct  timeval  mtime;


	pEngine->QueryInterface(IID_IRMAClientEngineSelector, 
				(void**) &pSelector);

	while (!pPlayer->IsDone())
	{
            mtime.tv_sec  = 0; 
            mtime.tv_usec = 10000;  // 20 ms is minimum
	    
            ret = pSelector->Select(0, 
				    (fd_set *) 0, 
				    (fd_set *) 0,
				    (fd_set *) 0,
				    &mtime);
				    
	    pEngine->EventOccurred(pNothing);
	}

#elif defined (_MACINTOSH)
	EventRecord event;
	char        theChar;

	while (!pPlayer->IsDone())
	{
	    WaitNextEvent(everyEvent, &event, 30, nil);
	    switch (event.what)
	    {
		case keyDown:
		case autoKey:
		    // Determine if command-period pressed
		    if( (event.modifiers & cmdKey) != 0 )
		    {
			theChar = (event.message & charCodeMask);
			if (theChar == kPeriodKey)
			{
			   goto cleanup;
			}
		    }
		    break;
	    }
	    
	    // Send all events to the player engine
	    PNxEvent pPNxEvent;   
	    pPNxEvent.event  = ((EventRecord*)&event)->what;
	    pPNxEvent.param1 = &event;
	    
	    if ( event.what == updateEvt ) {
	    	::BeginUpdate( (WindowPtr)event.message );
	    }
	    
	    pEngine->EventOccurred(&pPNxEvent);
	    
	    if ( event.what == updateEvt ) {
	    	::EndUpdate( (WindowPtr)event.message );
	    }
	}

#endif
	pPlayer->Stop();
    }

cleanup:

#ifdef _UNIX
    if (pSelector)
    {
        pSelector->Release();
    }
#endif

    if (pErrorSinkControl)
    {
	pErrorSinkControl->RemoveErrorSink(pErrorSink);
	pErrorSinkControl->Release();
	pErrorSinkControl = NULL;
    }

    if (exContext)
    {
	exContext->Release();
	exContext = NULL;
    }
    
    if (pPlayer)
    {
	pEngine->ClosePlayer(pPlayer);
	pPlayer->Release();
	pPlayer = NULL;
    }

    if (pEngine)
    {
	m_fpCloseEngine(pEngine);
	pEngine = NULL;
    }

    if (hDll)
    {
	FreeLibrary(hDll);
	hDll = NULL;
    }

    return theErr;
}
