/****************************************************************************
 * 
 *  $Id: winhdrs.cpp,v 1.18 1999/02/18 20:21:28 jfarr Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 * 
 *  winhdrs.cpp
 *
 *  Example application to demonstrate sending and receiving RFC822 headers
 *  NOTE: This example is intended for use with the HTTP protocol only
 *
 */


/****************************************************************************
 * Defines
 */
#define   INITGUID     /* Interface ID's */


/****************************************************************************
 * Includes
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "pncom.h"
#include "pntypes.h"
#include "pnresult.h"
#include "pnwintyp.h"
#include "rmafiles.h"
#include "rmaengin.h"
#include "rmapckts.h"
#include "rmacore.h"
#include "rmacomm.h"

#ifdef _WIN16
    #include <windows.h>
#endif

#ifdef _MACINTOSH
    #ifdef __MWERKS__
	#include <console.h>
	#include <sioux.h>
    #endif
    #include "os.h"
#endif

#include "winhdrs.h"


/****************************************************************************
 * Prototypes
 */
PN_RESULT parse_options(int argc, char **argv);


/****************************************************************************
 * Globals
 */
FPRMCREATEENGINE	g_fpCreateEngine = 0;
FPRMCLOSEENGINE	 	g_fpCloseEngine = 0;
IRMACommonClassFactory*	g_pCommonClassFactory = 0;
IRMAFileSystemManager*	g_pFileSystemManager = 0;
IUnknown*		g_pUnknown = 0;
IRMAFileObject*		g_pFileObject = 0;
IRMARequest*		g_pRequest = 0;
BOOL			g_bDone = FALSE;
const char*		g_pURL = 0;


/****************************************************************************
 *  main
 *
 */
int main(int argc, char ** argv)
{
    PN_RESULT	result = PNR_OK;
    HINSTANCE	hDll = NULL;

    #ifdef __MWERKS__
	argc = ccommand( &argv );
	SIOUXSettings.standalone     = FALSE;
	SIOUXSettings.setupmenus     = FALSE;
	SIOUXSettings.showstatusline = TRUE;
    #endif

    // parse command line
    result = parse_options(argc, argv);

    if (PNR_OK != result)
    {
	fprintf(stdout, "\n\nUSAGE: winhdrs URL\n");
	return result;
    }

    // initialize the globals
    g_fpCreateEngine	= NULL;
    g_fpCloseEngine	= NULL;

    IRMAClientEngine* pEngine = 0;
    CFSManagerResponse* pFSManagerResponse = 0;

    // prepare/load the RMACore module
#if defined(_MACINTOSH)    
    if (!(hDll = LoadLibrary("rmacore60.dll")))    
#elif defined(_UNIX)
    if (!(hDll = LoadLibrary("rmacore.dll")))
#else
    if (!(hDll = LoadLibrary("pnen3260.dll")))    
#endif
    {
	result = PNR_FAILED;
#ifndef _UNIX
 	fprintf(stdout, "Failed to load the 'rmacore' DLL\n");
#endif
#if defined(_UNIX)
 	fprintf(stdout, "Failed to load the 'rmacore.so.6.0' shared library\n");
	fprintf(stdout, dlerror());
#endif
	goto main_cleanup;
    }

    // retrieve the proc addresses from the module
    g_fpCreateEngine = (FPRMCREATEENGINE) GetProcAddress(hDll, "CreateEngine");
    g_fpCloseEngine  = (FPRMCLOSEENGINE)  GetProcAddress(hDll, "CloseEngine");
 
    if (g_fpCreateEngine == NULL || g_fpCloseEngine == NULL)
    {
	result = PNR_FAILED;
	goto main_cleanup;
    }

    // create client engine 
    result = g_fpCreateEngine((IRMAClientEngine**)&pEngine);

    if (PNR_OK != result)
    {
	goto main_cleanup;
    }

    result = pEngine->QueryInterface(IID_IRMACommonClassFactory,
                                     (void**)&g_pCommonClassFactory);

    if (PNR_OK != result)
    {
	goto main_cleanup;
    }

    result = g_pCommonClassFactory->CreateInstance(CLSID_IRMAFileSystemManager,
                                                   (void**)&g_pFileSystemManager);

    if (PNR_OK != result)
    {
	goto main_cleanup;
    }

    pFSManagerResponse = new CFSManagerResponse();
    pFSManagerResponse->AddRef();

    result = g_pFileSystemManager->Init(pFSManagerResponse);

    if (PNR_OK != result)
    {
	goto main_cleanup;
    }

    pFSManagerResponse->Release();

#if defined( _WIN32 ) || defined( _WINDOWS )
    MSG msg;

    while (GetMessage(&msg, NULL, 0, 0) && !g_bDone)
    {
	DispatchMessage(&msg);
    }


#elif defined (_MACINTOSH)
    EventRecord event;

    while (!g_bDone)
    {
        WaitNextEvent(everyEvent, &event, 30, nil);
        switch (event.what)
        {
    	    case keyDown:
	        goto main_cleanup;
	        break;
    	    
	    default:
	        PNxEvent pPNxEvent;   
	        pPNxEvent.event  = ((EventRecord*)&event)->what;
	        pPNxEvent.param1 = &event;
	        pEngine->EventOccurred(&pPNxEvent);
        }
    }
#endif

main_cleanup:

    if (g_pCommonClassFactory)
    {
	g_pCommonClassFactory->Release();
    }

    if (g_pFileSystemManager)
    {
	g_pFileSystemManager->Release();
    }

    if (g_pUnknown)
    {
	g_pUnknown->Release();
    }

    if (g_pFileObject)
    {
	g_pFileObject->Release();
    }

    if (g_pRequest)
    {
	g_pRequest->Release();
    }

    if (pEngine)
    {
	g_fpCloseEngine(pEngine);
	pEngine = NULL;
    }

    if (hDll)
    {
	FreeLibrary(hDll);
	hDll = NULL;
    }

    return result;
}


/****************************************************************************
 *  parse_options
 *
 *  This routine parses the command line arguments:
 *  1. Validates arguments
 *  2. Creates a list of names of input files
 *  3. Isolates the output filename
 */
PN_RESULT parse_options(int argc, char **argv)
{
    PN_RESULT	result		= PNR_OK;
    int		i		= 0;
  
    if (argc != 2)
    {
	result = PNR_FAILED;
	goto parse_cleanup;
    }

    g_pURL = argv[1];

parse_cleanup:

    return result;
}


// CFSManagerResponse Interface Methods

/****************************************************************************
 *  CFSManagerResponse::CFSManagerResponse                    ref:  winhdrs.h
 *
 *  Constructor
 */
CFSManagerResponse::CFSManagerResponse()
    : m_lRefCount(0)
{}


/****************************************************************************
 *  CFSManagerResponse::~CFSManagerResponse                   ref:  winhdrs.h
 *
 *  Destructor
 */
CFSManagerResponse::~CFSManagerResponse()
{}


/****************************************************************************
 *  CFSManagerResponse::InitDone                             ref:  rmafiles.h
 *
 */
STDMETHODIMP
CFSManagerResponse::InitDone
(
    PN_RESULT status
)
{
    if (PNR_OK != status)
    {
	g_bDone = TRUE;
	return PNR_OK;
    }

    /*
     * Setup IRMARequest object
     */

    PN_RESULT result = PNR_OK;

    result = g_pCommonClassFactory->CreateInstance(CLSID_IRMARequest,
                                                   (void**)&g_pRequest);

    if (PNR_OK != result)
    {
	g_bDone = TRUE;
	return result;
    }

    result = g_pRequest->SetURL(g_pURL);

    if (PNR_OK != result)
    {
	g_bDone = TRUE;
	return result;
    }
    
    IRMABuffer* pFooBuffer;
    IRMABuffer* pBarBuffer;
    IRMAValues* pHeaders;

    result = g_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,
                                                   (void**)&pFooBuffer);

    if (PNR_OK != result)
    {
	g_bDone = TRUE;
	goto init_cleanup;
    }

    result = g_pCommonClassFactory->CreateInstance(CLSID_IRMABuffer,
                                                   (void**)&pBarBuffer);

    if (PNR_OK != result)
    {
	g_bDone = TRUE;
	goto init_cleanup;
    }

    /*
     * Must include the '\0' in the buffer string
     */

    result = pFooBuffer->Set((BYTE*)"blech, blech", 13);

    if (PNR_OK != result)
    {
	g_bDone = TRUE;
	goto init_cleanup;
    }

    result = pBarBuffer->Set((BYTE*)"buzz; buzz=zzub", 16);

    if (PNR_OK != result)
    {
	g_bDone = TRUE;
	goto init_cleanup;
    }

    result = g_pCommonClassFactory->CreateInstance(CLSID_IRMAValues,
                                                   (void**)&pHeaders);

    if (PNR_OK != result)
    {
	g_bDone = TRUE;
	goto init_cleanup;
    }

    /*
     * Setting 2 RFC822 headers...foo:blech and bar:buzz
     */

    result = pHeaders->SetPropertyCString("foo", pFooBuffer);

    if (PNR_OK != result)
    {
	g_bDone = TRUE;
	goto init_cleanup;
    }

    result = pHeaders->SetPropertyCString("bar", pBarBuffer);

    if (PNR_OK != result)
    {
	g_bDone = TRUE;
	goto init_cleanup;
    }

    result = g_pRequest->SetRequestHeaders(pHeaders);

    if (PNR_OK != result)
    {
	g_bDone = TRUE;
	goto init_cleanup;
    }

    result = g_pFileSystemManager->GetFileObject(g_pRequest, 0);

    if (PNR_OK != result)
    {
	g_bDone = TRUE;
    }

init_cleanup:

    if (pFooBuffer)
    {
	pFooBuffer->Release();
    }

    if (pBarBuffer)
    {
	pBarBuffer->Release();
    }

    if (pHeaders)
    {
	pHeaders->Release();
    }

    return result;
}


/****************************************************************************
 *  CFSManagerResponse::FileObjectReady                      ref:  rmafiles.h
 *
 */
STDMETHODIMP
CFSManagerResponse::FileObjectReady
(
    PN_RESULT status,
    IUnknown* pUnknown
)
{
    if (PNR_OK != status || !pUnknown)
    {
	g_bDone = TRUE;
	return PNR_OK;
    }

    g_pUnknown = pUnknown;
    g_pUnknown->AddRef();

    PN_RESULT result = PNR_OK;

    CFileResponse* pFileResponse;

    result = g_pUnknown->QueryInterface(IID_IRMAFileObject,
                                        (void**)&g_pFileObject);

    if (PNR_OK != result)
    {
	g_bDone = TRUE;
	return result;
    }

    pFileResponse = new CFileResponse();
    pFileResponse->AddRef();

    result = g_pFileObject->Init(PN_FILE_READ,
                                 (IRMAFileResponse*)pFileResponse);

    if (PNR_OK != result)
    {
	g_bDone = TRUE;
    }

    pFileResponse->Release();

    return result;
}


/****************************************************************************
 *  CFSManagerResponse::DirObjectReady                       ref:  rmafiles.h
 *
 */
STDMETHODIMP
CFSManagerResponse::DirObjectReady
(
    PN_RESULT status,
    IUnknown* pDirObject
)
{
    return PNR_NOTIMPL;
}


// IUnknown COM Interface Methods

/****************************************************************************
 *  IUnknown::AddRef                                            ref:  pncom.h
 *
 *  This routine increases the object reference count in a thread safe
 *  manner. The reference count is used to manage the lifetime of an object.
 *  This method must be explicitly called by the user whenever a new
 *  reference to an object is used.
 */
STDMETHODIMP_(ULONG32)
CFSManagerResponse::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}


/****************************************************************************
 *  IUnknown::Release                                           ref:  pncom.h
 *
 *  This routine decreases the object reference count in a thread safe
 *  manner, and deletes the object if no more references to it exist. It must
 *  be called explicitly by the user whenever an object is no longer needed.
 */
STDMETHODIMP_(ULONG32)
CFSManagerResponse::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
        return m_lRefCount;
    }

    delete this;
    return 0;
}


/****************************************************************************
 *  IUnknown::QueryInterface                                    ref:  pncom.h
 *
 *  This routine indicates which interfaces this object supports. If a given
 *  interface is supported, the object's reference count is incremented, and
 *  a reference to that interface is returned. Otherwise a NULL object and
 *  error code are returned. This method is called by other objects to
 *  discover the functionality of this object.
 */
STDMETHODIMP
CFSManagerResponse::QueryInterface
(
    REFIID riid,
    void** ppvObj
)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
	AddRef();
	*ppvObj = (IUnknown*)(IRMAFileSystemManagerResponse*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMAFileSystemManagerResponse))
    {
	AddRef();
	*ppvObj = (IRMAFileSystemManagerResponse*)this;
	return PNR_OK;
    }

    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}


// IRMAFileResponse Interface Methods

/****************************************************************************
 *  CFileResponse::CFileResponse                              ref:  winhdrs.h
 *
 *  Constructor
 */
CFileResponse::CFileResponse()
    : m_lRefCount(0)
{}


/****************************************************************************
 *  CFileResponse::~CFileResponse                             ref:  winhdrs.h
 *
 *  Destructor
 */
CFileResponse::~CFileResponse()
{}


/****************************************************************************
 *  IRMAFileResponse::InitDone                               ref:  rmafiles.h
 *
 *  This routine notifies the RMA core that the initialization of the file is
 *  done. It is called automatically when the initialization of the file
 *  has completed.
 */
STDMETHODIMP
CFileResponse::InitDone
(
    PN_RESULT status
)
{
    if (PNR_OK != status)
    {
	g_bDone = TRUE;
	return PNR_OK;
    }

    PN_RESULT result = PNR_OK;

    /*
     * Print out the headers
     */

    IRMAValues* pHeaders = 0;
    UINT32 uContentLength = 0;

    result = g_pRequest->GetResponseHeaders(pHeaders);

    if (PNR_OK != result)
    {
	g_bDone = TRUE;
        return result;
    }

    if (pHeaders)
    {
	fprintf(stdout, "\nHEADERS...\n\n");

	const char* pName;
	IRMABuffer* pValue;

	result = pHeaders->GetFirstPropertyCString(pName, pValue);

	while (result == PNR_OK)
	{
	    fprintf(stdout, "\t%s: %s\n", pName, (char*)pValue->GetBuffer());

	    if (!strnicmp(pName, "content-length", 14))
	    {
		uContentLength = atoi((char*)pValue->GetBuffer());
	    }

	    pValue->Release();

	    result = pHeaders->GetNextPropertyCString(pName, pValue);
	}

	pHeaders->Release();
    }

    if (uContentLength)
    {
	result = g_pFileObject->Read(uContentLength);

	if (PNR_OK != result)
	{
	    g_bDone = TRUE;
	}

	return result;
    }

    result = g_pFileObject->Close();

    if (PNR_OK != result)
    {
	g_bDone = TRUE;
    }

    return result;
}


/****************************************************************************
 *  IRMAFileResponse::SeekDone                               ref:  rmafiles.h
 *
 *  This routine is automatically called when the Seek() operation in the
 *  file is complete. Other actions are then taken dependent on the current
 *  processing state.
 */
STDMETHODIMP
CFileResponse::SeekDone
(
    PN_RESULT status
)
{
    return PNR_NOTIMPL;
}


/****************************************************************************
 *  IRMAFileResponse::ReadDone                               ref:  rmafiles.h
 *
 *  This routine is automatically called when the Read() from the file is
 *  done. Other actions are then taken dependent on the current processing
 *  state.
 */
STDMETHODIMP
CFileResponse::ReadDone
(
    PN_RESULT status,
    IRMABuffer* pBuffer
)
{
    if (PNR_OK != status)
    {
	g_bDone = TRUE;
	return PNR_OK;
    }

    PN_RESULT result = PNR_OK;

    fprintf(stdout, "\nCONTENTS...\n\n");

    const BYTE* pBuf = pBuffer->GetBuffer();

    for (UINT32 i = 0; i < pBuffer->GetSize(); i++)
    {
	fprintf(stdout, "%c", pBuf[i]);
    }
 
    fprintf(stdout, "\n");

    result = g_pFileObject->Close();

    if (PNR_OK != result)
    {
	g_bDone = TRUE;
    }

    return result;
}


/****************************************************************************
 *  CFileResponse::WriteDone                                 ref:  rmafiles.h
 *
 */
STDMETHODIMP
CFileResponse::WriteDone
(
    PN_RESULT status
)
{
    return PNR_NOTIMPL;
}

  
/****************************************************************************
 *  CFileResponse::CloseDone                                 ref:  rmafiles.h
 *
 */
STDMETHODIMP
CFileResponse::CloseDone
(
    PN_RESULT status
)
{
    g_bDone = TRUE;

    return PNR_OK;
}


// IUnknown COM Interface Methods

/****************************************************************************
 *  IUnknown::AddRef                                            ref:  pncom.h
 *
 *  This routine increases the object reference count in a thread safe
 *  manner. The reference count is used to manage the lifetime of an object.
 *  This method must be explicitly called by the user whenever a new
 *  reference to an object is used.
 */
STDMETHODIMP_(ULONG32)
CFileResponse::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}


/****************************************************************************
 *  IUnknown::Release                                           ref:  pncom.h
 *
 *  This routine decreases the object reference count in a thread safe
 *  manner, and deletes the object if no more references to it exist. It must
 *  be called explicitly by the user whenever an object is no longer needed.
 */
STDMETHODIMP_(ULONG32)
CFileResponse::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
        return m_lRefCount;
    }

    delete this;
    return 0;
}


/****************************************************************************
 *  IUnknown::QueryInterface                                    ref:  pncom.h
 *
 *  This routine indicates which interfaces this object supports. If a given
 *  interface is supported, the object's reference count is incremented, and
 *  a reference to that interface is returned. Otherwise a NULL object and
 *  error code are returned. This method is called by other objects to
 *  discover the functionality of this object.
 */
STDMETHODIMP
CFileResponse::QueryInterface
(
    REFIID riid,
    void** ppvObj
)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
	AddRef();
	*ppvObj = (IUnknown*)(IRMAFileResponse*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMAFileResponse))
    {
	AddRef();
	*ppvObj = (IRMAFileResponse*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMAFileStatResponse))
    {
	AddRef();
	*ppvObj = (IRMAFileStatResponse*)this;
	return PNR_OK;
    }

    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}
