/****************************************************************************
 * $Id: converter.cpp,v 1.3 2000/05/12 01:26:58 paulm Exp $
 *
 *  Copyright (C) 1995,1996,1997,1998,1999,2000 RealNetworks, Inc.
 *  All rights reserved.
 *
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 *
 *  rnconvert.cpp
 *
 *  Example server side data converter plug-in for the RealMedia 
 *  Architecture (RMA).
 */

#include <stdio.h>
#include <stdlib.h>

#define INITGUID

#include "pncom.h"
#include "rmacomm.h"
#include "rmaplugn.h"
#include "rmapckts.h"
#include "rmadtcvt.h"
#include "rmafiles.h"

#include "converter.h"

/****************************************************************************
 * Utility function for inspecting the contents of an IRMAValues.
 */
void
DumpValues(IRMAValues* pValues)
{
    PN_RESULT res;
    IRMABuffer* pBuffer;
    const char* pName;
    ULONG32 ul;

    printf("BUFFER\n");
    res = pValues->GetFirstPropertyBuffer(pName, pBuffer);
    while (res == PNR_OK)
    {
	printf("%s: %s\n", pName, (const char*)pBuffer->GetBuffer());
	pBuffer->Release();
	res = pValues->GetNextPropertyBuffer(pName, pBuffer);
    }
    
    printf("ULONG32\n");
    res = pValues->GetFirstPropertyULONG32(pName, ul);
    while (res == PNR_OK)
    {
	printf("%s: %lu\n", pName, ul);
	res = pValues->GetNextPropertyULONG32(pName, ul);
    }
    
    printf("CSTRING\n");
    res = pValues->GetFirstPropertyCString(pName, pBuffer);
    while (res == PNR_OK)
    {
	printf("%s: %s\n", pName, (const char*)pBuffer->GetBuffer());
	pBuffer->Release();
	res = pValues->GetNextPropertyCString(pName, pBuffer);
    }
}

/****************************************************************************
 *  RMACreateInstance                                        ref:  rmaplugn.h
 *
 *  This routine creates a new instance of the CMyExampleFileSystem class.
 *  It is called when the RMA core application is launched, and whenever
 *  an URL with a protocol associated with this plug-in is opened.
 */
STDAPI
RMACreateInstance(IUnknown** pp)
{
    *pp = (IUnknown*)(IRMAPlugin*)new ConverterSystemObject;
    if (*pp != NULL)
    {
            (*pp)->AddRef();
            return PNR_OK;
    }

    return PNR_OUTOFMEMORY;
}

/*
 * ConverterSystemObject
 */
ConverterSystemObject::ConverterSystemObject()
: m_lRefCount(0)
, m_pContext(0)
, m_pOptions(0)
{
}

ConverterSystemObject::~ConverterSystemObject()
{
    PN_RELEASE(m_pContext);
    PN_RELEASE(m_pOptions);
}

/*
 *	IUnknown methods
 */
STDMETHODIMP
ConverterSystemObject::QueryInterface(REFIID riid, void** ppvObj)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
	AddRef();
	*ppvObj = (IRMAPlugin*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMAPlugin))
    {
	AddRef();
	*ppvObj = (IRMAPlugin*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMADataConvertSystemObject))
    {
	AddRef();
	*ppvObj = (IRMADataConvertSystemObject*)this;
	return PNR_OK;
    }
    
    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}

STDMETHODIMP_(ULONG32)
ConverterSystemObject::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}

STDMETHODIMP_(ULONG32)
ConverterSystemObject::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
	return m_lRefCount;
    }
    
    delete this;
    return 0;
}

/*
 *	IRMAPlugin methods
 */
STDMETHODIMP
ConverterSystemObject::GetPluginInfo(REF(BOOL)	 /*OUT*/ bMultipleLoad,
			REF(const char*) /*OUT*/ pDescription,
			REF(const char*) /*OUT*/ pCopyright,
			REF(const char*) /*OUT*/ pMoreInfoURL,
			REF(ULONG32)	 /*OUT*/ ulVersionNumber)
{
    bMultipleLoad = TRUE;
    pDescription = MY_DESCRIPTION;
    pCopyright = MY_COPYRIGHT;
    pMoreInfoURL = MY_MORE_INFO_URL;
    ulVersionNumber = MY_PLUGIN_VERSION;
    return PNR_OK;
}

STDMETHODIMP
ConverterSystemObject::InitPlugin(IUnknown* pContext)
{
    PN_RELEASE(m_pContext);
    m_pContext = pContext;
    m_pContext->AddRef();
    return PNR_OK;
}

/************************************************************************
 * IRMADataConvertSystemObject methods
 *
 * Purpose: This method tells the server core what my shortname is. 
 *          server core then uses this information to determine which
 *          data converter to instantiate based on mount point.
 *          Relevent config file section looks like this:
 *
 *	<List Name="DataConvertMount">
 *	    <List Name="Sample Data Converter">
 *		<Var ShortName="rn-sample-convert"/>
 *		<Var MountPoint="/convert/"/>
 *	    </List>
 *	</List>
 *
 *	    By Adding the above to your config file, this
 *	    IRMADataConvertSystemObject will get instantiated and 
 *	    asked to create an IRMADataConvert (through CreateDataConvert)
 *	    for every file request under the /convert/ path.
 *
 */
STDMETHODIMP
ConverterSystemObject::GetDataConvertInfo(REF(const char*) pShortName)
{
    pShortName = MY_SHORT_NAME;
    return PNR_OK;
}

/************************************************************************
 * Method: InitDataConvertSystemObject
 *
 * Purpose:  Pass in options from the config file.  This includes
 *           everything under the list for this plugin (for example,
 *           everything under "Sample Data Converter" from above).
 *           As an optional feature, this particular plugin will respond
 *           to a variable "ConvertHeaders" to turn on/off conversion
 *           of file and stream headers.
 */
STDMETHODIMP
ConverterSystemObject::InitDataConvertSystem(IRMAValues* pOptions)
{
    PN_RELEASE(m_pOptions);
    m_pOptions = pOptions;
    m_pOptions->AddRef();
    ULONG32 ul;
    if (PNR_OK == pOptions->GetPropertyULONG32("ConvertHeaders", ul))
    {
	m_ulConvertHeaders = ul;
    }
    else
    {
	m_ulConvertHeaders = 1;
    }
    return PNR_OK;
}

/************************************************************************
 * Method: CreateDataConvert
 *
 * Purpose: server core got a requets under your MountPoint and wants
 *          an object to do the conversions.
 */
STDMETHODIMP
ConverterSystemObject::CreateDataConvert(IUnknown** ppConvObj)
{
    Converter* pConv = new Converter(m_pContext, m_ulConvertHeaders);
    pConv->QueryInterface(IID_IUnknown, (void**)ppConvObj);
    return PNR_OK;
}

/*
 * Converter
 */
Converter::Converter(IUnknown* pContext, ULONG32 ul)
: m_lRefCount(0)
, m_pResponse(0)
, m_pContext(0)
, m_ulNumber(0)
, m_ulConvertHeaders(ul)
, m_type(UNICAST)
{
    m_pContext = pContext;
    m_pContext->AddRef();
    m_pContext->QueryInterface(IID_IRMACommonClassFactory,
	    (void**)&m_pClassFactory);
}

Converter::~Converter()
{
    PN_RELEASE(m_pResponse);
    PN_RELEASE(m_pContext);
    PN_RELEASE(m_pClassFactory);
}

/*
 *	IUnknown methods
 */
STDMETHODIMP
Converter::QueryInterface(REFIID riid, void** ppvObj)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
	AddRef();
	*ppvObj = (IRMAPlugin*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMADataConvert))
    {
	AddRef();
	*ppvObj = (IRMADataConvert*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMARequestHandler))
    {
	AddRef();
	*ppvObj = (IRMARequestHandler*)this;
	return PNR_OK;
    }
    
    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}

STDMETHODIMP_(ULONG32)
Converter::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}

STDMETHODIMP_(ULONG32)
Converter::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
	return m_lRefCount;
    }
    
    delete this;
    return 0;
}

/*
 *  IRMADataConvert
 */

/************************************************************************
 * Method: DataConvertInit
 *
 * Purpose: Intialize per player object with response object.  We use
 *          this to send our encryption key to the player.  This is
 *          not intended to be used as any form of encrytion, but
 *          is just a sample for how these things could be done.
 */
STDMETHODIMP
Converter::DataConvertInit(IRMADataConvertResponse* pResponse)
{
    PN_RELEASE(m_pResponse);
    m_pResponse = pResponse;
    m_pResponse->AddRef();
    m_pResponse->DataConvertInitDone(PNR_OK);
    return PNR_OK;
}

/************************************************************************
 * Method: GetConversionMimeType
 *
 * Purpose: Tell the core what conversion type you are using for the
 *          stream.  This doesn't get called until after your are
 *          initialized, so you could have a single plugin that
 *          implements different conversion types.
 */
STDMETHODIMP
Converter::GetConversionMimeType(REF(const char*)pConversionType)
{
    pConversionType = MY_CONVERSION_MIMETYPE;
    return PNR_OK;
}

/************************************************************************
 * Method: ConvertFileHeader
 *
 * Purpose: This is the beginning of the data the core is going to 
 *          pass you for conversion.  If you don't want to convert
 *          the headers, call the done method with PNR_OK,NULL.
 */
STDMETHODIMP
Converter::ConvertFileHeader(IRMAValues* pFileHeader)
{
    if (m_type != MULTICAST_CONTROL && !m_ulNumber)
    {
	m_ulNumber = (UINT32)(rand() / (double)RAND_MAX * 128 + 128);
    }
    if (m_type != MULTICAST_TRANSPORT)
    {
	XMitKey();
    }

    if (m_ulConvertHeaders)
    {
	IRMAValues* pValues = GetConvertedHeader(pFileHeader);
	m_pResponse->ConvertedFileHeaderReady(PNR_OK, pValues);
	pValues->Release();
    }
    else
    {
	m_pResponse->ConvertedFileHeaderReady(PNR_OK, NULL);
    }

    return PNR_OK;
}

/************************************************************************
 * Method: ConvertStreamHeader
 *
 * Purpose: Per stream headers coming to you for conversion.  Again, if
 *          you don't want to convert the headers, call the done function
 *          with PNR_OK, NULL.
 */
STDMETHODIMP
Converter::ConvertStreamHeader(IRMAValues* pStreamHeader)
{
    if (m_ulConvertHeaders)
    {
	IRMAValues* pValues = GetConvertedHeader(pStreamHeader);
	m_pResponse->ConvertedStreamHeaderReady(PNR_OK, pValues);
	pValues->Release();
    }
    else
    {
	m_pResponse->ConvertedStreamHeaderReady(PNR_OK, NULL);
    }

    return PNR_OK;
}

/************************************************************************
 * Method: ConvertData
 *
 * Pupose: Now the core is starting to send you the actual binary stream
 *         data.
 */
STDMETHODIMP
Converter::ConvertData(IRMAPacket* pPacket)
{
    IRMABuffer* pBuffer;
    UINT32 ulTime;
    UINT16 unStreamNumber;
    UINT8  unASMFlags;
    UINT16 unASMRuleNumber;

    pPacket->Get(pBuffer, ulTime, unStreamNumber, unASMFlags, unASMRuleNumber);
    IRMABuffer* pNewBuf = GetConvertedBuffer(pBuffer);
    IRMAPacket* pNewPacket;
    m_pClassFactory->CreateInstance(CLSID_IRMAPacket, (void**)&pNewPacket);
    pNewPacket->Set(pNewBuf, ulTime, unStreamNumber,
	    unASMFlags, unASMRuleNumber);
    pNewBuf->Release();
    pBuffer->Release();
    m_pResponse->ConvertedDataReady(PNR_OK, pNewPacket);
    pNewPacket->Release();
    return PNR_OK;
}

/************************************************************************
 * Method: SetMulticastTransportConverter
 *
 * Purpose:
 *  In this case the IRMADataConvert is only handling the header
 *  conversions per player and this call is handing you the data
 *  converter which is doing the data.  This will a different
 *  instance of the same object.
 */
STDMETHODIMP
Converter::SetMulticastTransportConverter(IRMADataConvert* pConverter)
{
    m_type = MULTICAST_CONTROL;
    m_ulNumber = ((Converter*)pConverter)->GetKey();
    XMitKey();

    return PNR_OK;
}

/************************************************************************
 * Method: AddMulticastControlConverter
 *
 * Purpose:
 *  In this case the IRMADataConvert is only handling the data 
 *  conversions for all of the players (but only once because it's
 *  multicast).  This call is handing you one of a possible many
 *  IRMADataConvert objects which will be handling the header
 *  conversions.
 */
STDMETHODIMP
Converter::AddMulticastControlConverter(IRMADataConvert* pConverter)
{
    m_type = MULTICAST_TRANSPORT;
    if (!m_ulNumber)
    {
	m_ulNumber = (UINT32)(rand() / (double)RAND_MAX * 128 + 128);
    }
    return PNR_OK;
}

/************************************************************************
 * Method: ControlBufferReady
 *
 * Purpose: Our reverter on the client side is sending us a message.  In
 *          this sample the client just sends us an acknowledgment, but
 *          we ignore it.  It comes in here though.
 */
STDMETHODIMP
Converter::ControlBufferReady(IRMABuffer* pBuffer)
{
    return PNR_OK;
}

/************************************************************************
 * Method: Done
 *
 * Purpose: clear up any circular refs because we are shutting down.  If
 *	    we had held on to and ref'd our multicast counterpart here
 *	    is where we would release.
 */
STDMETHODIMP
Converter::Done()
{
    return PNR_OK;
}

/************************************************************************
 *	Method:
 *	    IRMARequestHandler::SetRequest
 *	Purpose:
 *	    Associates an IRMARequest with an object
 */
STDMETHODIMP
Converter::SetRequest(IRMARequest*        /*IN*/  pRequest)
{
    return PNR_OK;
}

/************************************************************************
 *	Method:
 *	    IRMARequestHandler::GetRequest
 *	Purpose:
 *	    Gets the IRMARequest object associated with an object
 */
STDMETHODIMP
Converter::GetRequest(REF(IRMARequest*)        /*OUT*/  pRequest)
{
    return PNR_OK;
}

UINT32
Converter::GetKey()
{
    return m_ulNumber;
}

/*
 * Private helpers
 */

void
Converter::XMitKey()
{
    IRMABuffer* pBuffer;
    char numbuf[256];
    sprintf(numbuf, "init%lu %d", m_ulNumber, m_ulConvertHeaders);
    m_pClassFactory->CreateInstance(CLSID_IRMABuffer, (void**)&pBuffer);
    pBuffer->Set((const unsigned char*)numbuf, strlen(numbuf) + 1);
    m_pResponse->SendControlBuffer(pBuffer);
    pBuffer->Release();
}

/*
 * GetConvertedBuffer AND GetConvertedCString are needed because sometimes
 * the core will pass us buffers (which it got from other plugins) which
 * have CStrings which are in buffers which are WAY bigger than the size
 * of the string.  In this case we don't want to encode the whole buffer,
 * because it is only meant to be meaningfull up to the first NULL.
 */
IRMABuffer*
Converter::GetConvertedCString(IRMABuffer* pSource)
{
    return GetConverted(pSource, strlen((const char*)pSource->GetBuffer()) + 1);
}

IRMABuffer*
Converter::GetConvertedBuffer(IRMABuffer* pSource)
{
    return GetConverted(pSource, pSource->GetSize());
}

IRMABuffer*
Converter::GetConverted(IRMABuffer* pSource, int len)
{
    IRMABuffer* pNewBuf;
    m_pClassFactory->CreateInstance(CLSID_IRMABuffer, (void**)&pNewBuf);
    pNewBuf->SetSize(len);

    const char* pFrom;
    char* pTo;
    pFrom = (const char*)pSource->GetBuffer();
    pTo = (char*)pNewBuf->GetBuffer();
    int i;
    
    /*
     * Just XOR every other byte with that key that we sent to the player
     * in plaintext.  THIS IS NOT MEANT TO BE SECURE!
     */
    for (i = 0; i < len; i++)
    {
	if (i % 2 == 0)
	{
	    *pTo = (*pFrom) ^ (char)m_ulNumber;
	}
	else
	{
	    *pTo = *pFrom;
	}
	pTo++;
	pFrom++;
    }
    return pNewBuf;
}

IRMAValues*
Converter::GetConvertedHeader(IRMAValues* pHeader)
{
    const char* pName;
    IRMABuffer* pBuffer;
    PN_RESULT res;
    ULONG32 ul;
    
    IRMAValues* pValues;
    m_pClassFactory->CreateInstance(CLSID_IRMAValues, (void**)&pValues);
    
    
    /* ULONG32 */
    res = pHeader->GetFirstPropertyULONG32(pName, ul);
    while (res == PNR_OK)
    {
	pValues->SetPropertyULONG32(pName, ul);
	res = pHeader->GetNextPropertyULONG32(pName, ul);
    }

    /* CString */
    res = pHeader->GetFirstPropertyCString(pName, pBuffer);
    while (res == PNR_OK)
    {
	IRMABuffer* pNewBuf = GetConvertedCString(pBuffer);
	pValues->SetPropertyCString(pName, pNewBuf);
	pNewBuf->Release();
	pBuffer->Release();
	res = pHeader->GetNextPropertyCString(pName, pBuffer);
    }
    
    /* Buffer */
    res = pHeader->GetFirstPropertyBuffer(pName, pBuffer);
    while (res == PNR_OK)
    {
	IRMABuffer* pNewBuf = GetConvertedBuffer(pBuffer);
	pValues->SetPropertyBuffer(pName, pNewBuf);
	pNewBuf->Release();
	pBuffer->Release();
	res = pHeader->GetNextPropertyBuffer(pName, pBuffer);
    }

    return pValues;
}

