/***************************************************************************
 *
 * $Id: converter.h,v 1.2 2000/05/12 01:26:58 paulm Exp $
 *
 *  Copyright (C) 1995,1996,1997,1998,1999,2000 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 *
 *  converter.h
 *
 *  Example server side data converter plug-in for the RealMedia 
 *  Architecture (RMA).
 */

#ifndef _RNCONVERT_H
#define _RNCONVERT_H

/***************************************************************************
 * Constants
 */
#define MY_PLUGIN_VERSION       0
#define MY_DESCRIPTION         "Example DataConverter"
#define MY_COPYRIGHT           "(c) 1995-2000 RealNetworks, All rights reserved."
#define MY_MORE_INFO_URL       "http://www.real.com"
#define MY_SHORT_NAME	       "rn-sample-convert"
#define MY_CONVERSION_MIMETYPE "rn-sample-convert-mimetype"

class ConverterSystemObject : public IRMAPlugin,
			      public IRMADataConvertSystemObject
{
public:
    ConverterSystemObject();
    ~ConverterSystemObject();
    
    /*
     *  IUnknown methods
     */
    STDMETHOD(QueryInterface)           (THIS_
                                        REFIID riid,
                                        void** ppvObj);

    STDMETHOD_(ULONG32,AddRef)          (THIS);

    STDMETHOD_(ULONG32,Release)         (THIS);

    /*
     *	IRMAPlugin methods
     */
    STDMETHOD(GetPluginInfo)	(THIS_
				REF(BOOL)	 /*OUT*/ bMultipleLoad,
				REF(const char*) /*OUT*/ pDescription,
				REF(const char*) /*OUT*/ pCopyright,
				REF(const char*) /*OUT*/ pMoreInfoURL,
				REF(ULONG32)	 /*OUT*/ ulVersionNumber);

    STDMETHOD(InitPlugin)   (THIS_
			    IUnknown*   /*IN*/  pContext);


    /***********************************************************************
     * IRMADataConvertSystemObject
     */

    STDMETHOD(GetDataConvertInfo) (THIS_ REF(const char*) pShortName);

    STDMETHOD(InitDataConvertSystem) (THIS_ IRMAValues* pOptions);

    STDMETHOD(CreateDataConvert) (THIS_ IUnknown** /*OUT*/ ppConvObj);

private:
    INT32 m_lRefCount;
    IUnknown* m_pContext;
    ULONG32 m_ulConvertHeaders;
    IRMAValues* m_pOptions;
};

class Converter : public IRMADataConvert,
		  public IRMARequestHandler
{
public:
    Converter(IUnknown* pContext, ULONG32 ul);
    ~Converter();

    /*
     *	IUnknown methods
     */
    STDMETHOD(QueryInterface)		(THIS_
					REFIID riid,
					void** ppvObj);

    STDMETHOD_(ULONG32,AddRef)		(THIS);

    STDMETHOD_(ULONG32,Release)		(THIS);

    /************************************************************************
     *  IRMADataConvert
     */
    STDMETHOD(DataConvertInit) (THIS_ IRMADataConvertResponse* pResponse);

    STDMETHOD(ConvertFileHeader) (THIS_ IRMAValues* pFileHeader);
    
    STDMETHOD(ConvertStreamHeader) (THIS_ IRMAValues* pStreamHeader);
    
    STDMETHOD(GetConversionMimeType) (THIS_ REF(const char*) pConversionType);
    
    STDMETHOD(ConvertData) (THIS_ IRMAPacket* pPacket);
    
    STDMETHOD(ControlBufferReady) (THIS_ IRMABuffer* pBuffer);

    STDMETHOD(SetMulticastTransportConverter) (THIS_
	    IRMADataConvert* pConverter);
    
    STDMETHOD(AddMulticastControlConverter) (THIS_
	    IRMADataConvert* pConverter);

    STDMETHOD(Done)			(THIS);

    /************************************************************************
     *	Method:
     *	    IRMARequestHandler::SetRequest
     *	Purpose:
     *	    Associates an IRMARequest with an object
     */
    STDMETHOD(SetRequest)   (THIS_
			    IRMARequest*        /*IN*/  pRequest);

    /************************************************************************
     *	Method:
     *	    IRMARequestHandler::GetRequest
     *	Purpose:
     *	    Gets the IRMARequest object associated with an object
     */
    STDMETHOD(GetRequest)   (THIS_
			    REF(IRMARequest*)        /*OUT*/  pRequest);

    UINT32 GetKey();
private:
    enum RMA_DATA_CONVERT_TYPE
    {
	UNICAST,
	MULTICAST_TRANSPORT,
	MULTICAST_CONTROL
    };
    void XMitKey();
    IRMABuffer* GetConvertedCString(IRMABuffer* pSource);
    IRMABuffer* GetConvertedBuffer(IRMABuffer* pSource);
    IRMABuffer* GetConverted(IRMABuffer* pSource, int len);
    IRMAValues* GetConvertedHeader(IRMAValues* pHeader);

    INT32 m_lRefCount;
    IRMADataConvertResponse* m_pResponse;
    IUnknown* m_pContext;
    IRMACommonClassFactory* m_pClassFactory;
    UINT32 m_ulNumber;
    ULONG32 m_ulConvertHeaders;
    RMA_DATA_CONVERT_TYPE m_type;
    
};

#endif /* _RNCONVERT_H */
