/****************************************************************************
 *  $Id: exallow.h,v 1.1 1998/12/03 18:44:43 darrens Exp $
 *
 *  Copyright (C) 1998 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 *
 *  exallow.h
 * 
 *  Example Allowance plug-in for the RealMedia Architecture (RMA).
 */
#ifndef _EXALLOW_H_
#define _EXALLOW_H_


/****************************************************************************
 * Constants
 */
#define MY_PLUGIN_VERSION    0
#define MY_DESCRIPTION       "Example Allowance Plug-in"
#define MY_COPYRIGHT         "(c) 1998 RealNetworks, All rights reserved."
#define MY_MORE_INFO_URL     "http://www.real.com"


/****************************************************************************
 *
 *  CExampleAllowance Class
 *
 *  This class inherits the minimum number of interfaces required to create
 *  an RMA Allowance plug-in. The IRMAPlayerConnectionAdviseSink interface 
 *  provides notifications about significant events in a connected player's
 *  lifetime. The IRMAPlugin interface is required by all plug-ins to handle 
 *  fundamental plug-in operations. Since this is a COM object, this class 
 *  also inherits COM's IUnknown interface to handle object reference counting 
 *  and interface query.
 */

class CExampleAllowance : public IRMAPlayerConnectionAdviseSink,
                          public IRMAPlugin
{
public:
    CExampleAllowance (void);


    /************************************************************************
     *  IRMAPlayerConnectionAdviseSink Interface Methods     ref:  rmaallow.h
     */
    STDMETHOD (OnConnection)	    (THIS_ 
				     IRMAPlayerConnectionResponse* pResponse);
    STDMETHOD (SetPlayerController) (THIS_ 
				     IRMAPlayerController* pPlayerController);
    STDMETHOD (SetRegistryID)       (THIS_ 
				     UINT32 ulPlayerRegistryID);
    STDMETHOD (OnURL)		    (THIS_ 
				     IRMARequest* pRequest);
    STDMETHOD (OnBegin)		    (THIS);
    STDMETHOD (OnPause)		    (THIS);
    STDMETHOD (OnStop)		    (THIS);
    STDMETHOD (OnDone)		    (THIS);


    /************************************************************************
     *  IRMAPlugin Interface Methods                         ref:  rmaplugn.h
     */
    STDMETHOD(GetPluginInfo)
	    (THIS_ 
	      REF(BOOL)        bLoadMultiple,
	      REF(const char*) pDescription,
	      REF(const char*) pCopyright,
	      REF(const char*) pMoreInfoURL,
	      REF(UINT32)      versionNumber
	    );
    STDMETHOD(InitPlugin) (THIS_ IUnknown* pContext);


    /************************************************************************
     *  IUnknown COM Interface Methods                          ref:  pncom.h
     */
    STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
    STDMETHOD_(UINT32, AddRef ) (THIS);
    STDMETHOD_(UINT32, Release) (THIS);


private:
    /****** Private Class Methods ******************************************/
    ~CExampleAllowance(void);

    PN_RESULT HandleLimitedURLRequest  (void);
    PN_RESULT HandleJukeboxURLRequest  (void);
    void      DecrementLimitedURLCount (void);

    /****** Private Class Variables ****************************************/
    INT32		    m_RefCount;		// Object's reference count
    IRMACommonClassFactory* m_pClassFactory;	// Creates common RMA classes
    IRMAPNRegistry*	    m_pRegistry;	// Accesses server registry

    IRMAPlayerConnectionResponse* m_pPCResponse;
    IRMAPlayerController*   m_pPlayerController;
    
    // Location of player information in the server registry
    UINT32		    m_ulPlayerRegistryID;
    IRMABuffer*		    m_pPlayerRegistryName;

    BOOL		    m_bViewingLimitedURL;


    /****** Private Static Class Variables *********************************/
    static const char*      zm_pDescription;
    static const char*      zm_pCopyright;
    static const char*      zm_pMoreInfoURL;

    PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME       // Avoids GCC compiler warning
};

#endif  /* ifndef _EXALLOW_H_ */
