/****************************************************************************
 *  $Id: ff1rendr.h,v 1.15 1997/11/21 19:16:52 jfarr Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 *
 *  ff1rendr.h
 * 
 *  Example file Renderer plug-in for the RealMedia Architecture (RMA).
 */
#ifndef _FF1RENDR_H_
#define _FF1RENDR_H_


/****************************************************************************
 * Constants
 */
#define MY_PLUGIN_VERSION    0
#define MY_DESCRIPTION       "Example Renderer ff1 Plug-in"
#define MY_COPYRIGHT         "(c) 1995,1996,1997 Progressive Networks, All rights reserved."
#define MY_MORE_INFO_URL     "http://www.real.com"
#define MY_STREAM_MIME_TYPES {"application/x-pn-ff1_stream0", NULL}
#define MY_PACKET_LENGTH     11
#define MY_TIME_SYNC_FREQ    100


/****************************************************************************
 *
 *  CMyExampleRenderer Class
 *
 *  This class inherits the minimum number of interfaces required to create
 *  an RMA Renderer plug-in. The IRMARenderer interface provides routines
 *  for handling all rendering operations. The IRMAPlugin interface is
 *  required by all plug-ins to handle fundamental plug-in operations. Since
 *  this is a COM object, this class also inherits COM's IUnknown interface
 *  to handle object reference counting and interface query.
 */

class CMyExampleRenderer :  public IRMARenderer,
                            public IRMAPlugin
{
public:
	CMyExampleRenderer (void);


	/************************************************************************
	 *  IRMARenderer Interface Methods                       ref:  rmarendr.h
	 */
	STDMETHOD(StartStream) (THIS_ IRMAStream* pStream, IRMAPlayer* pPlayer);
	STDMETHOD(EndStream )  (THIS);
	STDMETHOD(OnHeader  )  (THIS_ IRMAValues* pStreamHeaderObj);
	STDMETHOD(OnPacket  )  (THIS_ IRMAPacket* pPacket, INT32 timeOffset);
	STDMETHOD(OnTimeSync)  (THIS_ UINT32 currentPlayBackTime);
	STDMETHOD(OnPreSeek )  (THIS_ UINT32 timeBefore, UINT32 timeAfter);
	STDMETHOD(OnPostSeek)  (THIS_ UINT32 timeBefore, UINT32 timeAfter);
	STDMETHOD(OnPause   )  (THIS_ UINT32 timeBeforePause);
	STDMETHOD(OnBegin   )  (THIS_ UINT32 timeAfterBegin);
	STDMETHOD(OnBuffering) (THIS_ UINT32 reason, UINT16 percentComplete);
	STDMETHOD(GetRendererInfo)
		(THIS_
		 REF(const char**) pStreamMimeTypes,
		 REF(UINT32)       initialGranularity
		);
	STDMETHOD(GetDisplayType)
		(THIS_
		 REF(RMA_DISPLAY_TYPE) displayType,
		 REF(IRMABuffer*)      pDisplayInfo
		);
	STDMETHOD(OnEndofPackets) (THIS);



	/************************************************************************
	 *  IRMAPlugin Interface Methods                         ref:  rmaplugn.h
	 */
	STDMETHOD(GetPluginInfo)
		(THIS_ 
		  REF(BOOL)        bLoadMultiple,
		  REF(const char*) pDescription,
		  REF(const char*) pCopyright,
		  REF(const char*) pMoreInfoURL,
		  REF(UINT32)      versionNumber
		);
	STDMETHOD(InitPlugin) (THIS_ IUnknown* pRMACore);


	/************************************************************************
	 *  IUnknown COM Interface Methods                          ref:  pncom.h
	 */
	STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
	STDMETHOD_(UINT32, AddRef ) (THIS);
	STDMETHOD_(UINT32, Release) (THIS);


private:
	/****** Private Class Methods ******************************************/
	~CMyExampleRenderer(void);

	/****** Private Class Variables ****************************************/
	INT32                    m_RefCount;       // Object's reference count
	IRMACommonClassFactory*  m_pClassFactory;  // Creates common RMA classes
	BOOL			 m_bInSeekMode;

	/****** Private Static Class Variables *********************************/
	static const char*      zm_pDescription;
	static const char*      zm_pCopyright;
	static const char*      zm_pMoreInfoURL;
	static const char*      zm_pStreamMimeTypes[];

	PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME       // Avoids GCC compiler warning
};

#endif  /* ifndef _FF1RENDR_H_ */
