/****************************************************************************
 *
 *  $Id: filefmt1.h,v 1.18 1997/10/21 20:42:31 christof Exp $
 *
 *  Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 *  All rights reserved.
 *  
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 *
 *  filefmt1.h
 *
 *  Example File Format plug-in for the RealMedia Architecture (RMA).
 */
#ifndef _FILEFMT1_H_
#define _FILEFMT1_H_


/****************************************************************************
 * Constants
 */
#define MY_PLUGIN_VERSION       0
#define MY_DESCRIPTION         "Example File Format 1 Plug-in"
#define MY_COPYRIGHT           "(c) 1995,1996,1997 RealNetworks, All rights reserved."
#define MY_MORE_INFO_URL       "http://www.real.com"
#define MY_FILE_MIME_TYPES     {"application/x-pn-example_fileformat1", NULL}
#define MY_FILE_EXTENSIONS     {"ff1", NULL}
#define MY_FILE_OPEN_NAMES     {"Example File Format (*.ff1)", NULL}
#define MY_STREAM_NO            0
#define MY_STREAM_MIME_TYPE    "application/x-pn-ff1_stream0"
#define MY_FILE_HEADER_START    0
#define MY_FILE_HEADER_LENGTH   7
#define MY_STREAM_HEADER_START  (MY_FILE_HEADER_START + MY_FILE_HEADER_LENGTH)
#define MY_STREAM_HEADER_LENGTH 29
#define MY_PACKET_DATA_START    (MY_STREAM_HEADER_START + MY_STREAM_HEADER_LENGTH)
#define MY_PACKET_LENGTH        11
#define MY_TIME_BETWEEN_PACKETS 1000


/****************************************************************************
 * Typedefs
 */
typedef enum
{
	Ready,
	InitPending,
	GetFileHeaderSeekPending,
	GetFileHeaderReadPending,
	GetStreamHeaderSeekPending,
	GetStreamHeaderReadPending,
	GetPacketSeekPending,
	GetPacketReadPending,
	SeekSeekPending
}
MyState;


/****************************************************************************
 *
 *  CMyExampleFileFormat Class
 *
 *  This class inherits the interfaces required to create a File Format
 *  plug-in. The IRMAFileFormatObject interface consists of routines specific
 *  to File Format plug-ins. The IRMAFileResponse interface is needed because
 *  of the asynchronous nature of the RealMedia file system. All plug-ins
 *  also require the IRMAPlugin interface to handle fundamental plug-in
 *  operations. Since we are using COM, this class also inherits COM's
 *  IUnknown interface to handle reference counting and interface query.
 */

class CMyExampleFileFormat :  public IRMAFileFormatObject,
                              public IRMAFileResponse,
                              public IRMAPlugin
{
	public:
	CMyExampleFileFormat (void);

	/************************************************************************
	 *  IRMAFileFormatObject Interface Methods               ref:  rmaformt.h
	 */
	STDMETHOD(GetFileFormatInfo)
		(THIS_
		  REF(const char**) pFileMimeTypes,
		  REF(const char**) pFileExtensions,
		  REF(const char**) pFileOpenNames
		);

	STDMETHOD(InitFileFormat)
		(THIS_
		  IRMARequest*        pRequest,
		  IRMAFormatResponse* pFormatResponse,
		  IRMAFileObject*     pFileObject
		);

	STDMETHOD(GetFileHeader  ) (THIS);
	STDMETHOD(GetStreamHeader) (THIS_ UINT16 streamNo);
	STDMETHOD(GetPacket      ) (THIS_ UINT16 streamNo);
	STDMETHOD(Seek           ) (THIS_ UINT32 requestedTime);
	STDMETHOD(Close          ) (THIS);


	/************************************************************************
	 *  IRMAFileResponse Interface Methods                   ref:  rmafiles.h
	 */
	STDMETHOD(InitDone ) (THIS_ PN_RESULT status);
	STDMETHOD(SeekDone ) (THIS_ PN_RESULT status);
	STDMETHOD(ReadDone ) (THIS_ PN_RESULT status, IRMABuffer* pBuffer);
	STDMETHOD(WriteDone) (THIS_ PN_RESULT status);
	STDMETHOD(CloseDone) (THIS_ PN_RESULT status);


	/************************************************************************
	 *  IRMAPlugin Interface Methods                         ref:  rmaplugn.h
	 */
	STDMETHOD(GetPluginInfo)
		(THIS_ 
		  REF(BOOL)        bLoadMultiple,
		  REF(const char*) pDescription,
		  REF(const char*) pCopyright,
		  REF(const char*) pMoreInfoURL,
		  REF(UINT32)      versionNumber
		);

	STDMETHOD(InitPlugin) (THIS_ IUnknown* pRMACore);


	/************************************************************************
	 *  IUnknown COM Interface Methods                          ref:  pncom.h
	 */
	STDMETHOD (QueryInterface ) (THIS_ REFIID ID, void** ppInterfaceObj);
	STDMETHOD_(UINT32, AddRef ) (THIS);
	STDMETHOD_(UINT32, Release) (THIS);


	private:
	/****** Private Class Variables ****************************************/
	INT32                   m_RefCount;      // Object's reference count
	IRMACommonClassFactory* m_pClassFactory; // Creates common RMA classes
	IRMAFileObject*         m_pFileObj;      // Used for file I/O
	IRMAFormatResponse*     m_pStatus;       // Reports status to RMA core
	MyState                 m_State;         // State used for asynch. calls
	UINT32                  m_NextPacketDeliveryTime;
	                                         // Delivery time of next packet
	                                         
	/****** Private Static Class Variables *********************************/
	static const char*     zm_pDescription;
	static const char*     zm_pCopyright;
	static const char*     zm_pMoreInfoURL;
	static const char*     zm_pFileMimeTypes[];
	static const char*     zm_pFileExtensions[];
	static const char*     zm_pFileOpenNames[];

	/****** Private Class Methods ******************************************/
	~CMyExampleFileFormat(void);
	void MyCreateFileHeaderObj   (PN_RESULT status, IRMABuffer* pDataRead);
	void MyCreateStreamHeaderObj (PN_RESULT status, IRMABuffer* pDataRead);
	void MyCreatePacketObj       (PN_RESULT status, IRMABuffer* pDataRead);

	PRIVATE_DESTRUCTORS_ARE_NOT_A_CRIME      // Avoids GCC compiler warning
};

#endif  /* ifndef _FILEFMT1_H_ */

