/****************************************************************************
 * $Id: reverter.cpp,v 1.3 2000/05/10 03:04:16 paulm Exp $
 *
 *  Copyright (C) 1995,1996,1997,1998,1999,2000 RealNetworks, Inc.
 *  All rights reserved.
 *
 *  http://www.real.com/devzone
 *
 *  This program contains proprietary information of RealNetworks, Inc.,
 *  and is licensed subject to restrictions on use and distribution.
 *
 *  reverter.cpp
 *
 *  Example client side data reverter plug-in for the RealMedia 
 *  Architecture (RMA).
 */

#define INITGUID

#include <stdio.h>
#include <stdlib.h>

#include "pncom.h"
#include "rmacomm.h"
#include "rmaplugn.h"
#include "rmapckts.h"
#include "rmadtcvt.h"

#include "reverter.h"

const char* Reverter::zm_pConvertTypes[] =
{
    MY_CONVERSION_MIMETYPE,
    NULL
};

/****************************************************************************
 *  RMACreateInstance                                        ref:  rmaplugn.h
 *
 *  This routine creates a new instance of the CMyExampleFileSystem class.
 *  It is called when the RMA core application is launched, and whenever
 *  an URL with a protocol associated with this plug-in is opened.
 */
STDAPI
RMACreateInstance(IUnknown** pp)
{
    *pp = (IUnknown*)(IRMAPlugin*)new Reverter;
    if (*pp != NULL)
    {
            (*pp)->AddRef();
            return PNR_OK;
    }

    return PNR_OUTOFMEMORY;
}

/*
 * Utility function for inspecting IRMAValues (Headers).
 */
void
DumpValues(IRMAValues* pValues)
{
    PN_RESULT res;
    IRMABuffer* pBuffer;
    const char* pName;
    ULONG32 ul;

    printf("BUFFER\n");
    res = pValues->GetFirstPropertyBuffer(pName, pBuffer);
    while (res == PNR_OK)
    {
	printf("%s: %s\n", pName, (const char*)pBuffer->GetBuffer());
	pBuffer->Release();
	res = pValues->GetNextPropertyBuffer(pName, pBuffer);
    }
    
    printf("ULONG32\n");
    res = pValues->GetFirstPropertyULONG32(pName, ul);
    while (res == PNR_OK)
    {
	printf("%s: %lu\n", pName, ul);
	res = pValues->GetNextPropertyULONG32(pName, ul);
    }
    
    printf("CSTRING\n");
    res = pValues->GetFirstPropertyCString(pName, pBuffer);
    while (res == PNR_OK)
    {
	printf("%s: %s\n", pName, (const char*)pBuffer->GetBuffer());
	pBuffer->Release();
	res = pValues->GetNextPropertyCString(pName, pBuffer);
    }
}

/*
 * Reverter
 */
Reverter::Reverter()
: m_lRefCount(0)
, m_pContext(0)
, m_pResp(0)
, m_pClassFactory(0)
, m_ulNumber(0)
{
}

Reverter::~Reverter()
{
    PN_RELEASE(m_pContext);
    PN_RELEASE(m_pResp);
    PN_RELEASE(m_pClassFactory);
}

/*
 *	IUnknown methods
 */
STDMETHODIMP
Reverter::QueryInterface(REFIID riid, void** ppvObj)
{
    if (IsEqualIID(riid, IID_IUnknown))
    {
	AddRef();
	*ppvObj = (IRMAPlugin*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMAPlugin))
    {
	AddRef();
	*ppvObj = (IRMAPlugin*)this;
	return PNR_OK;
    }
    else if (IsEqualIID(riid, IID_IRMADataRevert))
    {
	AddRef();
	*ppvObj = (IRMADataRevert*)this;
	return PNR_OK;
    }
    
    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}

STDMETHODIMP_(ULONG32)
Reverter::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}

STDMETHODIMP_(ULONG32)
Reverter::Release()
{
    if (InterlockedDecrement(&m_lRefCount) > 0)
    {
	return m_lRefCount;
    }
    delete this;
    return 0;
}

/*
 *	IRMAPlugin methods
 */
STDMETHODIMP
Reverter::GetPluginInfo(REF(BOOL)	 /*OUT*/ bMultipleLoad,
			REF(const char*) /*OUT*/ pDescription,
			REF(const char*) /*OUT*/ pCopyright,
			REF(const char*) /*OUT*/ pMoreInfoURL,
			REF(ULONG32)	 /*OUT*/ ulVersionNumber)
{
    bMultipleLoad = TRUE;
    pDescription = MY_DESCRIPTION;
    pCopyright = MY_COPYRIGHT;
    pMoreInfoURL = MY_MORE_INFO_URL;
    ulVersionNumber = MY_PLUGIN_VERSION;
    return PNR_OK;
}

STDMETHODIMP
Reverter::InitPlugin(IUnknown*   /*IN*/  pContext)
{
    PN_RELEASE(m_pContext);
    m_pContext = pContext;
    m_pContext->AddRef();
    m_pContext->QueryInterface(IID_IRMACommonClassFactory,
	    (void**)&m_pClassFactory);
    return PNR_OK;
}

/************************************************************************
 *  Method: IRMADataRevert
 *
 *  Purpose: Initialize the reverter object with response pointer.
 */
STDMETHODIMP
Reverter::DataRevertInit(IRMADataRevertResponse* pResponse)
{
    m_pResp = pResponse;
    m_pResp->AddRef();
    m_pResp->DataRevertInitDone(PNR_OK);
    return PNR_OK;
}

/************************************************************************
 * Method: GetDataRevertInfo
 *
 * Purpose: Return NULL terminated array of conversion mime types this
 *          reverter is willing to handle.
 */
STDMETHODIMP
Reverter::GetDataRevertInfo(REF(const char**)
				ppConversionTypes)
{
    ppConversionTypes = zm_pConvertTypes;
    return PNR_OK;
}

/************************************************************************
 * Method: RevertFileHeader
 *
 * Purpose: Player core is going to start sending you data beginning with
 *          these file headers right here.
 */
STDMETHODIMP
Reverter::RevertFileHeader(IRMAValues* pFileHeader)
{
    IRMAValues* pValues;
    if (m_ulRevertHeaders)
    {
	pValues = GetRevertedHeader(pFileHeader);
	DumpValues(pValues);
	m_pResp->RevertedFileHeaderReady(PNR_OK, pValues);
	pValues->Release();

    }
    else
    {
	DumpValues(pFileHeader);
	m_pResp->RevertedFileHeaderReady(PNR_OK, NULL);

    }

    return PNR_OK;
}

/************************************************************************
 * Method: RevertStreamHeader
 *
 * Purpose: Next, the core sends you stream headers for each of the
 *          steams.
 */
STDMETHODIMP
Reverter::RevertStreamHeader(IRMAValues* pStreamHeader)
{
    if (m_ulRevertHeaders)
    {
	IRMAValues* pValues = GetRevertedHeader(pStreamHeader);
	m_pResp->RevertedStreamHeaderReady(PNR_OK, pValues);
	pValues->Release();
    }
    else
    {
	m_pResp->RevertedStreamHeaderReady(PNR_OK, NULL);
    }

    return PNR_OK;
}

/************************************************************************
 * Method: RevertData
 *
 * Purpose: The player is now sending you the data that you converted
 *          on the server.
 */
STDMETHODIMP
Reverter::RevertData(IRMAPacket* pPacket)
{
    IRMABuffer* pBuffer;
    UINT32 ulTime;
    UINT16 unStreamNumber;
    UINT8  unASMFlags;
    UINT16 unASMRuleNumber;
    IRMAPacket* pNewPacket;
    m_pClassFactory->CreateInstance(CLSID_IRMAPacket, (void**)&pNewPacket);

    pPacket->Get(pBuffer, ulTime, unStreamNumber, unASMFlags, unASMRuleNumber);
    IRMABuffer* pNewBuf = GetRevertedBuffer(pBuffer);
    pNewPacket->Set(pNewBuf, ulTime, unStreamNumber,
	    unASMFlags, unASMRuleNumber);
    pNewBuf->Release();
    pBuffer->Release();
    m_pResp->RevertedDataReady(PNR_OK, pNewPacket);
    pNewPacket->Release();
    return PNR_OK;
}

/************************************************************************
 * Method: ControlBufferReady
 *
 * Purpose: The converter on the server is sending us a message.  This
 *          plugin parses the message for xor key and whether or not
 *          the headers are coming in reverted.
 */
STDMETHODIMP
Reverter::ControlBufferReady(IRMABuffer* pBuffer)
{
    const char* pToSend = "Ok, I got it, thanks.";
    const char* pc = (const char*)pBuffer->GetBuffer();
    sscanf(pc, "init%lu %d", &m_ulNumber, &m_ulRevertHeaders);
    IRMABuffer* pNewBuffer;
    m_pClassFactory->CreateInstance(CLSID_IRMABuffer, (void**)&pNewBuffer);
    pNewBuffer->Set((unsigned char*)pToSend, strlen(pToSend) + 1);
    m_pResp->SendControlBuffer(pNewBuffer);
    pNewBuffer->Release();
    return PNR_OK;
}

/*
 * Local helper funcs
 */
IRMABuffer*
Reverter::GetRevertedBuffer(IRMABuffer* pSource)
{
    IRMABuffer* pNewBuf;
    m_pClassFactory->CreateInstance(CLSID_IRMABuffer,
	    (void**)&pNewBuf);
    
    pNewBuf->SetSize(pSource->GetSize());

    const char* pFrom;
    char* pTo;
    
    pFrom = (const char*)pSource->GetBuffer();
    pTo = (char*)pNewBuf->GetBuffer();
    
    int i;
    
    for (i = 0; i < (int)pNewBuf->GetSize(); i++)
    {
	if (i % 2 == 0)
	{
	    *pTo = (*pFrom) ^ (char)m_ulNumber;
	}
	else
	{
	    *pTo = *pFrom;
	}
	pTo++;
	pFrom++;
    }
    return pNewBuf;
}

IRMAValues*
Reverter::GetRevertedHeader(IRMAValues* pHeader)
{
    const char* pName;
    IRMABuffer* pBuffer;
    PN_RESULT res;
    IRMAValues* pValues;
    ULONG32 ul;
    
    m_pClassFactory->CreateInstance(CLSID_IRMAValues, (void**)&pValues);
    
    /* ULONG32 */
    res = pHeader->GetFirstPropertyULONG32(pName, ul);
    while (res == PNR_OK)
    {
	pValues->SetPropertyULONG32(pName, ul);
	res = pHeader->GetNextPropertyULONG32(pName, ul);
    }
    
    /* CString */
    res = pHeader->GetFirstPropertyCString(pName, pBuffer);
    while (res == PNR_OK)
    {
	IRMABuffer* pNewBuf = GetRevertedBuffer(pBuffer);
	pValues->SetPropertyCString(pName, pNewBuf);
	pNewBuf->Release();
	pBuffer->Release();
	res = pHeader->GetNextPropertyCString(pName, pBuffer);
    }
    
    /* Buffer */
    res = pHeader->GetFirstPropertyBuffer(pName, pBuffer);
    while (res == PNR_OK)
    {
	IRMABuffer* pNewBuf = GetRevertedBuffer(pBuffer);
	pValues->SetPropertyBuffer(pName, pNewBuf);
	pNewBuf->Release();
	pBuffer->Release();
	res = pHeader->GetNextPropertyBuffer(pName, pBuffer);
    }
    
    return pValues;

}
